! =================================================================================================================================
! MODULE       : function_library
!
! CONTACT      : orchidee-help _at_ ipsl.jussieu.fr
!
! LICENCE      : IPSL (2006)
! This software is governed by the CeCILL licence see ORCHIDEE/ORCHIDEE_CeCILL.LIC
!
!>\BRIEF       Collection of functions that are used throughout the ORCHIDEE code
!!
!!\n DESCRIPTION: Collection of modules to : (1) convert one variable into another i.e. basal area
!! to diameter, diamter to tree height, diameter to crown area, etc. (2) ...
!!
!! RECENT CHANGE(S): None
!!
!! REFERENCE(S)	:
!!
!! SVN          :
!! $HeadURL: svn://forge.ipsl.jussieu.fr/orchidee/branches/ORCHIDEE-DOFOCO/ORCHIDEE/src_stomate/stomate_prescribe.f90 $
!! $Date: 2013-01-04 16:50:56 +0100 (Fri, 04 Jan 2013) $
!! $Revision: 1126 $
!! \n
!_ ================================================================================================================================

MODULE function_library

  ! modules used:

!$  USE ioipsl_para
  USE pft_parameters
  USE constantes

  IMPLICIT NONE

  ! private & public routines

  PRIVATE
  PUBLIC calculate_c0_alloc, wood_to_ba_eff_array, wood_to_ba_eff, wood_to_ba, &
       wood_to_height_eff, wood_to_height, wood_to_qmheight, wood_to_dia_eff, &
       wood_to_dia, wood_to_qmdia, wood_to_circ, wood_to_cn_array, &
       wood_to_cn, wood_to_cn_eff, wood_to_cv, wood_to_cv_eff, wood_to_volume, &
       biomass_to_lai, Nmax, Nmaxyield, Nminyield, distribute_mortality_biomass, &
       check_biomass_sync, biomass_to_coupled_lai, check_area 

  CONTAINS

 !! ================================================================================================================================
!! FUNCTION    : biomass_to_coupled_lai
!!
!>\BRIEF        Calculate the coupled_LAI based on biomass and veget of each pft
!!
!! DESCRIPTION : Calculates the lai that coupled with the atmosphere  
!! 
!!             
!! 
!! RECENT CHANGE(S): None
!!
!! RETURN VALUE : ::coupled_lai (m2/m2)
!!
!! REFERENCE(S)	: 
!!
!! FLOWCHART	: None
!! \n
!_ ================================================================================================================================
   
  FUNCTION biomass_to_coupled_lai(biomass_leaf, veget, pft)

 !! 0. Variable and parameter declaration

    !! 0.1 Input variables
    INTEGER(i_std)                                       :: pft                    !! PFT number (-)
    REAL(r_std)                                          :: veget                  !! 1-Pgap or the vegetation cover
    REAL(r_std)                                          :: biomass_leaf           !! Biomass of an individual tree within a circ 
                                                                                   !! class @tex $(m^{2} ind^{-1})$ @endtex 

    !! 0.2 Output variables         
    REAL(r_std)                                          :: biomass_to_coupled_lai !! The fraction of the LAI that is assumed to interact
                                                                                   !! with the atmosphere @tex $(m^{2} m^{-2})$ @endtex
    !! 0.3 Modified variables

    !! 0.4 Local variables
    REAL(r_std)                                          :: lai                    !! Leaf area index 
                                                                                   !! @tex $(m^{2} m^{-2})$ @endtex               
!_ ================================================================================================================================
 
    lai = biomass_to_lai(biomass_leaf, pft) 
   
    !+++CHECK+++
    ! In a closed canopy not all the leaves fully interact with the 
    ! atmosphere because leaves can shelter each other. In a more 
    ! open canopy most leaves can interact with the atmosphere.
    ! For the moment we are not clear how to calculate the fraction of
    ! the canopy that is coupled to the atmosphere. Also, based on the
    ! simulated evapotranspiration we need the entire canopy to be
    ! coupled to the atmosphere to obtain simulations which are close
    ! to the observations (Jung's upscaled product).
    IF (veget .GT. min_sechiba) THEN
     
       biomass_to_coupled_lai = lai   

    ELSE
   
       ! There are so many gaps that veget is extremely small. It is fair to
       ! assume that the whole canopy is coupled to the atmosphere.
       biomass_to_coupled_lai = lai
      
    ENDIF

  END FUNCTION biomass_to_coupled_lai
 
  
!! ================================================================================================================================
!! FUNCTION     : calculate_c0_alloc
!!
!>\BRIEF        Calculate the baseline root vs sapwood allocation
!!
!! DESCRIPTION : Calculates the baseline root vs sapwood allocation based on the 
!! parameters of the pipe model (hydraulic conductivities) and the
!! turnover of the different components              
!! 
!! RECENT CHANGE(S): None
!!
!! RETURN VALUE : ::c0_alloc (m)
!!
!! REFERENCE(S)	: 
!!
!! FLOWCHART	: None
!! \n
!_ ================================================================================================================================
   
  FUNCTION calculate_c0_alloc(pts, pft, tau_eff_root, tau_eff_sap)

 !! 0. Variable and parameter declaration

    !! 0.1 Input variables

    INTEGER(i_std)                             :: pts               !! Pixel number (-)
    INTEGER(i_std)                             :: pft               !! PFT number (-)
    REAL(r_std)                                :: tau_eff_root      !! Effective longivety for leaves (days)
    REAL(r_std)                                :: tau_eff_sap       !! Effective longivety for leaves (days)
    
    !! 0.2 Output variables
                
    REAL(r_std)                                :: calculate_c0_alloc  !! quadratic mean height (m) 

    !! 0.3 Modified variables

    !! 0.4 Local variables
    REAL(r_std)                                :: sapwood_density
    REAL(r_std)                                :: qm_dia            !! quadratic mean diameter (m)

!_ ================================================================================================================================
 
    !! 1. Calculate c0_alloc
    IF ( is_tree(pft) ) THEN

       sapwood_density = deux * pipe_density(pft) / kilo_to_unit
       calculate_c0_alloc = sqrt(k_root(pft)/k_sap(pft)*tau_eff_sap/tau_eff_root*sapwood_density)

    ! Grasses and croplands   
    ELSE

       !+++CHECK+++
       ! Simply copied the same formulation as for trees but note
       ! that the sapwood in trees vs grasses and crops has a very
       ! meaning. In grasses and crops is structural carbon to ensure
       ! that the allocation works. In trees it really is the sapwood
       sapwood_density = deux * pipe_density(pft) / kilo_to_unit
       calculate_c0_alloc = sqrt(k_root(pft)/k_sap(pft)*tau_eff_sap/tau_eff_root*sapwood_density)
       !+++++++++++

    ENDIF ! is_tree(j)

 END FUNCTION calculate_c0_alloc



!! ================================================================================================================================
!! FUNCTION     : wood_to_ba_eff_array
!!
!>\BRIEF        Calculate the effective basal area from woody biomass making use of allometric relationships.
!!
!! DESCRIPTION : Calculate basal area of an individual tree from the woody biomass of that tree making 
!! use of allometric relationships. Effective basal area accounts for both above and below ground carbon
!! and is the basis for the application of the rule of Deleuze and Dhote
!! 
!! RECENT CHANGE(S): None
!!
!! RETURN VALUE : effective basal area (m2 ind-1)
!!
!! REFERENCE(S)	: 
!!
!! FLOWCHART	: None
!! \n
!_ ================================================================================================================================
   
  FUNCTION wood_to_ba_eff_array(biomass_temp, npts, pft)

 !! 0. Variable and parameter declaration

    !! 0.1 Input variables

    INTEGER(i_std)                                          :: pft                  !! PFT number (-)
    INTEGER(i_std)                                          :: npts                 !! Number of pixels
    REAL(r_std), DIMENSION(:,:,:)                           :: biomass_temp         !! Biomass of an individual tree within a circ 
                                                                                    !! class @tex $(m^{2} ind^{-1})$ @endtex 

    !! 0.2 Output variables
                
    REAL(r_std), DIMENSION(npts,ncirc)                      :: wood_to_ba_eff_array !! Effective basal area of an individual tree within a circ
                                                                                    !! class @tex $(m^{2} ind^{-1})$ @endtex

    !! 0.3 Modified variables

    !! 0.4 Local variables
    INTEGER(i_std)                                          :: l                    !! Index
    REAL(r_std), DIMENSION(npts)                            :: woodmass_ind         !! Woodmass of an individual tree
                                                                                    !! @tex $(gC ind{-1})$ @endtex

!_ ================================================================================================================================
 
 !! 1. Calculate effective basal area from woodmass

    IF ( is_tree(pft) ) THEN
       
       DO l = 1,ncirc
         
          ! Woodmass of an individual tree. Note that for the effective basal area 
          ! both the above and belowground biomass are used.
          woodmass_ind(:) = biomass_temp(:,l,isapabove) + biomass_temp(:,l,isapbelow) + &
             biomass_temp(:,l,iheartabove) + biomass_temp(:,l,iheartbelow)

          ! Basal area of that individual (m2 ind-1)  
          wood_to_ba_eff_array(:,l) = (pi/4*(woodmass_ind(:)/&
               (tree_ff(pft)*pipe_density(pft)*pipe_tune2(pft))) &
               **(2./pipe_tune3(pft)))**(pipe_tune3(pft)/(pipe_tune3(pft)+2))
                 
       ENDDO

    ELSE

       WRITE(numout,*) 'function wood_to_ba_eff_array is not defined for this PFT'
       WRITE(numout,*) 'pft ',pft
       CALL ipslerr_p (3,'wood_to_ba_eff_array', &
            'wood_to_ba_eff_array is not defined for this PFT.', &
            'See the output file for more details.','')


    ENDIF

  END FUNCTION wood_to_ba_eff_array


!! ================================================================================================================================
!! FUNCTION     : wood_to_ba_eff
!!
!>\BRIEF        Calculate effective basal area from woody biomass making use of allometric relationships
!!
!! DESCRIPTION :  Calculate basal area of an individual tree from the woody biomass of that tree making 
!! use of allometric relationships. Effective basal area accounts for both above and below ground carbon
!! and is the basis for the application of the rule of Deleuze and Dhote.
!! (i) woodmass = tree_ff * pipe_density*ba*height
!! (ii) height = pipe_tune2 * sqrt(4/pi*ba) ** pipe_tune_3  
!! 
!! RECENT CHANGE(S): None
!!
!! RETURN VALUE : effective basal area (m2 ind-1)
!!
!! REFERENCE(S)	: 
!!
!! FLOWCHART	: None
!! \n
!_ ================================================================================================================================
 
 FUNCTION wood_to_ba_eff(biomass_temp, pft)

 !! 0. Variable and parameter declaration

    !! 0.1 Input variables

    INTEGER(i_std)                                          :: pft                  !! PFT number (-)
    REAL(r_std), DIMENSION(:,:)                             :: biomass_temp         !! Biomass of an individual tree within a circ 
                                                                                    !! class @tex $(m^{2} ind^{-1})$ @endtex 

    !! 0.2 Output variables
                
    REAL(r_std), DIMENSION(ncirc)                           :: wood_to_ba_eff       !! Effective basal area of an individual tree within a circ
                                                                                    !! class @tex $(m^{2} ind^{-1})$ @endtex

    !! 0.3 Modified variables

    !! 0.4 Local variables
    INTEGER(i_std)                                          :: l                    !! Index
    REAL(r_std)                                             :: woodmass_ind         !! Woodmass of an individual tree
                                                                                    !! @tex $(gC ind^{-1})$ @endtex

!_ ================================================================================================================================
 
 !! 1. Calculate basal area from woodmass

    IF ( is_tree(pft) ) THEN
       
       DO l = 1,ncirc
         
          ! Woodmass of an individual tree
          woodmass_ind = biomass_temp(l,isapabove) + biomass_temp(l,isapbelow) + &
             biomass_temp(l,iheartabove) + biomass_temp(l,iheartbelow)

          ! Basal area of that individual (m2 ind-1)  
          wood_to_ba_eff(l) = (pi/4*(woodmass_ind/(tree_ff(pft)*pipe_density(pft)*pipe_tune2(pft))) &
                  **(2./pipe_tune3(pft)))**(pipe_tune3(pft)/(pipe_tune3(pft)+2))
                 
       ENDDO

    ELSE

       WRITE(numout,*) 'pft ',pft
       CALL ipslerr_p (3,'wood_to_ba_eff', &
            'wood_to_ba_eff is not defined for this PFT.', &
            'See the output file for more details.','')

    ENDIF

 END FUNCTION wood_to_ba_eff



!! ================================================================================================================================
!! FUNCTION     : wood_to_ba
!!
!>\BRIEF        Calculate basal area from woody biomass making use of allometric relationships
!!
!! DESCRIPTION : Calculate basal area of an individual tree from the woody biomass of that tree making 
!! use of allometric relationships given below. Here basal area is defined in line with its classical 
!! forestry meaning.
!! (i) woodmass = tree_ff * pipe_density*ba*height
!! (ii) height = pipe_tune2 * sqrt(4/pi*ba) ** pipe_tune_3  
!! 
!! RECENT CHANGE(S): None
!!
!! RETURN VALUE : basal area (m2 ind-1)
!!
!! REFERENCE(S)	: 
!!
!! FLOWCHART	: None
!! \n
!_ ================================================================================================================================
 
 FUNCTION wood_to_ba(biomass_temp, pft)

 !! 0. Variable and parameter declaration

    !! 0.1 Input variables

    INTEGER(i_std)                                          :: pft               !! PFT number (-)
    REAL(r_std), DIMENSION(:,:)                             :: biomass_temp      !! Biomass of an individual tree within a circ 
                                                                                 !! class @tex $(m^{2} ind^{-1})$ @endtex 

    !! 0.2 Output variables
                
    REAL(r_std), DIMENSION(ncirc)                           :: wood_to_ba        !! Basal area of an individual tree within a circ
                                                                                 !! class @tex $(m^{2} ind^{-1})$ @endtex

    !! 0.3 Modified variables

    !! 0.4 Local variables
    INTEGER(i_std)                                          :: l                 !! Index
    REAL(r_std)                                             :: woodmass_ind      !! Woodmass of an individual tree
                                                                                 !! @tex $(gC ind^{-1})$ @endtex

!_ ================================================================================================================================
 
 !! 1. Calculate basal area from woodmass

    IF ( is_tree(pft) ) THEN
       
       DO l = 1,ncirc
         
          ! Woodmass of an individual tree
          woodmass_ind = biomass_temp(l,iheartabove) + biomass_temp(l,isapabove)


          ! Basal area of that individual (m2 ind-1)  
          wood_to_ba(l) = (pi/4*(woodmass_ind/(tree_ff(pft)*pipe_density(pft)*pipe_tune2(pft))) &
                  **(2./pipe_tune3(pft)))**(pipe_tune3(pft)/(pipe_tune3(pft)+2))
                 
       ENDDO

    ELSE

       WRITE(numout,*) 'pft ',pft
       CALL ipslerr_p (3,'wood_to_ba', &
            'wood_to_ba is not defined for this PFT.', &
            'See the output file for more details.','')

    ENDIF

 END FUNCTION wood_to_ba


!! ================================================================================================================================
!! FUNCTION     : wood_to_height_eff
!!
!>\BRIEF        Calculate the effective tree height from woody biomass making use of allometric relationships
!!
!! DESCRIPTION : Calculate the effective height of an individual tree from the woody biomass of that tree making 
!! use of allometric relationships. Effective height makes use of both above and belowground biomass and is
!! used in the calculation of the allocation according to deleuze and dhote, hydraulic architecture because also
!! the height of the belowground part should be included.
!! (i) height(:) = pipe_tune2(j)*(4/pi*ba(:))**(pipe_tune3(j)/2) and
!! (ii) woodmass_ind = tree_ff*pipe_density*ba*height   
!! 
!! RECENT CHANGE(S): None
!!
!! RETURN VALUE : height (m)
!!
!! REFERENCE(S)	: 
!!
!! FLOWCHART	: None
!! \n
!_ ================================================================================================================================
   
  FUNCTION wood_to_height_eff(biomass_temp, pft)

 !! 0. Variable and parameter declaration

    !! 0.1 Input variables

    INTEGER(i_std)                                          :: pft                !! PFT number (-)
    REAL(r_std), DIMENSION(:,:)                             :: biomass_temp       !! Biomass of an individual tree within a circ 
                                                                                  !! class @tex $(m^{2} ind^{-1})$ @endtex 

    !! 0.2 Output variables
                
    REAL(r_std), DIMENSION(ncirc)                           :: wood_to_height_eff !! Effective height of an individual tree within a circ
                                                                                  !! class (m)

    !! 0.3 Modified variables

    !! 0.4 Local variables
    INTEGER(i_std)                                          :: l                  !! Index
    REAL(r_std)                                             :: woodmass_ind       !! Woodmass of an individual tree
                                                                                  !! @tex $(gC ind{-1})$ @endtex
                                                                                  
    CHARACTER(len=256)  :: tmp_var_name                                           !! temporal variable for a text reading (from run.def)   
    REAL(r_std)         :: tmp_ratio                                              !! temporal variable for adjusting the height   
!_ ================================================================================================================================
 
 !! 1. Calculate height from woodmass

    IF ( is_tree(pft) ) THEN
       
       DO l = 1,ncirc
         
          ! Woodmass of an individual tree. Both above and belowground biomass are used
          woodmass_ind = biomass_temp(l,isapabove) + biomass_temp(l,isapbelow) + &
             biomass_temp(l,iheartabove) + biomass_temp(l,iheartbelow)

          ! Height of that individual
          wood_to_height_eff(l) = (((woodmass_ind/(tree_ff(pft)*pipe_density(pft))*4/pi)**(pipe_tune3(pft)/2))&
               *pipe_tune2(pft))**(1/(pipe_tune3(pft)/2+1))
          
          !+++++++ TEMP ++++++++++
          ! This code is only used evaluation of the performance of the multi-layer energy budget. 
          ! To reduce the complexity of the tests we want to impose the LAI and its vertical distribution. 
          ! The solution is not very elegant but it works. (To start the simulation, a retsart file with height stand is suggested) 
          ! This part of code use the CIRC_HEIGHT_RATIO to adjust the original heigth in each circonferance class.
          ! By doing so, the canopy sturcture can be adjust to match the site level observations. 
               IF (ld_fake_height)  THEN
          !        WRITE(numout,*)'+++ fake wood to height function +++'
                   !set initial the tmp_ratio reaul to 1.0
                   tmp_ratio=1.0
                   WRITE(tmp_var_name, '(A19,I5.5)') "CIRC_HEIGHT_RATIO__", l
                   CALL getin_p(TRIM(tmp_var_name),  tmp_ratio)
                   wood_to_height_eff(l) =tmp_ratio*wood_to_height_eff(l)        
           !       WRITE(numout,*) 'SET CIRC_', l, '_HEIGHT = ', wood_to_height_eff(l) 
               ENDIF                       
           !++++++++++++++++++++++               
 
       ENDDO

    ELSE

       WRITE(numout,*) 'pft ',pft
       CALL ipslerr_p (3,'wood_to_height_eff', &
            'wood_to_height_eff is not defined for this PFT.', &
            'See the output file for more details.','')

    ENDIF

 END FUNCTION wood_to_height_eff


!! ================================================================================================================================
!! FUNCTION     : wood_to_height
!!
!>\BRIEF        Calculate tree height from woody biomass making use of allometric relationships
!!
!! DESCRIPTION : Calculate height of an individual tree from the woody biomass of that tree making 
!! use of allometric relationships. This is the height used in forestry and for calculating the aerodynamic
!! interactions 
!! (i) height(:) = pipe_tune2(j)*(4/pi*ba(:))**(pipe_tune3(j)/2) and
!! (ii) woodmass_ind = tree_ff*pipe_density*ba*height   
!! 
!! RECENT CHANGE(S): None
!!
!! RETURN VALUE : height (m)
!!
!! REFERENCE(S)	: 
!!
!! FLOWCHART	: None
!! \n
!_ ================================================================================================================================
   
  FUNCTION wood_to_height(biomass_temp, pft)

 !! 0. Variable and parameter declaration

    !! 0.1 Input variables

    INTEGER(i_std)                                          :: pft                !! PFT number (-)
    REAL(r_std), DIMENSION(:,:)                             :: biomass_temp       !! Biomass of an individual tree within a circ 
                                                                                  !! class @tex $(m^{2} ind^{-1})$ @endtex 

    !! 0.2 Output variables
                
    REAL(r_std), DIMENSION(ncirc)                           :: wood_to_height     !! Height of an individual tree within a circ
                                                                                  !! class (m)

    !! 0.3 Modified variables

    !! 0.4 Local variables
    INTEGER(i_std)                                          :: l                  !! Index
    REAL(r_std)                                             :: woodmass_ind       !! Woodmass of an individual tree
                                                              !! @tex $(gC ind{-1})$ @endtex

!_ ================================================================================================================================
 
 !! 1. Calculate height from woodmass

    IF ( is_tree(pft) ) THEN
       
       DO l = 1,ncirc
         
          ! Woodmass of an individual tree (only the aboveground component)
          woodmass_ind = biomass_temp(l,isapabove) + biomass_temp(l,iheartabove)

          ! Height of that individual
          wood_to_height(l) = (((woodmass_ind/(tree_ff(pft)*pipe_density(pft))*4/pi)**(pipe_tune3(pft)/2))&
               *pipe_tune2(pft))**(1/(pipe_tune3(pft)/2+1))
            
       ENDDO

    ELSE

       WRITE(numout,*) 'pft ',pft
       CALL ipslerr_p (3,'wood_to_height', &
            'wood_to_height is not defined for this PFT.', &
            'See the output file for more details.','')

    ENDIF

 END FUNCTION wood_to_height


!! ================================================================================================================================
!! FUNCTION     : wood_to_qmheight
!!
!>\BRIEF        Calculate the quadratic mean height from the biomass
!!
!! DESCRIPTION : Calculates the quadratic mean height from the biomass
!! 
!! RECENT CHANGE(S): None
!!
!! RETURN VALUE : ::qm_height (m)
!!
!! REFERENCE(S)	: 
!!
!! FLOWCHART	: None
!! \n
!_ ================================================================================================================================
   
  FUNCTION wood_to_qmheight(biomass_temp, ind, pft)

 !! 0. Variable and parameter declaration

    !! 0.1 Input variables

    INTEGER(i_std)                             :: pft               !! PFT number (-)
    REAL(r_std), DIMENSION(ncirc,nparts)       :: biomass_temp      !! Biomass of the leaves @tex $(gC m^{-2})$ @endtex
    REAL(r_std), DIMENSION(ncirc)              :: ind               !! Number of individuals @tex $(m^{-2})$ @endtex


    !! 0.2 Output variables
                
    REAL(r_std)                                :: wood_to_qmheight  !! quadratic mean height (m) 

    !! 0.3 Modified variables

    !! 0.4 Local variables
    REAL(r_std), DIMENSION(ncirc)              :: circ_class_ba     !! basal area for each circ_class @tex $(m^{2})$ @endtex
    REAL(r_std)                                :: qm_dia            !! quadratic mean diameter (m)

!_ ================================================================================================================================
 
    !! 1. Calculate qm_height from the biomass
    IF ( is_tree(pft) ) THEN

       ! Basal area at the tree level (m2 tree-1)
       circ_class_ba(:) = wood_to_ba(biomass_temp(:,:),pft) 
       
       IF (SUM(ind(:)) .NE. zero) THEN

          qm_dia = SQRT( 4/pi*SUM(circ_class_ba(:)*ind(:))/SUM(ind(:)) )
          
       ELSE
          
          qm_dia = zero

       ENDIF
       
       wood_to_qmheight = pipe_tune2(pft)*(qm_dia**pipe_tune3(pft))
             

    ! Grasses and croplands   
    ELSE

       ! Calculate height as a function of the leaf and structural biomass. Use structural
       ! biomass to make sure that the grasslands have a roughness length during the winter
       ! If the biomass increases, vegetation height will increase as well. Divide by 
       ! ind(ipts,j) to obtain the height of the individual. biomass(ileaf) is in gC m-2 
       ! whereas qm is the height of the individual.
       IF (SUM(ind(:)) .NE. zero) THEN

          wood_to_qmheight = SUM(biomass_temp(:,ileaf) + biomass_temp(:,isapabove)) / &
               SUM(ind(:)) * sla(pft) * lai_to_height(pft)

       ELSE

          wood_to_qmheight = zero

       ENDIF

    ENDIF ! is_tree(j)

 END FUNCTION wood_to_qmheight



!! ================================================================================================================================
!! FUNCTION     : wood_to_dia_eff
!!
!>\BRIEF        Calculate effective diameter from woody biomass making use of allometric relationships
!!
!! DESCRIPTION : Calculate the effective diameter of an individual tree from the woody biomass of that tree making 
!! use of allometric relationships. Effective diameter accounts for both above and belowground biomass. 
!! (i) woodmass_ind = tree_ff * pipe_density * height * pi/4*dia**2
!! (ii) height = pipe_tune2 * dia * pipe_tune3 
!! 
!! RECENT CHANGE(S): None
!!
!! RETURN VALUE : diameter (m)
!!
!! REFERENCE(S)	: 
!!
!! FLOWCHART	: None
!! \n
!_ ================================================================================================================================
   
  FUNCTION wood_to_dia_eff(biomass_temp, pft)

 !! 0. Variable and parameter declaration

    !! 0.1 Input variables

    INTEGER(i_std)                                          :: pft               !! PFT number (-)
    REAL(r_std), DIMENSION(:,:)                             :: biomass_temp      !! Biomass of an individual tree within a circ 
                                                                                 !! class @tex $(m^{2} ind^{-1})$ @endtex 

    !! 0.2 Output variables
                
    REAL(r_std), DIMENSION(ncirc)                           :: wood_to_dia_eff   !! Diameter of an individual tree within a circ
                                                                                 !! class (m)

    !! 0.3 Modified variables

    !! 0.4 Local variables
    INTEGER(i_std)                                          :: l                 !! Index
    REAL(r_std)                                             :: woodmass_ind      !! Woodmass of an individual tree
                                                                                 !! @tex $(gC ind^{-1})$ @endtex

!_ ================================================================================================================================
 
 !! 1. Calculate basal area from woodmass

    IF ( is_tree(pft) ) THEN
       
       DO l = 1,ncirc
         
          ! Woodmass of an individual tree
          woodmass_ind = biomass_temp(l,isapabove) + biomass_temp(l,isapbelow) + &
             biomass_temp(l,iheartabove) + biomass_temp(l,iheartbelow)

          ! Basal area of that individual (m2 ind-1)  
          wood_to_dia_eff(l) = (4/pi*woodmass_ind/(tree_ff(pft)*pipe_density(pft)*pipe_tune2(pft))) ** &
                  (1./(2+pipe_tune3(pft)))
                 
       ENDDO

    ELSE

       WRITE(numout,*) 'pft ',pft
       CALL ipslerr_p (3,'wood_to_dia_eff', &
            'wood_to_dia_eff is not defined for this PFT.', &
            'See the output file for more details.','')

    ENDIF

 END FUNCTION wood_to_dia_eff



!! ================================================================================================================================
!! FUNCTION     : wood_to_dia
!!
!>\BRIEF        Calculate diameter from woody biomass making use of allometric relationships
!!
!! DESCRIPTION : Calculate diameter of an individual tree from the woody biomass of that tree making 
!! use of allometric relationships. Makes only use of the aboveground biomass and relates to the
!! typical forestry diameter (but not normalized at 1.3 m)
!! (i) woodmass_ind = tree_ff * pipe_density * height * pi/4*dia**2
!! (ii) height = pipe_tune2 * dia * pipe_tune3 
!! 
!! RECENT CHANGE(S): None
!!
!! RETURN VALUE : diameter (m)
!!
!! REFERENCE(S)	: 
!!
!! FLOWCHART	: None
!! \n
!_ ================================================================================================================================
   
  FUNCTION wood_to_dia(biomass_temp, pft)

 !! 0. Variable and parameter declaration

    !! 0.1 Input variables

    INTEGER(i_std)                                          :: pft               !! PFT number (-)
    REAL(r_std), DIMENSION(:,:)                             :: biomass_temp      !! Biomass of an individual tree within a circ 
                                                                                 !! class @tex $(m^{2} ind^{-1})$ @endtex 

    !! 0.2 Output variables
                
    REAL(r_std), DIMENSION(ncirc)                           :: wood_to_dia       !! Diameter of an individual tree within a circ
                                                                                 !! class (m)

    !! 0.3 Modified variables

    !! 0.4 Local variables
    INTEGER(i_std)                                          :: l                 !! Index
    REAL(r_std)                                             :: woodmass_ind      !! Woodmass of an individual tree
                                                                                 !! @tex $(gC ind^{-1})$ @endtex

!_ ================================================================================================================================
 
 !! 1. Calculate basal area from woodmass

    IF ( is_tree(pft) ) THEN
       
       DO l = 1,ncirc
         
          ! Woodmass of an individual tree
          woodmass_ind = biomass_temp(l,isapabove) + biomass_temp(l,iheartabove)

          ! Basal area of that individual (m2 ind-1)  
          wood_to_dia(l) = (4/pi*woodmass_ind/(tree_ff(pft)*pipe_density(pft)*pipe_tune2(pft))) ** &
                  (1./(2+pipe_tune3(pft)))
                 
       ENDDO

    ELSE

       WRITE(numout,*) 'pft ',pft
       CALL ipslerr_p (3,'wood_to_dia', &
            'wood_to_dia is not defined for this PFT.', &
            'See the output file for more details.','')

    ENDIF

 END FUNCTION wood_to_dia


!! ================================================================================================================================
!! FUNCTION     : wood_to_qmdia
!!
!>\BRIEF        Calculate the quadratic mean diameter from the biomass
!!
!! DESCRIPTION : Calculates the quadratic mean diameter from the aboveground biomss
!! 
!! RECENT CHANGE(S): None
!!
!! RETURN VALUE : ::qm_dia (m)
!!
!! REFERENCE(S)	: 
!!
!! FLOWCHART	: None
!! \n
!_ ================================================================================================================================
   
  FUNCTION wood_to_qmdia(biomass_temp, ind, pft)

 !! 0. Variable and parameter declaration

    !! 0.1 Input variables

    INTEGER(i_std)                             :: pft               !! PFT number (-)
    REAL(r_std), DIMENSION(ncirc,nparts)       :: biomass_temp      !! Biomass of the leaves @tex $(gC m^{-2})$ @endtex 
    REAL(r_std), DIMENSION(ncirc)              :: ind               !! Number of individuals @tex $(m^{-2})$ @endtex

    !! 0.2 Output variables
                
    REAL(r_std)                                :: wood_to_qmdia     !! quadratic mean diameter (m) 

    !! 0.3 Modified variables

    !! 0.4 Local variables
    REAL(r_std), DIMENSION(ncirc)              :: circ_class_ba     !! basal area for each circ_class @tex $(m^{2})$ @endtex

!_ ================================================================================================================================
 
    !! 1. Calculate qm_dia from the biomass
    IF ( is_tree(pft) ) THEN

       ! Basal area at the tree level (m2 tree-1)
       circ_class_ba(:) = wood_to_ba(biomass_temp(:,:),pft) 
       
       IF (SUM(ind(:)) .NE. zero) THEN

          wood_to_qmdia = SQRT( 4/pi*SUM(circ_class_ba(:)*ind(:))/SUM(ind(:)) )
          
       ELSE
          
          wood_to_qmdia = zero

       ENDIF


    ! Grasses and croplands   
    ELSE

       wood_to_qmdia = zero   
       
    ENDIF ! is_tree(pft)

 END FUNCTION wood_to_qmdia


!! ================================================================================================================================
!! FUNCTION     : wood_to_circ
!!
!>\BRIEF        Calculate circumference from woody biomass making use of allometric relationships
!!
!! DESCRIPTION : All this does it computer the diameter using a different routine, and then
!!               convert that into a circumference.  
!! 
!! RECENT CHANGE(S): None
!!
!! RETURN VALUE : circumference (m)
!!
!! REFERENCE(S)	: 
!!
!! FLOWCHART	: None
!! \n
!_ ================================================================================================================================
   
  FUNCTION wood_to_circ(biomass_temp, pft)

 !! 0. Variable and parameter declaration

    !! 0.1 Input variables

    INTEGER(i_std)                                          :: pft               !! PFT number (-)
    REAL(r_std), DIMENSION(:,:)                             :: biomass_temp      !! Biomass of an individual tree within a circ 
                                                                                 !! class @tex $(m^{2} ind^{-1})$ @endtex 

    !! 0.2 Output variables
                
    REAL(r_std), DIMENSION(ncirc)                           :: wood_to_circ      !! Circumference of an individual tree within a circ
                                                                                 !! class (m)

    !! 0.3 Modified variables

    !! 0.4 Local variables

!_ ================================================================================================================================
 
    !! 1. Calculate diameter from woodmass

    wood_to_circ(:)=val_exp

    wood_to_circ(:)=wood_to_dia(biomass_temp(:,:),pft)

    ! convert to a circumference (m)
    wood_to_circ(:) =  wood_to_circ(:)*pi

 END FUNCTION wood_to_circ



!! ================================================================================================================================
!! FUNCTION     : wood_to_cn_array
!!
!>\BRIEF        Calculate crown area from woody biomass making use of allometric relationships
!!
!! DESCRIPTION : Calculate crown area of an individual tree from the woody biomass of that tree
!! making use of allometric relationship that relates crown area (cn) to diameter (dia) as 
!! pipe_tune1*dia**pipe_tune_exp_coeff where the pipe_tune parameters are pft-specific. 
!! (i) Basal area is written as a function of wood_mass: woodmass_ind = tree_ff*pipe_density*ba*height
!! (ii) height = pipe_tune2*sqrt(4/pi*ba)**pipe_tune3
!! (iii) cn = pipe_tune1*sqrt(4/pi*ba)**pipe_tune_exp_coeff   
!! 
!! RECENT CHANGE(S): None
!!
!! RETURN VALUE : crown area (m2 ind-1)
!!
!! REFERENCE(S)	: 
!!
!! FLOWCHART	: None
!! \n
!_ ================================================================================================================================

  FUNCTION wood_to_cn_array(biomass_temp, npts, pft)

 !! 0. Variable and parameter declaration

    !! 0.1 Input variables

    INTEGER(i_std)                           :: pft              !! PFT number (-)
    INTEGER(i_std)                           :: npts             !! Pixel(s), this variable defines the dimensions of ba
    REAL(r_std), DIMENSION(:,:,:)            :: biomass_temp     !! Biomass of an individual tree within a circ 
                                                                 !! class @tex $(m^{2} ind^{-1})$ @endtex 
    

    !! 0.2 Output variables
                
    REAL(r_std), DIMENSION(npts,ncirc)       :: wood_to_cn_array !! Crown area of an individual tree @tex $(m^{2} ind{-1})$ @endtex

    !! 0.3 Modified variables

    !! 0.4 Local variables
 
    INTEGER(i_std)                           :: l                !! index
    REAL(r_std), DIMENSION(npts)             :: woodmass_ind     !! Woodmass of an individual tree @tex $(gC ind{-1})$ @endtex

!_ ================================================================================================================================
 
 !! 1. Calculate crown area from basal area

    IF ( is_tree(pft) ) THEN
       
       DO l = 1,ncirc

          ! Woodmass of an individual tree
          woodmass_ind(:) = biomass_temp(:,l,isapabove) + biomass_temp(:,l,iheartabove)

          ! Crown area of that individual
          ! cn = pipe_tune1*sqrt(4/pi*ba)**pipe_tune_exp_coeff 
          wood_to_cn_array(:,l) = pipe_tune1(pft) * SQRT( 4/pi*(pi/4*(woodmass_ind(:)/&
               (tree_ff(pft)*pipe_density(pft)*pipe_tune2(pft)))**(2./pipe_tune3(pft)))**&
               (pipe_tune3(pft)/(pipe_tune3(pft)+2)) ) ** pipe_tune_exp_coeff(pft)

       ENDDO

    ELSE

       WRITE(numout,*) 'pft ',pft
       CALL ipslerr_p (3,'wood_to_cn_array', &
            'wood_to_cn_array is not defined for this PFT.', &
            'See the output file for more details.','')

    ENDIF

  END FUNCTION wood_to_cn_array


!! ================================================================================================================================
!! FUNCTION     : wood_to_cn
!!
!>\BRIEF        Calculate crown area from woody biomass making use of allometric relationships
!!
!! DESCRIPTION : Calculate crown area of an individual tree from the woody biomass of that tree 
!! making use of allometric relationship that relates crown area (cn) to diameter (dia) as 
!! pipe_tune1*dia**pipe_tune_exp_coeff where the pipe_tune parameters are pft-specific. 
!! (i) Basal area is written as a function of wood_mass: woodmass_ind = tree_ff*pipe_density*ba*height
!! (ii) height = pipe_tune2*sqrt(4/pi*ba)**pipe_tune3
!! (iii) cn = pipe_tune1*sqrt(4/pi*ba)**pipe_tune_exp_coeff   
!! 
!! RECENT CHANGE(S): None
!!
!! RETURN VALUE : crown area (m2 ind-1)
!!
!! REFERENCE(S)	: 
!!
!! FLOWCHART	: None
!! \n
!_ ================================================================================================================================
   
  FUNCTION wood_to_cn(biomass_temp, pft)

 !! 0. Variable and parameter declaration

    !! 0.1 Input variables

    INTEGER(i_std)                                          :: pft               !! PFT number (-)
    REAL(r_std), DIMENSION(:,:)                             :: biomass_temp      !! biomass of an individual tree within a circ 
                                                                                 !! class @tex $(m^{2} ind^{-1})$ @endtex 

    !! 0.2 Output variables
                
    REAL(r_std), DIMENSION(ncirc)                           :: wood_to_cn        !! Crown area of an individual tree within a circ
                                                                                 !! class @tex $(m^{2} ind-1)$ @endtex

    !! 0.3 Modified variables

    !! 0.4 Local variables
    INTEGER(i_std)                                          :: l                 !! Index
    REAL(r_std)                                             :: woodmass_ind      !! Woodmass of an individual tree
                                                                                 !! @tex $(gC ind{-1})$ @endtex

!_ ================================================================================================================================
 
 !! 1. Calculate crown area from woodmass

    IF ( is_tree(pft) ) THEN
       
       DO l = 1,ncirc
         
          ! Woodmass of an individual tree
          woodmass_ind = biomass_temp(l,isapabove) + biomass_temp(l,iheartabove)
 
          ! Crown area of that individual
          ! cn = pipe_tune1*sqrt(4/pi*ba)**pipe_tune_exp_coeff 
          wood_to_cn(l) = pipe_tune1(pft) * SQRT( 4/pi*(pi/4*(woodmass_ind/&
               (tree_ff(pft)*pipe_density(pft)*pipe_tune2(pft)))**(2./pipe_tune3(pft)))**&
               (pipe_tune3(pft)&
               /(pipe_tune3(pft)+2)) ) ** pipe_tune_exp_coeff(pft)
                
       ENDDO

    ELSE

       WRITE(numout,*) 'pft ',pft
       CALL ipslerr_p (3,'wood_to_cn', &
            'wood_to_cn is not defined for this PFT.', &
            'See the output file for more details.','')

    ENDIF

 END FUNCTION wood_to_cn

!! ================================================================================================================================
!! FUNCTION     : wood_to_cn_eff
!!
!>\BRIEF        Calculate crown area from woody biomass making use of allometric relationships
!!
!! DESCRIPTION : Calculate crown area of an individual tree from the woody biomass of that tree 
!! making use of allometric relationship that relates crown area (cn) to diameter (dia) as 
!! pipe_tune1*dia**pipe_tune_exp_coeff where the pipe_tune parameters are pft-specific. 
!! (i) Basal area is written as a function of wood_mass: woodmass_ind = tree_ff*pipe_density*ba*height
!! (ii) height = pipe_tune2*sqrt(4/pi*ba)**pipe_tune3
!! (iii) cn = pipe_tune1*sqrt(4/pi*ba)**pipe_tune_exp_coeff   
!!
!! NOTE: This is different from wood_to_cn because this uses both the aboveground and
!!       belowground wood mass.
!! 
!! RECENT CHANGE(S): None
!!
!! RETURN VALUE : crown area (m2 ind-1)
!!
!! REFERENCE(S)	: 
!!
!! FLOWCHART	: None
!! \n
!_ ================================================================================================================================
   
  FUNCTION wood_to_cn_eff(biomass_temp, pft)

 !! 0. Variable and parameter declaration

    !! 0.1 Input variables

    INTEGER(i_std)                                          :: pft               !! PFT number (-)
    REAL(r_std), DIMENSION(:,:)                             :: biomass_temp      !! biomass of an individual tree within a circ 
                                                                                 !! class @tex $(m^{2} ind^{-1})$ @endtex 

    !! 0.2 Output variables
                
    REAL(r_std), DIMENSION(ncirc)                           :: wood_to_cn_eff    !! Crown area of an individual tree within a circ
                                                                                 !! class @tex $(m^{2} ind-1)$ @endtex

    !! 0.3 Modified variables

    !! 0.4 Local variables
    INTEGER(i_std)                                          :: l                 !! Index
    REAL(r_std)                                             :: woodmass_ind      !! Woodmass of an individual tree
                                                                                 !! @tex $(gC ind{-1})$ @endtex

!_ ================================================================================================================================
 
 !! 1. Calculate crown area from woodmass

    IF ( is_tree(pft) ) THEN
       
       DO l = 1,ncirc
         
          ! Woodmass of an individual tree
          woodmass_ind = biomass_temp(l,isapabove) + biomass_temp(l,iheartabove) + &
               biomass_temp(l,isapbelow) + biomass_temp(l,iheartbelow)
 
          ! Crown area of that individual
          ! cn = pipe_tune1*sqrt(4/pi*ba)**pipe_tune_exp_coeff 
          wood_to_cn_eff(l) = pipe_tune1(pft) * SQRT( 4/pi*(pi/4*(woodmass_ind/&
               (tree_ff(pft)*pipe_density(pft)*pipe_tune2(pft)))**(2./pipe_tune3(pft)))**&
               (pipe_tune3(pft)&
               /(pipe_tune3(pft)+2)) ) ** pipe_tune_exp_coeff(pft)
                
       ENDDO

    ELSE

       WRITE(numout,*) 'pft ',pft
       CALL ipslerr_p (3,'wood_to_cn_eff', &
            'wood_to_cn_eff is not defined for this PFT.', &
            'See the output file for more details.','')

    ENDIF

  END FUNCTION wood_to_cn_eff


!! ================================================================================================================================
!! FUNCTION     : wood_to_cv
!!
!>\BRIEF        Calculate crown volume from woody biomass making use of allometric relationships
!!
!! DESCRIPTION : Calculate basal areadiameter of an individual tree from the woody biomass of that tree making 
!! use of allometric relationships:
!! (i) Basal area is written as a function of wood_mass: woodmass_ind = tree_ff*pipe_density*ba*height
!! (ii) height = pipe_tune2 * sqrt( 4/pi*ba ) ** pipe_tune3
!! (iii) cn = pipe_tune1 * sqrt( 4/pi*ba ) ** pipe_tune_exp_coeff  
!! (iv) cv = 4.0 / 3.0 * pi * ( SQRT( cn/pi ) ) ** 3
!! 
!! RECENT CHANGE(S): None
!!
!! RETURN VALUE : diameter (m3 ind-1)
!!
!! REFERENCE(S)	: 
!!
!! FLOWCHART	: None
!! \n
!_ ================================================================================================================================
   
  FUNCTION wood_to_cv(biomass_temp, pft)

 !! 0. Variable and parameter declaration

    !! 0.1 Input variables

    INTEGER(i_std)                                          :: pft               !! PFT number (-)
    REAL(r_std), DIMENSION(:,:)                             :: biomass_temp      !! Biomass of an individual tree within a circ 
                                                                                 !! class @tex $(m^{2} ind^{-1})$ @endtex 

    !! 0.2 Output variables
                
    REAL(r_std), DIMENSION(ncirc)                           :: wood_to_cv        !! Crown volume of an individual tree 
                                                                                 !! @tex $(m^{2} ind{-1})$ @endtex

    !! 0.3 Modified variables

    !! 0.4 Local variables
    INTEGER(i_std)                                          :: l                 !! Index
    REAL(r_std)                                             :: woodmass_ind      !! Woodmass of an individual tree
                                                                                 !! @tex $(gC ind^{-1})$ @endtex
    REAL(r_std), DIMENSION(ncirc)                           :: dia               !! Diameter of an individual tree (m)

!_ ================================================================================================================================
 
 !! 1. Calculate crown volume from woodmass

    IF ( is_tree(pft) ) THEN

       ! Crown diameter of the individual tree (m2 ind-1)  
       dia(:) = (4./pi*wood_to_cn(biomass_temp(:,:),pft))**(1./2.)
       wood_to_cv(:) = pi/6.*dia(:)**3.  
       ! WRITE(numout,*) 'wood_to_cv, ', wood_to_cv(:)

    ELSE

       WRITE(numout,*) 'pft ',pft
       CALL ipslerr_p (3,'wood_to_cv', &
            'wood_to_cv is not defined for this PFT.', &
            'See the output file for more details.','')

    ENDIF

 END FUNCTION wood_to_cv 

   !! ================================================================================================================================
!! FUNCTION     : wood_to_cv_eff
!!
!>\BRIEF        Calculate crown volume from woody biomass making use of allometric relationships
!!
!! DESCRIPTION : Calculate basal areadiameter of an individual tree from the woody biomass of that tree making 
!! use of allometric relationships:
!! (i) Basal area is written as a function of wood_mass: woodmass_ind = tree_ff*pipe_density*ba*height
!! (ii) height = pipe_tune2 * sqrt( 4/pi*ba ) ** pipe_tune3
!! (iii) cn = pipe_tune1 * sqrt( 4/pi*ba ) ** pipe_tune_exp_coeff  
!! (iv) cv = 4.0 / 3.0 * pi * ( SQRT( cn/pi ) ) ** 3
!! 
!! NOTE: the difference between this and wood_to_cv is that this include aboveground and belowground
!!       biomass in the calculation
!!
!! RECENT CHANGE(S): None
!!
!! RETURN VALUE : diameter (m3 ind-1)
!!
!! REFERENCE(S)	: 
!!
!! FLOWCHART	: None
!! \n
!_ ================================================================================================================================
   
  FUNCTION wood_to_cv_eff(biomass_temp, pft)

 !! 0. Variable and parameter declaration

    !! 0.1 Input variables

    INTEGER(i_std)                                          :: pft               !! PFT number (-)
    REAL(r_std), DIMENSION(:,:)                             :: biomass_temp      !! Biomass of an individual tree within a circ 
                                                                                 !! class @tex $(m^{2} ind^{-1})$ @endtex 

    !! 0.2 Output variables
                
    REAL(r_std), DIMENSION(ncirc)                           :: wood_to_cv_eff    !! Crown volume of an individual tree 
                                                                                 !! @tex $(m^{2} ind{-1})$ @endtex

    !! 0.3 Modified variables

    !! 0.4 Local variables
    INTEGER(i_std)                                          :: l                 !! Index
    REAL(r_std)                                             :: woodmass_ind      !! Woodmass of an individual tree
                                                                                 !! @tex $(gC ind^{-1})$ @endtex
    REAL(r_std), DIMENSION(ncirc)                           :: dia               !! Diameter of an individual tree (m)

!_ ================================================================================================================================
 
 !! 1. Calculate crown volume from woodmass

    IF ( is_tree(pft) ) THEN

       ! Crown diameter of the individual tree (m2 ind-1)  
       dia(:) = (4./pi*wood_to_cn_eff(biomass_temp(:,:),pft))**(1./2.)
       wood_to_cv_eff(:) = pi/6.*dia(:)**3.  
       ! WRITE(numout,*) 'wood_to_cv_eff, ', wood_to_cv_eff(:)

    ELSE

       WRITE(numout,*) 'pft ',pft
       CALL ipslerr_p (3,'wood_to_cv_eff', &
            'wood_to_cv_eff is not defined for this PFT.', &
            'See the output file for more details.','')

    ENDIF

  END FUNCTION wood_to_cv_eff



!! ================================================================================================================================
!! FUNCTION     : wood_to_volume
!!
!>\BRIEF        This allometric function computes volume as a function of 
!! biomass at stand scale. Volume \f$(m^3 m^{-2}) = f(biomass (gC m^{-2}))\f$
!!
!! DESCRIPTION : None
!!
!! RECENT CHANGE(S): None
!! 
!! RETURN VALUE : biomass_to_volume
!!
!! REFERENCE(S)	: See above, module description.
!!
!! FLOWCHART	: None
!! \n
!_ ================================================================================================================================

  FUNCTION wood_to_volume(biomass,pft,branch_ratio,inc_branches)

 !! 0. Variable and parameter declaration

    !! 0.1 Input variables

    REAL(r_std), DIMENSION(:)   :: biomass              !! Stand biomass @tex $(gC m^{-2})$ @endtex  
    REAL(r_std)                 :: branch_ratio         !! Branch ratio of sap and heartwood biomass
                                                        !! unitless
    INTEGER(i_std)              :: pft                  !! Plant functional type (unitless)
    INTEGER(i_std)              :: inc_branches         !! Include the branches in the volume calculation?
                                                        !! 0: exclude the branches from the volume calculation 
                                                        !! (thus correct the biomass for the branch ratio)
                                                        !! 1: include the branches in the volume calculation
                                                        !! (thus use all aboveground biomass)
                                    
    

    !! 0.2 Output variables

    REAL(r_std)                 :: wood_to_volume       !! The volume of wood per square meter 
                                                        !!  @tex $(m^3 m^{-2})$ @endtex

    !! 0.3 Modified variables

    !! 0.4 Local variables
    
    REAL(r_std)                 :: woody_biomass        !! Woody biomass at the stand level 
                                                        !! @tex $(gC m^{-2})$ @endtex

!_ ================================================================================================================================

 !! 1. Volume to biomass

    ! Woody biomass used in the calculation
    IF (inc_branches .EQ. 0) THEN

       woody_biomass=(biomass(isapabove)+biomass(iheartabove))*(un - branch_ratio)

    ELSEIF (inc_branches .EQ. 1) THEN

       woody_biomass=(biomass(isapabove)+biomass(iheartabove))

    ELSE

    ENDIF

    ! Wood volume expressed in m**3 / m**2
    wood_to_volume = woody_biomass/(pipe_density(pft))

  END FUNCTION wood_to_volume



!! ================================================================================================================================
!! FUNCTION     : biomass_to_lai
!!
!>\BRIEF        Calculate the LAI based on the leaf biomass
!!
!! DESCRIPTION : Calculates the LAI of a PFT/grid square based on the leaf biomass
!! 
!! RECENT CHANGE(S): None
!!
!! RETURN VALUE : ::LAI [m**2 m**{-2}]
!!
!! REFERENCE(S)	: 
!!
!! FLOWCHART	: None
!! \n
!_ ================================================================================================================================
   
  FUNCTION biomass_to_lai(leaf_biomass, pft)

 !! 0. Variable and parameter declaration

    !! 0.1 Input variables

    INTEGER(i_std)                                          :: pft               !! PFT number (-)
    REAL(r_std)                                             :: leaf_biomass      !! Biomass of the leaves
                                                                                 !! @tex $(gC m^{-2})$ @endtex 


    !! 0.2 Output variables
                
    REAL(r_std)                                             :: biomass_to_lai    !! Leaf area index 
                                                                                 !! @tex $(m^{2} m^{-2})$ @endtex 

    !! 0.3 Modified variables

    !! 0.4 Local variables
    REAL(r_std)                                             :: impose_lai         !! LAI read from run.def
!_ ================================================================================================================================
 
    !! 1. Calculate the LAI from the leaf biomass

    biomass_to_lai = leaf_biomass * sla(pft)
     
!!$    !+++++++++ TEMP ++++++++++
!!$    ! This is a perfect place to hack the code to make it run with
!!$    ! constant lai
!!$    WRITE(numout,*) 'WARNING ERROR: Using fake lai values for testing!'
!!$    biomass_to_lai=3.79052
!!$    !+++++++++++++++++++++++++

            !+++++++ TEMP ++++++++++
            ! This code is only used evaluation of the performance of the multi-layer energy budget. 
            ! To reduce the complexity of the tests we want to impose the LAI and its vertical distribution. 
            ! The solution is not very elegant but it works. 
            !  IF (ld_fake_height) THEN
            ! In order to imposed lai, we read the TOTAL_LAI from run.def
            !  CALL getin_p('TOTAL_LAI', impose_lai)
            ! This part of code reset the sla vale to match which alow modeled LAI equal to TOTAL LAI. 
            ! Althought this is ugly way to match the modeled LAI and impose LAI. 
            ! You probably need to go to your ORCHIDEE out put file to find out the suitable SLA value 
            ! and reset it agin in the run.def.
            ! So, we impose LAI & structure for a quick testing the performance of multilayer energy budget
            ! without changing the leaf_biomass.    
            !     IF ( leaf_biomass .GT. 0.0) THEN
            !        sla(pft)=impose_lai/leaf_biomass
            !          WRITE(numout,'(A,F20.8)') 'USE A FAKE SLA BASED ON imposed LAI/LEAFMASS=', sla(pft)
            !     ENDIF 
            !        biomass_to_lai=leaf_biomass*sla(pft) 
            !  ENDIF 
            !++++++++++++++++++++++++

     END FUNCTION biomass_to_lai


!! ================================================================================================================================
!! FUNCTION     : Nmax
!!
!>\BRIEF        This function determines the maximum number of trees per hectare for 
!! a given quadratic mean diameter (Dg). It applies the self-thinning principle 
!! of Reineke (1933), with Dg instead of mean diameter (Dhote 1999).
!! Parameterization: Dhote (1999) for broad-leaved and Vacchiano (2008)
!! for needle-leaved.
!!
!! DESCRIPTION : None
!!
!! RECENT CHANGE(S): None
!!
!! RETURN VALUE : Nmax
!!
!! REFERENCE(S)   : See above, module description.
!!
!! FLOWCHART   : None
!! \n
!_ ================================================================================================================================
   
  FUNCTION Nmax(Dg,no_pft)

 !! 0. Variable and parameter declaration

    !! 0.1 Input variables

    REAL(r_std)      :: Dg              !! Quadratic mean diameter (cm)
    INTEGER(i_std)   :: no_pft          !! Plant functional type (unitless) 

    !! 0.2 Output variables

    REAL(r_std)      :: Nmax            !! Maximum number of trees according to the self-thinning model

    !! 0.3 Modified variables

    !! 0.4 Local variables

!_ ================================================================================================================================
 !! 1. maximum number of trees per hectare for a given quadratic mean diameter

    IF (is_tree(no_pft)) THEN

       ! thinning curve of the MTC
       Nmax = (Dg/alpha_self_thinning(no_pft))**(un/beta_self_thinning(no_pft))

       ! Truncate the range to avoid huge numbers due the exponental model used to describe self-thinning
       Nmax = MIN( Nmax, nmaxtrees(no_pft) )
       Nmax = MAX( Nmax, dens_target(no_pft) )

    ELSE

       WRITE(numout,*) 'Self thinning is not defined for PFT, ', no_pft
       CALL ipslerr_p (3,'nmax', &
            'Self thinning is not defined for this PFT.', &
            'See the output file for more details.','')

    ENDIF

  END FUNCTION Nmax


!! ================================================================================================================================
!! FUNCTION     : Nmaxyield
!!
!>\BRIEF        This function determines the maximum number of trees per hectare for 
!! a given quadratic mean diameter (Dg). It applies the 75 percentile  of the European 
!! yield tables
!!
!! DESCRIPTION : None
!!
!! RECENT CHANGE(S): None
!!
!! RETURN VALUE : Nmaxyield
!!
!! REFERENCE(S)   : See above, module description.
!!
!! FLOWCHART   : None
!! \n
!_ ================================================================================================================================
   
  FUNCTION Nmaxyield(Dg,no_pft)

 !! 0. Variable and parameter declaration

    !! 0.1 Input variables

    REAL(r_std)      :: Dg              !! Quadratic mean diameter (cm)
    INTEGER(i_std)   :: no_pft          !! Plant functional type (unitless) 

    !! 0.2 Output variables

    REAL(r_std)      :: Nmaxyield       !! Maximum number of trees according to the self-thinning model

    !! 0.3 Modified variables

    !! 0.4 Local variables

!_ ================================================================================================================================
 !! 1. maximum number of trees per hectare for a given quadratic mean diameter

    IF (is_tree(no_pft)) THEN

       ! thinning curve of the MTC
       Nmaxyield = (Dg/alpha_rdi_upper(no_pft))**(un/beta_rdi_upper(no_pft))

       ! Truncate the range to avoid huge numbers due the exponental model used to describe self-thinning
       Nmaxyield = MIN( Nmaxyield, nmaxtrees(no_pft) )
       Nmaxyield = MAX( Nmaxyield, dens_target(no_pft) )

    ELSE

       WRITE(numout,*) 'Self thinning is not defined for PFT, ', no_pft
       CALL ipslerr_p (3,'Nmaxyield', &
            'Self thinning is not defined for this PFT.', &
            'See the output file for more details.','')

    ENDIF

  END FUNCTION Nmaxyield


!! ================================================================================================================================
!! FUNCTION     : Nminyield
!!
!>\BRIEF        This function determines the minimum number of trees per hectare for 
!! a given quadratic mean diameter (Dg). It applies the 25 percentile of the European 
!! yield tables
!!
!! DESCRIPTION : None
!!
!! RECENT CHANGE(S): None
!!
!! RETURN VALUE : Nminyield
!!
!! REFERENCE(S)   : See above, module description.
!!
!! FLOWCHART   : None
!! \n
!_ ================================================================================================================================
   
  FUNCTION Nminyield(Dg,no_pft)

 !! 0. Variable and parameter declaration

    !! 0.1 Input variables

    REAL(r_std)      :: Dg              !! Quadratic mean diameter (cm)
    INTEGER(i_std)   :: no_pft          !! Plant functional type (unitless) 

    !! 0.2 Output variables

    REAL(r_std)      :: Nminyield       !! Minimum number of trees according to the self-thinning model

    !! 0.3 Modified variables

    !! 0.4 Local variables

!_ ================================================================================================================================
 !! 1. minimum number of trees per hectare for a given quadratic mean diameter

    IF (is_tree(no_pft)) THEN

       ! thinning curve of the MTC
       Nminyield = (Dg/alpha_rdi_lower(no_pft))**(un/beta_rdi_lower(no_pft))

       ! Truncate the range to avoid huge numbers due the exponental model used to describe self-thinning
        
       Nminyield = MIN( Nminyield, nmaxtrees(no_pft) )
       Nminyield = MAX( Nminyield, dens_target(no_pft) )

    ELSE

       WRITE(numout,*) 'Self thinning is not defined for PFT, ', no_pft
       CALL ipslerr_p (3,'Nminyield', &
            'Self thinning is not defined for this PFT.', &
            'See the output file for more details.','')


    ENDIF

  END FUNCTION Nminyield


!! ================================================================================================================================
!! SUBROUTINE  : distribute_mortality_biomass
!!
!>\BRIEF       Distributes biomass that is going to be killed by natural
!!             causes (not self thinning) over circ classes.
!!
!! DESCRIPTION : Mortality is going to kill a certain amount of biomass
!!               in forests every day.  Since we have circumference classes
!!               now for our forests, we need to determine which classes
!!               of trees will suffer from this environmental mortality.
!!               Right now we are taking an exponential distribution.
!!               Notice that this is NOT the same as redistributing biomass
!!               after one of the circ classes becomes empty.
!!
!! RECENT CHANGE(S): None
!!
!! MAIN OUTPUT VARIABLE(S): ::circ_class_kill
!!
!! REFERENCE(S) : None
!!
!! FLOWCHART    : None
!! \n
!_ ================================================================================================================================
  SUBROUTINE distribute_mortality_biomass ( bm_difference, ddf_temp, circ_class_n_temp, &
       circ_class_biomass_temp, circ_class_kill_temp )

 !! 0. Variable and parameter description

    !! 0.1 Input variables
    REAL(r_std),INTENT(in)                                    :: bm_difference !! the biomass to distribute
    REAL(r_std),INTENT(in)                                    :: ddf_temp !! the death_distribution_factor for this pft
    REAL(r_std),DIMENSION(:),INTENT(in)                       :: circ_class_n_temp !! circ_class_n for this point/PFT
    REAL(r_std),DIMENSION(:,:),INTENT(in)                     :: circ_class_biomass_temp !! circ_class_biomass for 
                                                                          !! this point/PFT
    !! 0.2 Output variables

    !! 0.3 Modified variables
    REAL(r_std),DIMENSION(:),INTENT(inout)                    :: circ_class_kill_temp !! circ_class_kill for 
                                                                                      !! this point/PFT/pool

    !! 0.4 Local variables
    REAL(r_std), DIMENSION(ncirc)                             :: biomass_desired    !! The biomass that dies naturally
    REAL(r_std), DIMENSION(ncirc)                             :: death_distribution !! The fraction of biomass taken from
                                                                                    !! each circ class for mortality.
    LOGICAL                                                   :: ldone              !! Flag to exit a loop
    REAL(r_std)                                               :: scale_factor       !! 
    REAL(r_std)                                               :: sum_total          !! 
    REAL(r_std)                                               :: leftover_bm        !! excess biomass that we need to kill 
    REAL(r_std)                                               :: living_biomass      !! summed biomass
    REAL(r_std)                                               :: living_trees       !! summed biomass
    INTEGER                                                   :: icir

!_ ================================================================================================================================

    IF(ncirc == 1)THEN

       ! This is the easy case.  All of our biomass will be taken from the only
       ! circumference class that we have.
       circ_class_kill_temp(1)=circ_class_kill_temp(1)+&
            bm_difference/SUM(circ_class_biomass_temp(1,:))

       RETURN
    ENDIF

    ! Here we assume an exponential distribution, arranged so that
    ! ddf_temp times more biomass is taken from the largest circ class
    ! compared to the smallest.
    biomass_desired(:)=zero
    death_distribution(:)=un
    scale_factor=ddf_temp**(un/REAL(ncirc-1))
    DO icir=2,ncirc
       death_distribution(icir)=death_distribution(icir-1)*scale_factor
    ENDDO
    ! Normalize it
    sum_total=SUM(death_distribution(:))
    death_distribution(:)=death_distribution(:)/sum_total

    ! Ideally, how much is killed from each class?  Be careful to include
    ! what was killed in self-thinning here!  If we don't, we may try to kill
    ! more biomass than is available in the loop below.
    DO icir=1,ncirc
       biomass_desired(icir)=death_distribution(icir)*bm_difference+&
            circ_class_kill_temp(icir)*SUM(circ_class_biomass_temp(icir,:))
    ENDDO

    ! Right now, we know how much biomass we want to kill in each
    ! class (biomass_desired).  What we will do now is to loop through
    ! all the circ_classes and see if we have this much biomass
    ! in each class still alive.  The total amount of vegetation still
    ! alive is circ_class_n_temp(icir)-circ_class_kill_temp(icir).  If
    ! this number of individuals cannot give us the total biomass that
    ! we need, we keep track of a residual quantity, leftover_bm, which
    ! we try to take from other circ_classes.  It's possible that we
    ! will have to loop several times, which makes things more complicated.

    ldone=.FALSE.
    leftover_bm=zero
    DO
       DO icir=ncirc,1,-1

          living_trees=circ_class_n_temp(icir)-circ_class_kill_temp(icir)
          living_biomass=&
               SUM(circ_class_biomass_temp(icir,:))*living_trees
          biomass_desired(icir)=biomass_desired(icir)+leftover_bm

          IF(living_biomass .LE. biomass_desired(icir))THEN

             ! We can't get everything from this class, so we kill whatever is left.
             leftover_bm=biomass_desired(icir)-living_biomass
             biomass_desired(icir)=zero
             circ_class_kill_temp(icir)=circ_class_kill_temp(icir)+&
                  living_trees

          ELSE

             ! We have enough in this class.
             circ_class_kill_temp(icir)=circ_class_kill_temp(icir)+&
                  biomass_desired(icir)/SUM(circ_class_biomass_temp(icir,:))
             biomass_desired(icir)=zero
             leftover_bm=zero

          ENDIF ! living_biomass .LE. biomass_desired(icir)

       ENDDO ! loop over circ classes

       IF(leftover_bm .LE. min_stomate) EXIT

       ! it's possible that we don't have enough biomass left to kill what needs to be
       ! killed, so everything just dies.  I cannot think of a case where this
       ! would happen, though, since mortality should always be a percentage of the
       ! total biomass.
       ldone=.TRUE.
       DO icir=1,ncirc

          IF( circ_class_kill_temp(icir) .LT. circ_class_n_temp(icir) ) &
               ldone=.FALSE.  

       ENDDO

       IF(ldone)EXIT ! All our biomass is dead, and we still want to kill more!

    ENDDO

    

  END SUBROUTINE distribute_mortality_biomass


!! ================================================================================================================================
!! SUBROUTINE  : check_biomass_sync
!!
!>\BRIEF       
!!
!! DESCRIPTION : 
!! RECENT CHANGE(S): None
!!
!! MAIN OUTPUT VARIABLE(S):
!!
!! REFERENCE(S) : None
!!
!! FLOWCHART    : None
!! \n
!_ ================================================================================================================================
  SUBROUTINE check_biomass_sync ( check_point, npts, biomass, &
       circ_class_biomass, circ_class_n , ind, &
       lsync, bm_sync)

 !! 0. Variable and parameter description

    !! 0.1 Input variables
    INTEGER(i_std), INTENT(in)                                 :: npts                 !! Domain size (unitless)
    REAL(r_std), DIMENSION(:,:,:,:,:), INTENT(in)              :: circ_class_biomass   !! Biomass of the componets of the model  
                                                                                       !! tree within a circumference
                                                                                       !! class @tex $(gC ind^{-1})$ @endtex  
    REAL(r_std), DIMENSION(:,:,:), INTENT(in)                  :: circ_class_n         !! Number of individuals in each circ class
                                                                                       !! @tex $(m^{-2})$ @endtex
    REAL(r_std), DIMENSION(:,:,:,:), INTENT(in)                :: biomass              !! Stand level biomass 
                                                                                       !! @tex $(gC m^{-2})$ @endtex
    CHARACTER(*),INTENT(in)                                    :: check_point          !! A flag to indicate at which
                                                                                       !! point in the code we're doing
                                                                                       !! this check
    REAL(r_std), DIMENSION(:,:), INTENT(in)                    :: ind                  !! Density of individuals 
                                                                                       !! @tex $(m^{-2})$ @endtex 

    !! 0.2 Output variables
    LOGICAL,INTENT(out)                                        :: lsync
    REAL(r_std), DIMENSION(:,:,:), INTENT(out)                 :: bm_sync              !! The difference betweeen the
                                                                                       !! biomass in the circ_classes and
                                                                                       !! the total biomass 
                                                                                       !! @tex $(gC m^{-2})$ @endtex
    !! 0.3 Modified variables

    !! 0.4 Local variables
    INTEGER                                                    :: iele,ipts,ivm,ipar,icir
    REAL(r_std)                                                :: total_circ_class_biomass
    REAL(r_std),DIMENSION(ncirc)                               :: tree_size
    LOGICAL                                                    :: lnegative
    
!_ ================================================================================================================================

    lsync=.TRUE.
    lnegative=.FALSE.

    bm_sync(:,:,:)=zero

    !++++++ TEMP ++++++
    ! We gain 5-10% speed by skipping this routine
       
    !++++++++++++

    ! Check to see if the biomass is not equal to the total biomass
    ! in circ_class_biomass anywhere.
    DO ipts=1,npts

       DO ivm=1,nvm

          ! Only woody PFTs have circumference classes therefore
          ! only woody PFTs need to be syncronized
          IF(.NOT. lbypass_cc)THEN
             IF(is_tree(ivm)) THEN
                tree_size(:)=zero
                DO icir=1,ncirc
                   tree_size(icir)=SUM(circ_class_biomass(ipts,ivm,icir,:,1))
                ENDDO
                DO icir=2,ncirc
                   IF(tree_size(icir) .LT. tree_size(icir-1)-min_stomate)THEN
                      WRITE(numout,*) 'ERROR: stopping in sync'
                      WRITE(numout,*) check_point
                      WRITE(numout,*) 'ipts,ivm: ',ipts,ivm
                      WRITE(numout,*) 'tree_size(icir), tree_size(icir-1), ',&
                           tree_size(icir), tree_size(icir-1), tree_size(icir) - tree_size(icir-1)  
                      WRITE(numout,*) 'icir, tree_size: ',icir, tree_size(:)
                      !+++ TEMP +++
                      !This would not STOP the ORCHIDEE beacause the mass balance is due to imposed LAI   
                      IF(ld_fake_height)  THEN 
                           CALL ipslerr_p (2,'check_biomass_sync', &
                           'The size of the trees in the circ class are not monotonically increasing!',&
                           'Look in the output file for more details.',&
                           '')
                      ELSE
                           CALL ipslerr_p (3,'check_biomass_sync', &
                           'The size of the trees in the circ class are not monotonically increasing!',&
                           'Look in the output file for more details.',&
                           '')
                      ENDIF
                      !++++++++++++    
                   ENDIF
                ENDDO
             ENDIF
          ENDIF
          
          DO iele=1,icarbon
                
             DO ipar=1,nparts
                
                total_circ_class_biomass=zero
                DO icir=1,ncirc
                   
                   total_circ_class_biomass=total_circ_class_biomass+&
                        circ_class_biomass(ipts,ivm,icir,ipar,iele)*circ_class_n(ipts,ivm,icir)

                   ! Check as well to see if our biomass is ever negative.
                   ! It really should not be.
                   IF(circ_class_biomass(ipts,ivm,icir,ipar,iele) .LT. -min_stomate)THEN

                      lnegative=.TRUE.
                      WRITE(numout,*) '!***********************************'
                      WRITE(numout,*) 'Error: Negative biomass component!'
                      WRITE(numout,*) 'Check point: ',TRIM(check_point)
                      WRITE(numout,*) 'circ_class_biomass(ipts,ivm,icir,ipar,iele) ',&
                           circ_class_biomass(ipts,ivm,icir,ipar,iele)
                      WRITE(numout,'(A,5I5)') 'ipts,ivm,icir,ipar,iele',ipts,ivm,icir,ipar,iele
                      WRITE(numout,*) '!***********************************'

                   ENDIF
                ENDDO

                IF(ABS(biomass(ipts,ivm,ipar,iele) -  &
                     total_circ_class_biomass) .GT. sync_threshold)THEN

                   WRITE(numout,*) '!***********************************'
                   WRITE(numout,*) 'Biomass and circ_class_biomass are not equal!'
                   WRITE(numout,*) 'Check point: ',TRIM(check_point)
                   WRITE(numout,100) 'biomass(ipts,ivm,ipar,iele) ',&
                        biomass(ipts,ivm,ipar,iele)
                   WRITE(numout,100) 'total_circ_class_biomass ',&
                        total_circ_class_biomass
                   WRITE(numout,100) 'Difference: ',&
                        ABS(biomass(ipts,ivm,ipar,iele) - total_circ_class_biomass)
                   WRITE(numout,*) 'ipts,ivm,ipar,iele',ipts,ivm,ipar,iele
                   WRITE(numout,*) '!***********************************'
100                FORMAT(A,E20.10)
                   lsync=.FALSE.

                ENDIF

             ENDDO

             ! we are not going to save the biomass for every component right now,
             ! just the total
             bm_sync(ipts,ivm,iele)=zero

             DO ipar=1,nparts
                   

                DO icir=1,ncirc

                   bm_sync(ipts,ivm,iele)=bm_sync(ipts,ivm,iele)+&
                        circ_class_biomass(ipts,ivm,icir,ipar,iele)*circ_class_n(ipts,ivm,icir)
                ENDDO ! ncirc

             ENDDO ! nparts

             bm_sync(ipts,ivm,iele)=ABS(bm_sync(ipts,ivm,iele)-&
                  SUM(biomass(ipts,ivm,:,iele)))

          ENDDO ! nelements



       ENDDO ! loop over PFTs

    ENDDO ! loop over points
 
    !---TEMP---
    IF(ld_biomass)THEN
       WRITE(numout,*) 'Check point: ',TRIM(check_point)
       WRITE(numout,*) 'test_pft, test_grid: ',test_pft,test_grid
       WRITE(numout,*) 'biomass (ileaf), ', biomass(test_grid,test_pft,ileaf,icarbon)
       WRITE(numout,*) 'biomass (iwood), ', biomass(test_grid,test_pft,isapabove,icarbon) + &
           biomass(test_grid,test_pft,isapbelow,icarbon) + biomass(test_grid,test_pft,iheartabove,icarbon) + &
           biomass(test_grid,test_pft,iheartbelow,icarbon)
       WRITE(numout,*) 'biomass (iroot), ', biomass(test_grid,test_pft,iroot,icarbon)
       WRITE(numout,'(A,20F14.6)') 'biomassHHH, ',biomass(test_grid,test_pft,:,icarbon)
       DO icir=1,ncirc
          WRITE(numout,'(A,I1,20F14.6)') 'ccbiomass',icir,circ_class_biomass(test_grid,test_pft,icir,:,icarbon)
       ENDDO
       WRITE(numout,*) 'circ_class_biomass, ',&
            SUM (SUM(circ_class_biomass(test_grid,test_pft,:,:,icarbon),2) * &
            circ_class_n(test_grid,test_pft,:))
       WRITE(numout,*) 'circ_class_n, ', SUM(circ_class_n(test_grid,test_pft,:))
       WRITE(numout,*) 'circ_class_n(:), ', circ_class_n(test_grid,test_pft,:)
       WRITE(numout,*) 'ind, ', ind(test_grid,test_pft)
    ENDIF

!!$    !----------

    IF(.NOT. lsync) THEN
       WRITE(numout,*) 'ERROR: stopping in sync #2'
       WRITE(numout,*) 'Stopping'
       CALL ipslerr_p (3,'check_biomass_sync', &
            'circ_class_biomass*circ_class_n is not equal to the total biomass',&
            'Look in the output file for more details.',&
            '')

    ENDIF
    IF(lnegative) THEN
       WRITE(numout,*) 'ERROR: negative biomass'
       WRITE(numout,*) 'Stopping'
       CALL ipslerr_p (3,'check_biomass_sync', &
            'One of the biomass pools is negative!',&
            'Look in the output file for more details.',&
            '')
    ENDIF

  END SUBROUTINE check_biomass_sync

!! ================================================================================================================================
!! SUBROUTINE  : check_area
!!
!>\BRIEF: Check whether surface area is preserved after forest
!! management, age_class_distribution and land use change
!!
!! DESCRIPTION : After processes where biomass pools could become zero, e.g., forest management
!!               a mass abalnce check alone is not good enough because the check is based on
!!               biomass*veget_max. Biomass is zero so veget_max could take any value. Here it is
!!               explicitly checked whether veget_max is preserved.
!!
!! RECENT CHANGE(S): None
!!
!! MAIN OUTPUT VARIABLE(S): None
!!
!! REFERENCE(S) : None
!!
!! FLOWCHART    : None
!! \n
!_ ================================================================================================================================
  SUBROUTINE check_area (check_point, npts, veget_max_begin, veget_max)

 !! 0. Variable and parameter description

    !! 0.1 Input variables
    CHARACTER(*),INTENT(in)                 :: check_point     !! A flag to indicate at which point in the code we're doing
                                                               !! this check
    INTEGER(i_std), INTENT(in)              :: npts            !! Number of pixels
    REAL(r_std), INTENT(in), DIMENSION(:,:) :: veget_max_begin !! Veget_max at the start of the routine that needs to be checked
    REAL(r_std), INTENT(in), DIMENSION(:,:) :: veget_max       !! Veget_max at the moment this routine is checked

    !! 0.2 Output variables

    !! 0.3 Modified variables

    !! 0.4 Local variables
    INTEGER(i_std)                          :: ipts,ivm        !! Indices
    REAL(r_std)                             :: area_begin      !! Temporairy variable to check area conservation
    REAL(r_std)                             :: area_end        !! Temporairy variable to check area conservation

!_ ================================================================================================================================

    ! Which subroutine is being checked?
    WRITE(numout,*) 'Check point: ',TRIM(check_point)

    ! Check for surface area presevation
    DO ipts = 1,npts

       DO ivm = 1,nvm

          ! Area balance closure
          IF (ivm .EQ. 1) THEN

             ! Initialize
             area_begin = veget_max_begin(ipts,ivm)
             area_end = veget_max(ipts,ivm)

          ELSEIF (agec_group(ivm-1) .EQ. agec_group(ivm)) THEN

             ! The current PFT belongs to the same species
             ! as the previous. Cumulate the areas.
             area_begin = area_begin + veget_max_begin(ipts,ivm)
             area_end = area_end + veget_max(ipts,ivm)

          ELSEIF (agec_group(ivm-1) .NE. agec_group(ivm)) THEN

             ! The current PFT belongs to a different species
             ! as the previous. Check the areas of the previous group
             ! and reset
             IF (ABS(area_begin-area_begin) .LE. min_stomate) THEN

                ! The code preserves the surface area
                IF (ld_massbal) WRITE(numout,*) &
                     'Preserved veget_max in ', check_point, ', igroup, ', &
                     agec_group(ivm-1)

             ELSE

                ! Surface area is NOT preserved. Write some
                ! additional information to diagnose the problem
                WRITE(numout,*) 'Error: veget_max is not closed in ', check_point
                WRITE(numout,*) ' ipts,igroup; ', ipts,agec_group(ivm-1)
                WRITE(numout,*) ' Difference is, ', area_end-area_begin
                WRITE(numout,*) ' veget_max_begin,veget_max_end: ', area_begin, &
                     area_end
                IF(ld_stop)THEN
                   CALL ipslerr_p (3,check_point, 'Area error','','')
                ENDIF

             ENDIF

             ! Write the surface area to the next 
             ! species group
             area_begin = veget_max_begin(ipts,ivm)
             area_end = veget_max(ipts,ivm)

          ELSE
             
             ! Shouldn't be here
             CALL ipslerr_p (3,check_point, &
                  'unexpected case in if-loop','','')
          ENDIF

         ! last pft
         IF (ivm .EQ. nvm) THEN

            ! The code preserves the surface area
            IF (ABS(area_begin-area_begin) .LE. min_stomate) THEN

               IF (ld_massbal) WRITE(numout,*) &
                    'Preserved veget_max in ',check_point, ', igroup, ', &
                    agec_group(ivm)
            
            ELSE

               ! Surface area is NOT preserved. Write some
               ! additional information to diagnose the problem
               WRITE(numout,*) 'Error: veget_max is not closed in ',check_point
               WRITE(numout,*) ' ipts,igroup; ', ipts,agec_group(ivm)
               WRITE(numout,*) ' Difference is, ', area_end-area_begin
               WRITE(numout,*) ' veget_max_begin,veget_max_end: ', area_begin, &
                    area_end
               IF(ld_stop)THEN
                  CALL ipslerr_p (3,check_point, 'Area error','','')
               ENDIF

            ENDIF ! preserve area
         ENDIF ! last ivm

      ENDDO ! nvm
   ENDDO ! npts

END SUBROUTINE check_area


END MODULE function_library
