!  ==============================================================================================================================
!  MODULE                   : enerbil
!
!  CONTACT                  : orchidee-help _at_ ipsl.jussieu.fr
!
!  LICENCE                      : IPSL (2006)
!  This software is governed by the CeCILL licence see ORCHIDEE/ORCHIDEE_CeCILL.LIC
!
!>\BRIEF                    This module computes the energy balance on 
!! continental surfaces. The module contains the following subroutines: enerbil_main, enerbil_init,
!! enerbil_clear, enerbil_var_init, enerbil_begin, enerbil_surftemp, enerbil_flux, enerbil_evapveg,
!! enerbil_fusion and enerbil_t2mdiag
!!
!!\n DESCRIPTION                                : 
!! \n
!! \latexonly 
!!     \input{enerbil_intro2.tex}
!! \endlatexonly
!!
!! IMPORTANT NOTE: The coefficients A and B are defined differently than those in the referenced 
!! literature and from those in the code and documentation of the atmospheric model LMDZ. For the
!! avoidance of doubt, the coefficients as described here always refer to the ORCHIDEE coefficients, 
!! and are denoted as such (with the marker: ORC). The re-definition of the coefficients takes place 
!! within LMDZ before they are passed to ORCHIDEE. The following sequence of expressions is to be 
!! found within the LMDZ module 'surf_land_orchidee':\n
!!
!! \latexonly 
!!     \input{surflandLMDZ1.tex}
!!     \input{surflandLMDZ2.tex}
!!     \input{surflandLMDZ3.tex}
!!     \input{surflandLMDZ4.tex}
!! \endlatexonly
!! \n
!!
!! \latexonly 
!!     \input{enerbil_symbols.tex}
!! \endlatexonly
!!
!! +++CHECK++++
!! NOTE: the code for the explicit snow scheme is added to the original code but
!! is not yet integrated in the DOFOCO code! When running DOFOCO the single layer
!! implementation of the multi-layer code is used. The multi-layer code has the
!! new water stress. Hence, the explicit snow can currently NOT be used with the
!! DOFOCO changes.
!! +++++++++++++
!!
!! RECENT CHANGE(S)                             : None
!!
!! REFERENCE(S)                                 : None 
!!
!! SVN          :
!! $HeadURL$
!! $Date$
!! $Revision$
!! \n
!_ ================================================================================================================================

MODULE enerbil

  ! routines called : restput, restget
  !
  ! modules used
  USE ioipsl
  USE xios_orchidee
  USE ioipsl_para 
  USE constantes
  USE pft_parameters
  USE qsat_moisture
  USE sechiba_io
  USE slowproc ! JR 151013 - because we need access to tot_bare_soil
  USE explicitsnow
  USE stomate_laieff ! JR 161214 - we need access to z_level_photo
  ! USE write_field_p, only : WriteFieldI_p  
  IMPLICIT NONE

  PRIVATE
  PUBLIC :: enerbil_main, enerbil_fusion,enerbil_clear, jdriver, enerbil_write

  ! variables used inside enerbil module : declaration and initialisation
 
  LOGICAL, SAVE                              :: l_first_enerbil=.TRUE.  !! Initialisation has to be done one time

  LOGICAL, SAVE                              :: l_first_jenerbil=.TRUE.  !! similar initialisation for multi-layer scheme

!$OMP THREADPRIVATE(l_first_enerbil)
  ! one dimension array allocated, computed and used in enerbil module exclusively

  LOGICAL, SAVE                              :: l_first_enerbil_column=.TRUE.  !! Initialisation has to be done one time
!$OMP THREADPRIVATE(l_first_enerbil_column)
  ! one dimension array allocated, computed and used in enerbil module exclusively

  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:) :: psold         !! Old surface dry static energy (J kg^{-1})
!$OMP THREADPRIVATE(psold)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:) :: qsol_sat      !! Saturated specific humudity for old temperature (kg kg^{-1})
!$OMP THREADPRIVATE(qsol_sat)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:) :: pdqsold       !! Deriv. of saturated specific humidity at old temp
                                                                 !! (kg (kg s)^{-1})
!$OMP THREADPRIVATE(pdqsold)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:) :: psnew         !! New surface static energy (J kg^{-1})
!$OMP THREADPRIVATE(psnew)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:) :: qsol_sat_new  !! New saturated surface air moisture (kg kg^{-1})
!$OMP THREADPRIVATE(qsol_sat_new)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:) :: netrad        !! Net radiation (W m^{-2})
!$OMP THREADPRIVATE(netrad)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:) :: lwabs         !! LW radiation absorbed by the surface (W m^{-2})
!$OMP THREADPRIVATE(lwabs)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:) :: lwup          !! Long-wave up radiation (W m^{-2})
!$OMP THREADPRIVATE(lwup)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:) :: lwnet         !! Net Long-wave radiation (W m^{-2})
!$OMP THREADPRIVATE(lwnet)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:) :: fluxsubli     !! Energy of sublimation (mm day^{-1})
!$OMP THREADPRIVATE(fluxsubli)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:) :: qsat_air      !! Air saturated specific humidity (kg kg^{-1})
!$OMP THREADPRIVATE(qsat_air)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:) :: tair          !! Air temperature (K)
!$OMP THREADPRIVATE(tair)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:) :: q_sol_pot     !! Potential surface humidity
!$OMP THREADPRIVATE(q_sol_pot)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:) :: temp_sol_pot  !! Potential surface temperature (unstressed)
!$OMP THREADPRIVATE(temp_sol_pot)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:) :: box_height_old !! division of canopy levels from the previous time step
!$OMP THREADPRIVATE(box_height_old)



  ! %%% JR 230813 start variables %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  ! variables that are used throughout the program / module
  INTEGER                                         :: timesteps = 1                ! number of half-hour timesteps
  REAL(r_std)                                     :: canopy_height = 50.0d0               ! canopy height (m)
  REAL(r_std)                                     :: u_h                                  ! total wind speed at canopy height (m/s)
  REAL(r_std)                                     :: rho_veg = 1000.0d0                   ! temporary for number checking
  REAL(r_std)                                     :: leaf_tks = 0.001d0                   ! leaf thickness
  REAL(r_std)                                     :: rho_soil = 1.0d0
  INTEGER, PARAMETER                              :: jnlvls_integ = 1
  REAL, DIMENSION(0:jnlvls_integ+1)               :: lai_profile, lai_cum
  

  REAL(r_std), SAVE                               :: top_jnlvls
  REAL(r_std), SAVE                               :: big_k_lw
  REAL(r_std), SAVE                               :: big_k_sw

  REAL(r_std), SAVE                               :: k_eddy_ceil    ! coefficients for S-shape fuction for K_eddy
  REAL(r_std), SAVE                               :: k_eddy_slope
  REAL(r_std), SAVE                               :: k_eddy_ustar

  REAL(r_std), SAVE                               :: ks_tune        ! coefficients for S-shape fuction for KS (K_eddy only for surface)
  REAL(r_std), SAVE                               :: ks_slope
  REAL(r_std), SAVE                               :: ks_veget

  REAL(r_std), SAVE                               :: sr_fac
  REAL(r_std), SAVE                               :: br_fac

  REAL(r_std), SAVE                               :: hd_big_r
  REAL(r_std), SAVE                               :: hd_big_r_prime
  CHARACTER(LEN=256), SAVE                        :: jamescdfname           !! To store the name of the multilayer netCDF output file (seperate to the rest)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)   :: pad_deltah
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)   :: box_height

  
  ! %%%% JR 230813 end variables %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


  CONTAINS  
    
   
!!  ===========================================================================================================================
!! SUBROUTINE                   : enerbil_main
!!
!!
!>\BRIEF                    Calls each part of the energy budget calculation in sequence 
!!
!! DESCRIPTION              : 
!! This is the main routine for the 'enerbil' module. It is called
!! once during the initialisation of ORCHIDEE, and then once for each time step. It is called a final
!! time at the culmination of the last time step, for the writing of a restart file.\n
!!
!! The algorithm first calls 'enerbil_begin' for initialisation, followed by 'enerbil_surftemp' to
!! calculate the surface static energy and the saturated surface humidity for the new time step.
!! Next is the module 'enerbil_flux' which calculates the new surface temperature, the net radiation in
!! the surface layer, the total evaporation and the latent and sensible heat fluxes. Finally comes
!! 'enerbil_evapveg', which calculates the evaporation and transpiration from the vegetation.\n
!! \n
!!
!! +++CHECK++++
!! NOTE: the code for the explicit snow scheme is added to the original code but
!! is not yet integrated in the DOFOCO code! When running DOFOCO the single layer
!! implementation of the multi-layer code is used. The multi-layer code has the
!! new water stress. Hence, the explicit snow can currently NOT be used with the
!! DOFOCO changes.
!! +++++++++++++
!!
!! RECENT CHANGE(S): None
!!
!! MAIN OUTPUT VARIABLE(S)  : evapot, evapot_corr, temp_sol, qsurf, fluxsens, fluxlat, tsol_rad,
!! vevapp, temp_sol_new, vevapnu, vevapsno, transpir, vevapwet, t2mdiag
!!
!! REFERENCE(S)     :
!! - Best, MJ, Beljaars, A, Polcher, J & Viterbo, P, 2004. A proposed structure for coupling tiled
!! surfaces with the planetary boundary layer. Journal of Hydrometeorology, 5, pp.1271-1278
!! - de Noblet-Ducoudré, N, Laval, K & Perrier, A, 1993. SECHIBA, a new set of parameterisations
!! of the hydrologic exchanges at the land-atmosphere interface within the LMD Atmospheric General
!! Circulation Model. Journal of Climate, 6, pp.248-273
!! - Dufresne, J-L & Ghattas, J, 2009. Description du schéma de la couche limite turbulente et la
!! interface avec la surface planetaire dans LMDZ, Technical note, available (22/12/11):
!! http://lmdz.lmd.jussieu.fr/developpeurs/notes-techniques/ressources/pbl_surface.pdf
!! - Guimberteau, M, 2010. Modélisation de l'hydrologie continentale et influences de l'irrigation
!! sur le cycle de l'eau, PhD Thesis, available (22/12/11):
!! http://www.sisyphe.upmc.fr/~guimberteau/docs/manuscrit_these.pdf 
!! - Polcher, J. McAvaney, B, Viterbo, P, Gaertner, MA, Hahmann, A, Mahfouf, J-F, Noilhan, J
!! Phillips, TJ, Pitman, AJ, Schlosser, CA, Schulz, J-P, Timbal, B, Verseghy, D &
!! Xue, Y, 1998. A proposal for a general interface between land surface schemes and
!! general circulation models. Global and Planetary Change, 19, pp.261-276
!! - Richtmeyer, RD, Morton, KW, 1967. Difference Methods for Initial-Value Problems.
!! Interscience Publishers\n
!! - Schulz, Jan-Peter, Lydia Dümenil, Jan Polcher, 2001: On the Land Surface–Atmosphere 
!! Coupling and Its Impact in a Single-Column Atmospheric Model. J. Appl. Meteor., 40, 642–663.
!!
!! FLOWCHART            :
!! \latexonly 
!!     \includegraphics[scale=0.5]{enerbil_main_flowchart.png}
!! \endlatexonly
!! \n
!_ ==============================================================================================================================
   
  ! %%%% 230813 JR function insert %%%%%%%%

  REAL(r_std) FUNCTION gu (x_in)
    
  REAL(r_std) :: k_ext = 0.9 
  REAL(r_std) :: x_in, k_ext_fac
  
      !+++ TEMP +++
       k_ext_fac = 1.0 
       CALL getin_p('K_EXT_FAC',k_ext_fac)
           k_ext=k_ext*k_ext_fac 
      !++++++++++++
      gu = exp(-k_ext*x_in)    
  END FUNCTION ! gu(x_in)

  ! %%%% 230813 end JR function insert %%%%
  SUBROUTINE enerbil_main (kjit, kjpindex, dtradia, ldrestart_read, ldrestart_write, &
            & index, indexveg, zlev, lwdown, swnet, swdown, &
            & epot_air, temp_air, u, v, petAcoef, &
            & petBcoef, qair, peqAcoef, peqBcoef, pb, &
            & rau, vbeta, valpha, vbeta1, vbeta2, &
            & vbeta3, vbeta3pot, vbeta4, vbeta5, emis, &
            & soilflx, soilcap, q_cdrag, humrel, fluxsens, &
            & fluxlat, vevapp, transpir, transpot, vevapnu, &
            & vevapwet, vevapsno, vevapflo, t2mdiag, temp_sol, &
            & tsol_rad, temp_sol_new, qsurf, evapot, evapot_corr, &
            & rest_id, hist_id, hist2_id, beta_diffuco, &
            & precip_rain, snow, sfcfrz, radsink, &
            & veget_max, osfcmelt, pgflux, gthick, gtemp, &
            & gpkappa, grndflux, albedo_undersnow, snowtemp, snowdz, &
            & radxs, pkappa_snow, precip_snow, snowliq, snowheat, & 
            & snowrho, snowgrain, snowdz_o, soilflxresid)

    !! 0 Variable and parameter description

    !! 0.1 Input variables

    INTEGER(i_std), INTENT(in)                           :: kjit             !! Time step number (-)
    INTEGER(i_std), INTENT(in)                           :: kjpindex         !! Domain size (-)
    INTEGER(i_std),INTENT (in)                           :: rest_id,hist_id  !! _Restart_ file and _history_ file identifier (-)
    INTEGER(i_std),INTENT (in)                           :: hist2_id         !! _history_ file 2 identifier (-)
    REAL(r_std), INTENT (in)                             :: dtradia          !! Time (s)
    LOGICAL, INTENT(in)                                  :: ldrestart_read   !! Logical for _restart_ file to read (-)
    LOGICAL, INTENT(in)                                  :: ldrestart_write  !! Logical for _restart_ file to write (-)

    INTEGER(i_std),DIMENSION (kjpindex), INTENT (in)     :: index            !! Indeces of the points on the map (-)
    INTEGER(i_std),DIMENSION(kjpindex*nvm), INTENT(in)   :: indexveg         !! Indeces of the points on the 3D map
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)        :: zlev             !! Height of first layer (m)
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)        :: lwdown           !! Down-welling long-wave flux (W m^{-2})
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)        :: swnet            !! Net surface short-wave flux (W m^{-2})
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)        :: swdown           !! Down-welling short-wave flux (W/m^2) 
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)        :: epot_air         !! Air potential energy (?? J)
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)        :: temp_air         !! Air temperature (K)
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)        :: u                !! Eastward Lowest level wind speed  (m s^{-1})
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)        :: v                !! Northward Lowest level wind speed (m s^{-1})
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)        :: petAcoef         !! PetAcoef (see note)
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)        :: petBcoef         !! PetBcoef (see note)
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)        :: qair             !! Lowest level specific humidity (kg kg^{-1})
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)        :: peqAcoef         !! PeqAcoef (see note)
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)        :: peqBcoef         !! PeqBcoef (see note)
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)        :: pb               !! Lowest level pressure (hPa)
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)        :: rau              !! Air density (kg m^{-3})
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)        :: vbeta            !! Resistance coefficient (-)
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)        :: valpha           !! Resistance coefficient (-)
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)        :: vbeta1           !! Snow resistance (-) 
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)        :: vbeta4           !! Bare soil resistance (-)
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)        :: vbeta5           !! Floodplains resistance
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)        :: emis             !! Emissivity (-)
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)        :: soilflx          !! Soil flux (W m^{-2})
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)        :: soilcap          !! Soil calorific capacity (J K^{-1])
    REAL(r_std),DIMENSION (kjpindex), INTENT (inout)     :: q_cdrag          !! Surface drag (m s^{-1})
    REAL(r_std),DIMENSION (kjpindex,nvm), INTENT (in)    :: humrel           !! Soil moisture stress (within range 0 to 1)
    REAL(r_std),DIMENSION (kjpindex,nvm), INTENT (in)    :: vbeta2           !! Interception resistance (-)
    REAL(r_std),DIMENSION (kjpindex,nvm), INTENT (in)    :: vbeta3           !! Vegetation resistance (-)
    REAL(r_std),DIMENSION (kjpindex,nvm), INTENT (in)    :: vbeta3pot        !! Vegetation resistance for potential transpiration
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)        :: snow             !! Snow mass [kg/m2]
    REAL(r_std),DIMENSION (kjpindex),INTENT (in)         :: precip_rain      !! Rainfall
    REAL(r_std),DIMENSION (kjpindex),INTENT (in)         :: precip_snow      !! Snowfall
    REAL(r_std),DIMENSION (kjpindex,nvm),INTENT(in)      :: veget_max        !! Maximum vegetation fraction
    REAL(r_std),DIMENSION (kjpindex), INTENT(in)         :: gthick           !! First soil layer thickness
    REAL(r_std),DIMENSION (kjpindex), INTENT(in)         :: gpkappa          !! First soil layer thermal conductivity
    REAL(r_std),DIMENSION (kjpindex),INTENT(in)          :: albedo_undersnow !! Albedo under snow
  

    !! 0.2 Output variables
    REAL(r_std),DIMENSION (kjpindex), INTENT (out)       :: vevapnu          !! Bare soil evaporation (mm day^{-1})
    REAL(r_std),DIMENSION (kjpindex), INTENT (out)       :: vevapsno         !! Snow evaporation (mm day^{-1})
    REAL(r_std),DIMENSION (kjpindex), INTENT (out)       :: vevapflo         !! Floodplains evaporation
    REAL(r_std),DIMENSION (kjpindex,nvm), INTENT (out)   :: transpir         !! Transpiration (mm day^{-1})
    REAL(r_std),DIMENSION (kjpindex,nvm), INTENT (out)   :: transpot         !! Potential transpiration
    REAL(r_std),DIMENSION (kjpindex,nvm), INTENT (out)   :: vevapwet         !! Interception (mm day^{-1})
    REAL(r_std),DIMENSION (kjpindex), INTENT (out)       :: t2mdiag          !! Diagnostic temperature at 2m above ground (K)
    REAL(r_std),DIMENSION (kjpindex)                     :: beta_diffuco

    
    !! 0.3 Modified variables
    REAL(r_std),DIMENSION (kjpindex), INTENT (inout)     :: evapot           !! Soil Potential Evaporation (mm day^{-1})
    REAL(r_std),DIMENSION (kjpindex), INTENT (inout)     :: evapot_corr      !! Soil Potential Evaporation Correction (mm day^{-1})
    REAL(r_std),DIMENSION (kjpindex), INTENT (inout)     :: temp_sol         !! Soil temperature (K)
    REAL(r_std),DIMENSION (kjpindex), INTENT (inout)     :: qsurf            !! Surface specific humidity (kg kg^{-1})
    REAL(r_std),DIMENSION (kjpindex), INTENT (inout)     :: fluxsens         !! Sensible heat flux (W m^{-2})
    REAL(r_std),DIMENSION (kjpindex), INTENT (inout)     :: fluxlat          !! Latent heat flux (W m^{-2})
    REAL(r_std),DIMENSION (kjpindex), INTENT (inout)     :: tsol_rad         !! Tsol_rad (W m^{-2})
    REAL(r_std),DIMENSION (kjpindex), INTENT (inout)     :: vevapp           !! Total of evaporation (mm day^{-1})
    REAL(r_std),DIMENSION (kjpindex), INTENT (inout)     :: temp_sol_new     !! New soil temperature (K)
    REAL(r_std),DIMENSION (kjpindex),INTENT (inout)      :: sfcfrz           !! Surface frozen fraction
    REAL(r_std),DIMENSION (kjpindex,nsnow),INTENT(inout) :: radsink          !! Radiation term at each snow layer
    LOGICAL, DIMENSION(kjpindex), INTENT(inout)          :: osfcmelt         !! Indicate snow melting in each gridcell
    REAL(r_std),DIMENSION (kjpindex), INTENT (inout)     :: pgflux           !! Net energy into snowpack(W/m^2)
    REAL(r_std),DIMENSION (kjpindex), INTENT(inout)      :: gtemp            !! First soil layer temperature
    REAL(r_std),DIMENSION (kjpindex), INTENT(inout)      :: grndflux         !! Energy flux into the soil
    REAL(r_std),DIMENSION (kjpindex,nsnow),INTENT(inout) :: snowrho          !! Snow density at each snow layer
    REAL(r_std),DIMENSION (kjpindex,nsnow),INTENT(inout) :: snowliq          !! Liquid water content at each snow layer
    REAL(r_std),DIMENSION (kjpindex,nsnow),INTENT(inout) :: snowheat         !! Heat capacity at each snow layer
    REAL(r_std),DIMENSION (kjpindex,nsnow),INTENT(inout) :: snowtemp         !! Snow temperature at each snow layer
    REAL(r_std),DIMENSION (kjpindex,nsnow),INTENT(inout) :: snowdz           !! Snow depth at each snow layer
    REAL(r_std),DIMENSION (kjpindex,nsnow),INTENT(inout) :: snowgrain        !! Snow grain size at each snow layer
    REAL(r_std),DIMENSION (kjpindex),INTENT(inout)       :: radxs            !! Residual radiation at bottom of the snowpack
    REAL(r_std), DIMENSION (kjpindex,nsnow),INTENT(inout):: pkappa_snow      !! Snow thermal conductivity at each snow layer
    REAL(r_std), DIMENSION (kjpindex,nsnow),INTENT(inout):: snowdz_o         !! Snow depth at previous time step
    REAL(r_std),DIMENSION(kjpindex),INTENT(inout)        :: soilflxresid     !! Energy flux to the snowpack


    !! 0.4 Local variables
    REAL(r_std), DIMENSION(kjpindex)                     :: dev_qsol
    REAL(r_std),DIMENSION (kjpindex)                     :: epot_air_new
    REAL(r_std),DIMENSION (kjpindex)                     :: qair_new
    CHARACTER(LEN=80)                                    :: var_name         !! To store variables names for I/O
    REAL(r_std),DIMENSION (kjpindex)                     :: ptso1
    REAL(r_std),DIMENSION (kjpindex)                     :: s2flux           !! Surface and sub-surface snow layer heat  conduction 
    REAL(r_std),DIMENSION (kjpindex)                     :: psnowhmass       !! heat content of snowfall (J/m2)
    INTEGER(i_std)                                       :: ji,ii,iv
    REAL(r_std)                                          :: M_1_h, N_1_h, M_1_q, N_1_q

!_ ================================================================================================================================

   
    !! 1. Run the initialisation routines 
    IF (l_first_enerbil) THEN

        IF (long_print) WRITE (numout,*) ' l_first_enerbil : call enerbil_init '
        CALL enerbil_init (kjit, ldrestart_read, kjpindex, index, rest_id, qair, temp_sol, temp_sol_new,  &
             &             qsurf, tsol_rad, vevapp, fluxsens, fluxlat, evapot, evapot_corr)

        CALL enerbil_var_init (kjpindex, temp_air, t2mdiag)
        ! This needs initialization too, since it is used in hydrol.  I have no idea if this is
        ! the correct value.
        vevapsno(:)=zero
        RETURN 

    ENDIF
        
    !! 2. Prepare a restart file for the next simulation
    
    IF (ldrestart_write) THEN

        IF (long_print) WRITE (numout,*) ' we have to complete restart file with ENERBIL variables '

        var_name= 'temp_sol'  
        CALL restput_p(rest_id, var_name, nbp_glo, 1, 1, kjit,  temp_sol, 'scatter',  nbp_glo, index_g)

        var_name= 'qsurf'  
        CALL restput_p(rest_id, var_name, nbp_glo, 1, 1, kjit,  qsurf, 'scatter',  nbp_glo, index_g)

        var_name= 'evapot'  
        CALL restput_p(rest_id, var_name, nbp_glo, 1, 1, kjit,  evapot, 'scatter',  nbp_glo, index_g)

        var_name= 'evapot_corr'  
        CALL restput_p(rest_id, var_name, nbp_glo, 1, 1, kjit,  evapot_corr, 'scatter',  nbp_glo, index_g)

        var_name= 'tsolrad'
        CALL restput_p(rest_id, var_name, nbp_glo, 1, 1, kjit,  tsol_rad, 'scatter',  nbp_glo, index_g)

        var_name= 'evapora'
        CALL restput_p(rest_id, var_name, nbp_glo, 1, 1, kjit,  vevapp, 'scatter',  nbp_glo, index_g)

        var_name= 'fluxlat'
        CALL restput_p(rest_id, var_name, nbp_glo, 1, 1, kjit,  fluxlat, 'scatter',  nbp_glo, index_g)

        var_name= 'fluxsens'
        CALL restput_p(rest_id, var_name, nbp_glo, 1, 1, kjit,  fluxsens, 'scatter',  nbp_glo, index_g)

        var_name= 'tempsolpot'
        CALL restput_p(rest_id, var_name, nbp_glo, 1, 1, kjit, temp_sol_pot, 'scatter',  nbp_glo, index_g)

        var_name= 'qsolpot'
        CALL restput_p(rest_id, var_name, nbp_glo, 1, 1, kjit, q_sol_pot, 'scatter',  nbp_glo, index_g)
       
        RETURN 

    ENDIF
      
    
    IF (.not. control%ok_nenerbil ) THEN
    
       !!------Do standard calculation & set variable out if ok_enbile is FALSE ------       
       !! 3. Computes some initialisation variables
       !  Computes some initialisation variables psold (the old surface static energy), 
       !  qsol_sat (the saturated surface humidity) and pdqsold (the derivative of the 
       !  saturated surface humidity,
       ! calculated with respect to the surface temperature at the 'old' timestep)
       CALL enerbil_begin (kjpindex, temp_sol, lwdown, swnet, pb, &
            & psold, qsol_sat, pdqsold, netrad, emis, &
            & dtradia, u, v, &
            & precip_snow, temp_air, albedo_undersnow, snowtemp, snowdz, &
            & snowrho, snowheat, snowliq, snowgrain, radsink, &
            & radxs, psnowhmass, s2flux, ptso1)
 
       !! 4. Computes psnew (the surface static energy at the 'new' timestep) 
       ! Computes psnew (the surface static energy at the 'new' timestep), qsol_sat_new 
       ! (the surface saturated humidity at the 'new' timestep), temp_sol_new (the 
       ! surface temperature at the 'new timestep), qair_new (the lowest atmospheric 
       ! humidity at the 'new' timestep) and epot_air_new
       ! (the lowest atmospheric evaporation potential at the 'new' timestep)
       CALL enerbil_surftemp (kjpindex, dtradia, zlev, emis, &
        & epot_air, petAcoef, petBcoef, qair, peqAcoef, &
                & peqBcoef, soilflx, rau, u, v, &
                & q_cdrag, vbeta, valpha, vbeta1, vbeta5, &
                & soilcap, lwdown, swnet, psnew, qsol_sat_new, &
                & temp_sol_new, qair_new, epot_air_new, beta_diffuco, &
                & temp_sol, ptso1, radsink, snowrho, &
                & snowdz, snowtemp, snowliq, sfcfrz, pb, &
                & temp_air, precip_rain, gthick, gtemp, gpkappa, &
                & grndflux, s2flux, pkappa_snow, snowdz_o, soilflxresid)
  
       !  computes surface temperature and humidity for a saturated surface
       CALL enerbil_pottemp (kjpindex, dtradia, zlev, emis, &
            & epot_air, petAcoef, petBcoef, qair, peqAcoef, &
            & peqBcoef, soilflx, rau, u, v, &
            & q_cdrag, vbeta, valpha, vbeta1, vbeta5, &
            & soilcap, lwdown, swnet, q_sol_pot, temp_sol_pot)
       
       !! 5. Diagnoses components of the energy budget  
       ! Diagnoses lwup (upwards long wave radiation), lwnet (net long wave radiation), tsol_rad (radiative
       ! ground temperature), netrad (net radiation), qsurf (surface humidity), vevapp (total evaporation),
       ! evapot (evaporation potential), evapot_corr (evaporation potential correction factor), 
       ! fluxlat (latent heat flux), fluxsubli (sublimination heat flux) and fluxsens (sensible heat flux).

       CALL enerbil_flux (kjpindex, dtradia, emis, temp_sol, rau, &
            & u, v, q_cdrag, vbeta, valpha, &
            & vbeta1, vbeta5, qair, epot_air, psnew, &
            & qsurf, fluxsens, fluxlat, fluxsubli, vevapp, &
            & temp_sol_new, lwdown, swnet, lwup, lwnet, &
            & pb, tsol_rad, netrad, evapot, evapot_corr, &
            & beta_diffuco, soilflx, soilcap, precip_rain, &
            & snowdz, snowrho, snowtemp, radsink, s2flux, &
            & osfcmelt, ptso1, temp_air, pgflux)

    !! 6. Diagnoses the values for evaporation and transpiration 
     
    ! Diagnoses the values for evaporation and transpiration: vevapsno (snow evaporation), vevapnu 
    ! (bare soil evaporation), transpir (transpiration), and vevapwet (interception)
        CALL enerbil_evapveg (kjpindex, dtradia, vbeta1, vbeta2, vbeta3, &
           & vbeta3pot, vbeta4, vbeta5, rau, u, &
           & v, q_cdrag, qair, humrel, vevapsno, &
           & vevapnu , vevapflo, vevapwet, transpir, transpot, &
           & evapot, snowdz, snowrho, evapot_corr)

    !! 7. Assigns a temperature 2 metres above the ground 

    ! Assigns a temperature 2 metres above the ground. The output variable - t2mdiag, is used just for
    ! diagnostic purposes, for now.
    CALL enerbil_t2mdiag (kjpindex, temp_air, t2mdiag)

    ! write(numout, *)  "YC standard energy budget:"

    !CALL xios_orchidee_send_field("netrad",netrad)
    CALL xios_orchidee_send_field("evapot",evapot*one_day/dt_sechiba)
    CALL xios_orchidee_send_field("evapot_alma",evapot/dt_sechiba)
    CALL xios_orchidee_send_field("evapot_corr",evapot_corr*one_day/dt_sechiba)
    CALL xios_orchidee_send_field("evapot_corr_alma",evapot_corr/dt_sechiba)
    CALL xios_orchidee_send_field("lwdown",lwabs)
    CALL xios_orchidee_send_field("lwnet",lwnet)
    CALL xios_orchidee_send_field("Qv",fluxsubli)
    CALL xios_orchidee_send_field("PotSurfT",temp_sol_pot)
          
        IF ( .NOT. almaoutput ) THEN
            !CALL histwrite_p(hist_id, 'netrad', kjit, netrad, kjpindex, index)
            CALL histwrite_p(hist_id, 'evapot', kjit, evapot, kjpindex, index)
            CALL histwrite_p(hist_id, 'evapot_corr', kjit, evapot_corr, kjpindex, index)
            CALL histwrite_p(hist_id, 'lwdown', kjit, lwabs,  kjpindex, index)
            CALL histwrite_p(hist_id, 'lwnet',  kjit, lwnet,  kjpindex, index)
             IF ( hist2_id > 0 ) THEN
            !CALL histwrite_p(hist2_id, 'netrad', kjit, netrad, kjpindex, index)
            CALL histwrite_p(hist2_id, 'evapot', kjit, evapot, kjpindex, index)
            CALL histwrite_p(hist2_id, 'evapot_corr', kjit, evapot_corr, kjpindex, index)
            CALL histwrite_p(hist2_id, 'lwdown', kjit, lwabs,  kjpindex, index)
            CALL histwrite_p(hist2_id, 'lwnet',  kjit, lwnet,  kjpindex, index)
             ENDIF
        ELSE
            CALL histwrite_p(hist_id, 'LWnet', kjit, lwnet, kjpindex, index)
            CALL histwrite_p(hist_id, 'Qv', kjit, fluxsubli, kjpindex, index)
            CALL histwrite_p(hist_id, 'PotEvap', kjit, evapot_corr, kjpindex, index)
            CALL histwrite_p(hist_id, 'PotEvapOld', kjit, evapot, kjpindex, index)
            CALL histwrite_p(hist_id, 'PotSurfT', kjit, temp_sol_pot, kjpindex, index)
             IF ( hist2_id > 0 ) THEN
            CALL histwrite_p(hist2_id, 'LWnet', kjit, lwnet, kjpindex, index)
            CALL histwrite_p(hist2_id, 'Qv', kjit, fluxsubli, kjpindex, index)
            CALL histwrite_p(hist2_id, 'PotEvap', kjit, evapot_corr, kjpindex, index)
             ENDIF
        ENDIF
    
    ELSE
    
!---If ok_enbile is TRUE, we write/set the vaiables out to histoty file here. 

        !! New energy budget scheme don't follow the standard energy budget calling grapgh.
    !! The calculation process from "Section 3" to "Section 7" were modified and called in the jdriver.f90, 
    !! which could be called in the rountine of sechiba.f90 
    !!     
    !write(numout, *) "YC new energy budget:"
    
        !CALL xios_orchidee_send_field("netrad",netrad)
        CALL xios_orchidee_send_field("evapot",evapot*one_day/dt_sechiba)
        CALL xios_orchidee_send_field("evapot_alma",evapot/dt_sechiba)
        CALL xios_orchidee_send_field("evapot_corr",evapot_corr*one_day/dt_sechiba)
        CALL xios_orchidee_send_field("evapot_corr_alma",evapot_corr/dt_sechiba)
        CALL xios_orchidee_send_field("lwdown",lwabs)
        CALL xios_orchidee_send_field("lwnet",lwnet)
        CALL xios_orchidee_send_field("Qv",fluxsubli)
        CALL xios_orchidee_send_field("PotSurfT",temp_sol_pot)
          
        IF ( .NOT. almaoutput ) THEN
            !CALL histwrite_p(hist_id, 'netrad', kjit, netrad, kjpindex, index)
            CALL histwrite_p(hist_id, 'evapot', kjit, evapot, kjpindex, index)
            CALL histwrite_p(hist_id, 'evapot_corr', kjit, evapot_corr, kjpindex, index)
            CALL histwrite_p(hist_id, 'lwdown', kjit, lwabs,  kjpindex, index)
            CALL histwrite_p(hist_id, 'lwnet',  kjit, lwnet,  kjpindex, index)
            IF ( hist2_id > 0 ) THEN
            !CALL histwrite_p(hist2_id, 'netrad', kjit, netrad, kjpindex, index)
            CALL histwrite_p(hist2_id, 'evapot', kjit, evapot, kjpindex, index)
            CALL histwrite_p(hist2_id, 'evapot_corr', kjit, evapot_corr, kjpindex, index)
            CALL histwrite_p(hist2_id, 'lwdown', kjit, lwabs,  kjpindex, index)
            CALL histwrite_p(hist2_id, 'lwnet',  kjit, lwnet,  kjpindex, index)
            ENDIF
        ELSE
            CALL histwrite_p(hist_id, 'LWnet', kjit, lwnet, kjpindex, index)
            CALL histwrite_p(hist_id, 'Qv', kjit, fluxsubli, kjpindex, index)
            CALL histwrite_p(hist_id, 'PotEvap', kjit, evapot_corr, kjpindex, index)
            CALL histwrite_p(hist_id, 'PotEvapOld', kjit, evapot, kjpindex, index)
            CALL histwrite_p(hist_id, 'PotSurfT', kjit, temp_sol_pot, kjpindex, index)
            IF ( hist2_id > 0 ) THEN
            CALL histwrite_p(hist2_id, 'LWnet', kjit, lwnet, kjpindex, index)
            CALL histwrite_p(hist2_id, 'Qv', kjit, fluxsubli, kjpindex, index)
            CALL histwrite_p(hist2_id, 'PotEvap', kjit, evapot_corr, kjpindex, index)
            ENDIF
        ENDIF 
       
    ENDIF  

    IF (long_print) WRITE (numout,*) ' enerbil_main Done '

  END SUBROUTINE enerbil_main


  !!  =============================================================================================================================
  !! SUBROUTINE                 : enerbil_init
  !!
  !>\BRIEF                      Performs dynamic allocation for local arrays, and retrieves
  !! previous data from restart file (if the switch is set to do so)
  !!
  !! DESCRIPTION                : The purpose of this module is, firstly, to allocate space
  !! in memory for key variables within the 'enerbil' module. The second task is to retrieve previous data
  !! from the restart file, if the 'ldrestart-read' flag is set.
  !!
  !! RECENT CHANGE(S)               : None
  !!
  !! MAIN OUTPUT VARIABLE(S)                : temp_sol, temp_sol_new, qsurf, tsol_rad, vevapp, fluxsens,
  !! fluxlat, evapot, evapot_corr
  !!
  !! REFERENCE(S)               : None
  !! 
  !! FLOWCHART                              : None
  !! \n
  !_ ==============================================================================================================================
  
  SUBROUTINE enerbil_init (kjit, ldrestart_read, kjpindex, index, rest_id, qair, temp_sol, temp_sol_new, &
       &                   qsurf, tsol_rad, vevapp, fluxsens, fluxlat, evapot, evapot_corr)

    !! 0. Variable and parameter declaration

    !! 0.1 Input variables
  
    INTEGER(i_std), INTENT (in)                              :: kjit               !! Time step number (-) 
    LOGICAL ,INTENT (in)                                     :: ldrestart_read     !! Logical for _restart_ file to read (-)
    INTEGER(i_std), INTENT (in)                              :: kjpindex           !! Domain size (-)
    INTEGER(i_std),DIMENSION (kjpindex), INTENT (in)         :: index              !! Indeces of the points on the map (-)
    INTEGER(i_std), INTENT (in)                              :: rest_id            !! _Restart_ file identifier (-)
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: qair               !! Lowest level specific humidity (kg kg^{-1})
    
    !! 0.2 Output variables
    
    REAL(r_std),DIMENSION (kjpindex), INTENT (out)           :: temp_sol_new       !! New ground temperature (K)
    REAL(r_std),DIMENSION (kjpindex), INTENT (out)           :: qsurf              !! Near surface specific humidity (kg kg^{-1})
    REAL(r_std),DIMENSION (kjpindex), INTENT (out)           :: tsol_rad           !! Tsol_rad (W m^{-2})
    REAL(r_std),DIMENSION (kjpindex), INTENT (out)           :: vevapp             !! Total of evaporation (mm day^{-1})
    REAL(r_std),DIMENSION (kjpindex), INTENT (out)           :: fluxsens           !! Sensible heat flux (W m^{-2})
    REAL(r_std),DIMENSION (kjpindex), INTENT (out)           :: fluxlat            !! Latent heat flux (W m^{-2})
    REAL(r_std),DIMENSION (kjpindex), INTENT (out)           :: evapot             !! Soil Potential Evaporation (mm day^{-1})
    REAL(r_std),DIMENSION (kjpindex), INTENT (out)           :: evapot_corr        !! Soil Potential Evaporation Correction 
                                                                                   !! (mm day^{-1})

    !! 0.3 Modified variables

    REAL(r_std),DIMENSION (kjpindex), INTENT (inout)         :: temp_sol           !! Ground temperature (K)

    !! 0.4 Local variables

    INTEGER(i_std)                                           :: ier
    CHARACTER(LEN=80)                                        :: var_name            !! To store variables names for I/O

!_ ================================================================================================================================

  !! 1 Initialisation of variables
 
    !! Initialisation of variables psold (old surface static energy), qsol_sat (saturated surface air moisture), pdqsold
    !! (derivative of satured specific humidity at the old temperature), psnew (new surface static energy), qsol_sat_new
    !! (new saturated surface air moisture), netrad (net radiation), lwabs (long wave absolute radiation), lwup (long wave
    !! (upwelling radiation), lwnet (long wave net radiation), fluxsubli (sublimination flux), qsat_air (saturated air humidity)
    !! and tair (air temperature).
    IF (l_first_enerbil) THEN 
        l_first_enerbil=.FALSE.
    ELSE 
        WRITE (numout,*) ' l_first_enerbil false . we stop'
        CALL ipslerr(3,'enerbil.f90','enerbil_init','','')  
    ENDIF

    ALLOCATE (psold(kjpindex),stat=ier)
    IF (ier.NE.0) THEN
        WRITE (numout,*) ' error in psold allocation. We stop.We need kjpindex words = ',kjpindex
        CALL ipslerr(3,'enerbil.f90','enerbil_init','','')
    END IF

    ALLOCATE (qsol_sat(kjpindex),stat=ier)
    IF (ier.NE.0) THEN
        WRITE (numout,*) ' error in qsol_sat allocation. We stop. We need kjpindex words = ',kjpindex
        CALL ipslerr(3,'enerbil.f90','enerbil_init','','')
    END IF

    ALLOCATE (pdqsold(kjpindex),stat=ier)
    IF (ier.NE.0) THEN
        WRITE (numout,*) ' error in pdqsold allocation. We stop. We need kjpindex words = ',kjpindex
        CALL ipslerr(3,'enerbil.f90','enerbil_init','','')
    END IF

    ALLOCATE (psnew(kjpindex),stat=ier)
    IF (ier.NE.0) THEN
        WRITE (numout,*) ' error in psnew allocation. We stop. We need kjpindex words = ',kjpindex
        CALL ipslerr(3,'enerbil.f90','enerbil_init','','')
    END IF

    ALLOCATE (qsol_sat_new(kjpindex),stat=ier)
    IF (ier.NE.0) THEN
        WRITE (numout,*) ' error in qsol_sat_new allocation. We stop. We need kjpindex words = ',kjpindex
        CALL ipslerr(3,'enerbil.f90','enerbil_init','','')
    END IF

    ALLOCATE (netrad(kjpindex),stat=ier)
    IF (ier.NE.0) THEN
        WRITE (numout,*) ' error in netrad allocation. We stop. We need kjpindex words = ',kjpindex
        CALL ipslerr(3,'enerbil.f90','enerbil_init','','')
    END IF

    ALLOCATE (lwabs(kjpindex),stat=ier)
    IF (ier.NE.0) THEN
        WRITE (numout,*) ' error in lwabs allocation. We stop. We need kjpindex words = ',kjpindex
        CALL ipslerr(3,'enerbil.f90','enerbil_init','','')
    END IF

    ALLOCATE (lwup(kjpindex),stat=ier)
    IF (ier.NE.0) THEN
        WRITE (numout,*) ' error in lwup allocation. We stop. We need kjpindex words = ',kjpindex
        CALL ipslerr(3,'enerbil.f90','enerbil_init','','')
    END IF

    ALLOCATE (lwnet(kjpindex),stat=ier)
    IF (ier.NE.0) THEN
        WRITE (numout,*) ' error in lwnet allocation. We stop. We need kjpindex words = ',kjpindex
        CALL ipslerr(3,'enerbil.f90','enerbil_init','','')
    END IF

    ALLOCATE (fluxsubli(kjpindex),stat=ier)
    IF (ier.NE.0) THEN
        WRITE (numout,*) ' error in fluxsubli allocation. We stop. We need kjpindex words = ',kjpindex
        CALL ipslerr(3,'enerbil.f90','enerbil_init','','')
    END IF

    ALLOCATE (qsat_air(kjpindex),stat=ier)
    IF (ier.NE.0) THEN
        WRITE (numout,*) ' error in qsat_air allocation. We stop. We need kjpindex words = ',kjpindex
        CALL ipslerr(3,'enerbil.f90','enerbil_init','','')
    END IF

    ALLOCATE (tair(kjpindex),stat=ier)
    IF (ier.NE.0) THEN
        WRITE (numout,*) ' error in tair allocation. We stop. We need kjpindex words = ',kjpindex
        CALL ipslerr(3,'enerbil.f90','enerbil_init','','')
    END IF


    ALLOCATE (q_sol_pot(kjpindex),stat=ier)
    IF (ier.NE.0) THEN
        WRITE (numout,*) ' error in q_sol_pot allocation. We stop. We need kjpindex words = ',kjpindex
        CALL ipslerr(3,'enerbil.f90','enerbil_init','','')
    END IF

    ALLOCATE (temp_sol_pot(kjpindex),stat=ier)
    IF (ier.NE.0) THEN
        WRITE (numout,*) ' error in temp_sol_pot allocation. We stop. We need kjpindex words = ',kjpindex
        CALL ipslerr(3,'enerbil.f90','enerbil_init','','')
    END IF

    ALLOCATE (box_height_old(jnlvls + 1),stat=ier)
    IF (ier.NE.0) THEN
        WRITE (numout,*) ' error in box_height_old allocation. We stop. We need kjpindex words = ',kjpindex
        STOP 'enerbil_init'
    END IF

  !! 2. Open restart input file and read data from the restart input file

    !! Open restart input file done by enerbil_init and read data from the restart input file
    !! that corresponds to processes within the 'enerbil' module (if the 'ldrestart_read' flag is set).\n
    !! The variables read are: temp_sol (surface temperature), qsurf (near surface specific humidity),
    !! evapot (soil potential evaporation), evapot_corr (corrected soil potential evaporation), tsolrad
    !! (radiative surface temperature), evapora (evaporation), fluxlat (latent heat flux), fluxsens
    !! (sensible heat flux), and temp_sol_new (new surface temperature).
    IF (ldrestart_read) THEN

        IF (long_print) WRITE (numout,*) ' we have to read a restart file for ENERBIL variables'

        var_name='temp_sol'
        CALL ioconf_setatt_p('UNITS', 'K')
        CALL ioconf_setatt_p('LONG_NAME','Surface temperature')
        CALL restget_p (rest_id, var_name, nbp_glo, 1, 1, kjit, .TRUE., temp_sol, "gather", nbp_glo, index_g)

        IF ( MINVAL(temp_sol) < 0. .OR. MAXVAL(temp_sol) > 500.) THEN
           WRITE(numout, *) "Big problem with surface temperature at restart : ",  MINVAL(temp_sol), MAXVAL(temp_sol)
           WHERE ( temp_sol < 0.) temp_sol = 280.
        ENDIF

        !
        !Config Key   = ENERBIL_TSURF
        !Config Desc  = Initial temperature if not found in restart
        !Config If    = OK_SECHIBA
        !Config Def   = 280.
        !Config Help  = The initial value of surface temperature if its value is not found
        !Config         in the restart file. This should only be used if the model is 
        !Config         started without a restart file.
        !Config Units = Kelvin [K]
        CALL setvar_p (temp_sol, val_exp,'ENERBIL_TSURF', 280._r_std)
        
        var_name= 'qsurf'
        CALL ioconf_setatt_p('UNITS', 'g/g')
        CALL ioconf_setatt_p('LONG_NAME','near surface specific humidity')
        CALL restget_p (rest_id, var_name, nbp_glo, 1, 1, kjit, .TRUE., qsurf, "gather", nbp_glo, index_g)
        IF ( ALL( qsurf(:) .EQ. val_exp ) ) THEN 
           qsurf(:) = qair(:)
        ENDIF
        
        var_name= 'evapot'
        CALL ioconf_setatt_p('UNITS', 'mm day^{-1}')
        CALL ioconf_setatt_p('LONG_NAME','Soil Potential Evaporation')
        CALL restget_p (rest_id, var_name, nbp_glo, 1, 1, kjit, .TRUE., evapot, "gather", nbp_glo, index_g)
        CALL setvar_p (evapot, val_exp, 'ENERBIL_EVAPOT', zero)
        
        var_name= 'evapot_corr'
        ! if this variable is not initialized, it is tried to use in the setvar_p loop below uninitialised
        evapot_corr(:)=zero
        CALL ioconf_setatt_p('UNITS', 'mm day^{-1}')
        CALL ioconf_setatt_p('LONG_NAME','Corrected Soil Potential Evaporation')
        CALL restget_p (rest_id, var_name, nbp_glo, 1, 1, kjit, .TRUE., evapot_corr, "gather", nbp_glo, index_g)
        
        !Config Key   = ENERBIL_EVAPOT
        !Config Desc  = Initial Soil Potential Evaporation
        !Config If    = OK_SECHIBA       
        !Config Def   = 0.0
        !Config Help  = The initial value of soil potential evaporation if its value 
        !Config         is not found in the restart file. This should only be used if
        !Config         the model is started without a restart file. 
        !Config Units = 
        CALL setvar_p (evapot_corr, val_exp, 'ENERBIL_EVAPOT', zero)
                
        var_name= 'tsolrad'
        CALL ioconf_setatt_p('UNITS', 'K')
        CALL ioconf_setatt_p('LONG_NAME','Radiative surface temperature')
        CALL restget_p (rest_id, var_name, nbp_glo, 1, 1, kjit, .TRUE., tsol_rad, "gather", nbp_glo, index_g)
        IF ( ALL( tsol_rad(:) .EQ. val_exp ) ) THEN 
           tsol_rad(:) = temp_sol(:)
        ENDIF
        
        !! 1.3 Set the fluxes so that we have something reasonable and not NaN on some machines
        var_name= 'evapora'
        CALL ioconf_setatt_p('UNITS', 'Kg/m^2/dt')
        CALL ioconf_setatt_p('LONG_NAME','Evaporation')
        CALL restget_p (rest_id, var_name, nbp_glo, 1, 1, kjit, .TRUE., vevapp, "gather", nbp_glo, index_g)
        IF ( ALL( vevapp(:) .EQ. val_exp ) ) THEN 
           vevapp(:) = zero
        ENDIF
        
        var_name= 'fluxlat'
        CALL ioconf_setatt_p('UNITS', 'W/m^2')
        CALL ioconf_setatt_p('LONG_NAME','Latent heat flux')
        CALL restget_p (rest_id, var_name, nbp_glo, 1, 1, kjit, .TRUE., fluxlat, "gather", nbp_glo, index_g)
        IF ( ALL( fluxlat(:) .EQ. val_exp ) ) THEN 
           fluxlat(:) = zero
        ENDIF
        
        var_name= 'fluxsens'
        CALL ioconf_setatt_p('UNITS', 'W/m^2')
        CALL ioconf_setatt_p('LONG_NAME','Sensible heat flux')
        CALL restget_p (rest_id, var_name, nbp_glo, 1, 1, kjit, .TRUE., fluxsens, "gather", nbp_glo, index_g)
        IF ( ALL( fluxsens(:) .EQ. val_exp ) ) THEN 
           fluxsens(:) = zero
        ENDIF
        
        var_name= 'tempsolpot'
        CALL ioconf_setatt_p('UNITS', 'K')
        CALL ioconf_setatt_p('LONG_NAME','Potential surface temperature')
        CALL restget_p (rest_id, var_name, nbp_glo, 1, 1, kjit, .TRUE., temp_sol_pot, "gather", nbp_glo, index_g)
        IF ( ALL( temp_sol_pot(:) .EQ. val_exp ) ) THEN 
           temp_sol_pot = temp_sol
        ENDIF
        !
        var_name= 'qsolpot'
        CALL ioconf_setatt_p('UNITS', 'kg/m^2')
        CALL ioconf_setatt_p('LONG_NAME','Potential saturated surface humidity')
        CALL restget_p (rest_id, var_name, nbp_glo, 1, 1, kjit, .TRUE., q_sol_pot, "gather", nbp_glo, index_g)
        IF ( ALL( q_sol_pot(:) .EQ. val_exp ) ) THEN 
           q_sol_pot = qsurf
        ENDIF

        ! initialises temp_sol_new saved in thermosoil
        var_name='temp_sol_new'
        CALL ioconf_setatt_p('UNITS', 'K')
        CALL ioconf_setatt_p('LONG_NAME','New Surface temperature')
        CALL restget_p (rest_id, var_name, nbp_glo, 1, 1, kjit, .TRUE., temp_sol_new, "gather", nbp_glo, index_g)
        IF ( ALL( temp_sol_new(:) .EQ. val_exp ) ) THEN
           ! Variables not found in restart file.
           temp_sol_new(:) = temp_sol(:)
        ENDIF
        
    ELSE

       ! initialises temp_sol_new 
       temp_sol_new(:) = temp_sol(:)
    ENDIF

    IF (long_print) WRITE (numout,*) ' enerbil_init done '

  END SUBROUTINE enerbil_init


  !!  =============================================================================================================================
  !! SUBROUTINE                 : enerbil_clear
  !!
  !>\BRIEF                  Routine deallocates clear output variables if already allocated.
  !!
  !! DESCRIPTION                : This is a 'housekeeping' routine that deallocates the key output
  !! variables, if they have already been allocated. The variables that are deallocated are psold,
  !! qsol_sat, pdqsold, psnew, qsol_sat_new, netrad, lwabs, lwup, lwnet, fluxsubli, qsat_air, tair
  !!
  !! RECENT CHANGE(S)               : None
  !!
  !! MAIN OUTPUT VARIABLE(S)                : None
  !!
  !! REFERENCES                 : None
  !! 
  !! FLOWCHART                              : None
  !! \n
  !_ ==============================================================================================================================

  SUBROUTINE enerbil_clear ()
    l_first_enerbil=.TRUE.
    l_first_enerbil_column = .TRUE.
    IF ( ALLOCATED (psold)) DEALLOCATE (psold)
    IF ( ALLOCATED (qsol_sat)) DEALLOCATE (qsol_sat)
    IF ( ALLOCATED (pdqsold)) DEALLOCATE (pdqsold)
    IF ( ALLOCATED (psnew)) DEALLOCATE (psnew)
    IF ( ALLOCATED (qsol_sat_new)) DEALLOCATE (qsol_sat_new)
    IF ( ALLOCATED (netrad)) DEALLOCATE (netrad)
    IF ( ALLOCATED (lwabs)) DEALLOCATE (lwabs)
    IF ( ALLOCATED (lwup)) DEALLOCATE (lwup)
    IF ( ALLOCATED (lwnet)) DEALLOCATE (lwnet)
    IF ( ALLOCATED (fluxsubli)) DEALLOCATE (fluxsubli)
    IF ( ALLOCATED (qsat_air)) DEALLOCATE (qsat_air)
    IF ( ALLOCATED (tair)) DEALLOCATE (tair)
    IF ( ALLOCATED (q_sol_pot)) DEALLOCATE (q_sol_pot)
    IF ( ALLOCATED (temp_sol_pot)) DEALLOCATE (temp_sol_pot)
   
    IF ( ALLOCATED (pad_deltah)) DEALLOCATE (pad_deltah)
    IF ( ALLOCATED (box_height)) DEALLOCATE (box_height)   
    IF ( ALLOCATED (box_height_old)) DEALLOCATE (box_height_old)
   ! ??open restart input file done by enerbil_init
   ! and read data from restart input file for ENERBIL process??
 
  END SUBROUTINE enerbil_clear


  !!  =============================================================================================================================
  !! SUBROUTINE                 : enerbil_var_init
  !!
  !>\BRIEF                  Housekeeping routine that calls enerbil_t2mdiag.
  !!
  !! DESCRIPTION                : Housekeeping routine that calls the 2 metre temperature
  !! diagnostic subroutine 'enerbil_2tmdiag'.
  !!
  !! MAIN OUTPUT VARIABLE(S)                    : t2mdiag
  !!
  !! REFERENCE(S)               : None
  !!
  !! FLOWCHART                                  : None
  !! \n
  !_ ==============================================================================================================================
 
  SUBROUTINE enerbil_var_init (kjpindex, temp_air, t2mdiag)

    !! 0. Variable and parameter declaration
    
    !! 0.1 Input variables
 
    INTEGER(i_std), INTENT(in)                         :: kjpindex       !! Domain size (-)
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)      :: temp_air       !! Air temperature (K)

    !! 0.2 Output variables

    !! 0.3 Modified variables

    !! 0.4 Output fields
    REAL(r_std),DIMENSION (kjpindex), INTENT (out)     :: t2mdiag        !! Temperature at 2 metres above ground(K)
!_ ================================================================================================================================

    CALL enerbil_t2mdiag (kjpindex, temp_air, t2mdiag)

    IF (long_print) WRITE (numout,*) ' enerbil_var_init done '

  END SUBROUTINE enerbil_var_init


  !!  =============================================================================================================================
  !! SUBROUTINE                 : enerbil_begin
  !!
  !>\BRIEF                  Preliminary variables required for the calculation of
  !! the energy budget are derived.
  !!
  !! DESCRIPTION                : This routines computes preliminary variables required
  !! for the calculation of the energy budget: the old surface static energy (psold), the surface saturation
  !! humidity (qsol_sat), the derivative of satured specific humidity at the old temperature (pdqsold) 
  !! and the net radiation (netrad).
  !!
  !! MAIN OUTPUT VARIABLE(S)                    : psold, qsol_sat, pdqsold, netrad
  !!
  !! REFERENCE(S)               :
  !! - Best, MJ, Beljaars, A, Polcher, J & Viterbo, P, 2004. A proposed structure for coupling tiled
  !! surfaces with the planetary boundary layer. Journal of Hydrometeorology, 5, pp.1271-1278
  !! - de Noblet-Ducoudré, N, Laval, K & Perrier, A, 1993. SECHIBA, a new set of parameterisations
  !! of the hydrologic exchanges at the land-atmosphere interface within the LMD Atmospheric General
  !! Circulation Model. Journal of Climate, 6, pp.248-273
  !! - Dufresne, J-L & Ghattas, J, 2009. Description du schéma de la couche limite turbulente et la
  !! interface avec la surface planetaire dans LMDZ, Technical note, available (22/12/11):
  !! http://lmdz.lmd.jussieu.fr/developpeurs/notes-techniques/ressources/pbl_surface.pdf
  !! - Polcher, J. McAvaney, B, Viterbo, P, Gaertner, MA, Hahmann, A, Mahfouf, J-F, Noilhan, J
  !! Phillips, TJ, Pitman, AJ, Schlosser, CA, Schulz, J-P, Timbal, B, Verseghy, D &
  !! Xue, Y, 1998. A proposal for a general interface between land surface schemes and
  !! general circulation models. Global and Planetary Change, 19, pp.261-276
  !! - Richtmeyer, RD, Morton, KW, 1967. Difference Methods for Initial-Value Problems.
  !! Interscience Publishers\n
  !! - Schulz, Jan-Peter, Lydia Dümenil, Jan Polcher, 2001: On the Land Surface–Atmosphere 
  !! Coupling and Its Impact in a Single-Column Atmospheric Model. J. Appl. Meteor., 40, 642–663.
  !!
  !! FLOWCHART  : None                     
  !! \n
  !_ ==============================================================================================================================
  
  SUBROUTINE enerbil_begin (kjpindex, temp_sol, lwdown, swnet, pb, &
       & psold, qsol_sat, pdqsold, netrad, emis, &
       & dtradia, u, v, &
       & precip_snow, temp_air, albedo_undersnow, snowtemp, snowdz, &
       & snowrho, snowheat, snowliq, snowgrain, radsink, &
       & radxs, psnowhmass, s2flux, ptso1)

    !! 0. Variable and parameter declaration

    !! 0.1 Input variables

    INTEGER(i_std), INTENT(in)                         :: kjpindex         !! Domain size (-)
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)      :: temp_sol         !! Soil temperature (K)
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)      :: lwdown           !! Down-welling long-wave flux (W m^{-2})
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)      :: swnet            !! Net surface short-wave flux (W m^{-2})
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)      :: pb               !! Lowest level pressure (hPa)
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)      :: emis             !! Emissivity (-)
    REAL(r_std), INTENT(in)                            :: dtradia          !! Time step [s]
    REAL(r_std), DIMENSION (kjpindex), INTENT(in)      :: u, v             !! Horizontal wind speed
    REAL(r_std), DIMENSION (kjpindex), INTENT(in)      :: precip_snow      !! Snowfall
    REAL(r_std), DIMENSION (kjpindex), INTENT(in)      :: temp_air         !! Air temperature
    REAL(r_std), DIMENSION(kjpindex), INTENT(in)       :: albedo_undersnow !! Albedo under snow
    

    !! 0.2 Output variables

    REAL(r_std),DIMENSION (kjpindex), INTENT (out)     :: psold            !! Old surface dry static energy (J kg^{-1})
    REAL(r_std),DIMENSION (kjpindex), INTENT (out)     :: qsol_sat     !! Saturated specific humudity for old temperature 
                                                                           !! (kg kg^{-1})    
    REAL(r_std),DIMENSION (kjpindex), INTENT (out)     :: pdqsold      !! Derivative of satured specific humidity at the old 
                                                                           !! temperature (kg (kg s)^{-1})
    REAL(r_std),DIMENSION (kjpindex), INTENT (out)     :: netrad           !! Net radiation (W m^{-2})
    REAL(r_std), DIMENSION (kjpindex), INTENT(out)     :: psnowhmass       !! Heat content of snowfall (J/m2)

    !! 0.3 Modified variables
    REAL(r_std), DIMENSION (kjpindex,nsnow),INTENT(inout) :: snowdz        !! Snow depth at each snow layer
    REAL(r_std), DIMENSION (kjpindex,nsnow),INTENT(inout) :: snowrho       !! Snow density
    REAL(r_std), DIMENSION (kjpindex,nsnow),INTENT(inout) :: snowtemp      !! Snow temperature
    REAL(r_std), DIMENSION (kjpindex,nsnow),INTENT(inout) :: snowgrain     !! Snow grain size
    REAL(r_std), DIMENSION (kjpindex,nsnow),INTENT(inout) :: snowheat      !! Snow heat capacity
    REAL(r_std), DIMENSION (kjpindex,nsnow),INTENT(inout) :: snowliq       !! Liquid water content
    REAL(r_std), DIMENSION(kjpindex,nsnow),INTENT(inout)  :: radsink       !! Radiation term at each snow layer
    REAL(r_std), DIMENSION(kjpindex),INTENT(inout)        :: radxs         !! Residual radiation at bottom of the snowpack
    REAL(r_std), DIMENSION (kjpindex),INTENT(inout)       :: s2flux        !! Heat conduction between first and second snow layer
    REAL(r_std), DIMENSION (kjpindex),INTENT(inout)       :: ptso1         !! First snow layer temperature

    !! 0.4 Local variables
    REAL(r_std), DIMENSION(kjpindex)                   :: dev_qsol
    INTEGER(i_std)                                     :: ji
    REAL(r_std), PARAMETER                             :: missing = 999998.
    REAL(r_std), DIMENSION (kjpindex)                  :: snow_depth_tmp
    REAL(r_std), DIMENSION (kjpindex,nsnow)            :: snowdz_old

!_ ================================================================================================================================

    !! 1. Computes psold (the surface static energy for the old timestep)
    
    !! We here define the surface static energy for the 'old' timestep, in terms of the surface
    !! temperature and heat capacity.
    !! \latexonly 
    !!     \input{enerbilbegin1.tex}
    !! \endlatexonly
    psold(:) = temp_sol(:)*cp_air

    !! 2. Computes qsol_sat (the surface saturated humidity).
    !! We call the routine 'qsatcalc' from within the module 'src_parameters/constantes_veg'.

    CALL qsatcalc (kjpindex, temp_sol, pb, qsol_sat)

    IF ( diag_qsat ) THEN
       IF ( ANY(ABS(qsol_sat(:)) .GT. missing) ) THEN
          DO ji = 1, kjpindex
             IF ( ABS(qsol_sat(ji)) .GT. missing) THEN
                WRITE(numout,*) 'ERROR on ji = ', ji
                WRITE(numout,*) 'temp_sol(ji),  pb(ji) :', temp_sol(ji),  pb(ji)
                CALL ipslerr_p (3,'enerbil_begin', &
                     &           'qsol too high ','','')
             ENDIF
          ENDDO
       ENDIF
    ENDIF

    !! 3. Computes pdqsold 
    !! Computes pdqsold (the derivative of the saturated humidity with respect to temperature
    !! analysed at the surface temperature at the 'old' timestep.
    !! We call the routine 'dev_qsatcalc' from within the module qsat_moisture module
    !! \latexonly 
    !!     \input{enerbilbegin2.tex}
    !! \endlatexonly
    CALL dev_qsatcalc (kjpindex, temp_sol, pb, dev_qsol)
    pdqsold(:) = dev_qsol(:)

    IF ( diag_qsat ) THEN
       IF ( ANY(ABS( pdqsold(:)) .GT. missing) ) THEN
          DO ji = 1, kjpindex
             IF ( ABS( pdqsold(ji)) .GT. missing ) THEN
                WRITE(numout,*) 'ERROR on ji = ', ji
                WRITE(numout,*) 'temp_sol(ji),  pb(ji) :', temp_sol(ji),  pb(ji)
                CALL ipslerr_p (3,'enerbil_begin', &
                     &           'pdqsold too high ','','')
             ENDIF
          ENDDO
       ENDIF
    ENDIF

    !! 4. Computes the net radiation and the absorbed LW radiation absorbed at the surface.  
    !! Long wave radiation absorbed by the surface is the product of emissivity and 
    !! downwelling LW radiation    
    !! \latexonly 
    !!     \input{enerbilbegin3.tex}
    !! \endlatexonly
    lwabs(:) = emis(:) * lwdown(:)

    !! Net radiation is calculated as:
    !! \latexonly 
    !!     \input{enerbilbegin4.tex}
    !! \endlatexonly
    netrad(:) = lwdown(:) + swnet (:) - (emis(:) * c_stefan * temp_sol(:)**4 + &
         (un - emis(:)) * lwdown(:)) 

    ! 5. Snow processes
    IF (ok_explicitsnow) THEN
       ! 5.1 Snow fall
       CALL explicitsnow_fall(kjpindex,dtradia,precip_snow,temp_air,u,v,snowrho,snowdz,&
            & snowheat,snowgrain,snowtemp,psnowhmass)

       ! 5.2 calculate the new snow discretization
       snow_depth_tmp(:) = SUM(snowdz(:,:),2)
       snowdz_old = snowdz
       CALL explicitsnow_levels(kjpindex,snow_depth_tmp, snowdz)

       ! 5.3 Snow heat redistribution
       CALL explicitsnow_transf(kjpindex,snowdz_old,snowdz,snowrho,snowheat,snowgrain)

       ! 5.4 Diagonize water portion of the snow from snow heat content:
       DO ji=1, kjpindex
          IF (SUM(snowdz(ji,:)) .GT. 0.0) THEN
             snowtemp(ji,:) = snow3ltemp_1d(snowheat(ji,:),snowrho(ji,:),snowdz(ji,:))
             snowliq(ji,:) = snow3lliq_1d(snowheat(ji,:),snowrho(ji,:),snowdz(ji,:),snowtemp(ji,:))
          ENDIF
       END DO

       ! 5.5 snow compaction
       CALL explicitsnow_compactn(kjpindex,dtradia,snowtemp,snowrho,snowdz)

       ! 5.6 Update snow heat 
       DO ji = 1, kjpindex
          snowheat(ji,:) = snow3lheat_1d(snowliq(ji,:),snowrho(ji,:),snowdz(ji,:),snowtemp(ji,:))
       ENDDO

       ! 5.7 Calculating snow radiation within snowpack
       CALL explicitsnow_rad(kjpindex,swnet,albedo_undersnow,snowdz,snowrho,radsink,radxs)

       ! Save first snow layer temperature 
       s2flux(:) = 0.0
       ptso1(:) = snowtemp(:,1)

    ENDIF
    !
    IF (long_print) WRITE (numout,*) ' enerbil_begin done '

  END SUBROUTINE enerbil_begin


  !!  =============================================================================================================================
  !! SUBROUTINE                 : enerbil_surftemp
  !!
  !>\BRIEF                  This routine computes the new surface static energy
  !! (psnew) and the saturated humidity at the surface (qsol_sat_new). 
  !!
  !! DESCRIPTION                : This is the key part of the enerbil module, for which
  !! the energy budget in the surface layer is solved and changes over the model timestep 'dtradia' are
  !! quantified for the surface static energy, surface temperature, surface humidity, the 'air' (or lowest
  !! level atmospheric model) temperature, the 'air' (or lowest level atmospheric model) humidity, 
  !! according to the method that is laid out by Dufresne \& Ghattas (2009) and Shultz et al. (2001).
  !!
  !! It computes the energy balance at the surface with an implicit scheme, that is connected to the 
  !! Richtmyer and Morton algorithm of the PBL. By computing the surface temperature and surface humidity 
  !! the routine also implicitly estimates the various fluxes which balance in order to give the new 
  !! temperature. Thus once the surface temperature has been obtained all the fluxes need to be diagnosed. 
  !!
  !! If ok_explicitsnow is used, the notion of skin layer in ORCHIDEE is abandoned and 
  !! the first thin snow layer is adopted to solve the surface energy budget. 
  !! Implicit method is still used for coupling to the atmosphere. 
  !! In this new scheme, the snow temperature profile is simulatenously solved 
  !! along with the surface snow temperature, and the details are referenced to Boone et al. (2010) 
  !!
  !! MAIN OUTPUT VARIABLE(S)    : psnew, qsol_sat_new, temp_sol_new, qair_new, epot_air_new
  !!
  !! REFERENCE(S)                   :
  !! - Best, MJ, Beljaars, A, Polcher, J & Viterbo, P, 2004. A proposed structure for coupling tiled
  !! surfaces with the planetary boundary layer. Journal of Hydrometeorology, 5, pp.1271-1278
  !! - de Noblet-Ducoudré, N, Laval, K & Perrier, A, 1993. SECHIBA, a new set of parameterisations
  !! of the hydrologic exchanges at the land-atmosphere interface within the LMD Atmospheric General
  !! Circulation Model. Journal of Climate, 6, pp.248-273
  !! - Dufresne, J-L & Ghattas, J, 2009. Description du schéma de la couche limite turbulente et la
  !! interface avec la surface planetaire dans LMDZ, Technical note, available (22/12/11):
  !! http://lmdz.lmd.jussieu.fr/developpeurs/notes-techniques/ressources/pbl_surface.pdf
  !! - Guimberteau, M, 2010. Modélisation de l'hydrologie continentale et influences de l'irrigation
  !! sur le cycle de l'eau, PhD Thesis, available (22/12/11):
  !! http://www.sisyphe.upmc.fr/~guimberteau/docs/manuscrit_these.pd 
  !! - Polcher, J. McAvaney, B, Viterbo, P, Gaertner, MA, Hahmann, A, Mahfouf, J-F, Noilhan, J
  !! Phillips, TJ, Pitman, AJ, Schlosser, CA, Schulz, J-P, Timbal, B, Verseghy, D &
  !! Xue, Y, 1998. A proposal for a general interface between land surface schemes and
  !! general circulation models. Global and Planetary Change, 19, pp.261-276
  !! - Richtmeyer, RD, Morton, KW, 1967. Difference Methods for Initial-Value Problems.
  !! Interscience Publishers
  !! - Schulz, Jan-Peter, Lydia Dümenil, Jan Polcher, 2001: On the Land Surface–Atmosphere 
  !! Coupling and Its Impact in a Single-Column Atmospheric Model. J. Appl. Meteor., 40, 642–663.
  !!
  !! FLOWCHART    : None
  !!
  !_ ==============================================================================================================================  
  

  SUBROUTINE enerbil_surftemp (kjpindex, dtradia, zlev, emis, &
       & epot_air, petAcoef, petBcoef, qair, peqAcoef, &
       & peqBcoef, soilflx, rau, u, v, &
       & q_cdrag, vbeta, valpha, vbeta1, vbeta5, &
       & soilcap, lwdown, swnet, psnew, qsol_sat_new, &
       & temp_sol_new, qair_new, epot_air_new, beta_diffuco, &
       & temp_sol, ptso1, radsink, snowrho, &
       & snowdz, snowtemp, snowliq, sfcfrz, pb, &
       & temp_air, precip_rain, gthick, gtemp, gpkappa, &
       & grndflux, s2flux, pkappa_snow, snowdz_o, soilflxresid) 

    !! 0. Variable and parameter declaration 

    !! 0.1 Input variables

    INTEGER(i_std), INTENT(in)                               :: kjpindex      !! Domain size (-)
    REAL(r_std), INTENT(in)                                  :: dtradia       !! Time step (s)
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: zlev          !! Height of first layer (m)
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: emis          !! Emissivity (-)
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: epot_air      !! Air potential energy (?? J)
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: petAcoef      !! PetAcoef (see note)
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: petBcoef      !! PetBcoef (see note)
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: qair          !! Lowest level specific humidity (kg kg^{-1})
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: peqAcoef      !! PeqAcoef (see note)
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: peqBcoef      !! PeqBcoef (see note)
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: soilflx       !! Soil flux (W m^{-2})
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: rau           !! Air density (kg m^{-3})
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: u, v          !! Wind velocity by directional components 
                                                                              !! u (Eastwards) and v (Northwards) (m s^{-1})
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: q_cdrag       !! Surface drag (m s^{-1})
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: vbeta         !! Resistance coefficient (-)
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: valpha        !! Resistance coefficient (-)
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: vbeta1        !! Snow resistance (-)
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: vbeta5        !! Floodplains resistance
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: soilcap       !! Soil calorific capacity (J K^{-1])
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: lwdown        !! Down-welling long-wave flux (W m^{-2})
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: swnet         !! Net surface short-wave flux (W m^{-2})
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: temp_sol         !! Soil temperature (K)
    REAL(r_std), DIMENSION (kjpindex,nsnow), INTENT(in)      :: radsink       !! Radiation term at each snow layer
    REAL(r_std), DIMENSION (kjpindex,nsnow), INTENT(in)      :: snowrho       !! Snow density
    REAL(r_std), DIMENSION (kjpindex,nsnow), INTENT(in)      :: snowdz        !! Snow depth
    REAL(r_std), DIMENSION (kjpindex,nsnow), INTENT(in)      :: snowliq       !! Liquid water content
    REAL(r_std), DIMENSION (kjpindex), INTENT(in)            :: pb            !! Surface pressure
    REAL(r_std), DIMENSION (kjpindex), INTENT(in)            :: temp_air      !! Air temperature
    REAL(r_std), DIMENSION (kjpindex), INTENT(in)            :: precip_rain   !! Rainfall
    REAL(r_std), DIMENSION (kjpindex), INTENT(in)            :: gthick        !! Surface soil layer thickness
    REAL(r_std), DIMENSION (kjpindex), INTENT(in)            :: gpkappa       !! Surface soil conductivity
    REAL(r_std), DIMENSION (kjpindex), INTENT(in)            :: ptso1         !! First snow layer temperature 

    !! 0.2 Output variables

    REAL(r_std),DIMENSION (kjpindex), INTENT (out)           :: psnew         !! New surface static energy (J kg^{-1})
    REAL(r_std),DIMENSION (kjpindex), INTENT (out)           :: qsol_sat_new  !! New saturated surface air moisture (kg kg^{-1})
    REAL(r_std),DIMENSION (kjpindex), INTENT (out)           :: temp_sol_new  !! New soil temperature (K)
    REAL(r_std),DIMENSION (kjpindex), INTENT (out)           :: qair_new      !! New air moisture (kg kg^{-1})
    REAL(r_std),DIMENSION (kjpindex), INTENT (out)           :: epot_air_new  !! New air temperature (K)

    !! 0.3 Modified variables
    REAL(r_std), DIMENSION (kjpindex,nsnow), INTENT(inout)   :: snowtemp      !! Snow temperature
    REAL(r_std), DIMENSION (kjpindex), INTENT(inout)         :: sfcfrz        !! Snow surface frozen fraction
    REAL(r_std), DIMENSION (kjpindex), INTENT(inout)         :: gtemp         !! Surface soil temperature
    REAL(r_std), DIMENSION (kjpindex), INTENT(inout)         :: grndflux      !! Energy flux into the soil
    REAL(r_std), DIMENSION (kjpindex), INTENT(inout)         :: s2flux        !! Heat flux between surface and 2nd snow layers(W/m2)
    REAL(r_std), DIMENSION (kjpindex), INTENT(inout)         :: soilflxresid  !! Energy flux to the snowpack
    REAL(r_std), DIMENSION (kjpindex,nsnow),INTENT(inout)    :: pkappa_snow   !! Snow conductivity
    REAL(r_std), DIMENSION (kjpindex,nsnow),INTENT(inout)    :: snowdz_o      !! Snow depth at previous time step


    !! 0.4 Local variables
    INTEGER(i_std)                          :: ji
    REAL(r_std),DIMENSION (kjpindex)        :: zicp
    REAL(r_std)                             :: fevap
    REAL(r_std), DIMENSION(kjpindex)        :: dev_qsol
    REAL(r_std)                             :: sensfl_old, larsub_old, lareva_old, dtheta, sum_old, sum_sns
    REAL(r_std)                             :: zikt, zikq, netrad_sns, sensfl_sns, larsub_sns, lareva_sns
    REAL(r_std)                             :: speed


    REAL(r_std)                             :: test_delta_temp, test_delta_temp2, test_delta_temp3
    REAL(r_std)                             :: test_delta_temp4

    REAL(r_std),DIMENSION (kjpindex)        :: beta_diffuco, beta_diffuco_sub, beta_diffuco_evap

    REAL(r_std)                             :: M_1_h, N_1_h, M_1_q, N_1_q
    REAL(r_std), DIMENSION (kjpindex)       :: ZSNOWDZM1,ZSNOWDZM2
    REAL(r_std), DIMENSION (kjpindex)       :: PCT     ! Surface thermal inertia
    REAL(r_std), DIMENSION (kjpindex)       :: ZSCONDA ! Thermal conductivity between uppermost and lower snow layer  
    REAL(r_std), DIMENSION (kjpindex,nsnow) :: pcapa_snow ! snow heat capacity
    REAL(r_std), DIMENSION (kjpindex)       :: PHPSNOW ! rainfall heat (advective) flux      (W/m2)
    REAL(r_std), DIMENSION (kjpindex)       :: ZA, ZB, ZC, PTSTERM2,PTSTERM1
    INTEGER(i_std)                          :: jg
    REAL(r_std)                             :: qc
    REAL(r_std)                             :: grndflux_tmp,s2flux_tmp,fromsoilflux
    REAL(r_std), DIMENSION (nsnow)          :: snowtemp_tmp
    REAL(r_std), DIMENSION (kjpindex)       :: AQA,BQA,ATA,BTA,peqA,peqB,petA,petB,pdqsold_newsnow

!_ ================================================================================================================================
    
    zicp = un / cp_air

    !! Case A : The first snow layer is used to solve the surface energy buget  
    !!          only if there is snow on the ground and if ok_explictsnow is activated 
    !! Case B : Otherwise the skin layer is used to solve the surface energy buget
    DO ji=1,kjpindex
  
       IF (SUM(snowdz(ji,:)) .GT. 0.0 .AND. ok_explicitsnow)  THEN 
 
         !! Case A : The first snow layer is used to solve the surface energy buget 
          !1. Calculate some useful coefficients  
          speed = MAX(min_wind, SQRT (u(ji)*u(ji) + v(ji)*v(ji))) 
          qc = speed * q_cdrag(ji) 

          ! old qsol_sat and dev_qsol 
          qsol_sat(ji)=qsat(ptso1(ji), pb(ji)) 
          dev_qsol(ji)=dev_qsat(ptso1(ji), pb(ji)) 
          pdqsold_newsnow(ji) = dev_qsol(ji) * ( pb(ji)**kappa ) / cp_air 

          ! To prevent numerical difficulties for very thin snow layers, limit 
          ! the grid "thinness": this is important as 
          ! layers become vanishing thin: 
          ZSNOWDZM1(ji) = MAX(snowdz(ji,1), psnowdzmin) 
          ZSNOWDZM2(ji) = MAX(snowdz(ji,2), psnowdzmin) 

          ! Snow surface thermal inertia: 
          PCT(ji)      = 1.0/(snowrho(ji,1)*xci*ZSNOWDZM1(ji)) 

          ! Fraction of surface frozen (sublimation) with the remaining 
          ! fraction being liquid (evaporation): 
          sfcfrz(ji)  = 1.0 - snowliq(ji,1)*ph2o/(ZSNOWDZM1(ji)*snowrho(ji,1)) 

          ! snow thermal conductivity and capacity 
          pkappa_snow(ji,:) = (ZSNOWTHRMCOND1 + ZSNOWTHRMCOND2*snowrho(ji,:)*snowrho(ji,:)) +      & 
               MAX(0.0,(ZSNOWTHRMCOND_AVAP+(ZSNOWTHRMCOND_BVAP/(snowtemp(ji,:)+ & 
               ZSNOWTHRMCOND_CVAP)))*(XP00/(pb(ji)*100.))) 
          pcapa_snow(ji,:) = snowrho(ji,:) * xci 

          ! Thermal conductivity between uppermost and lower snow layers: 
          ZSCONDA(ji)  = (ZSNOWDZM1(ji)*pkappa_snow(ji,1) + ZSNOWDZM2(ji)*pkappa_snow(ji,2))/           & 
               (ZSNOWDZM1(ji)+ZSNOWDZM2(ji)) 

          ! Heat advected by rainfall 
          PHPSNOW(ji) = precip_rain(ji)*(4.218E+3)*(MAX(tp_00,temp_air(ji))-tp_00)/dtradia ! (w/m2) 


          !2. Prepare the coefficients for the implicit linearized snow 
          !   surface energy budget  
          ! Calculate the coefficients for the new qair and tair 
          peqB(ji)=peqBcoef(ji) 
          peqA(ji)=peqAcoef(ji) 

          BQA(ji) = (peqB(ji) - peqA(ji)*qc * rau(ji) * (qsol_sat(ji)-pdqsold_newsnow(ji)*ptso1(ji))) / & 
               (1-peqA(ji)*qc*rau(ji)) 

          AQA(ji) = (-1.)* (peqA(ji)*qc*rau(ji)*pdqsold_newsnow(ji)) / & 
               (1-peqA(ji)*qc*rau(ji)) 

          petA(ji) = petAcoef(ji) 
          petB(ji) = petBcoef(ji)/cp_air 

          BTA(ji) = petB(ji)/(1-petA(ji)*cp_air*qc*rau(ji)) 
          ATA(ji) = (-1.)* (petA(ji)*cp_air*qc*rau(ji))/(1-petA(ji)*cp_air*qc*rau(ji)) 

          ! Compute the coefficients   
          ZA(ji) = 1. / dtradia + PCT(ji) * (4. * emis(ji) * c_stefan * (ptso1(ji)**3) +     & 
               rau(ji) * qc * (pdqsold_newsnow(ji)-AQA(ji)) * ( (1.-sfcfrz(ji))*chalev0 + sfcfrz(ji)*chalsu0) + & 
               rau(ji) * qc * cp_air* (1-ATA(ji)) +         & 
               (2*ZSCONDA(ji)/(ZSNOWDZM2(ji)+ZSNOWDZM1(ji))) ) 

          ZB(ji) = 1. / dtradia + PCT(ji) * (3. * emis(ji) * c_stefan * (ptso1(ji)** 3) +    & 
               rau(ji) * qc * pdqsold_newsnow(ji) * & 
               ( (1.-sfcfrz(ji))*chalev0 + sfcfrz(ji)*chalsu0) ) 

          ZC(ji) = PCT(ji) * (rau(ji) * qc * cp_air * BTA(ji) + & 
               swnet(ji)+ radsink(ji,1) + emis(ji)*lwdown(ji) - & 
               rau(ji) * qc * ( (1.-sfcfrz(ji))*chalev0 + sfcfrz(ji)*chalsu0) * (qsol_sat(ji)-BQA(ji)) & 
               + PHPSNOW(ji)  ) 

          PTSTERM2(ji) = 2*ZSCONDA(ji)*PCT(ji)/(ZA(ji)*(ZSNOWDZM2(ji)+ZSNOWDZM1(ji))) 
          PTSTERM1(ji) = (ptso1(ji)*ZB(ji) + ZC(ji))/ZA(ji) 

          !3. Calculate the snow temperature profile based on heat diffusion equation  
          snowtemp_tmp=snowtemp(ji,:) 

          CALL explicitsnow_solvt(dtradia,snowdz(ji,:),pkappa_snow(ji,:),pcapa_snow(ji,:),gtemp(ji), & 
               gpkappa(ji),gthick(ji),radsink(ji,:),PCT(ji),PTSTERM1(ji),PTSTERM2(ji),grndflux_tmp,& 
               snowtemp_tmp,s2flux_tmp,fromsoilflux) 

          s2flux(ji) = s2flux_tmp 
          snowtemp(ji,:)=snowtemp_tmp 
          snowdz_o(ji,:)=snowdz(ji,:) 
          soilflxresid(ji) =  fromsoilflux 

          !4. Calculate the new qair, tair, qsol_sat,'temp_sol_new' 
          epot_air_new(ji) = (BTA(ji) + ATA(ji)*snowtemp(ji,1))*cp_air 
          qair_new(ji) = BQA(ji) + AQA(ji)*snowtemp(ji,1) 
          qsol_sat_new(ji) = qsol_sat(ji) + pdqsold_newsnow(ji) * (snowtemp(ji,1) - ptso1(ji)) 
          psnew(ji) = snowtemp(ji,1) * cp_air 
          temp_sol_new(ji) = snowtemp(ji,1) 

       ELSE 

          !! Case B: The skin layer is used to solve the surface energy buget 
          snowdz_o(ji,:)=0 

          !! 1. Derivation of auxiliary variables
          !! \latexonly 
          !!     \input{enerbilsurftemp1.tex}
          !! \endlatexonly
          speed = MAX(min_wind, SQRT (u(ji)*u(ji) + v(ji)*v(ji)))
          
          !! \latexonly 
          !!     \input{enerbilsurftemp2.tex}
          !! \endlatexonly
          zikt = 1/(rau(ji) * speed * q_cdrag(ji))
          zikq = 1/(rau(ji) * speed * q_cdrag(ji))
          beta_diffuco(ji) = 1.0d0 * vbeta1(ji) * (un - vbeta5(ji)) + &
               &  (un - vbeta1(ji)) * (un - vbeta5(ji)) * vbeta(ji) + vbeta5(ji)
          beta_diffuco_sub(ji) = vbeta1(ji) * (1.0d0 - vbeta5(ji))
          beta_diffuco_evap(ji) = (1.0d0 - vbeta1(ji)) * (1.0d0 -vbeta5(ji)) * vbeta(ji) + &
               &   vbeta5(ji)

          !! 2. Computation of fluxes for the old surface conditions
          !! As a reference, we first calculate the sensible and latent heat for the 'old' timestep.

          !! 2.1 Sensible heat for the 'old' timestep
          !! This is equation (64) of (Dufresne & Ghattas, 2009), rewritten in terms of H_old. We make the 
          !! approximation that (P_r/P)^{kappa}=1 and convert the A and B coefficients to the ORCHIDEE
          !! format (see introductory note, above). Also, the equation here is in terms of surface static
          !! energy, which is defined as per section 3.1, above.
          !! \latexonly 
          !!     \input{enerbilsurftemp3.tex}
          !! \endlatexonly
          sensfl_old = (petBcoef(ji) -  psold(ji)) / (zikt -  petAcoef(ji))

          !! 2.2 Latent heat for the 'old' timestep
          !! This is equation (70) of (Dufresne & Ghattas, 2009), rewritten in terms of {\lambda E}_{old}.
          !! Again we convert the A and B coefficients from the LMDZ format to the ORCHIDEE format.\n
          !! There are two forms of the equation - first for the latent heat as a result of sublimination
          !! processes and then as a result of evaporative processes:
          !! \latexonly 
          !!     \input{enerbilsurftemp4.tex}
          !! \endlatexonly
          larsub_old = chalsu0 * vbeta1(ji) * (un - vbeta5(ji)) * (peqBcoef(ji) -  &
               qsol_sat(ji)) / (zikq - peqAcoef(ji))

          !! \latexonly 
          !!     \input{enerbilsurftemp5.tex}
          !! \endlatexonly
          lareva_old = chalev0 * (un - vbeta1(ji)) * (un - vbeta5(ji)) * vbeta(ji) * &
               & (peqBcoef(ji) -  valpha(ji) * qsol_sat(ji)) / (zikq - peqAcoef(ji)) &
               & + chalev0 * vbeta5(ji) * (peqBcoef(ji) -  qsol_sat(ji)) / (zikq - peqAcoef(ji))

          !! 3. Calculation of sensitivity terms
          !! We here calculate the sensitivity for the different fluxes to changes in dtheta, 
          !! which is the change in the surface static energy over the model time step (dtradia).

          !! 3.1 Net radiation sensitivity
          !! This is an explicit-implicit representation of the Stefan-Boltzmann law - the explicit terms 
          !! are ${ps}_{old}^3$, and it is completed when multiplied by dtheta (defined above).
          !! \latexonly 
          !!     \input{enerbilsurftemp6.tex}
          !! \endlatexonly
          netrad_sns = zicp(ji) * quatre * emis(ji) * c_stefan * ((zicp(ji) * psold(ji))**3)

          !! 3.2 Sensible heat flux sensitivity
          !! This is the temperature sensitivity term N_1^h derived in equation (66) of 
          !! (Dufresne & Ghattas, 2009), where we again assume that (P_r/P)^{kappa}=1, 
          !! convert the A and B coefficients to the ORCHIDEE format, and rewrite in terms 
          !! of surface static energy, rather than temperature (see section 3.1, above).
          !! \latexonly 
          !!     \input{enerbilsurftemp7.tex}
          !! \endlatexonly
          sensfl_sns = un / (zikt -  petAcoef(ji))

          !! 3.3 Latent heat flux sensitivity 
          !! This is the humidity sensitivity term N_1^q derived in equation (72) of 
          !! (Dufresne & Ghattas, 2009), where the A and B coefficients are written in 
          !! the ORCHIDEE format. larsub_sns is the latent heat sensitivity for sublimination. 
          !! The coefficient vbeta1 is the evaporation coefficient. It represents the relationship 
          !! between the real and potential evaporation. It is derived
          !! in the module 'src_sechiba/diffuco_snow'.
          !! \latexonly 
          !!     \input{enerbilsurftemp8.tex}
          !! \endlatexonly
          larsub_sns = chalsu0 * vbeta1(ji) * (un - vbeta5(ji)) * zicp(ji) * pdqsold(ji) / &
               (zikq - peqAcoef(ji))

          !! lareva_sns is the latent heat sensitivity for evaporation. vbeta1 (src_sechiba/diffuco_snow), 
          !! vbeta (src_sechiba/diffuco_comb) and valpha (src_sechiba/diffuco_comb) are the evaporation
          !! coefficients.
          !! \latexonly 
          !!     \input{enerbilsurftemp9.tex}
          !! \endlatexonly
          lareva_sns = chalev0 * ((un - vbeta1(ji))*(un - vbeta5(ji)) * vbeta(ji) * valpha(ji) + &
               vbeta5(ji)) * &
               & zicp(ji) * pdqsold(ji) / (zikq - peqAcoef(ji))

          !! 4. Solution of the energy balance

          !! 4.1 We calculate the total flux for the 'old' timestep.
          !! \latexonly 
          !!     \input{enerbilsurftemp10.tex}
          !! \endlatexonly
          sum_old = netrad(ji) + sensfl_old + larsub_old + lareva_old + soilflx(ji)

          !! 4.2 We calculate the total sensitivity dtheta (the change in the
          !! surface static energy over the timestep.
          !! \latexonly 
          !!     \input{enerbilsurftemp11.tex}
          !! \endlatexonly
          sum_sns = netrad_sns + sensfl_sns + larsub_sns + lareva_sns

          !! 4.3 Calculation of dtheta (change in surface static energy over the
          !! timestep.
          !! \latexonly 
          !!     \input{enerbilsurftemp12.tex}
          !! \endlatexonly
          dtheta = dtradia * sum_old / (zicp(ji) * soilcap(ji) + dtradia * sum_sns)

          !! 4.4 Determination of state variables for the 'new' timestep
          !! No we have dtheta, we can determine the surface static energy that
          !! corresponds to the 'new' timestep.
          !! \latexonly 
          !!     \input{enerbilsurftemp13.tex}
          !! \endlatexonly
          psnew(ji) =  psold(ji) + dtheta

          !! The new surface saturated humidity can be calculated by equation (69)
          !! of (Dufresne & Ghattas, 2009), in which we substitute dtheta for the
          !! change between old and new temperature using the relationship from 3.1,
          !! above.
          !! \latexonly 
          !!     \input{enerbilsurftemp14.tex}
          !! \endlatexonly
          qsol_sat_new(ji) = qsol_sat(ji) + zicp(ji) * pdqsold(ji) * dtheta

          !! The new surface temperature is determined from the new surface static temperature,
          !! again by using the relationship in section 3.1.
          !! \latexonly 
          !!     \input{enerbilsurftemp15.tex}
          !! \endlatexonly
          temp_sol_new(ji) = psnew(ji) / cp_air

          !! 4.5 Calculation of new evaporation potential and new evaporation latent heat
          !! flux (???)
          !! \latexonly 
          !!     \input{enerbilsurftemp16.tex}
          !! \endlatexonly
          epot_air_new(ji) = zikt * (sensfl_old - sensfl_sns * dtheta) + psnew(ji)

          !! \latexonly 
          !!     \input{enerbilsurftemp17.tex}
          !! \endlatexonly
          fevap = (lareva_old - lareva_sns * dtheta) + (larsub_old - larsub_sns * dtheta)
          IF ( ABS(fevap) < EPSILON(un) ) THEN

             !! \latexonly 
             !!     \input{enerbilsurftemp18.tex}
             !! \endlatexonly
             qair_new(ji) = qair(ji)

          ELSE

             !! \latexonly 
             !!     \input{enerbilsurftemp19.tex}
             !! \endlatexonly     
             qair_new(ji) = zikq * un / ( chalsu0 *  vbeta1(ji) * (un - vbeta5(ji)) + &
                  & chalev0 * ((un - vbeta1(ji))* &
                  & (un - vbeta5(ji)) * vbeta(ji) * valpha(ji) + vbeta5(ji)) ) &
                  & * fevap + qsol_sat_new(ji)
          ENDIF
       ENDIF
    ENDDO

    IF (long_print) WRITE (numout,*) ' enerbil_surftemp done '

  END SUBROUTINE enerbil_surftemp


 !! =============================================================================================================================
 !! SUBROUTINE                  : enerbil_pottemp
 !!
 !>\BRIEF       This subroutine computes the surface temperature and humidity should the surface been unstressed    
 !!
 !! DESCRIPTION             : 
 !!
 !! MAIN OUTPUT VARIABLE(S) : 
 !!
 !! REFERENCE(S)        :
 !!
 !! FLOWCHART    : None
 !!
 !_ ==============================================================================================================================  

  SUBROUTINE enerbil_pottemp (kjpindex, dtradia, zlev, emis, epot_air, &
     & petAcoef, petBcoef, qair, peqAcoef, peqBcoef, soilflx, rau, u, v, q_cdrag, vbeta,&
     & valpha, vbeta1, vbeta5, soilcap, lwdown, swnet, q_sol_pot, temp_sol_pot) 

    ! interface 
    ! input scalar 
    INTEGER(i_std), INTENT(in)                               :: kjpindex      !! Domain size
    REAL(r_std), INTENT(in)                                  :: dtradia       !! Time step in seconds
    ! input fields
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: zlev          !! Height of first layer
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: emis          !! Emissivity
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: epot_air      !! Air potential energy
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: petAcoef      !! PetAcoef
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: petBcoef      !! PetBcoef
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: qair          !! Lowest level specific humidity
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: peqAcoef      !! PeqAcoef
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: peqBcoef      !! PeqBcoef
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: soilflx       !! Soil flux
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: rau           !! Density
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: u, v          !! Wind
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: q_cdrag       !!
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: vbeta         !! Resistance coefficient
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: valpha        !! Resistance coefficient
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: vbeta1        !! Snow resistance
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: vbeta5        !! Floodplains resistance
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: soilcap       !! Soil calorific capacity
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: lwdown        !! Down-welling long-wave flux
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: swnet         !! Net surface short-wave flux
    ! output fields
    REAL(r_std),DIMENSION (kjpindex), INTENT (out)           :: q_sol_pot     !! Potential surface air moisture
    REAL(r_std),DIMENSION (kjpindex), INTENT (out)           :: temp_sol_pot  !! Potential soil temperature


    ! local declaration
    INTEGER(i_std)                  :: ji
    REAL(r_std),DIMENSION (kjpindex) :: zicp
    REAL(r_std)                      :: fevap
    REAL(r_std)                      :: sensfl_old, larsub_old, lareva_old, dtheta, sum_old, sum_sns
    REAL(r_std)                      :: zikt, zikq, netrad_sns, sensfl_sns, larsub_sns, lareva_sns
    REAL(r_std)                      :: speed

!_ ==============================================================================================================================

    !++++ CHECK ++++
    ! temp_sol_pot is an output variable?  Need to initialize it, otherwise
    ! it goes into the loop with no value.
    temp_sol_pot(:)=zero
    q_sol_pot(:)=zero
    !+++++++++++++++

    zicp = un / cp_air
    !
    DO ji=1,kjpindex

       dtheta = zero
       fevap = zero
       temp_sol_pot(ji) = temp_sol_pot(ji) + dtheta

       q_sol_pot(ji) = q_sol_pot(ji) + fevap

    ENDDO

  END SUBROUTINE enerbil_pottemp


  !!  =============================================================================================================================
  !! SUBROUTINE                 : enerbil_flux
  !!
  !>\BRIEF                  Computes the new soil temperature, net radiation and
  !! latent and sensible heat flux for the new time step.
  !!
  !! DESCRIPTION                : This routine diagnoses, based on the new soil temperature, 
  !! the net radiation, the total evaporation, the latent heat flux, the sensible heat flux and the 
  !! sublimination flux. It also diagnoses the potential evaporation used for the fluxes (evapot) and the potential
  !! as defined by Penman & Monteith (Monteith, 1965) based on the correction term developed by Chris 
  !! Milly (1992). This Penman-Monteith formulation is required for the estimation of bare soil evaporation 
  !! when the 11 layer CWRR moisture scheme is used for the soil hydrology.
  !!
  !! MAIN OUTPUT VARIABLE(S)    : qsurf, fluxsens, fluxlat, fluxsubli, vevapp, lwup, lwnet,
  !! tsol_rad, netrad, evapot, evapot_corr
  !!
  !! REFERENCE(S)                   :
  !! - Best, MJ, Beljaars, A, Polcher, J & Viterbo, P, 2004. A proposed structure for coupling tiled
  !! surfaces with the planetary boundary layer. Journal of Hydrometeorology, 5, pp.1271-1278
  !! - de Noblet-Ducoudré, N, Laval, K & Perrier, A, 1993. SECHIBA, a new set of parameterisations
  !! of the hydrologic exchanges at the land-atmosphere interface within the LMD Atmospheric General
  !! Circulation Model. Journal of Climate, 6, pp.248-273
  !! - Dufresne, J-L & Ghattas, J, 2009. Description du schéma de la couche limite turbulente et la
  !! interface avec la surface planetaire dans LMDZ, Technical note, available (22/12/11):
  !! http://lmdz.lmd.jussieu.fr/developpeurs/notes-techniques/ressources/pbl_surface.pdf
  !! - Guimberteau, M, 2010. Modélisation de l'hydrologie continentale et influences de l'irrigation
  !! sur le cycle de l'eau, PhD Thesis, available (22/12/11):
  !! http://www.sisyphe.upmc.fr/~guimberteau/docs/manuscrit_these.pdf
  !! - Monteith, JL, 1965. Evaporation and Environment, paper presented at Symposium of the Society
  !! for Experimental Biology
  !! - Monteith & Unsworth, 2008. Principles of Environmental Physics (third edition), published Elsevier
  !! ISBN 978-0-12-505103-3
  !! - Milly, P. C. D., 1992: Potential Evaporation and Soil Moisture in General Circulation Models. 
  !! Journal of Climate, 5, pp. 209–226.
  !! - Polcher, J. McAvaney, B, Viterbo, P, Gaertner, MA, Hahmann, A, Mahfouf, J-F, Noilhan, J
  !! Phillips, TJ, Pitman, AJ, Schlosser, CA, Schulz, J-P, Timbal, B, Verseghy, D &
  !! Xue, Y, 1998. A proposal for a general interface between land surface schemes and
  !! general circulation models. Global and Planetary Change, 19, pp.261-276
  !! - Richtmeyer, RD, Morton, KW, 1967. Difference Methods for Initial-Value Problems.
  !! Interscience Publishers
  !! - Schulz, Jan-Peter, Lydia Dümenil, Jan Polcher, 2001: On the Land Surface–Atmosphere 
  !! Coupling and Its Impact in a Single-Column Atmospheric Model. J. Appl. Meteor., 40, 642–663.
  !!
  !! FLOWCHART                  : None
  !! \n
  !_ ==============================================================================================================================

  SUBROUTINE enerbil_flux (kjpindex, dtradia, emis, temp_sol, rau, &
       & u, v, q_cdrag, vbeta, valpha, &
       & vbeta1, vbeta5, qair, epot_air, psnew, &
       & qsurf, fluxsens, fluxlat, fluxsubli, vevapp, &
       & temp_sol_new, lwdown, swnet, lwup, lwnet, &
       & pb, tsol_rad, netrad, evapot, evapot_corr, &
       & beta_diffuco, soilflx, soilcap, precip_rain, &
       & snowdz, snowrho, snowtemp, radsink, s2flux, &
       & osfcmelt, ptso1, temp_air, pgflux)

    !! 0. Variable and parameter declaration
    
    !! 0.1 Input variables

    INTEGER(i_std), INTENT(in)                               :: kjpindex      !! Domain size (-)
    REAL(r_std), INTENT(in)                                  :: dtradia       !! Time step (s)
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: emis          !! Emissivity (-)
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: temp_sol      !! Surface temperature (K)
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: rau           !! Density (kg m^{-3})
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: u,v           !! Wind velocity in components u and v (m s^{-1}) 
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: q_cdrag       !! Surface drag (m s^{-1})
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: vbeta         !! Resistance coefficient (-)
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: valpha        !! Resistance coefficient (-)
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: vbeta1        !! Snow resistance  (-)
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: vbeta5        !! Flood resistance 
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: qair          !! Lowest level specific humidity (kg kg^{-1})
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: epot_air      !! Air potential energy (J)
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: psnew         !! New surface static energy (J kg^{-1})
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: temp_sol_new  !! New soil temperature (K)
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: pb            !! Lowest level pressure (hPa)

    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: soilflx       !! Soil flux (W m^{-2})
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: soilcap       !! Soil calorific capacity (J K^{-1])
    REAL(r_std), DIMENSION (kjpindex,nsnow), INTENT(in)      :: snowdz        !! Snow depth 
    REAL(r_std), DIMENSION (kjpindex,nsnow), INTENT(in)      :: snowrho       !! Snow density 
    REAL(r_std), DIMENSION (kjpindex,nsnow), INTENT(in)      :: radsink       !! Radiation term 
    REAL(r_std), DIMENSION (kjpindex),INTENT(in)             :: ptso1         !! First snow layer temperature from enerbil_begin 
    REAL(r_std), DIMENSION (kjpindex),INTENT(in)             :: s2flux        !! Heat flux between surface and 2nd snow layers(W/m2) 
    REAL(r_std), DIMENSION (kjpindex),INTENT(in)             :: precip_rain   !! Rainfall 
    REAL(r_std), DIMENSION (kjpindex),INTENT(in)             :: temp_air      !! Air temperature 
    

    !! 0.2 Output variables

    REAL(r_std),DIMENSION (kjpindex), INTENT (out)           :: qsurf         !! Surface specific humidity (kg kg^{-1})
    REAL(r_std),DIMENSION (kjpindex), INTENT (out)           :: fluxsens      !! Sensible heat flux (W m^{-2})
    REAL(r_std),DIMENSION (kjpindex), INTENT (out)           :: fluxlat       !! Latent heat flux (W m^{-2})
    REAL(r_std),DIMENSION (kjpindex), INTENT (out)           :: fluxsubli     !! Energy of sublimation (W m^{-2})
    REAL(r_std),DIMENSION (kjpindex), INTENT (out)           :: vevapp        !! Total of evaporation (mm day^{-1})
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: lwdown        !! Downward long wave radiation (W m^{-2})
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: swnet         !! Net short wave radiation (W m^{-2})
    REAL(r_std),DIMENSION (kjpindex), INTENT (out)           :: lwup          !! Long-wave up radiation (W m^{-2})
    REAL(r_std),DIMENSION (kjpindex), INTENT (out)           :: lwnet         !! Long-wave net radiation (W m^{-2})
    REAL(r_std),DIMENSION (kjpindex), INTENT (out)           :: tsol_rad      !! Radiative surface temperature (W m^{-2})

    
    !! 0.3 Modified variables

    REAL(r_std),DIMENSION (kjpindex), INTENT (inout)         :: netrad        !! Net radiation (W m^{-2})
    REAL(r_std),DIMENSION (kjpindex), INTENT (inout)         :: evapot        !! Soil Potential Evaporation (mm day^{-1})
    REAL(r_std),DIMENSION (kjpindex), INTENT (inout)         :: evapot_corr   !! Soil Potential Evaporation Correction 
    LOGICAL, DIMENSION(kjpindex), INTENT(inout)              :: osfcmelt      !! Indicate snow melting in each gridcell 
    REAL(r_std), DIMENSION (kjpindex,nsnow), INTENT(inout)   :: snowtemp      !! Snow temperature 
    REAL(r_std), DIMENSION (kjpindex),INTENT(inout)          :: pgflux        !! Net energy into snowpack(W/m^2)                                                                             
                                                                              !! (?? mm day^{-1})
 
    !! 0.4 Local variables

    INTEGER(i_std)                                           :: ji
    REAL(r_std),DIMENSION (kjpindex)                     :: grad_qsat
    REAL(r_std)                                          :: correction
    REAL(r_std)                                          :: speed          !! Speed (m s^{-1}), 
    REAL(r_std)                                          :: qc             !! Surface drag coefficient (??)
    LOGICAL,DIMENSION (kjpindex)                         :: warning_correction
    REAL(r_std),DIMENSION (kjpindex)                         :: beta_diffuco
    REAL(r_std), DIMENSION (kjpindex)                        :: zsmsnow,zgflux 
    REAL(r_std), DIMENSION (kjpindex)                        :: ZSNOWTEMP,PCT,ZSNOWTO3,ZDELTAT,PHPSNOW 
    REAL(r_std)                                              :: allowed_err 

!_ ================================================================================================================================
    
    !! Case A : Calculation of surface energy fluxes by taking into account surface snow 
    !!          state (frozen vs. melt). only if there is snow on the ground and if 
    !!          ok_explictsnow is activated 
    !! Case B : Otherwise calculation of surface energy fluxes using the default scheme 
    DO ji=1,kjpindex
       IF (ok_explicitsnow .AND. SUM(snowdz(ji,:)) .GT. 0.0) THEN

          !! Case A: Calculation of surface energy fluxes by taking into 
          !! account surface snow state (frozen vs. melt) 

          !! 0. Initialization:
          ZSNOWTEMP(ji)  = snowtemp(ji,1)
          speed = MAX(min_wind, SQRT (u(ji)*u(ji) + v(ji)*v(ji)))
          qc = speed * q_cdrag(ji)
          lwup(ji) = emis(ji) * c_stefan * ptso1(ji)**4 + &
               &     quatre * emis(ji) * c_stefan * ptso1(ji)**3 * &
               &     (snowtemp(ji,1) - ptso1(ji))

          ! Add the reflected LW radiation
          lwup(ji) = lwup(ji)  +  (un - emis(ji)) * lwdown(ji)
          lwnet(ji) = lwdown(ji) - lwup(ji)
          tsol_rad(ji) = emis(ji) * c_stefan * ptso1(ji)**4 + lwup(ji)
          PCT(ji)      = 1.0/(snowrho(ji,1)*xci*MAX(snowdz(ji,1),psnowdzmin))
          osfcmelt(ji)   = .FALSE.
          ZSNOWTO3(ji)   = ptso1(ji) ** 3 

          !! 1. Flux calculations when melt not occuring at surface (W/m2):
          ! Surface temperature change:
          ZDELTAT(ji) = snowtemp(ji,1) - ptso1(ji)
          netrad(ji)     = swnet(ji) + emis(ji) * (lwdown(ji) -    &
               c_stefan * ZSNOWTO3(ji)*( ptso1(ji) + 4.* ZDELTAT(ji)) )
          qsurf(ji) = (vbeta1(ji) * (un - vbeta5(ji)) + vbeta5(ji)) *qsol_sat_new(ji) + &
               & (un - vbeta1(ji))*(un - vbeta5(ji)) * vbeta(ji)*valpha(ji) * qsol_sat_new(ji)
          qsurf(ji) = MAX(qsurf(ji), qair(ji))
          fluxsens(ji)      = rau(ji)* qc * cp_air * (snowtemp(ji,1) -epot_air(ji)/cp_air)
          vevapp(ji) = dtradia * rau(ji) * qc * (vbeta1(ji) * (un -vbeta5(ji)) + vbeta5(ji)) * &
               & (qsol_sat_new(ji) - qair(ji)) + &
               &  dtradia * rau(ji) * qc * (un - vbeta1(ji))*(un-vbeta5(ji)) * vbeta(ji) * &
               & (valpha(ji) * qsol_sat_new(ji) - qair(ji))
          fluxlat(ji) = chalsu0 * rau(ji) * qc * vbeta1(ji) * (un -vbeta5(ji)) * &
               & (qsol_sat_new(ji) - qair(ji)) + &
               &  chalev0 * rau(ji) * qc * vbeta5(ji) *&
               & (qsol_sat_new(ji) - qair(ji)) + &
               &  chalev0 * rau(ji) * qc * (un - vbeta1(ji)) * (un-vbeta5(ji)) * vbeta(ji) * &
               & (valpha(ji) * qsol_sat_new(ji) - qair(ji))
          fluxsubli(ji) = chalsu0 * rau(ji) * qc * vbeta1(ji) * (un-vbeta5(ji)) * &
               & (qsol_sat_new(ji) - qair(ji))
          PHPSNOW(ji) =precip_rain(ji)*(4.218E+3)*(MAX(tp_00,temp_air(ji))-tp_00)/dtradia ! (w/m2)
          pgflux(ji)  = netrad(ji) - fluxsens(ji) - fluxlat(ji) + PHPSNOW(ji)

          !! 2. Initial melt adjustment
          !  If energy avalabile to melt snow, then recalculate fluxes
          !  at the freezing point and add residual heat to layer
          !  average heat.
         
          !! A) If temperature change is > 0 and passes freezing point this
          !! timestep,
          !     then recalculate fluxes at freezing point and excess energy
          !     will be used outside of this routine to change snow heat
          !     content:
          IF (snowtemp(ji,1) > tp_00 .AND. ptso1(ji) < tp_00) THEN

             osfcmelt(ji)= .TRUE.
             ZDELTAT(ji) = tp_00 - ptso1(ji)
             lwup(ji) = emis(ji) * c_stefan * ptso1(ji)**4 + &
                  &     quatre * emis(ji) * c_stefan * ptso1(ji)**3 * &
                  &     (tp_00 - ptso1(ji))

             ! Add the reflected LW radiation
             lwup(ji) = lwup(ji)  +  (un - emis(ji)) * lwdown(ji)
             lwnet(ji) = lwdown(ji) - lwup(ji)
             tsol_rad(ji) = emis(ji) * c_stefan * ptso1(ji)**4 + lwup(ji)
             netrad(ji)     = swnet(ji) + emis(ji) * (lwdown(ji) -    &
                  c_stefan * ZSNOWTO3(ji)*( ptso1(ji) + 4.* ZDELTAT(ji) ) )
             fluxsens(:)      = rau(ji) * qc * cp_air * (tp_00 - epot_air(ji)/cp_air)
             vevapp(ji) = dtradia * rau(ji) * qc * (vbeta1(ji) * (un -vbeta5(ji)) + vbeta5(ji)) * &
                  & (qsol_sat_new(ji) - qair(ji)) + &
                  &  dtradia * rau(ji) * qc * (un -vbeta1(ji))*(un-vbeta5(ji)) *vbeta(ji) * &
                  & (valpha(ji) * qsol_sat_new(ji) - qair(ji))
             fluxlat(ji) = chalsu0 * rau(ji) * qc * vbeta1(ji) * (un-vbeta5(ji)) * &
                  & (qsol_sat_new(ji) - qair(ji)) + &
                  &  chalev0 * rau(ji) * qc * vbeta5(ji) *&
                  & (qsol_sat_new(ji) - qair(ji)) + &
                  &  chalev0 * rau(ji) * qc * (un - vbeta1(ji)) * (un-vbeta5(ji))* vbeta(ji) * &
                  & (valpha(ji) * qsol_sat_new(ji) - qair(ji))
             fluxsubli(ji) = chalsu0 * rau(ji) * qc * vbeta1(ji) * (un-vbeta5(ji))* &
                  & (qsol_sat_new(ji) - qair(ji))
             zgflux(ji)  = netrad(ji) - fluxsens(ji) - fluxlat(ji) +PHPSNOW(ji)
             zsmsnow(ji) = pgflux(ji) - zgflux(ji)
             pgflux(ji)  = zgflux(ji)
             
             ! This will be used to change heat content of snow:
             ZSNOWTEMP(ji) = snowtemp(ji,1) - zsmsnow(ji)*dtradia*PCT(ji)

          END IF

          !! 3. Ongoing melt adjustment: explicit solution
          !    If temperature change is 0 and at freezing point this
          !    timestep,
          !    then recalculate fluxes and surface temperature *explicitly*
          !    as this is *exact* for snow at freezing point (Brun, Martin)
          allowed_err = 50000*EPSILON(un)

          IF (snowtemp(ji,1) > tp_00 .AND. ABS(ptso1(ji)-tp_00) .LE. allowed_err ) THEN

             osfcmelt(ji)  = .TRUE.
             netrad(ji)     = swnet(ji) + emis(ji) * (lwdown(ji) -    &
                  c_stefan * (tp_00 ** 4) )
             lwup(ji) = emis(ji) * c_stefan * ptso1(ji)**4 + &
                  &     quatre * emis(ji) * c_stefan * ptso1(ji)**3 * &
                  &     (tp_00 - ptso1(ji))
             
             ! Add the reflected LW radiation
             lwup(ji) = lwup(ji)  +  (un - emis(ji)) * lwdown(ji)
             lwnet(ji) = lwdown(ji) - lwup(ji)
             tsol_rad(ji) = emis(ji) * c_stefan * ptso1(ji)**4 + lwup(ji)
             fluxsens(ji)      = rau(ji) * qc * cp_air * (tp_00 - epot_air(ji)/cp_air)
             vevapp(ji) = dtradia * rau(ji) * qc * (vbeta1(ji) *(un-vbeta5(ji)) + vbeta5(ji)) * &
                  & (qsol_sat_new(ji) - qair(ji)) + &
                  &  dtradia * rau(ji) * qc * (un-vbeta1(ji))*(un-vbeta5(ji)) *vbeta(ji) * &
                  & (valpha(ji) * qsol_sat_new(ji) - qair(ji))
             fluxlat(ji) = chalsu0 * rau(ji) * qc * vbeta1(ji) *(un-vbeta5(ji)) * &
                  & (qsol_sat_new(ji) - qair(ji)) + &
                  &  chalev0 * rau(ji) * qc * vbeta5(ji) *&
                  & (qsol_sat_new(ji) - qair(ji)) + &
                  &  chalev0 * rau(ji) * qc * (un - vbeta1(ji)) *(un-vbeta5(ji))* vbeta(ji) * &
                  & (valpha(ji) * qsol_sat_new(ji) - qair(ji))
             fluxsubli(ji) = chalsu0 * rau(ji) * qc * vbeta1(ji) *(un-vbeta5(ji))* &
                  & (qsol_sat_new(ji) - qair(ji))
             pgflux(ji)  =netrad(ji) - fluxsens(ji) - fluxlat(ji) +PHPSNOW(ji)
             ZSNOWTEMP(ji) = tp_00 + dtradia*PCT(ji)*(pgflux(ji) +radsink(ji,1) - s2flux(ji))

          END IF
          
          !! 4. Update surface temperature:
          snowtemp(ji,1) = ZSNOWTEMP(ji)

          !! 5. soil evaporation flux 
          evapot(ji) =  MAX(zero, dtradia * rau(ji) * qc * (qsol_sat_new(ji)-qair(ji)))
          tair(ji)  =  epot_air(ji) / cp_air

       ELSE

          !! 1. Determination of 'housekeeping' variables
          !! The horizontal wind speed is calculated from the velocities along each axis 
          !! using Pythagorus' theorem. 
          !! \latexonly 
          !!     \input{enerbilflux1.tex}
          !! \endlatexonly
          speed = MAX(min_wind, SQRT (u(ji)*u(ji) + v(ji)*v(ji)))

          !! From definition, the surface drag coefficient is defined:
          !! \latexonly 
          !!     \input{enerbilflux2.tex}
          !! \endlatexonly
          qc = speed * q_cdrag(ji)

          !! 2. Calculation of the net upward flux of longwave radiation
          !! We first of all calculate the radiation as a result of the Stefan-Boltzmann equation,
          !!  which is the sum of the calculated values at the surface temperature  at the 'old' 
          !!  temperature and the value that corresponds to the difference between the old temperature
          !!  and the temperature at the 'new' timestep.
          !! \latexonly 
          !!     \input{enerbilflux3.tex}
          !! \endlatexonly
          lwup(ji) = emis(ji) * c_stefan * temp_sol(ji)**4 + &
               &     quatre * emis(ji) * c_stefan * temp_sol(ji)**3 * &
               &     (temp_sol_new(ji) - temp_sol(ji))      

          !! We then add to this value that from the reflected longwave radiation:
          !! \latexonly 
          !!     \input{enerbilflux4.tex}
          !! \endlatexonly
          lwup(ji) = lwup(ji)  +  (un - emis(ji)) * lwdown(ji)

          !! The radiative surface temperature is inverting the Stefan-Boltzmann relation:
          !! \latexonly 
          !!     \input{enerbilflux5.tex}
          !! \endlatexonly      
          !! The implicit solution computes an emitted long-wave flux which is a limited Taylor expansion 
          !! of the future surface temperature around the current values. Thus the long-wave flux does not  
          !! correspond to the new surface temperature but some intermediate value. So we need to deduce the 
          !! radiative surface temperature to which this flux corresponds. 
          tsol_rad(ji) = (lwup(ji)/ (emis(ji) * c_stefan)) **(1./quatre) 

          !! qsurf (the surface specific humidity) is a simple diagnostic which will be used by the 
          !! GCM to compute the dependence of of the surface layer stability on moisture.
          !! \latexonly 
          !!     \input{enerbilflux6.tex}
          !! \endlatexonly
          qsurf(ji) = (vbeta1(ji) * (un - vbeta5(ji)) + vbeta5(ji)) * qsol_sat_new(ji) + &
               & (un - vbeta1(ji))*(un - vbeta5(ji)) * vbeta(ji) * valpha(ji) * qsol_sat_new(ji)

          !! \latexonly 
          !!     \input{enerbilflux7.tex}
          !! \endlatexonly
          qsurf(ji) = MAX(qsurf(ji), qair(ji))

          !! Net downward radiation is the sum of the down-welling less the up-welling long wave flux, plus
          !! the short wave radiation.
          !! \latexonly 
          !!     \emissivity absorbed lw radiationinput{enerbilflux8.tex}
          !! \endlatexonly
          netrad(ji) = lwdown(ji) + swnet(ji) - lwup(ji) 

          !! 'vevapp' is the sum of the total evaporative processes (snow plus non-snow processes).
          !! \latexonly 
          !!     \input{enerbilflux9.tex}
          !! \endlatexonly
          vevapp(ji) = dtradia * rau(ji) * qc * (vbeta1(ji) * (un - vbeta5(ji)) + vbeta5(ji)) * &
               & (qsol_sat_new(ji) - qair(ji)) + &
               &  dtradia * rau(ji) * qc * (un - vbeta1(ji))*(un-vbeta5(ji)) * vbeta(ji) * &
               & (valpha(ji) * qsol_sat_new(ji) - qair(ji))

          !! The total latent heat flux is the sum of the snow plus non-snow processes.
          !! \latexonly 
          !!     \input{enerbilflux10.tex}
          !! \endlatexonly
          fluxlat(ji) = chalsu0 * rau(ji) * qc * vbeta1(ji) * (un - vbeta5(ji)) * &
               & (qsol_sat_new(ji) - qair(ji)) + &
               &  chalev0 * rau(ji) * qc * vbeta5(ji) *&
               & (qsol_sat_new(ji) - qair(ji)) + &
               &  chalev0 * rau(ji) * qc * (un - vbeta1(ji)) * (un - vbeta5(ji)) * vbeta(ji) * &
               & (valpha(ji) * qsol_sat_new(ji) - qair(ji))


          !! The sublimination flux concerns is calculated using vbeta1, the snow resistance.
          !! \latexonly 
          !!     \input{enerbilflux11.tex}
          !! \endlatexonly
          fluxsubli(ji) = chalsu0 * rau(ji) * qc * vbeta1(ji) * (un - vbeta5(ji)) * &
               & (qsol_sat_new(ji) - qair(ji)) 

          !! The sensible heat flux is a factor of the difference between the new surface static energy
          !! and the potential energy of air.
          !! \latexonly 
          !!     \input{enerbilflux12.tex}
          !! \endlatexonly
          fluxsens(ji) =  rau(ji) * qc * (psnew(ji) - epot_air(ji))

          !! This is the net longwave downwards radiation.
          !! \latexonly 
          !!     \input{enerbilflux13.tex}
          !! \endlatexonly
          lwnet(ji) = lwdown(ji) - lwup(ji)

          !! Diagnoses the potential evaporation used for the fluxes (evapot)
          !! \latexonly 
          !!     \input{enerbilflux14.tex}
          !! \endlatexonly  
          evapot(ji) =  MAX(zero, dtradia * rau(ji) * qc * (qsol_sat_new(ji) - qair(ji))) 

          !! From definition we can say:
          !! \latexonly 
          !!     \input{enerbilflux15.tex}
          !! \endlatexonly 
          tair(ji)  =  epot_air(ji) / cp_air

       ENDIF

    ENDDO

    !! 3. Define qsat_air with the subroutine src_parameter:

    CALL qsatcalc(kjpindex, tair, pb, qsat_air)
    CALL dev_qsatcalc(kjpindex, tair, pb, grad_qsat)

    ! grad_qsat(:)= (qsol_sat_new(:)- qsat_air(:)) / ((psnew(:) - epot_air(:)) / cp_air) ! * dtradia
    !- Penser a sortir evapot en meme temps qu'evapot_corr tdo.
    warning_correction(:)=.FALSE.
    DO ji=1,kjpindex

       !! \latexonly 
       !!     \input{enerbilflux16.tex}
       !! \endlatexonly 
       speed = MAX(min_wind, SQRT (u(ji)*u(ji) + v(ji)*v(ji)))

       !! \latexonly 
       !!     \input{enerbilflux17.tex}
       !! \endlatexonly 
       qc = speed * q_cdrag(ji)

       !! Derive the potential as defined by Penman & Monteith (Monteith, 1965) based on the correction 
       !! term developed by Chris Milly (1992).
       IF ((evapot(ji) .GT. zero) .AND. ((psnew(ji) - epot_air(ji)) .NE. zero )) THEN
          
          !! \latexonly 
          !!     \input{enerbilflux18.tex}
          !! \endlatexonly 
          correction =  (quatre * emis(ji) * c_stefan * tair(ji)**3 + rau(ji) * qc * cp_air + &
               &                  chalev0 * rau(ji) * qc * grad_qsat(ji) * vevapp(ji) / evapot(ji) )

          !! \latexonly 
          !!     \input{enerbilflux19.tex}
          !! \endlatexonly 
          IF (ABS(correction) .GT. min_sechiba) THEN
             correction = chalev0 * rau(ji) * qc * grad_qsat(ji) * (un - vevapp(ji)/evapot(ji)) / correction
          ELSE
             warning_correction(ji)=.TRUE.
          ENDIF

       ELSE
          correction = zero
       ENDIF

       correction = MAX (zero, correction)

       !! \latexonly 
       !!     \input{enerbilflux20.tex}
       !! \endlatexonly 
       evapot_corr(ji) = evapot(ji) / (un + correction)

    ENDDO

    IF ( ANY(warning_correction) ) THEN
       DO ji=1,kjpindex
          IF ( warning_correction(ji) ) THEN
             WRITE(numout,*) ji,"Denominateur de la correction de milly nul! Aucune correction appliquee"
          ENDIF
       ENDDO
    ENDIF

    IF (long_print) WRITE (numout,*) ' enerbil_flux done '

  END SUBROUTINE enerbil_flux


  !!  ===========================================================================================================================
  !! SUBROUTINE                 : enerbil_evapveg
  !!
  !>\BRIEF                  Splits total evaporation loss into individual process
  !! components and calculates the assimilation.
  !!
  !! DESCRIPTION                : Based on the estimation of the fluxes in enerbil_flux, 
  !! this routine splits the total evaporation into transpiration interception loss from vegetation, 
  !! bare soil evaporation and snow sublimation. It then calculates the assimilation.
  !!
  !! MAIN OUTPUT VARIABLE(S)                    : vevapsno, vevapnu, transpir, vevapwet
  !!
  !! REFERENCE(S)                   :
  !! - Best, MJ, Beljaars, A, Polcher, J & Viterbo, P, 2004. A proposed structure for coupling tiled
  !! surfaces with the planetary boundary layer. Journal of Hydrometeorology, 5, pp.1271-1278
  !! - de Noblet-Ducoudré, N, Laval, K & Perrier, A, 1993. SECHIBA, a new set of parameterisations
  !! of the hydrologic exchanges at the land-atmosphere interface within the LMD Atmospheric General
  !! Circulation Model. Journal of Climate, 6, pp.248-273
  !! - Dufresne, J-L & Ghattas, J, 2009. Description du schéma de la couche limite turbulente et la
  !! interface avec la surface planetaire dans LMDZ, Technical note, available (22/12/11):
  !! http://lmdz.lmd.jussieu.fr/developpeurs/notes-techniques/ressources/pbl_surface.pdf
  !! - Guimberteau, M, 2010. Modélisation de l'hydrologie continentale et influences de l'irrigation
  !! sur le cycle de l'eau, PhD Thesis, available (22/12/11): 
  !! http://www.sisyphe.upmc.fr/~guimberteau/docs/manuscrit_these.pdf
  !! - Polcher, J. McAvaney, B, Viterbo, P, Gaertner, MA, Hahmann, A, Mahfouf, J-F, Noilhan, J
  !! Phillips, TJ, Pitman, AJ, Schlosser, CA, Schulz, J-P, Timbal, B, Verseghy, D &
  !! Xue, Y, 1998. A proposal for a general interface between land surface schemes and
  !! general circulation models. Global and Planetary Change, 19, pp.261-276
  !! - Richtmeyer, RD, Morton, KW, 1967. Difference Methods for Initial-Value Problems.
  !! Interscience Publishers
  !! - Schulz, Jan-Peter, Lydia Dümenil, Jan Polcher, 2001: On the Land Surface–Atmosphere 
  !! Coupling and Its Impact in a Single-Column Atmospheric Model. J. Appl. Meteor., 40, 642–663. 
  !!
  !! FLOWCHART   : None
  !! \n
  !_ ==============================================================================================================================

  SUBROUTINE enerbil_evapveg (kjpindex, dtradia, vbeta1, vbeta2, vbeta3, &
       & vbeta3pot, vbeta4, vbeta5, rau, u, &
       & v, q_cdrag, qair, humrel, vevapsno, &
       & vevapnu , vevapflo, vevapwet, transpir, transpot, &
       & evapot, snowdz, snowrho, evapot_corr)

    !! 0. Variable and parameter declaration
    
    !! 0.1 Input variables

    INTEGER(i_std), INTENT(in)                          :: kjpindex          !! Domain size (-)
    REAL(r_std), INTENT(in)                             :: dtradia           !! Time step (s)
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)       :: vbeta1            !! Snow resistance (-)
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)       :: vbeta4            !! Bare soil resistance (-)
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)       :: vbeta5            !! Floodplains resistance
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)       :: rau               !! Density (kg m^{-3})
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)       :: u, v              !! Wind velocity in directions u and v (m s^{-1})
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)       :: q_cdrag           !! Surface drag (m s^{-1})
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)       :: qair              !! Lowest level specific humidity (kg kg^{-1})
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)       :: evapot            !! Soil Potential Evaporation (mm day^{-1})
    REAL(r_std),DIMENSION (kjpindex, nvm), INTENT (in)  :: humrel            !! Soil moisture stress (within range 0 to 1)
!!$ DS 15022011 humrel was used in a previous version of Orchidee, developped by Nathalie. Need to be discussed if it should be introduce again             
    REAL(r_std),DIMENSION (kjpindex,nvm), INTENT (in)   :: vbeta2            !! Interception resistance (-)
    REAL(r_std),DIMENSION (kjpindex,nvm), INTENT (in)   :: vbeta3            !! Vegetation resistance (-)
    REAL(r_std),DIMENSION (kjpindex,nvm), INTENT (in)   :: vbeta3pot         !! Vegetation resistance for potential transpiration
    REAL(r_std), DIMENSION(kjpindex,nsnow),INTENT(in)   :: snowrho           !! Snow density
    
    !! 0.2 Output variables

    REAL(r_std),DIMENSION (kjpindex), INTENT (out)      :: vevapsno          !! Snow evaporation (mm day^{-1})
    REAL(r_std),DIMENSION (kjpindex), INTENT (out)      :: vevapnu           !! Bare soil evaporation (mm day^{-1})
    REAL(r_std),DIMENSION (kjpindex), INTENT (out)      :: vevapflo          !! Floodplains evaporation
    REAL(r_std),DIMENSION (kjpindex,nvm), INTENT (out)  :: transpir          !! Transpiration (mm day^{-1})
    REAL(r_std),DIMENSION (kjpindex,nvm), INTENT (out)  :: transpot          !! Potential Transpiration
    REAL(r_std),DIMENSION (kjpindex,nvm), INTENT (out)  :: vevapwet          !! Interception (mm day^{-1})

    !! 0.3 Modified variables
    REAL(r_std), DIMENSION(kjpindex,nsnow),INTENT(inout):: snowdz            !! Snow depth
    REAL(r_std), DIMENSION(kjpindex),INTENT(inout)      :: evapot_corr       !! Correction term in evapotranspiration
    !! 0.4 Local variables

    INTEGER(i_std)                                      :: ji, jv
    REAL(r_std), DIMENSION(kjpindex)                    :: xx
    REAL(r_std), DIMENSION(kjpindex)                    :: vbeta2sum, vbeta3sum
    REAL(r_std)                                     :: speed         !! Speed (m s^{-1})
    REAL(r_std), DIMENSION(kjpindex)                    :: ZSNOWEVAPS,ZEVAPCOR,ZSNOWDZ

!_ ==============================================================================================================================
    ! initialisation: utile pour calculer l'evaporation des floodplains dans lesquelles il y a de la vegetation
    vbeta2sum(:) = 0.
    vbeta3sum(:) = 0.
    DO jv = 1, nvm
      vbeta2sum(:) = vbeta2sum(:) + vbeta2(:,jv)
      vbeta3sum(:) = vbeta3sum(:) + vbeta3(:,jv)
    ENDDO 
    
    !! 1. Compute vevapsno (evaporation from snow) and vevapnu (bare soil evaporation)        
    DO ji=1,kjpindex 

      !! \latexonly 
      !!     \input{enerbilevapveg1.tex}
      !! \endlatexonly
      speed = MAX(min_wind, SQRT (u(ji)*u(ji) + v(ji)*v(ji)))

      !! 1.1 Snow sublimation
      !! \latexonly 
      !!     \input{enerbilevapveg2.tex}
      !! \endlatexonly
      vevapsno(ji) = (un - vbeta5(ji)) * vbeta1(ji) * dtradia * rau(ji) * speed * &
           q_cdrag(ji) * (qsol_sat_new(ji) - qair(ji))

      ! For the case of ok_explicitsnow activated, remove mass from uppermost snow layer in 
      ! response to sublimation. Do not take into account evaporation. The evaporation on the
      ! snow surface could be possible in this case since the new snow model allows liquid 
      ! water to be existed.
      IF ( ok_explicitsnow ) THEN
        
         ! Sublimation of snow ice:
         ! Reduce snow layer thickness and total snow depth if sublimation. 
         ! Add to correction term if potential sublimation exceeds available snow cover.
         ZSNOWEVAPS(ji)  = vevapsno(ji)/(chalsu0*snowrho(ji,1))
         ZSNOWDZ(ji)     = snowdz(ji,1) - ZSNOWEVAPS(ji)
         ZEVAPCOR(ji)    = MAX(0.0,-ZSNOWDZ(ji))*snowrho(ji,1)/dtradia

         ! Update evaporation correction term:
          evapot_corr(ji) = evapot_corr(ji) + ZEVAPCOR(ji)

      ENDIF

      !! 1.2 Bare soil evaporation
      !! \latexonly 
      !!     \input{enerbilevapveg3.tex}
      !! \endlatexonly
      vevapnu(ji) = (un - vbeta1(ji)) * (un-vbeta5(ji)) * vbeta4(ji) * dtradia * rau(ji) * &
           &  speed * q_cdrag(ji) &
           & * (qsol_sat_new(ji) - qair(ji))
      
      ! 1.3 floodplains evaporation - transpiration et interception prioritaires dans les floodplains
      vevapflo(ji) = vbeta5(ji) * (1 - vbeta2sum(ji) - vbeta3sum(ji)) &
           & * dtradia * rau(ji) * speed * q_cdrag(ji) * (qsol_sat_new(ji) - qair(ji))

    END DO

    !! 2. Compute transpir (transpiration) and vevapwet (interception)
    ! Preliminaries
    DO ji = 1, kjpindex
       
       !! \latexonly 
       !!     \input{enerbilevapveg4.tex}
       !! \endlatexonly
       speed = MAX(min_wind, SQRT (u(ji)*u(ji) + v(ji)*v(ji)))
       
       !! \latexonly 
       !!     \input{enerbilevapveg5.tex}
       !! \endlatexonly
       xx(ji) = dtradia * (un-vbeta1(ji)) * (qsol_sat_new(ji)-qair(ji)) * rau(ji) * speed * q_cdrag(ji)
     
    ENDDO
    
    DO jv=1,nvm 
      DO ji=1,kjpindex 
        
        !! 2.1 Calculate interception loss
        !! \latexonly 
        !!     \input{enerbilevapveg6.tex}
        !! \endlatexonly
        vevapwet(ji,jv) = xx(ji) * vbeta2(ji,jv)
         
        !! 2.2 Calculate transpiration
        !! \latexonly 
        !!     \input{enerbilevapveg7.tex}
        !! \endlatexonly
        transpir (ji,jv) = xx(ji) * vbeta3(ji,jv)

        transpot(ji,jv) = xx(ji) * vbeta3pot(ji,jv)

      END DO
    END DO

    IF (long_print) WRITE (numout,*) ' enerbil_evapveg done '

  END SUBROUTINE enerbil_evapveg

 
  !!  =============================================================================================================================
  !! SUBROUTINE                 : enerbil_fusion
  !!
  !>\BRIEF                  Computes new soil temperature due to
  !! ice and snow melt. 
  !!
  !! DESCRIPTION                : This routine computes new soil temperature due to
  !! ice and snow melt. It is the second part of main routine for enerbil module, and is called every
  !! time step. 
  !!
  !! MAIN OUTPUT VARIABLE(S)            : temp_sol_new, fusion
  !!
  !! REFERENCE(S)               :
  !! - Best, MJ, Beljaars, A, Polcher, J & Viterbo, P, 2004. A proposed structure for coupling tiled
  !! surfaces with the planetary boundary layer. Journal of Hydrometeorology, 5, pp.1271-1278
  !! - de Noblet-Ducoudré, N, Laval, K & Perrier, A, 1993. SECHIBA, a new set of parameterisations
  !! of the hydrologic exchanges at the land-atmosphere interface within the LMD Atmospheric General
  !! Circulation Model. Journal of Climate, 6, pp.248-273
  !! - Dufresne, J-L & Ghattas, J, 2009. Description du schéma de la couche limite turbulente et la
  !! interface avec la surface planetaire dans LMDZ. Technical note, available (22/12/11):
  !! http://lmdz.lmd.jussieu.fr/developpeurs/notes-techniques/ressources/pbl_surface.pdf
  !! - Guimberteau, M, 2010. Modélisation de l'hydrologie continentale et influences de l'irrigation
  !! sur le cycle de l'eau. PhD Thesis, available (22/12/11):
  !! http://www.sisyphe.upmc.fr/~guimberteau/docs/manuscrit_these.pdf
  !! - Polcher, J. McAvaney, B, Viterbo, P, Gaertner, MA, Hahmann, A, Mahfouf, J-F, Noilhan, J
  !! Phillips, TJ, Pitman, AJ, Schlosser, CA, Schulz, J-P, Timbal, B, Verseghy, D &
  !! Xue, Y, 1998. A proposal for a general interface between land surface schemes and
  !! general circulation models. Global and Planetary Change, 19, pp.261-276
  !! - Richtmeyer, RD, Morton, KW, 1967. Difference Methods for Initial-Value Problems.
  !! Interscience Publishers
  !! - Schulz, Jan-Peter, Lydia Dümenil, Jan Polcher, 2001: On the Land Surface–Atmosphere 
  !! Coupling and Its Impact in a Single-Column Atmospheric Model. J. Appl. Meteor., 40, 642–663.
  !!
  !! FLOWCHART  : None
  !! \n
  !_ ==============================================================================================================================
  
  SUBROUTINE enerbil_fusion (kjpindex, dtradia, tot_melt, soilcap, snowdz, &
       temp_sol_new, fusion)


    !! 0. Variable and parameter declaration
    
    !! 0.1 Input variables

    INTEGER(i_std), INTENT(in)                               :: kjpindex       !! Domain size (-)
    REAL(r_std), INTENT(in)                                  :: dtradia        !! Time step (s)
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: tot_melt       !! Total melt (??)
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: soilcap        !! Soil heat capacity (J K^{-1])
    REAL(r_std),DIMENSION (kjpindex,nsnow),INTENT(in)        :: snowdz         !! Snow layer thickness (m)
    
    !! 0.2 Output variables
    
    REAL(r_std),DIMENSION (kjpindex), INTENT (out)           :: fusion         !! Fusion (??)

    !! 0.3 Modified variables

    REAL(r_std),DIMENSION (kjpindex), INTENT (inout)         :: temp_sol_new   !! New soil temperature (K)

    !! 0.4 Local variables
    INTEGER(i_std)                                       :: ji         !! (-)

  !_ ==============================================================================================================================
    
    !! Initialisation
    IF (long_print) WRITE (numout,*) ' enerbil_fusion start ', MINVAL(soilcap), MINLOC(soilcap),&
         & MAXVAL(soilcap), MAXLOC(soilcap)
    
    !! 1. Calculate new soil temperature due to ice and snow melt   
    IF  (ok_explicitsnow) THEN

       ! Case with explicit snow activated :
       ! Surface temperature is forced to zero celcius if its value is larger than melting point 
       DO ji=1,kjpindex 
          IF  (SUM(snowdz(ji,:)) .GT. 0.0) THEN
             IF (temp_sol_new(ji) .GE. tp_00) THEN
                temp_sol_new(ji) = tp_00
             ENDIF
          END IF
       END DO

    ELSE

       ! Default case :
       ! Surface temperature is reduced if there is snow melt
       DO ji=1,kjpindex 

          !! \latexonly 
          !!     \input{enerbilfusion1.tex}
          !! \endlatexonly
          fusion(ji) = tot_melt(ji) * chalfu0 / dtradia
          
          !! \latexonly 
          !!     \input{enerbilfusion2.tex}
          !! \endlatexonly
          temp_sol_new(ji) = temp_sol_new(ji) - ((tot_melt(ji) * chalfu0) / soilcap(ji))

       END DO

    ENDIF
           
    IF (long_print) WRITE (numout,*) ' enerbil_fusion done '

  END SUBROUTINE enerbil_fusion

  !!  =============================================================================================================================
  !! SUBROUTINE                 : enerbil_t2mdiag
  !!
  !> BRIEF                  This routine diagnoses the air temperature at 2
  !! metres above ground.
  !!
  !! DESCRIPTION                : The 2 metre above ground air temperature is currently
  !! set to be the same as the general air temperaturem, as the surface model currently simulates the 
  !! above ground atmosphere as a uniform box (and the lowest layer of the LMDZ atmospheric model).
  !!
  !! MAIN OUTPUT VARIABLE(S)            : t2mdiag
  !!
  !! REFERENCE(S)               :
  !! - Best, MJ, Beljaars, A, Polcher, J & Viterbo, P, 2004. A proposed structure for coupling tiled
  !! surfaces with the planetary boundary layer. Journal of Hydrometeorology, 5, pp.1271-1278
  !! - de Noblet-Ducoudré, N, Laval, K & Perrier, A, 1993. SECHIBA, a new set of parameterisations
  !! of the hydrologic exchanges at the land-atmosphere interface within the LMD Atmospheric General
  !! Circulation Model. Journal of Climate, 6, pp.248-273
  !! - Dufresne, J-L & Ghattas, J, 2009. Description du schéma de la couche limite turbulente et la
  !! interface avec la surface planetaire dans LMDZ, Technical note, available from:
  !! http://lmdz.lmd.jussieu.fr/developpeurs/notes-techniques/ressources/pbl_surface.pdf
  !! - Guimberteau, M, 2010. Modélisation de l'hydrologie continentale et influences de l'irrigation
  !! sur le cycle de l'eau, PhD Thesis, available from:
  !! http://www.sisyphe.upmc.fr/~guimberteau/docs/manuscrit_these.pdf
  !! - Polcher, J. McAvaney, B, Viterbo, P, Gaertner, MA, Hahmann, A, Mahfouf, J-F, Noilhan, J
  !! Phillips, TJ, Pitman, AJ, Schlosser, CA, Schulz, J-P, Timbal, B, Verseghy, D &
  !! Xue, Y, 1998. A proposal for a general interface between land surface schemes and
  !! general circulation models. Global and Planetary Change, 19, pp.261-276
  !! - Richtmeyer, RD, Morton, KW, 1967. Difference Methods for Initial-Value Problems.
  !! Interscience Publishers\n
  !! - Schulz, Jan-Peter, Lydia Dümenil, Jan Polcher, 2001: On the Land Surface–Atmosphere 
  !! Coupling and Its Impact in a Single-Column Atmospheric Model. J. Appl. Meteor., 40, 642–663.
  !!
  !! FLOWCHART  : None
  !! \n
  !_ ==============================================================================================================================

  SUBROUTINE enerbil_t2mdiag (kjpindex, temp_air, t2mdiag)

    !! 0. Variable and parameter declaration

    !! 0.1 Input variables

    INTEGER(i_std), INTENT(in)                              :: kjpindex       !! Domain size (-)
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)           :: temp_air       !! Air temperature (K)
    
    !! 0.2 Output variables
    REAL(r_std),DIMENSION (kjpindex), INTENT (out)          :: t2mdiag        !! 2-metre temperature (K)

    !! 0.3 Modified variables

    !! 0.4 Local variables

  !_ ==============================================================================================================================

    !! The 2 metre air temperature is assumed to be the same of the value of temp_air - the atmospheric air
    !! temperature, which also taken as the temperature at the lowest level of the LMDZ. 't2mdiag' is just
    !! a value used for diagnostic purposes, admd it is no applied amywhere else.
    !! \latexonly 
    !!     \input{enerbilt2mdiag1.tex}
    !! \endlatexonly
    t2mdiag(:) = temp_air(:)

    IF (long_print) WRITE (numout,*) ' enerbil_t2mdiag done '

  END SUBROUTINE enerbil_t2mdiag





   ! %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


   SUBROUTINE jdriver (kjit, kjpindex, dtradia, ldrestart_read, ldrestart_write, &
                      & index, indexveg, zlev, lwdown, swnet, swdown, &
                      & epot_air, temp_air, u, v, petAcoef, petBcoef, &
                      & qair, peqAcoef, peqBcoef, pb, rau, &
                      & vbeta, valpha, vbeta1, vbeta2, vbeta3, &
                      & vbeta3pot, vbeta4, vbeta5, &
!!$                      & vbeta3pot, vbeta4, vbeta5, cimean, ccanopy, &
                      & emis, soilflx, soilcap, q_cdrag, &
                      & humrel, fluxsens, fluxlat, vevapp, transpir, &
                      & transpot, vevapnu, vevapwet, vevapsno, &
!!$                      & transpot, gpp, vevapnu, vevapwet, vevapsno, &
                      & vevapflo, t2mdiag, temp_sol, tsol_rad, temp_sol_new, &
                      & qsurf, evapot, evapot_corr, rest_id, hist_id, & 
                      & hist2_id, &
                      & flux_ground_h, flux_ground_le, flux_h_grid, flux_le_grid, &
                      & t_a_next_grid, q_a_next_grid, temp_surf_next, &
                      & temp_atmos_pres_grid, q_atmos_pres_grid, &
                      & james_netcdf_flag, james_step_count, &
                      & temp_leaf_pres_grid, temp_surf_pres, E_supply, hydrol_flag, &
                      & hydrol_flag2, hydrol_flag3, vbeta2sum, vbeta3sum, veget_max, &
                      & qsol_sat_new_out, qair_new, delta_temp_save, netrad, veget, &
                      & Collim_Abs_Tot, Collim_Alb_Tot, &
                      & laieff_isotrop, h_array_out, z_array_out, &
                      & transpir_supply_column, u_speed, profile_vbeta3, profile_rveget, &
                      & max_height_store, u_speed_grid, flux_rn_grid, t2m_month_out)
     IMPLICIT NONE



     !! 0 Variable and parameter description 

     !! 0.1 Input variables

     INTEGER(i_std), INTENT(in)                              :: kjit             !! Time step number (-)
     INTEGER(i_std), INTENT(in)                              :: kjpindex         !! Domain size (-) 

     INTEGER(i_std), INTENT (in)                             :: rest_id,hist_id  !! _Restart_ file and _history_ file identifier (-)
     INTEGER(i_std), INTENT (in)                             :: hist2_id         !! _history_ file 2 identifier (-)

     REAL(r_std), INTENT (in)                                :: dtradia          !! Time (s)

     LOGICAL, INTENT(in)                                     :: hydrol_flag      !! Flag that recalculates the energy budget for the new supply term (true/false)    
     LOGICAL, DIMENSION (kjpindex), INTENT(in)               :: hydrol_flag2     !! flag that 'trips' the alternative energy budget for each grid square
     LOGICAL, DIMENSION (kjpindex, nvm), INTENT(in)          :: hydrol_flag3     !! flag that 'trips' the alternative energy budget for each grid square and PFT
 
     REAL(r_std), DIMENSION(kjpindex,nvm), INTENT(in)        :: E_supply         !! Supply of water for transpiration (mm timestep^{-1})
     REAL(r_std), DIMENSION(kjpindex)                        :: E_supply_tot     !! total supply of water for transpiration for each domain (mm timestep^{-1}))

     LOGICAL, INTENT(in)                                     :: ldrestart_read   !! Logical for _restart_ file to read (-)
     LOGICAL, INTENT(in)                                     :: ldrestart_write  !! Logical for _restart_ file to write (-)

     INTEGER(i_std), DIMENSION (kjpindex), INTENT (in)       :: index            !! Indeces of the points on the map (-)
     INTEGER(i_std), DIMENSION(kjpindex*nvm), INTENT(in)     :: indexveg         !! Indeces of the points on the 3D map

     REAL(r_std), DIMENSION (kjpindex), INTENT (in)          :: zlev             !! Height of first layer (m)
     REAL(r_std), DIMENSION (kjpindex), INTENT (in)          :: lwdown           !! Down-welling long-wave flux (W m^{-2})
     REAL(r_std), DIMENSION (kjpindex), INTENT (in)          :: swnet            !! Net surface short-wave flux (W m^{-2})
     REAL(r_std), DIMENSION (kjpindex), INTENT (in)          :: swdown           !! Down-welling short-wave flux (W/m^2)
     REAL(r_std), DIMENSION (kjpindex), INTENT (in)          :: epot_air         !! Air potential energy (?? J)
    
     REAL(r_std), DIMENSION (kjpindex)                       :: temp_air         !! Air temperature (K)
     REAL(r_std), DIMENSION (kjpindex), INTENT (in)          :: u                !! Eastward Lowest level wind speed  (m s^{-1})
     REAL(r_std), DIMENSION (kjpindex), INTENT (in)          :: v                !! Northward Lowest level wind speed (m s^{-1})
     REAL(r_std), DIMENSION (kjpindex), INTENT (in)          :: petAcoef         !! PetAcoef (see note)
     REAL(r_std), DIMENSION (kjpindex), INTENT (in)          :: petBcoef         !! PetBcoef (see note)
     REAL(r_std), DIMENSION (kjpindex), INTENT (in)          :: peqAcoef         !! PeqAcoef (see note)
     REAL(r_std), DIMENSION (kjpindex), INTENT (in)          :: peqBcoef         !! PeqBcoef (see note)
     REAL(r_std), DIMENSION (kjpindex), INTENT (in)          :: pb               !! Lowest level pressure hPa)
     REAL(r_std), DIMENSION (kjpindex), INTENT (in)          :: rau              !! Air density (kg m^{-3})
    
     REAL(r_std), DIMENSION (kjpindex), INTENT (inout)          :: vbeta            !! Resistance coefficient (-)
     REAL(r_std), DIMENSION (kjpindex), INTENT (in)          :: valpha           !! Resistance coefficient (-)
     REAL(r_std), DIMENSION (kjpindex), INTENT (in)          :: vbeta1           !! Snow resistance (-) 
     REAL(r_std), DIMENSION (kjpindex), INTENT (in)          :: vbeta4           !! Bare soil resistance (-)
     REAL(r_std), DIMENSION (kjpindex), INTENT (in)          :: vbeta5           !! Floodplains resistance
     REAL(r_std), DIMENSION (kjpindex), INTENT (in)          :: emis             !! Emissivity (-)
    
     REAL(r_std), DIMENSION (kjpindex), INTENT (in)          :: soilflx          !! Soil flux (W m^{-2})
     REAL(r_std), DIMENSION (kjpindex), INTENT (in)          :: soilcap          !! Soil calorific capacity (J K^{-1])
    
     REAL(r_std), DIMENSION (kjpindex), INTENT (inout)          :: q_cdrag          !! Surface drag (m s^{-1})
!!$     REAL(r_std), DIMENSION (kjpindex), INTENT (in)          :: ccanopy          !! CO_2 concentration in the canopy (ppm)
     REAL(r_std), DIMENSION (kjpindex,nvm), INTENT (in)      :: humrel           !! Soil moisture stress (within range 0 to 1)
     REAL(r_std), DIMENSION (kjpindex,nvm), INTENT (in)      :: vbeta2           !! Interception resistance (-)
     REAL(r_std), DIMENSION (kjpindex,nvm), INTENT (in)      :: vbeta3pot        !! Vegetation resistance for potential transpiration
!!$     REAL(r_std), DIMENSION (kjpindex,nvm), INTENT (in)      :: vbetaco2         !! Vegetation resistance to CO2 (-)
!!$     REAL(r_std), DIMENSION (kjpindex,nvm), INTENT (in)      :: cimean           !! mean intercellular ci (from STOMATE) 
!!$                                                                                !! (\mu mole m^{-2} s^{-1})

     REAL(r_std), DIMENSION (kjpindex)                       :: qair             !! Lowest level specific humidity (kg kg^{-1})

     INTEGER, INTENT(in)                                     :: james_step_count

     REAL(r_std),  DIMENSION (kjpindex, nvm), INTENT(in)     :: veget_max      !! Max. fraction of vegetation type (LAI -> infty, unitless)
     REAL(r_std),  DIMENSION (kjpindex, nvm), INTENT(in)     :: veget          !! Vegetation fraction for each pft 
  
!!$     REAL(r_std), DIMENSION (kjpindex,nvm), INTENT(in)       :: gpp              !! Assimilation ((gC m^{-2}), total area)                , INTENT (inout)

     ! variables for netCDF preparation
     LOGICAL, INTENT(in)                                     :: james_netcdf_flag !! Flag that controls the netCDF output routine

     ! REAL(r_std), INTENT(in)                                 :: top_jnlvls
     ! CHARACTER(LEN=256), INTENT(in)                          :: jamescdfname           !! To store the name of the multilayer netCDF output file (seperate to the rest)

     REAL(r_std),DIMENSION(kjpindex,nvm,ncirc,nlevels_tot), INTENT(in) &
                                                             :: h_array_out  !! An output of h_array, to use in sechiba               
     REAL(r_std),DIMENSION(kjpindex,nvm,ncirc,nlevels_tot), INTENT(in) &
                                                             :: z_array_out  !! An output of h_array, to use in sechiba                             

     REAL(r_std), DIMENSION(nlevels_tot, kjpindex, nvm), INTENT(in) &
                                                             :: transpir_supply_column !! Supply of water for transpiration 

     REAL(r_std), DIMENSION (kjpindex,nvm), INTENT(in)       :: max_height_store

     REAL(r_std), DIMENSION (kjpindex), INTENT(in)           :: t2m_month_out      !! An output of t2m_month, from stomate season 

     REAL(r_std), DIMENSION(nlevels_tot,kjpindex,nvm), INTENT(in) &
                                                             :: laieff_isotrop             !! Effective LAI
     REAL(r_std),DIMENSION (kjpindex,nvm,nlevels_tot), INTENT(in) &     
                                                             :: profile_vbeta3
     REAL(r_std),DIMENSION (kjpindex,nvm,nlevels_tot), INTENT(in) &
                                                             :: profile_rveget

     !! 0.2 Output variables

     REAL(r_std), DIMENSION (kjpindex), INTENT(out)          :: vevapnu          !! Bare soil evaporation (mm day^{-1})            , INTENT (out) 
     REAL(r_std), DIMENSION (kjpindex), INTENT(out)          :: vevapsno         !! Snow evaporation (mm day^{-1})                 , INTENT (out)
     REAL(r_std), DIMENSION (kjpindex), INTENT(out)          :: vevapflo         !! Floodplains evaporation                        , INTENT (out) 
     REAL(r_std), DIMENSION (kjpindex,nvm), INTENT(out)      :: transpir         !! Transpiration (mm day^{-1})                    , INTENT (out)
     REAL(r_std), DIMENSION (kjpindex,nvm), INTENT(out)      :: transpot         !! Potential transpiration                        , INTENT (out)
     REAL(r_std), DIMENSION (kjpindex,nvm), INTENT(out)      :: vevapwet         !! Interception (mm day^{-1})                     , INTENT (out)
     REAL(r_std), DIMENSION (kjpindex), INTENT(out)          :: t2mdiag          !! JJJ Diagnostic temperature at 2m above ground (K)  , INTENT (out)
     REAL(r_std), INTENT(out)                                :: temp_surf_pres   !! temperature at surface (K) PRESENT STEP
     REAL(r_std), INTENT(out)                                :: temp_surf_next   !! temperature at surface (K) NEXT STEP
     REAL(r_std), INTENT(out)                                :: flux_ground_h    !! ground level sensible heat flux (W/m^2)
     REAL(r_std), INTENT(out)                                :: flux_ground_le   !! ground level latent heat flux (W/m^2)

     REAL(r_std), DIMENSION(kjpindex,jnlvls), INTENT(out)       :: flux_rn_grid               ! net radiationflux flux down the column (W/m^2) 
     REAL(r_std), DIMENSION(kjpindex,jnlvls), INTENT(out)       :: flux_h_grid                ! sensible heat flux down the column (W/m^2)
     REAL(r_std), DIMENSION(kjpindex,jnlvls), INTENT(out)       :: flux_le_grid               ! latent heat flux down the column (W/m^2)
     REAL(r_std), DIMENSION(kjpindex,jnlvls), INTENT(out)       :: u_speed_grid               ! wind_speed down the column (m/s)
     REAL(r_std), DIMENSION(kjpindex,jnlvls), INTENT(out)       :: q_atmos_pres_grid          ! atmospheric specific humididy down the column (kg/kg) PRESENT STEP
     REAL(r_std), DIMENSION(kjpindex,jnlvls), INTENT(out)       :: t_a_next_grid              ! t
     REAL(r_std), DIMENSION(kjpindex,jnlvls), INTENT(out)       :: q_a_next_grid
     REAL(r_std), DIMENSION (kjpindex), INTENT(out)            :: qsol_sat_new_out  !! New saturated surface air moisture (kg kg^{-1})
     REAL(r_std),DIMENSION (kjpindex), INTENT(out)             ::  qair_new


     !! 0.3 Modified variables
     REAL(r_std),DIMENSION(nlevels_tot), INTENT(inout)          :: Collim_Abs_Tot   !! collimated total absorption for a given level
     REAL(r_std),DIMENSION(nlevels_tot), INTENT(inout)          :: Collim_Alb_Tot   !! Collimated (direct) total albedo for a given level

     REAL(r_std), DIMENSION (kjpindex,nvm), INTENT(inout)    :: vbeta3           !! Vegetation resistance (-)
     REAL(r_std), DIMENSION (kjpindex), INTENT(inout)        :: evapot           !! Soil Potential Evaporation (mm day^{-1})              , INTENT (inout)
     REAL(r_std), DIMENSION (kjpindex), INTENT(inout)        :: evapot_corr      !! Soil Potential Evaporation Correction (mm day^{-1})   , INTENT (inout)
     REAL(r_std), DIMENSION (kjpindex), INTENT(inout)        :: temp_sol         !!      JJJ     Soil temperature (K)                     , INTENT (inout)
     REAL(r_std), DIMENSION (kjpindex), INTENT(inout)        :: qsurf            !! Surface specific humidity (kg kg^{-1})                , INTENT (inout)
     REAL(r_std), DIMENSION (kjpindex), INTENT(inout)        :: fluxsens         !!     ! JJJ     Sensible heat flux (W m^{-2})           , INTENT (inout)
     REAL(r_std), DIMENSION (kjpindex), INTENT(inout)        :: fluxlat          !!     ! JJJ      Latent heat flux (W m^{-2})            , INTENT (inout)
     REAL(r_std), DIMENSION (kjpindex), INTENT(inout)        :: netrad           !!     ! YC     Net radiation flux (W m^{-2})            , INTENT (inout)
     REAL(r_std), DIMENSION (kjpindex), INTENT(inout)        :: tsol_rad         !! Tsol_rad (W m^{-2})                                   , INTENT (inout)
     REAL(r_std), DIMENSION (kjpindex), INTENT(inout)        :: vevapp           !! Total of evaporation (mm day^{-1})                    , INTENT (inout)

     REAL(r_std), DIMENSION (kjpindex), INTENT(inout)        :: temp_sol_new     !!     ! JJJ     New soil temperature (K)                , INTENT (out)

     REAL(r_std), DIMENSION(kjpindex), INTENT(inout)         :: vbeta2sum
     REAL(r_std), DIMENSION(kjpindex), INTENT(inout)         :: vbeta3sum


     REAL(r_std), DIMENSION (kjpindex,jnlvls), INTENT(inout) :: temp_atmos_pres_grid    ! atmospheric temperature down the column (K) PRESENT STEP
     REAL(r_std), DIMENSION (kjpindex,jnlvls), INTENT(inout) :: temp_leaf_pres_grid     ! leaf temperature down the column (K) PRESENT STEP

     REAL(r_std),DIMENSION (kjpindex), INTENT(inout)         :: delta_temp_save     

     
     !! 0.4 Local variables
     INTEGER(i_std)                          :: i, j
     REAL(r_std), DIMENSION(jnlvls)          :: horiz_flux_t,  horiz_flux_q
     REAL(r_std)                             :: u_star                ! friction velocity (m/s)
     REAL(r_std), DIMENSION (0:jnlvls)       :: jrpad                  ! Plant Area Density, replacement for values above (as definitions between different
                                                                      !    parts of the model was becoming confusing)
                                                                      ! 'jrpad' is the total one-sided leaf area per unit of layer volume
     REAL(r_std), DIMENSION (jnlvls)         :: h_flux, le_flux
     REAL(r_std), DIMENSION (jnlvls)         :: u_speed               ! wind speed at specified canopy level (m/s)
     REAL(r_std)                             :: q_surf_next           ! specific humidity at surface (kg/kg) NEXT STEP
     REAL(r_std), DIMENSION (jnlvls)         :: temp_atmos_next       ! atmospheric temperature down the column (K) NEXT STEP
     REAL(r_std), DIMENSION (jnlvls)         :: temp_leaf_next        ! leaf temperature down the column (K) NEXT STEP
     REAL(r_std), DIMENSION (jnlvls)         :: q_leaf_pres           ! leaf specific humidity down the column (kg/kg) PRESENT STEP
     REAL(r_std), DIMENSION (jnlvls)         :: q_atmos_next          ! atmospheric specific humididy down the column (kg/kg) NEXT STEP
     REAL(r_std), DIMENSION (jnlvls)         :: q_leaf_next           ! leaf specific humidity down the column (kg/kg) NEXT STEP
     REAL(r_std), DIMENSION (jnlvls)         :: jalpha  
     REAL(r_std), DIMENSION (jnlvls)         :: jbeta 
     REAL(r_std), DIMENSION (jnlvls)         :: k_eddy                 ! eddy transport coefficient (m^2 s-1)
     REAL(r_std)                             :: k_eddy_surf            ! surface eddy transport coefficient (m^2 s-1)
     REAL(r_std), DIMENSION (jnlvls)         :: down_sw_tab, down_lw_tab
     REAL(r_std), DIMENSION (jnlvls)         :: big_r, big_r_prime
     REAL(r_std), DIMENSION (timesteps)      :: ustar_ac               ! input friction velocity above the canopy (m/s)
     REAL(r_std)                             :: layer_height
     REAL(r_std), DIMENSION(jnlvls)          :: delta_h                ! height of each level (m)
     REAL(r_std)                             :: delta_h_surf           ! height of surface level (m)
     REAL(r_std), DIMENSION (jnlvls)         :: delta_z                ! vertical difference between the middle of each box (m)
     REAL(r_std)                             :: delta_z_surf           ! vertical difference between the middle of the surface layer
                                                                       !    and the middle of the first layer (m)
     REAL(r_std)                             :: box_height_surf        ! height of bottom of surface layer box (m)
     REAL(r_std), DIMENSION(kjpindex)        :: jalpha_surf, jbeta_surf
     REAL(r_std), DIMENSION(jnlvls)          :: big_e, big_f, big_g
     REAL(r_std), DIMENSION(jnlvls)          :: a_t, b_t, c_t, d_t
     REAL(r_std), DIMENSION(jnlvls)          :: a_q, b_q, c_q, d_q
     REAL(r_std), DIMENSION(jnlvls)          :: jemissivity
     REAL(r_std), DIMENSION(jnlvls)          :: big_pad                ! plant area density at each level (m^2/m^3)
     REAL(r_std), DIMENSION(jnlvls)          :: eta_1, eta_2, eta_3, eta_4, jomega
     REAL(r_std)                             :: eta_1_surf, eta_2_surf
     REAL(r_std)                             :: eta_3_surf, eta_4_surf, eta_3_above
     REAL(r_std)                             :: eta_1_above, eta_4_above, eta_2_above           

     REAL(r_std), DIMENSION (kjpindex,jnlvls)     :: temp_leaf_next_grid     ! leaf temperature down the column (K) PRESENT STEP

     REAL(r_std)                                  :: delta_energy_surf
     REAL(r_std), DIMENSION (kjpindex,jnlvls)     :: delta_energy
     REAL(r_std), DIMENSION (jnlvls)              :: jtheta
     REAL(r_std)                                  :: r_sto_v_fac, big_r_h2o_fac
     REAL(r_std)                                  :: big_r_heat_fac          !
     REAL(r_std), DIMENSION(jnlvls)               :: jomega_sw
     REAL(r_std)                                  :: jomega_surf
     REAL(r_std)                                  :: jemissivity_surf
     REAL(r_std)                                  :: jomega_sw_surf         ! emissivity of surface layer
     REAL(r_std)                                  :: surf_lev
     REAL(r_std), DIMENSION(jnlvls+1)               :: top_lev    
     REAL(r_std), DIMENSION(kjpindex)             :: q_sat_surf
     REAL(r_std)                                  :: zikt, zikq, speed
     REAL(r_std)                                  :: delta_temp             ! change in temperature over the time step (Kelvins)
     REAL(r_std), DIMENSION(kjpindex)             :: jpdqsold
     REAL(r_std),DIMENSION (kjpindex)             :: epot_air_new
     CHARACTER(LEN=256)                           :: var_name               !! To store variables names for I/O
     CHARACTER(LEN=256)                           :: jamescdfname           !! To store the name of the multilayer netCDF output file (seperate to the rest)
     REAL(r_std), DIMENSION (jnlvls)              :: t_l_pres, t_l_next
     REAL, DIMENSION (0:jnlvls+1, 0:jnlvls+1)     :: ngu_alpha
     REAL(r_std), DIMENSION(kjpindex)             :: dev_qsol
     REAL(r_std), DIMENSION(kjpindex)             :: gs_effective   !! 'effective' stomatal conductance
     REAL(r_std), DIMENSION(kjpindex)             :: vbeta3_effective

     INTEGER(i_std)                               :: ji, jv
     INTEGER(i_std)                               :: ier
     REAL(r_std), DIMENSION (kjpindex)            :: grad_qsat
     REAL(r_std)                                  :: correction
     REAL(r_std)                                  :: qc                 !! Surface drag coefficient (??)
     LOGICAL, DIMENSION (kjpindex)                :: warning_correction
     REAL(r_std), DIMENSION (kjpindex)            :: E_supply_sum

     REAL(r_std)                                  :: jr_hcm             !! heat capacity multiplier (for testing)
     REAL(r_std)                                  :: matt_surf_absorbed
     REAL(r_std), DIMENSION(jnlvls)               :: matt_layer_absorbed

     REAL(r_std)                                  :: pft_running_total
     REAL(r_std), DIMENSION(0:jnlvls)               :: pad_deltah_running_total
     REAL(r_std), DIMENSION(jnlvls+1)             :: box_height_running_total


    ! each potential input variable is listed separately - can't see a good way that this 
    !    can be avoided in Fortran

    REAL(r_std)          :: pad_lev_1, pad_lev_2, pad_lev_3, pad_lev_4 
    REAL(r_std)          :: pad_lev_5, pad_lev_6, pad_lev_7, pad_lev_8 
    REAL(r_std)          :: pad_lev_9, pad_lev_10, pad_lev_11, pad_lev_12 
    REAL(r_std)          :: pad_lev_13, pad_lev_14, pad_lev_15, pad_lev_16 
    REAL(r_std)          :: pad_lev_17, pad_lev_18, pad_lev_19, pad_lev_20
    REAL(r_std)          :: pad_lev_21, pad_lev_22, pad_lev_23, pad_lev_24 
    REAL(r_std)          :: pad_lev_25, pad_lev_26, pad_lev_27, pad_lev_28 
    REAL(r_std)          :: pad_lev_29, pad_lev_30, pad_lev_31, pad_lev_32 
    REAL(r_std)          :: pad_lev_33, pad_lev_34, pad_lev_35, pad_lev_36 
    REAL(r_std)          :: pad_lev_37, pad_lev_38, pad_lev_39, pad_lev_40
    REAL(r_std)          :: pad_lev_41, pad_lev_42, pad_lev_43, pad_lev_44 
    REAL(r_std)          :: pad_lev_45, pad_lev_46, pad_lev_47, pad_lev_48 
    REAL(r_std)          :: pad_lev_49, pad_lev_50    

    REAL(r_std)          :: height_lev_1, height_lev_2, height_lev_3, height_lev_4 
    REAL(r_std)          :: height_lev_5, height_lev_6, height_lev_7, height_lev_8 
    REAL(r_std)          :: height_lev_9, height_lev_10, height_lev_11, height_lev_12 
    REAL(r_std)          :: height_lev_13, height_lev_14, height_lev_15, height_lev_16 
    REAL(r_std)          :: height_lev_17, height_lev_18, height_lev_19, height_lev_20
    REAL(r_std)          :: height_lev_21, height_lev_22, height_lev_23, height_lev_24 
    REAL(r_std)          :: height_lev_25, height_lev_26, height_lev_27, height_lev_28 
    REAL(r_std)          :: height_lev_29, height_lev_30, height_lev_31, height_lev_32 
    REAL(r_std)          :: height_lev_33, height_lev_34, height_lev_35, height_lev_36 
    REAL(r_std)          :: height_lev_37, height_lev_38, height_lev_39, height_lev_40
    REAL(r_std)          :: height_lev_41, height_lev_42, height_lev_43, height_lev_44 
    REAL(r_std)          :: height_lev_45, height_lev_46, height_lev_47, height_lev_48 
    REAL(r_std)          :: height_lev_49, height_lev_50

    REAL(r_std)          :: top_jnlvls
    
    CHARACTER(len=256)  :: jname
    CHARACTER(len=256)  :: jname2
    CHARACTER(len=256)  :: format_string


    REAL(r_std), DIMENSION (0:jnlvls+1)                  :: ngu_bign

    REAL(r_std), DIMENSION (0:jnlvls+1)                  :: ngu_alpha_insert

     
    ! Parameters set to free for adjust site level calibation (25032015) 
    LOGICAL                                      :: WC02         ! logical paramter for applying (Wohlfahrt & Cernusca 2002) Dynamic drag 
    REAL(r_std)                                  :: c_drag                  ! in canopy drag coefficient for sub-canopy wind profile 
    REAL(r_std)                                  :: a_1                     ! a1 to a5:parameters for the sub-canopy wind profile  
    REAL(r_std)                                  :: a_2                     ! the default values were set as the 
    REAL(r_std)                                  :: a_3                     ! a1= 0.065; a2=0.001; a3=0.434; a4= -0.751; a5=0.071 
    REAL(r_std)                                  :: a_4  
    REAL(r_std)                                  :: a_5

    REAL(r_std), DIMENSION(jnlvls)               :: flux_netrad             ! net radiation flux for each layer
    REAL(r_std), DIMENSION(jnlvls)               :: flux_netsw              ! net radiation contributed from short-wave
    REAL(r_std), DIMENSION(jnlvls)               :: flux_netlw              ! net radiation contributed from long-wave
 
!_ =======================================================================================

    IF (l_first_jenerbil) THEN

        l_first_jenerbil=.FALSE.

        ALLOCATE (pad_deltah(0:jnlvls),stat=ier)
        IF (ier.NE.0) THEN
            WRITE (numout,*) ' error in pad_deltah allocation. '
        END IF
        
        ALLOCATE (box_height(jnlvls+1),stat=ier)
        IF (ier.NE.0) THEN
            WRITE (numout,*) ' error in box_height allocation.'
        END IF

    END IF ! (l_first_jenerbil)

    ! set a default value of top_jnlvls
    top_jnlvls = 1.0d0

    ! set a default value of jamescdfname
    jamescdfname = 'jamescdf.nc'
    
    ! pad_deltah(0) = 0.0d0
    pad_deltah(0) = 0.001d0

    IF (ld_jameswrite) THEN
        WRITE (numout,*) '240215 start of enerbil, tair is: ', tair
        WRITE (numout,*) '20150327 lwdown in jdriver:', lwdown(:)
    END IF ! (ld_jameswrite) THEN


    IF (jnlvls .eq. 1) THEN
        ! no read-in profile for single layer case

        ! set a default value of box_height
        box_height(:) = 1.0d0        
        
        ! set a default value of pad_deltah
        pad_deltah(:) = 1.0d0

        jr_hcm = 1.0_r_std

        k_eddy_ceil = 1.0_r_std
        k_eddy_slope = 1.0_r_std
        k_eddy_ustar = 0.001_r_std
        
    ELSE
        ! set a default value of hd_big_r_prime
        jr_hcm = 1.0_r_std
        !
        !Config Key   = JR_HCM
        !Config If    = JENERBIL
        !Config Desc  = heat capacity multiplier, for testing
        !Config Def   = one_day??
        !Config Help  = heat capacity multiplier, for testing 
        !Config         Default value is: 1.0
        !Config Units = [-]
        !
        CALL getin_p('JR_HCM', jr_hcm)

        IF (ld_jameswrite) THEN
            WRITE (numout,*) '170214 jr_hcm is: ', jr_hcm
        END IF !(ld_jameswrite)

        

        ! These variables will require the configuration infrastructure
        !
        !Config Key   = top_jnlvls
        !Config If    = JENERBIL
        !Config Desc  = Value of the boundary resistance factor
        !Config Def   = one_day??
        !Config Help  = This is the value of the boundary resistance factor. 
        !Config         Default value is: 1.0
        !Config Units = [-]
        !
        ! set a default value of top_jnlvls
        top_jnlvls = 1.0d0

        CALL getin_p('TOP_jnlvls', top_jnlvls)




        ! set a default value of jamescdfname
        jamescdfname = 'jamescdf.nc'

        ! These variables will require the configuration infrastructure
        !
        !Config Key   = jamescdfname
        !Config If    = JAMESCDFNAME
        !Config Desc  = Name of the output CDF file
        !Config Def   = one_day??
        !Config Help  = This values gives the name of the output CDF file. 
        !Config         Default value is: jamescdf.nc
        !Config Units = [-]
        !    
        CALL getin_p('JAMESCDFNAME', jamescdfname)



        ! read in both the plant area density and the height of each level      


       !Config Key   = K_EDDY_CEIL
       !Config Desc  = 
       !Config If    = 
       !Config Def   = 1.0
       !Config Help  = 
       !Config Units = [-]
       ! set a default value of k_eddy_ceil
       k_eddy_ceil = 1.0_r_std
       CALL getin_p('K_EDDY_CEIL', k_eddy_ceil)

       !Config Key   = K_EDDY_SLOPE
       !Config Desc  = 
       !Config If    = 
       !Config Def   = 1.0
       !Config Help  = 
       !Config Units = [-]
       ! set a default value of k_eddy_ceil
       k_eddy_slope = 10.0_r_std
       CALL getin_p('K_EDDY_SLOPE', k_eddy_slope)

       !Config Key   = K_EDDY_USTAR
       !Config Desc  = 
       !Config If    = 
       !Config Def   = 0.1
       !Config Help  = 
       !Config Units = [-]
       ! set a default value of k_eddy_ceil
       k_eddy_ustar = 0.1_r_std
       CALL getin_p('K_EDDY_USTAR', k_eddy_ustar)


       !Config Key   = KS_TUNE
       !Config Desc  = 
       !Config If    = 
       !Config Def   = 1.0
       !Config Help  = 
       !Config Units = [-]
       ! set a default value of k_eddy_ceil
       ks_tune = 1.0_r_std
       CALL getin_p('KS_TUNE', ks_tune)

       !Config Key   = KS_SLOPE
       !Config Desc  = 
       !Config If    = 
       !Config Def   = 20.0
       !Config Help  = 
       !Config Units = [-]
       ! set a default value of 
       ks_slope = 15.0_r_std
       CALL getin_p('KS_SLOPE', ks_slope)

       !Config Key   = KS_VEGET
       !Config Desc  = 
       !Config If    = 
       !Config Def   = 0.35
       !Config Help  = 
       !Config Units = [-]
       ! set a default value of k_eddy_ceil
       ks_veget = 0.35_r_std
       CALL getin_p('KS_VEGET', ks_veget) 

       ! These variables will require the configuration infrastructure
       !
       !Config Key   = SR_FAC
       !Config If    = JENERBIL
       !Config Desc  = Value of the stomatal resistance factor
       !Config Def   = one_day??
       !Config Help  = This is the value of the stomatal resistance factor. 
       !Config         Default value is: 1.0
       !Config Units = [-]
       ! set a default value of sr_fac
       sr_fac = 1.0_r_std
       CALL getin_p('SR_FAC', sr_fac)

       ! These variables will require the configuration infrastructure
       !
       !Config Key   = BR_FAC
       !Config If    = JENERBIL
       !Config Desc  = Value of the boundary resistance factor
       !Config Def   = one_day??
       !Config Help  = This is the value of the boundary resistance factor. 
       !Config         Default value is: 1.0
       !Config Units = [-]
       ! set a default value of br_fac
       br_fac = 1.0_r_std
       CALL getin_p('BR_FAC', br_fac)



       !Config Key   =  WC02
       !Config If    =  WC02
       !Config Desc  =  this is the logic variable for using vertical dynamic drag coefficient (Wohlfahrt and Cernusca, 2002)
       !Config Def   =  Default is FALSE
       !Config Help  =  
       !Config          Default value is: FALSE
       !Config Units = [-]
       !
       WC02 = .FALSE.
       CALL getin_p('WC02', WC02)  
      
       !Config Key  = a_1 to a_5 
       !Config IF   = WC02
       !Config Desc = Sub-canopy wind spped parametrization 
       !Config Defi = Default is FALSE 
       !Config Help = 
       !Config        Default value is: FALSE
       !Config Units = [-]
        a_1 =  6.140d0
        a_2 =  0.001d0
        a_3 =  0.434d0
        a_4 = -0.751d0
        a_5 =  0.071d0
        CALL getin_p('A_1',a_1)
        CALL getin_p('A_2',a_2)
        CALL getin_p('A_3',a_3)
        CALL getin_p('A_4',a_4)
        CALL getin_p('A_5',a_5)

       big_r=un    

    ENDIF  ! jnlvls  e.q. 1.0



    ! set a default value of top_jnlvls
    top_jnlvls = 1.0d0  
    ! pad_deltah(0) = 0.0d0
    pad_deltah(0) = 0.001d0

    ! for cases in which we exit the loop if hydrol_flag2 is false
    epot_air_new = epot_air
    temp_sol_new = temp_sol

    ! initialise variables that are required for multilayer scheme

    u_h = 0.0d0
    q_leaf_pres(:) = 0.0d0                
    temp_atmos_next(:) = 0.0d0; temp_leaf_next(:) = 0.0d0     
    q_atmos_next(:) = 0.0d0; q_leaf_next(:) = 0.0d0       
    jalpha = 0.0d0; jbeta = 0.0d0
    temp_surf_next = 0.0d0; q_surf_next = 0.0d0
    jemissivity(:) = 0.96d0
    r_sto_v_fac = 1.0d0
    jtheta = 4.18d3                      ! specific heat capacity of vegetation (assumed equal to water J/(kg K))
    rho_veg = 1000.0d0                   ! assume that density of vegetation is as for pure water for now (1000 kg/m^3)
    leaf_tks = 0.0017d0                  ! leaf thickness (derived from TRY database) 


    jamescdfname = '/home/orchidee03/jryder/'//jamescdfname

    ! for cases in which we exit the loop if hydrol_flag2 is false
    epot_air_new = epot_air
    temp_sol_new = temp_sol

    u_h = 0.0d0
    q_leaf_pres(:) = 0.0d0                
    temp_atmos_next(:) = 0.0d0; temp_leaf_next(:) = 0.0d0     
    q_atmos_next(:) = 0.0d0; q_leaf_next(:) = 0.0d0       
    jalpha = 0.0d0; jbeta = 0.0d0
    temp_surf_next = 0.0d0; q_surf_next = 0.0d0
    jemissivity(:) = 0.96d0 !0.96d0

    r_sto_v_fac = 1.0d0
    big_r_h2o_fac = 1.0d0
    big_r_heat_fac = 1.0d0


   !jtheta = jtheta * 0.5                 ! see reference of  http://link.springer.com/article/10.1007%2Fs10765-010-0877-7
                                          ! M. S. Jayalakshmy and  J. Philip  
                                          ! Thermophysical Properties of Plant Leaves and Their Influence on the Environment Temperature,
                                          ! International Journal of Thermophysics, 2010, Volume 31, Issue 11-12, pp 2295-2304 

    rho_veg = 1000.0d0                    ! assume that density of vegetation is as for pure water for now (1000 kg/m^3)
    ! leaf_tks = 0.001d0                  ! leaf thickness (assumed to be 1mm) 
    leaf_tks = 0.00014d0                  ! leaf thickness (try thinner leaf) 

    ! 110914 there are two ways to arrive at the MLA (mass per unit leaf area, kg/m^2), 
    ! which we then multiply by the LAD (leaf area density, m^2/m^3) to arrive at the 
    ! vegetation matter per m in height

    ! 1) as in Nobel, Park S. (2005) 'Physiochemical and Environmental Plant Physiology' 
    ! Edition 3, published Elsevier, chapter 7.1, p. 309
    !    - assume leaf has a specific heat of water (4.19 kJ kg^-1 degC^-1 at 20 degC)
    !    - assume leaf has thickness 300micrometres
    !    - assume leaf has density of 700kg/m^3 (water is 1000kg/m^3 but leaves are often 30% air)
    !    - this gives (300 x 10^-6 m)(700 kg m^-3) = 0.21 kg m^-2

    ! 2) Using LMA (leaf per unit mass area, m^2/kg) from the TRY database
    !    - for Eucalyptus delegatensis we have 155 cm^2/g ---> 0.155 m^2/g. 
    !      This is dried weight of the leaf.
    !    - 100 gC/m^2 soil
    !    - 100 * 2 * 10 = 2000 g C / m^2 soil
    !    Dry weight / Fresh weight = 8 / SLA ---> g FW / m^2 (here we are assuming a dry 
    !                                               weight to fresh weight ratio of 1:9)
    !                              = 8 / 0.155
    !                              = 516 g FW / m^2
    !                              = 0.516 kg / m^2


    ! jjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjj


     IF (ld_jameswrite) THEN  
         WRITE (numout, *) '100315 SUM(laieff_isotrop(1:jnlvls_canopy,test_grid,test_pft)) is: ', SUM(laieff_isotrop(1:jnlvls_canopy,test_grid,test_pft)) 
     END IF !(ld_jameswrite)


     IF (jnlvls .ne. 1) THEN
        IF (jnlvls .ne. (jnlvls_under + jnlvls_canopy + jnlvls_over)) THEN
            WRITE (numout, *) 'number of levels in canopy column imbalance'
            STOP
        END IF
     END IF       

     IF (ld_jameswrite) THEN
         WRITE (numout, *) '020315a in jdriver, SUM(laieff_isotrop(1:jnlvls_canopy,test_grid,test_pft)) is: ', SUM(laieff_isotrop(1:jnlvls_canopy,test_grid,test_pft))
     END IF !(ld_jameswrite)

     IF (control%ok_new_enerbil_nextstep ) THEN 
       
          pft_running_total = 0.0d0
          box_height_running_total = 0.0d0
          pad_deltah_running_total = 0.0d0
       
          DO ji = 1,1
              DO jv = 1,nvm
                IF (veget_max(ji,jv) .GT. zero .AND. veget(ji,jv) .GT. zero) THEN 

            IF ((SUM(laieff_isotrop(1:jnlvls_canopy, ji, jv)) .ne. 0.0) .AND. (SUM(z_array_out(ji, jv, 1, :)) .ne. 0.0)) THEN

                      ! update plant area density
                      pad_deltah(:) = 0.001d0
                      ! n.b. the first level of lai_eff_isotrop represents the understorey.
                      ! We set this equal to the min!
                      DO i = (jnlvls_under + 1) , (jnlvls_under + jnlvls_canopy)
                          pad_deltah(i) = laieff_isotrop(i-jnlvls_under+1, ji, jv) * cos(pi/3.0d0)
                          IF (pad_deltah(i) .lt. 1.0d-3) THEN
                              pad_deltah(i) = 1.0d-3
                          END IF
                          IF (ld_jameswrite) THEN
                               WRITE (numout, *) '160315 pad_deltah level: ', i,' is: ', pad_deltah(i)
                          END IF ! (ld_jameswrite)
                       END DO ! i = (jnlvls_under + 1) , (jnlvls_under + jnlvls_canopy)

                       IF (ld_jameswrite) THEN
                           WRITE (numout, *) '080515 in calculate box_height section'
                           WRITE (numout, *) '080515y max_height_store(1,4) is: ', max_height_store(ji, jv)
                           WRITE (numout, *) '080515y z_array_out(1,4,1,:) is: ', z_array_out(ji, jv, 1, :)
                       END IF ! (ld_jameswrite) THEN

                       DO i = 1, jnlvls_under - 1
                           box_height(i) = (i) * (z_array_out(ji, jv, 1, 2) - z_array_out(ji, jv, 1, 1)) / (jnlvls_under)
                       END DO ! 1, jnlvls_under - 1 

                       box_height(jnlvls_under) = z_array_out(ji, jv, 1, 2)



                       DO i = (jnlvls_under + 1), (jnlvls_under + jnlvls_canopy - 1)
                           box_height(i) = z_array_out(ji, jv, 1, i - jnlvls_under + 2)
                       END DO ! i = (jnlvls_under + 1), (jnlvls_under + jnlvls_canopy - 1)


                       box_height(jnlvls_under + jnlvls_canopy) = max_height_store(ji, jv)

                       DO i = (jnlvls_under + jnlvls_canopy + 1), (jnlvls_under + jnlvls_canopy + jnlvls_over - 1)
                           box_height(i) = max_height_store(ji, jv) + &
                                           & (( i - (jnlvls_under + jnlvls_canopy) ) * (49.0d0 - max_height_store(ji, jv) )/ jnlvls_over )
                       END DO ! i = (jnlvls_under + jnlvls_canopy + 1), (jnlvls_under + jnlvls_canopy + jnlvls_over)


                       box_height(jnlvls_under + jnlvls_canopy + jnlvls_over) = 49.0d0
                       box_height(jnlvls_under + jnlvls_canopy + jnlvls_over+1) = 50.0d0

                       jrpad(:) = pad_deltah(:)

                       IF (ld_jameswrite) THEN
                           DO i = 1, jnlvls
                               WRITE(numout, *) 'lev: ', i, ' b_h: ', box_height(i), ' p_d: ', pad_deltah(i)
                           END DO ! i = 1, jnlvls
                       END IF ! (ld_jameswrite) THEN


            END IF ! (SUM(laieff_isotrop(1:jnlvls_canopy, ji, jv)) .ne. 0.0)


                  pft_running_total = pft_running_total + 1
                  box_height_running_total = box_height_running_total + box_height
                  pad_deltah_running_total = pad_deltah_running_total + pad_deltah

                END IF !(veget_max(ji,jv) .GT. zero .AND. veget(ji,jv) .GT. zero) THEN 

               END DO ! jv = 2, nvm
          END DO ! ji = 1, kjpindex

          IF (ld_jameswrite) THEN
              WRITE(numout, *) '100515 pft_running_total is: ', pft_running_total
              WRITE(numout, *) '100515 box_height_running_total(:) is: ', box_height_running_total(:)
              WRITE(numout, *) '100515 pad_deltah_running_total(:) is: ', pad_deltah_running_total(:)
          END IF ! (ld_jameswrite) THEN

          box_height = box_height_running_total / pft_running_total
          pad_deltah = pad_deltah_running_total / pft_running_total

          box_height_old = box_height

          IF (l_first_enerbil_column) THEN
              l_first_enerbil_column = .FALSE.
              IF (ld_jameswrite) WRITE(numout, *) '020315 not calling reallocate_column'
          ELSE
              IF (ld_jameswrite) WRITE(numout, *) '020315 calling reallocate_column'
              CALL reallocate_column(box_height_old, box_height,  &
                  &  temp_atmos_pres_grid, q_atmos_pres_grid, temp_leaf_pres_grid, &
                  &  jtheta, rho_veg, leaf_tks, rau, pad_deltah, kjpindex)
          ENDIF

          box_height_old = box_height

     END IF ! (control%ok_new_enerbil_nextstep ) THEN 


     a_t = 0; b_t = 0; c_t = 0; d_t = 0
     a_q = 0; b_q = 0; c_q = 0; d_q = 0

     ! top_lev(:) = box_height(:)

     surf_lev = 0.0d0

     box_height_surf = 0.0d0 ! infinitesimal surface layer
     delta_h_surf = 0.0d0    ! infinitesimal surface layer
    
     delta_z_surf = 0.5d0

     delta_z(:) = 1.0d0 

     delta_h(:) = 1.0d0


     speed = MAX(min_wind, SQRT (u(1)*u(1) + v(1)*v(1)))

     E_supply_sum(:) = 0.0d0
     E_supply_tot(:) = 0.0d0

     a_t = 0; b_t = 0; c_t = 0; d_t = 0
     a_q = 0; b_q = 0; c_q = 0; d_q = 0

    top_lev(:) = box_height(:)
    surf_lev = 0.0d0
    box_height_surf = 0.0d0 ! infinitesimal surface layer
    delta_h_surf = 0.0d0    ! infinitesimal surface layer
    delta_z_surf = 0.5d0
    delta_z(:) = 1.0d0 
    delta_h(:) = 1.0d0


    ! initialise variables that are required for the enerbil to hydrol interface
    E_supply_sum(:) = 0.0d0
    E_supply_tot(:) = 0.0d0


    ! calculate wind speed based on input velocities
    speed = MAX(min_wind, SQRT (u(1)*u(1) + v(1)*v(1)))


    ! collate E_supply by grid square - this is necessary as the energy budget is calculated as a combination of all PFTs
    !    within one grid square, whereas the hydrological architecture is calculated per individual PFT, per grid square.
    IF (hydrol_flag) THEN
       DO ji = 1, kjpindex
          DO jv = 1, nvm
              IF (hydrol_flag3(ji,jv)) THEN
                 E_supply_sum(ji) = E_supply_sum(ji) + E_supply(ji, jv) / veget_max(ji,jv)
              ELSE
              END IF
          END DO ! jv = 1, nvm
       END DO ! ji = 1, kjpindex
       E_supply_tot(:) = E_supply_sum(:)
    ELSE 
       ! do nothing as the hydrol_flag has not been set (within sechiba)
    END IF


    ! <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>

    ! as per the original scheme
    CALL jenerbil_begin (kjpindex, temp_sol, lwdown, swnet, pb, &
                         & psold, qsol_sat, pdqsold, netrad, emis, &
                         & dev_qsol)

    ! calculates the in-canopy transport profile
    ! IF (jnlvls .eq. 1) THEN
        ! no transport profile for single layer case
    ! ELSE
        CALL jprofile (pad_deltah, layer_height, ustar_ac, delta_z, &
                    & delta_h, box_height, k_eddy, delta_h_surf, delta_z_surf, &
                    & box_height_surf, u_star, speed, u_speed, big_pad, &
                    & k_eddy_ceil,k_eddy_slope,k_eddy_ustar, k_eddy_surf, surf_lev, top_lev, swnet(1), jrpad, &
                    & a_1, a_2, a_3, a_4, a_5, WC02)

    ! END IF !(jnlvls .eq. 1)    

    IF (jnlvls .eq. 1) THEN
       ! no radiation profile for single layer case
       eta_1(:)=zero
       eta_2(:)=zero
       eta_3(:)=zero
       eta_4(:)=zero
       jomega(:)=zero
       jomega_sw(:)=zero
       jemissivity(:)=zero
       jemissivity_surf=zero
       eta_1_surf=zero
       eta_2_surf=zero
       eta_3_surf=zero
       eta_4_surf=zero
       ngu_alpha(:,:)=zero
       jomega_surf=zero
       jomega_sw_surf=zero
       eta_1_above=zero
       eta_2_above=zero
       eta_3_above=zero
       eta_4_above=zero
    ELSE
        CALL calc_jradiation_scheme (eta_1, eta_2, eta_3, eta_4, jomega, &
                & jemissivity, temp_leaf_pres_grid, temp_surf_pres, &
                & pad_deltah, jomega_sw, eta_1_surf, eta_2_surf, &
                & eta_3_surf, eta_4_surf, jemissivity_surf, &
                & jomega_surf, jomega_sw_surf, & 
                & eta_3_above, eta_1_above, eta_4_above, &
                & eta_2_above, ngu_alpha, &
                & ngu_alpha_insert, kjpindex, &
                & big_k_lw, big_k_sw, delta_h)
    END IF !(jnlvls .eq. 1)    

    ! calculates the stomatal resistance
    ! IF (jnlvls .eq. 1) THEN
        ! use assigned beta coefficients for single layer case
    ! ELSE
        CALL calc_stomatal_resistance (pad_deltah, swnet, lwdown, pb, & 
                & down_sw_tab, down_lw_tab, big_r_prime, &
                & temp_atmos_pres_grid, q_atmos_pres_grid, big_r, &
                & u_speed, r_sto_v_fac, big_r_h2o_fac, rau, kjpindex, &
                & sr_fac, hd_big_r_prime, q_cdrag, &
                & delta_h, jrpad, profile_rveget)
    ! END IF !(jnlvls .eq. 1)        

    ! calculates the boundary layer resistance
    ! IF (jnlvls .eq. 1) THEN
        ! use assigned beta coefficients for single layer case
    ! ELSE
        CALL calc_boundarylayer_resistance (big_r, u_speed, temp_atmos_pres_grid, &
                & big_r_heat_fac, kjpindex, br_fac, hd_big_r, q_cdrag, pad_deltah, delta_h, jrpad) 
    ! END IF !(jnlvls .eq. 1)        

    ! calculates the jalpha and jbeta coefficients (only required in the multilayer case)
    CALL calc_jalpha_and_jbeta (temp_atmos_pres_grid, temp_leaf_pres_grid, &
                    & jalpha, jbeta, jalpha_surf, jbeta_surf, & 
                    & temp_surf_pres, q_sat_surf, &
                    & kjpindex, pb, temp_sol, &
                    & jpdqsold)

    ! the central energy budget calculation routine

    CALL jcalc (pad_deltah, layer_height, big_r, big_r_prime, delta_z, &
                 & k_eddy, swnet, swdown, lwdown, &
                 & temp_atmos_pres_grid, q_atmos_pres_grid, & 
                 & a_t, b_t, c_t, d_t, & 
                 & a_q, b_q, c_q, d_q, & 
                 & big_e, big_f, big_g, &
                 & jalpha, jbeta, temp_leaf_pres_grid, delta_h, &
                 & eta_1, eta_2, eta_3, eta_4, &
                 & k_eddy_surf, delta_z_surf, jtheta, & 
                 & jalpha_surf, jbeta_surf, & 
                 & rau, temp_surf_pres, &
                 & soilcap, soilflx, speed, & 
                 & q_cdrag, &
                 & kjpindex, &
                 & flux_ground_h, flux_ground_le, flux_h_grid, flux_le_grid, &
                 & u_speed, t_a_next_grid, q_a_next_grid, & 
                 & temp_surf_next, &
                 & eta_1_surf, eta_2_surf, eta_3_surf, eta_4_surf, &
                 & temp_leaf_next_grid, &
                 & epot_air_new, delta_temp, &
                 & horiz_flux_t, horiz_flux_q, &
                 & ngu_bign, ngu_alpha, ngu_alpha_insert, qair_new, qair, &
                 & emis, temp_sol_new, vbeta1, vbeta4, vbeta5, vbeta2, &
                 & vbeta3, vbeta3pot, vbeta, fluxsens, fluxlat, netrad, &
                 & petAcoef, petBcoef, peqAcoef, peqBcoef, &
                 & u, v, dtradia, &
                 & E_supply, E_supply_sum, E_supply_tot, hydrol_flag, hydrol_flag2, &
                 & hydrol_flag3, &
                 & vbeta2sum, vbeta3sum, veget_max, veget, &
                 & delta_temp_save, james_step_count, &
                 & Collim_Abs_Tot, Collim_Alb_Tot, matt_surf_absorbed, matt_layer_absorbed, &
                 & transpir_supply_column, pb, profile_vbeta3, profile_rveget, u_speed_grid, &
                 & flux_netrad, flux_netsw, flux_netlw, flux_rn_grid, &
                 & ks_tune, ks_slope, ks_veget, t2m_month_out )



    ! call the modified potential temperature subroutine

    CALL jenerbil_pottemp (kjpindex, dtradia, zlev, emis, &
       & epot_air, petAcoef, petBcoef, qair, peqAcoef, peqBcoef, &
       & soilflx, rau, u, v, q_cdrag, &
       & vbeta, valpha, vbeta1, vbeta5, soilcap, lwdown, swnet, &
       & q_sol_pot, temp_sol_pot)




    ! call the modified flux calculation subroutine 
    CALL jenerbil_flux (kjpindex, dtradia, emis, temp_sol, rau, u, v, &
        & q_cdrag, vbeta, valpha, vbeta1, vbeta5, &
        & qair_new, epot_air_new, psnew, qsurf, &
        & fluxsens , fluxlat , fluxsubli, vevapp, &
        & temp_sol_new, lwdown, swnet, &
        & lwup, lwnet, pb, tsol_rad, netrad, &
        & evapot, evapot_corr)

    ! Diagnoses the values for evaporation and transpiration: vevapsno 
    !(snow evaporation), vevapnu 
    ! (bare soil evaporation), transpir (transpiration), gpp (assimilation) 
    ! and vevapwet (interception)
    CALL jenerbil_evapveg (kjpindex, dtradia, vbeta1, vbeta2, vbeta3, &
                 & vbeta3pot, vbeta4, vbeta5, &
                 & rau, u, v, q_cdrag, &
!!$              & cimean, ccanopy, rau, u, v, q_cdrag,     &
                 & qair_new, humrel, &
                 & vevapsno, vevapnu, vevapflo, vevapwet, &
                 & transpir, transpot, evapot)

     CALL enerbil_t2mdiag (kjpindex, temp_air, t2mdiag)

     !+++TEMP+++
     IF(ld_enerbil)THEN
        WRITE (numout,*) 'before q_leaf_next calculation'
        WRITE (numout,*) 'jalpha(:) is: ', jalpha(:)
        WRITE (numout,*) 'jbeta(:) is: ', jbeta(:)
        WRITE (numout,*) 'temp_leaf_next(:) is: ', temp_leaf_next(:)
     ENDIF
     !++++++++++

     q_leaf_next(:) = jalpha(:) * temp_leaf_next(:) + jbeta(:)

     ! surface layer check
     delta_energy_surf = (temp_surf_next - temp_surf_pres) * rho_soil * soilcap(1)

     ! outputs the multilayer components to a seperate netCDF file - call removed for now
     IF (jnlvls .eq. 1) THEN
        ! rely on history file output single layer case
     ELSE
        CALL jnetcdf(james_netcdf_flag, t_a_next_grid(1,:), q_a_next_grid(1,:), &
                 & temp_leaf_next_grid(1,:), q_leaf_next, &
                 & flux_h_grid(1,:), flux_le_grid(1,:), flux_ground_h, flux_ground_le, &
                 & k_eddy, u_speed, temp_surf_next, &
                 & eta_1, eta_2, eta_3, eta_4, jomega, &
                 & eta_1_surf, eta_2_surf, eta_3_surf, eta_4_surf, &
                 & lwdown(1), swnet(1), &
                 & temp_atmos_pres_grid(1,:), q_atmos_pres_grid(1,:), b_t, b_q, soilflx(1), &
                 & jomega_surf, jemissivity_surf, jomega_sw_surf, jemissivity, &
                 & jomega_sw, temp_leaf_pres_grid(1,:), temp_surf_pres, &
                 & big_r, big_r_prime, horiz_flux_t,  horiz_flux_q, &
                 & soilcap(1), eta_3_above, eta_1_above, eta_4_above, &
                 & eta_2_above, pad_deltah, jamescdfname, &
                 & ngu_bign, ngu_alpha(1,:), ngu_alpha_insert, qsol_sat_new(1), kjit, &
                 & matt_surf_absorbed, matt_layer_absorbed, box_height)                 
     END IF !(jnlvls .eq. 1)        
 
     ! Write to history files
!!$     CALL histwrite_p(hist_id, 'evapot', kjit, evapot, kjpindex, index)
!!$     CALL histwrite_p(hist_id, 'evapot_corr', kjit, evapot_corr, kjpindex, index)
!!$     CALL histwrite_p(hist2_id, 'transpot', kjit, transpot, kjpindex, index)

     ! prepare for the next timestep
     temp_atmos_pres_grid(:,:) = t_a_next_grid(:,:)
     q_atmos_pres_grid(:,:) = q_a_next_grid(:,:)
     temp_leaf_pres_grid(:,:) = temp_leaf_next_grid(:,:)

     temp_surf_pres = temp_surf_next

     qsol_sat_new_out(:) = qsol_sat_new
          
 END SUBROUTINE jdriver

 
 ! ---------------------------------------------------------------------------------------


 !!  ===========================================================================================================================
 !! SUBROUTINE                  : jnetcdf
 !!
 !!
 !>\BRIEF                   : produces an output netCDF file for the multilayer scheme (though significantly
 !!                                           slows the runtime, so this is not for general use) 
 !!
 !! DESCRIPTION                 :
 !! 
 !!
 !! RECENT CHANGE(S): None
 !!
 !! MAIN OUTPUT VARIABLE(S) : none
 !!
 !! REFERENCE(S)        :
 !!
 !! FLOWCHART           :
 !_ ==============================================================================================================================



 SUBROUTINE jnetcdf(james_netcdf_flag, temp_atmos_next, q_atmos_next, &
                     & temp_leaf_next, q_leaf_next, &
                     & flux_h, flux_le, flux_ground_h, flux_ground_le, &
                     & k_eddy, u_speed, temp_surf_next, &
                     & eta_1, eta_2, eta_3, eta_4, jomega, &
                     & eta_1_surf, eta_2_surf, eta_3_surf, eta_4_surf, &
                     & down_lw_ac, down_sw_ac, &
                     & temp_atmos_pres, q_atmos_pres, b_t, b_q, heat_soil, &
                     & jomega_surf, jemissivity_surf, jomega_sw_surf, jemissivity, & 
                     & jomega_sw, temp_leaf_pres, temp_surf_pres, &
                     & big_r, big_r_prime, horiz_flux_h, horiz_flux_le, &
                     & theta_zero, eta_3_above, eta_1_above, eta_4_above, &
                     & eta_2_above, pad_deltah, jamescdfname, &
                     & ngu_bign, ngu_alpha, ngu_alpha_insert, qsol_sat_new, kjit, &
                     & matt_surf_absorbed, matt_layer_absorbed, box_height)  
 
    USE netcdf

    IMPLICIT NONE

    !! 0.1 Input variables

    CHARACTER(LEN=80), INTENT(in)                        :: jamescdfname          

    REAL(r_std), DIMENSION (jnlvls), INTENT(in)          :: jemissivity       
    REAL(r_std), DIMENSION (jnlvls), INTENT(in)               :: temp_atmos_pres    
    REAL(r_std), DIMENSION (jnlvls), INTENT(in)          :: b_t, b_q
    REAL(r_std), DIMENSION (jnlvls), INTENT(in)          :: q_atmos_pres     
    REAL(r_std), DIMENSION (jnlvls), INTENT(in)          :: big_r, big_r_prime  
    REAL(r_std), DIMENSION (jnlvls), INTENT(in)          :: temp_atmos_next     
    REAL(r_std), DIMENSION (jnlvls), INTENT(in)          :: temp_leaf_next, temp_leaf_pres         ! leaf temperature down the column (K) PRESENT STEP
    REAL(r_std), DIMENSION (jnlvls), INTENT(in)          :: q_atmos_next        ! atmospheric specific humididy down the column (kg/kg) PRESENT STEP
    REAL(r_std), DIMENSION (jnlvls), INTENT(in)          :: q_leaf_next         ! leaf specific humidity down the column (kg/kg) PRESENT STEP
    REAL(r_std), DIMENSION (jnlvls), INTENT(in)          :: flux_h, flux_le     ! leaf temperature down the column (K) PRESENT STEP
    ! REAL(r_std), DIMENSION (jnlvls), INTENT(in)          :: horiz_flux_h,  horiz_flux_le        
    REAL(r_std), DIMENSION (jnlvls)                      :: horiz_flux_h,  horiz_flux_le        
    REAL(r_std), DIMENSION (jnlvls), INTENT(in)          :: k_eddy              ! eddy transport coefficient (m^2 s-1)
    REAL(r_std), DIMENSION (jnlvls), INTENT(in)          :: u_speed             ! (m/s)
    REAL(r_std), DIMENSION (jnlvls), INTENT(in)          :: jomega_sw
    REAL(r_std), DIMENSION (jnlvls), INTENT(in)          :: eta_1, eta_2, eta_3, eta_4, jomega

    REAL(r_std), DIMENSION (jnlvls+1), INTENT(in)        :: box_height

    REAL, DIMENSION (0:jnlvls+1), INTENT(in)             :: ngu_alpha
    REAL(r_std), DIMENSION (0:jnlvls+1), INTENT(in)      :: ngu_alpha_insert

    REAL(r_std), DIMENSION (0:jnlvls), INTENT(in)        :: pad_deltah         ! eddy transport coefficient (m^2 s-1)    
    REAL(r_std), DIMENSION (0:jnlvls+1), INTENT(in)      :: ngu_bign

    REAL(r_std), INTENT(in)                              :: flux_ground_h, flux_ground_le
    REAL(r_std), INTENT(in)                              :: jomega_surf   
    REAL(r_std), INTENT(in)                              :: jemissivity_surf    
    REAL(r_std), INTENT(in)                              :: jomega_sw_surf    
    REAL(r_std), INTENT(in)                              :: temp_surf_next, temp_surf_pres, qsol_sat_new    
    REAL(r_std), INTENT(in)                              :: eta_1_surf, eta_2_surf
    REAL(r_std), INTENT(in)                              :: eta_3_surf, eta_4_surf, eta_3_above
    REAL(r_std), INTENT(in)                              :: eta_1_above, eta_4_above, eta_2_above   
    REAL(r_std), INTENT(in)                              :: down_lw_ac             ! input downwelling lw radiation above the canopy (watts/m^2)
    REAL(r_std), INTENT(in)                              :: down_sw_ac             ! input downwelling sw radiation above the canopy (watts/m^2) 
    REAL(r_std), INTENT(in)                              :: heat_soil              ! heat flux from or to deeper soil (layered scheme in ORCHIDEE) (W m^-2)
    REAL(r_std), INTENT(in)                              :: theta_zero             ! heat capacity of the infinitesimal surface layer (J/ (K m^2))

    INTEGER(i_std), INTENT(in)                           :: kjit

    REAL(r_std), INTENT(in)                              :: matt_surf_absorbed
    REAL(r_std), DIMENSION(:), INTENT(in)                :: matt_layer_absorbed

    !! 0.2 Output variables

    !! 0.3 Modified variables

    !! 0.4 Local variables

    INTEGER(i_std), SAVE                     :: tap_id, qap_id, tlp_id, qlp_id              ! ID integers
    INTEGER(i_std), SAVE                     :: tan_id, qan_id, tln_id, qln_id              ! ID integers
    INTEGER(i_std), SAVE                     :: bt_id, bq_id                                ! ID integers
    INTEGER(i_std), SAVE                     :: bigr_id, bigrprime_id                       ! ID integers
    INTEGER(i_std), SAVE                     :: horiz_f_h_id, horiz_f_le_id                 ! ID integers
    INTEGER(i_std), SAVE                     :: hflux_id, leflux_id, keddy_id, uspeed_id    ! ID integers
    INTEGER(i_std), SAVE                     :: tsn_id, tsp_id, qsn_id                              ! ID integers
    INTEGER(i_std), SAVE                     :: e1_id, e2_id, e3_id, e4_id                  ! ID integers
    INTEGER(i_std), SAVE                     :: e1s_id, e2s_id, e3s_id, e4s_id              ! ID integers
    INTEGER(i_std), SAVE                     :: e3a_id, e1a_id, e2a_id, e4a_id              ! ID integers
    INTEGER(i_std), SAVE                     :: lw_id, sw_id, hs_id                         ! ID integers
    INTEGER(i_std), SAVE                     :: jos_id, jes_id, josws_id                    ! ID integers
    INTEGER(i_std), SAVE                     :: je_id, jo_id, josw_id                       ! ID integers
    INTEGER(i_std), SAVE                     :: tz_id, pad_id                               ! ID integers  
    INTEGER(i_std), SAVE                     :: ngu_bign_id, ngu_bign_s_id, ngu_bign_a_id   ! ID integers
    INTEGER(i_std), SAVE                     :: ngu_alpha_id, ngu_alpha_s_id, ngu_alpha_a_id   ! ID integers
    INTEGER(i_std), SAVE                     :: ngu_alphai_id, ngu_alphai_s_id, ngu_alphai_a_id   ! ID integers
    INTEGER(i_std), SAVE                     :: msa_id, mla_id                            ! ID integers
    INTEGER(i_std), SAVE                     :: james_step_count    
    INTEGER(i_std), SAVE                     :: iret, iret2, fid, fid2, fid3
    INTEGER(i_std), SAVE                     :: ibh_id

    INTEGER(i_std)                           :: ii, jj, nlonid, nlatid, varid
    INTEGER(i_std)                           :: TimeDimID, jtid, ts_id, HeightDimID, i
    INTEGER(i_std)                           :: height_dim_id, height_dim_id2, time_dim_id
    INTEGER(i_std)                           :: jstartnc_b, jcountnc_b
    INTEGER(i_std)                           :: dimids_b
    INTEGER(i_std)                           :: dimids(2) 
    INTEGER(i_std)                           :: dimids2(2) 
    INTEGER, DIMENSION(2)                    :: jstartnc, jcountnc    
    LOGICAL                                  :: james_netcdf_flag    

    REAL(r_std), DIMENSION(jnlvls)           :: flux_h_combo, flux_le_combo
    REAL(r_std), DIMENSION (jnlvls)          :: array_jomega_surf
    REAL(r_std), DIMENSION (jnlvls)          :: array_jemissivity_surf 
    REAL(r_std), DIMENSION (jnlvls)          :: array_jomega_sw_surf 
    REAL(r_std), DIMENSION (jnlvls)          :: array_temp_surf_pres
    REAL(r_std), DIMENSION (jnlvls)          :: array_temp_surf_next
    REAL(r_std), DIMENSION (jnlvls)          :: array_qsol_sat_new
    REAL(r_std), DIMENSION (jnlvls)          :: array_theta_zero
    REAL(r_std), DIMENSION (jnlvls)          :: array_eta_1_surf, array_eta_2_surf
    REAL(r_std), DIMENSION (jnlvls)          :: array_eta_3_surf, array_eta_4_surf 
    REAL(r_std), DIMENSION (jnlvls)          :: array_eta_3_above, array_eta_1_above
    REAL(r_std), DIMENSION (jnlvls)          :: array_eta_2_above, array_eta_4_above
    REAL(r_std), DIMENSION (jnlvls)          :: array_down_lw_ac, array_down_sw_ac
    REAL(r_std), DIMENSION (jnlvls)          :: array_heat_soil
    REAL(r_std), DIMENSION (jnlvls)          :: array_ngu_bign_surf
    REAL(r_std), DIMENSION (jnlvls)          :: array_ngu_bign_above
    REAL(r_std), DIMENSION (jnlvls)          :: array_ngu_bign
    REAL(r_std), DIMENSION (jnlvls)          :: array_ngu_alpha_surf
    REAL(r_std), DIMENSION (jnlvls)          :: array_ngu_alpha_above
    REAL(r_std), DIMENSION (jnlvls)          :: array_ngu_alpha

    REAL(r_std), DIMENSION (jnlvls)          :: array_ngu_alphai_surf
    REAL(r_std), DIMENSION (jnlvls)          :: array_ngu_alphai_above
    REAL(r_std), DIMENSION (jnlvls)          :: array_ngu_alphai

    REAL(r_std), DIMENSION (jnlvls)          :: array_matt_surf_absorbed
    REAL(r_std), DIMENSION (jnlvls)          :: array_matt_layer_absorbed
 !_ ==============================================================================================================================

    REAL(r_std), DIMENSION (jnlvls)          :: intermediate_box_height
 
    ! parameters, dimensions and variable set-ups (only runs once)
 
    IF (james_netcdf_flag) THEN

        james_netcdf_flag = .FALSE.

        ! … define the parameters here
 
        iret = nf90_create(jamescdfname, nf90_write, fid)
        
  
        ! define the dimensions
        
        iret = nf90_def_dim(fid, "height", jnlvls, height_dim_id)
        iret = nf90_def_dim(fid, "time", nf90_unlimited, time_dim_id)

        dimids = (/ height_dim_id, time_dim_id /)

 
        ! define the variables
        iret = nf90_def_var(fid,"temp_atmos_pres",nf90_real4, dimids, tap_id)
        iret = nf90_def_var(fid,"q_atmos_pres",nf90_real4, dimids, qap_id)

        iret = nf90_def_var(fid,"temp_atmos_next",nf90_real4, dimids, tan_id)
        iret = nf90_def_var(fid,"q_atmos_next",nf90_real4, dimids, qan_id)

        iret = nf90_def_var(fid,"temp_leaf_pres",nf90_real4, dimids, tlp_id)
        
        iret = nf90_def_var(fid,"temp_leaf_next",nf90_real4, dimids, tln_id)
        iret = nf90_def_var(fid,"q_leaf_next",nf90_real4, dimids, qln_id)

        iret = nf90_def_var(fid,"big_r",nf90_real4, dimids, bigr_id)
        iret = nf90_def_var(fid,"big_r_prime",nf90_real4, dimids, bigrprime_id)

        iret = nf90_def_var(fid,"h_flux",nf90_real4, dimids, hflux_id)
        iret = nf90_def_var(fid,"le_flux",nf90_real4, dimids, leflux_id)

        iret = nf90_def_var(fid,"horiz_flux_h",nf90_real4, dimids, horiz_f_h_id)
        iret = nf90_def_var(fid,"horiz_flux_le",nf90_real4, dimids, horiz_f_le_id)

        iret = nf90_def_var(fid,"k_eddy",nf90_real4, dimids, keddy_id)

        iret = nf90_def_var(fid,"pad_deltah",nf90_real4, dimids, pad_id)

        iret = nf90_def_var(fid,"u_speed",nf90_real4, dimids, uspeed_id)

        iret = nf90_def_var(fid,"b_t",nf90_real4, dimids, bt_id)
        iret = nf90_def_var(fid,"b_q",nf90_real4, dimids, bq_id)

        iret = nf90_def_var(fid,"temp_surf_pres",nf90_real4, dimids, tsp_id)
        iret = nf90_def_var(fid,"temp_surf_next",nf90_real4, dimids, tsn_id)
        iret = nf90_def_var(fid,"qsol_sat_new",nf90_real4, dimids, qsn_id)

        iret = nf90_def_var(fid,"qsol_sat_new",nf90_real4, dimids, qsn_id)
        iret = nf90_def_var(fid,"theta_zero",nf90_real4, dimids, tz_id)

        iret = nf90_def_var(fid,"eta_1",nf90_real4, dimids, e1_id)
        iret = nf90_def_var(fid,"eta_2",nf90_real4, dimids, e2_id)
        iret = nf90_def_var(fid,"eta_3",nf90_real4, dimids, e3_id)
        iret = nf90_def_var(fid,"eta_4",nf90_real4, dimids, e4_id)

        iret = nf90_def_var(fid,"eta_1_surf",nf90_real4, dimids, e1s_id)
        iret = nf90_def_var(fid,"eta_2_surf",nf90_real4, dimids, e2s_id)
        iret = nf90_def_var(fid,"eta_3_surf",nf90_real4, dimids, e3s_id)
        iret = nf90_def_var(fid,"eta_4_surf",nf90_real4, dimids, e4s_id)

        iret = nf90_def_var(fid,"eta_1_above",nf90_real4, dimids, e1a_id)
        iret = nf90_def_var(fid,"eta_2_above",nf90_real4, dimids, e2a_id)
        iret = nf90_def_var(fid,"eta_3_above",nf90_real4, dimids, e3a_id)
        iret = nf90_def_var(fid,"eta_4_above",nf90_real4, dimids, e4a_id)

        iret = nf90_def_var(fid,"lw_down_ac",nf90_real4, dimids, lw_id)
        iret = nf90_def_var(fid,"sw_down_ac",nf90_real4, dimids, sw_id)

        iret = nf90_def_var(fid,"heat_soil",nf90_real4, dimids, hs_id)

        iret = nf90_def_var(fid,"jomega",nf90_real4, dimids, jo_id)
        iret = nf90_def_var(fid,"jomega_sw",nf90_real4, dimids, josw_id)
        iret = nf90_def_var(fid,"jomega_surf",nf90_real4, dimids, jos_id)
        iret = nf90_def_var(fid,"jemissivity",nf90_real4, dimids, je_id)
        iret = nf90_def_var(fid,"jemissivity_surf",nf90_real4, dimids, jes_id)
        iret = nf90_def_var(fid,"jomega_sw_surf",nf90_real4, dimids, josws_id)

        iret = nf90_def_var(fid,"ngu_bign",nf90_real4, dimids, ngu_bign_id)
        iret = nf90_def_var(fid,"ngu_bign_surf",nf90_real4, dimids, ngu_bign_s_id)
        iret = nf90_def_var(fid,"ngu_bign_above",nf90_real4, dimids, ngu_bign_a_id)
        iret = nf90_def_var(fid,"ngu_alpha",nf90_real4, dimids, ngu_alpha_id)
        iret = nf90_def_var(fid,"ngu_alpha_surf",nf90_real4, dimids, ngu_alpha_s_id)
        iret = nf90_def_var(fid,"ngu_alpha_above",nf90_real4, dimids, ngu_alpha_a_id)

        iret = nf90_def_var(fid,"ngu_alphai",nf90_real4, dimids, ngu_alphai_id)
        iret = nf90_def_var(fid,"ngu_alphai_surf",nf90_real4, dimids, ngu_alphai_s_id)
        iret = nf90_def_var(fid,"ngu_alphai_above",nf90_real4, dimids, ngu_alphai_a_id)

        iret = nf90_def_var(fid,"matt_surf_absorbed",nf90_real4, dimids, msa_id)
        iret = nf90_def_var(fid,"matt_layer_absorbed",nf90_real4, dimids, mla_id)

        iret = nf90_def_var(fid,"intermediate_box_height",nf90_real4, dimids, ibh_id)

        iret = nf90_enddef (fid)
         
    END IF !(james_netcdf_flag)

    intermediate_box_height(1:jnlvls) = box_height(1:jnlvls)

    ! open the netCDF file at each step
    iret = nf90_open(jamescdfname, nf90_Write, fid) 

    jstartnc = (/ 1, 1 /)  ! vector of integers specifying the index in the variable from 
                           !   which the first of the data values will be read
    jcountnc = (/ jnlvls, 1 /)  ! vector of integers specifying the number of indices 
                                 !   selected along each dimension

    james_step_count = james_step_count + 1    ! this is the time step counter
    
    jstartnc(2) = james_step_count   ! (2) refers to the time dimension at present (it 
                                     !   must always come last)
 
    flux_h_combo(1) = flux_ground_h
    flux_h_combo(2:jnlvls) = flux_h(2:jnlvls)
    
    flux_le_combo(1) = flux_ground_le
    flux_le_combo(2:jnlvls) = flux_le(2:jnlvls) 

    array_temp_surf_pres(1) = temp_surf_pres
    array_temp_surf_pres(2:jnlvls) = 0.0d0
    
    array_temp_surf_next(1) = temp_surf_next
    array_temp_surf_next(2:jnlvls) = 0.0d0
    
    array_qsol_sat_new(1) = qsol_sat_new
    array_qsol_sat_new(2:jnlvls) = 0.0d0

    array_eta_1_surf(1) = eta_1_surf
    array_eta_1_surf(2:jnlvls) = 0.0d0
    
    array_eta_2_surf(1) = eta_2_surf
    array_eta_2_surf(2:jnlvls) = 0.0d0    
 
    array_eta_3_surf(1) = eta_3_surf
    array_eta_3_surf(2:jnlvls) = 0.0d0    

    array_eta_1_above(1) = eta_1_above
    array_eta_1_above(2:jnlvls) = 0.0d0  

    array_eta_2_above(1) = eta_2_above
    array_eta_2_above(2:jnlvls) = 0.0d0  

    array_eta_3_above(1) = eta_3_above
    array_eta_3_above(2:jnlvls) = 0.0d0  

    array_eta_4_above(1) = eta_4_above
    array_eta_4_above(2:jnlvls) = 0.0d0  

    array_eta_4_surf(1) = eta_4_surf
    array_eta_4_surf(2:jnlvls) = 0.0d0
    
    array_down_lw_ac(1) = down_lw_ac
    array_down_lw_ac(2:jnlvls) = 0.0d0 
    array_down_sw_ac(1) = down_sw_ac
    array_down_sw_ac(2:jnlvls) = 0.0d0
    
    array_heat_soil(1) = heat_soil
    array_heat_soil(2:jnlvls) = 0.0d0

    array_jomega_surf(1) = jomega_surf
    array_jomega_surf(2:jnlvls) = 0.0d0

    array_theta_zero(1) = theta_zero
    array_theta_zero(2:jnlvls) = 0.0d0
    
    array_jemissivity_surf(1) = jemissivity_surf
    array_jemissivity_surf(2:jnlvls) = 0.0d0

    array_jomega_sw_surf(1) = jomega_sw_surf    
    array_jomega_sw_surf(2:jnlvls) = 0.0d0     

    array_ngu_bign_surf(1) = ngu_bign(0)    
    array_ngu_bign_surf(2:jnlvls) = 0.0d0            

    array_ngu_bign_above(1) = ngu_bign(jnlvls+1)
    array_ngu_bign_above(2:jnlvls) = 0.0d0 

    array_ngu_bign(1:jnlvls) = ngu_bign(1:jnlvls)    



    array_ngu_alpha_surf(1) = ngu_alpha(0)    
    array_ngu_alpha_surf(2:jnlvls) = 0.0d0            

    array_ngu_alpha_above(1) = ngu_alpha(jnlvls+1)
    array_ngu_alpha_above(2:jnlvls) = 0.0d0 

    array_ngu_alpha(1:jnlvls) = ngu_alpha(1:jnlvls) 



    array_ngu_alphai_surf(1) = ngu_alpha_insert(0)    
    array_ngu_alphai_surf(2:jnlvls) = 0.0d0            

    array_ngu_alphai_above(1) = ngu_alpha_insert(jnlvls+1)
    array_ngu_alphai_above(2:jnlvls) = 0.0d0 

    array_ngu_alphai(1:jnlvls) = ngu_alpha_insert(1:jnlvls) 

    array_matt_surf_absorbed(1) = matt_surf_absorbed
    array_matt_surf_absorbed(2:jnlvls) = 0.0d0 

    array_matt_layer_absorbed(1:jnlvls) = matt_layer_absorbed(1:jnlvls) 

    IF (ld_jameswrite) THEN
        WRITE(numout, *) '240215 in jnetcdf, after column allocation'
    END IF ! (ld_jameswrite) THEN


    iret = nf90_put_var(fid, bt_id, b_t(:), start = jstartnc, count = jcountnc) 
    iret = nf90_put_var(fid, bq_id, b_q(:), start = jstartnc, count = jcountnc)

    iret = nf90_put_var(fid, tap_id, temp_atmos_pres(:), start = jstartnc, count = jcountnc) 
    iret = nf90_put_var(fid, qap_id, q_atmos_pres(:), start = jstartnc, count = jcountnc)
     
    iret = nf90_put_var(fid, tan_id, temp_atmos_next(:), start = jstartnc, count = jcountnc) 
    iret = nf90_put_var(fid, qan_id, q_atmos_next(:), start = jstartnc, count = jcountnc)         

    iret = nf90_put_var(fid, qsn_id, array_qsol_sat_new(:), start = jstartnc, count = jcountnc)         

    iret = nf90_put_var(fid, tlp_id, temp_leaf_pres(:), start = jstartnc, count = jcountnc)         

    iret = nf90_put_var(fid, tln_id, temp_leaf_next(:), start = jstartnc, count = jcountnc)         
    iret = nf90_put_var(fid, qln_id, q_leaf_next(:), start = jstartnc, count = jcountnc)         

    iret = nf90_put_var(fid, bigr_id, big_r(:), start = jstartnc, count = jcountnc)         
    iret = nf90_put_var(fid, bigrprime_id, big_r_prime(:), start = jstartnc, count = jcountnc)
     
    iret = nf90_put_var(fid, hflux_id, flux_h_combo(:), start = jstartnc, count = jcountnc)         
    iret = nf90_put_var(fid, leflux_id, flux_le_combo(:), start = jstartnc, count = jcountnc)         

    iret = nf90_put_var(fid, horiz_f_h_id, horiz_flux_h(:), start = jstartnc, count = jcountnc)         
    iret = nf90_put_var(fid, horiz_f_le_id, horiz_flux_le(:), start = jstartnc, count = jcountnc)

    iret = nf90_put_var(fid, keddy_id, k_eddy(:), start = jstartnc, count = jcountnc)              
    
    iret = nf90_put_var(fid, pad_id, pad_deltah(:), start = jstartnc, count = jcountnc)              
    
    iret = nf90_put_var(fid, uspeed_id, u_speed(:), start = jstartnc, count = jcountnc)              

    iret = nf90_put_var(fid, e1_id, eta_1(:), start = jstartnc, count = jcountnc)              
    iret = nf90_put_var(fid, e2_id, eta_2(:), start = jstartnc, count = jcountnc)              
    iret = nf90_put_var(fid, e3_id, eta_3(:), start = jstartnc, count = jcountnc)              
    iret = nf90_put_var(fid, e4_id, eta_4(:), start = jstartnc, count = jcountnc)              

    iret = nf90_put_var(fid, tsp_id, array_temp_surf_pres(:), start = jstartnc, count = jcountnc)              

    iret = nf90_put_var(fid, tsn_id, array_temp_surf_next(:), start = jstartnc, count = jcountnc)              

    iret = nf90_put_var(fid, e1s_id, array_eta_1_surf(:), start = jstartnc, count = jcountnc)              
    iret = nf90_put_var(fid, e2s_id, array_eta_2_surf(:), start = jstartnc, count = jcountnc)              
    iret = nf90_put_var(fid, e3s_id, array_eta_3_surf(:), start = jstartnc, count = jcountnc)              
    iret = nf90_put_var(fid, e4s_id, array_eta_4_surf(:), start = jstartnc, count = jcountnc)              

    iret = nf90_put_var(fid, e1a_id, array_eta_1_above(:), start = jstartnc, count = jcountnc)              
    iret = nf90_put_var(fid, e2a_id, array_eta_2_above(:), start = jstartnc, count = jcountnc)              
    iret = nf90_put_var(fid, e3a_id, array_eta_3_above(:), start = jstartnc, count = jcountnc)              
    iret = nf90_put_var(fid, e4a_id, array_eta_4_above(:), start = jstartnc, count = jcountnc)              


    iret = nf90_put_var(fid, lw_id, array_down_lw_ac(:), start = jstartnc, count = jcountnc)              
    iret = nf90_put_var(fid, sw_id, array_down_sw_ac(:), start = jstartnc, count = jcountnc) 

    iret = nf90_put_var(fid, hs_id, array_heat_soil(:), start = jstartnc, count = jcountnc) 

    iret = nf90_put_var(fid, tz_id, array_theta_zero(:), start = jstartnc, count = jcountnc) 

    iret = nf90_put_var(fid, jo_id, jomega(:), start = jstartnc, count = jcountnc) 
    iret = nf90_put_var(fid, josw_id, jomega_sw(:), start = jstartnc, count = jcountnc) 
    iret = nf90_put_var(fid, jos_id, array_jomega_surf(:), start = jstartnc, count = jcountnc) 
    iret = nf90_put_var(fid, je_id, jemissivity(:), start = jstartnc, count = jcountnc) 
    iret = nf90_put_var(fid, jes_id, array_jemissivity_surf(:), start = jstartnc, count = jcountnc) 
    iret = nf90_put_var(fid, josws_id, array_jomega_sw_surf(:), start = jstartnc, count = jcountnc) 

    iret = nf90_put_var(fid, ngu_bign_id, array_ngu_bign(:), start = jstartnc, count = jcountnc)              
    iret = nf90_put_var(fid, ngu_bign_s_id, array_ngu_bign_surf(:), start = jstartnc, count = jcountnc)              
    iret = nf90_put_var(fid, ngu_bign_a_id, array_ngu_bign_above(:), start = jstartnc, count = jcountnc)     


    iret = nf90_put_var(fid, ngu_alpha_id, array_ngu_alpha(:), start = jstartnc, count = jcountnc)              
    iret = nf90_put_var(fid, ngu_alpha_s_id, array_ngu_alpha_surf(:), start = jstartnc, count = jcountnc)              
    iret = nf90_put_var(fid, ngu_alpha_a_id, array_ngu_alpha_above(:), start = jstartnc, count = jcountnc)   


    iret = nf90_put_var(fid, ngu_alphai_id, array_ngu_alphai(:), start = jstartnc, count = jcountnc)              
    iret = nf90_put_var(fid, ngu_alphai_s_id, array_ngu_alphai_surf(:), start = jstartnc, count = jcountnc)              
    iret = nf90_put_var(fid, ngu_alphai_a_id, array_ngu_alphai_above(:), start = jstartnc, count = jcountnc)   

    iret = nf90_put_var(fid, msa_id, array_matt_surf_absorbed(:), start = jstartnc, count = jcountnc)              
    iret = nf90_put_var(fid, mla_id, array_matt_layer_absorbed(:), start = jstartnc, count = jcountnc) 

    iret = nf90_put_var(fid, ibh_id, intermediate_box_height(:), start = jstartnc, count = jcountnc) 

    ! close the netCDF file and write to disk                   
    iret = nf90_close(fid)

 
 END SUBROUTINE jnetcdf

 
 ! ------------------------------------------------------------------------------------------------------------------------------------- 

 !!  ===========================================================================================================================
 !! SUBROUTINE                  : calc_jradiation_scheme
 !!
 !!
 !>\BRIEF                   : calculates the radiation balance with a multilayer canopy 
 !!
 !! DESCRIPTION                 :
 !! 
 !!
 !! RECENT CHANGE(S): None
 !!
 !! MAIN OUTPUT VARIABLE(S) : eta_1, eta_2, eta_3, eta_4, jomega, jemissivity, temp_leaf_pres_grid, temp_surf_pres,
 !!                               pad_deltah, jomega_sw, eta_1_surf, eta_2_surf, eta_3_surf, eta_4_surf,
 !!                               jemissivity_surf, jomega_surf, jomega_sw_surf, eta_3_above, eta_1_above, eta_4_above, eta_2_above,
 !!                               ngu_alpha, ngu_alpha_insert, kjpindex, big_k_lw, big_k_sw
 !!
 !! REFERENCE(S)        : Gu, L. et al., 1999. Micrometeorology, biophysical exchanges and NEE decomposition in a two-storey
 !!                                   boreal forest - development and test of an integrated model. Agricultural and Forest 
 !!                                   Meteorology, 94, pp.123–148.
 !!
 !! FLOWCHART           :
 !_ ==============================================================================================================================

 SUBROUTINE calc_jradiation_scheme (eta_1, eta_2, eta_3, eta_4, &
                                & jomega, jemissivity, temp_leaf_pres_grid, temp_surf_pres, &
                                & pad_deltah, jomega_sw, &
                                & eta_1_surf, eta_2_surf, eta_3_surf, eta_4_surf, &
                                & jemissivity_surf, jomega_surf, jomega_sw_surf, &
                                & eta_3_above, eta_1_above, eta_4_above, eta_2_above, &
                                & ngu_alpha, ngu_alpha_insert, kjpindex, &
                                & big_k_lw, big_k_sw, delta_h)


     IMPLICIT NONE


     !! 0.1 Input variables
     INTEGER(i_std), INTENT(in)                             :: kjpindex               !! Domain size (-)

     REAL(r_std), DIMENSION (kjpindex,jnlvls), INTENT(in)   :: temp_leaf_pres_grid    ! leaf temperature down the column (K) PRESENT STEP
     REAL(r_std), INTENT(in)                                :: temp_surf_pres
     REAL(r_std), DIMENSION (0:jnlvls), INTENT(in)          :: pad_deltah
     REAL(r_std), INTENT(in)                                :: big_k_lw
     REAL(r_std), INTENT(in)                                :: big_k_sw
     REAL(r_std), DIMENSION(jnlvls), INTENT(in)             :: delta_h                ! height of each level (m)


     !! 0.2 Output variables
     REAL(r_std), DIMENSION(jnlvls), INTENT(out)            :: eta_1, eta_2, eta_3, eta_4
     REAL(r_std), DIMENSION(jnlvls), INTENT(out)            :: jomega
     REAL(r_std), DIMENSION (jnlvls), INTENT(out)           :: jomega_sw
     REAL(r_std), DIMENSION(jnlvls), INTENT(out)            :: jemissivity
     REAL(r_std), INTENT(out)                               :: jemissivity_surf                     ! emissivity of surface layer
     REAL(r_std), INTENT(out)                               :: jomega_surf
     REAL(r_std), INTENT(out)                               :: jomega_sw_surf
     REAL(r_std), INTENT(out)                               :: eta_1_surf, eta_2_surf
     REAL(r_std), INTENT(out)                               :: eta_3_surf, eta_4_surf
     REAL(r_std), INTENT(out)                               :: eta_3_above, eta_1_above
     REAL(r_std), INTENT(out)                               :: eta_4_above, eta_2_above 
     REAL, DIMENSION (0:jnlvls+1, 0:jnlvls+1), INTENT(out)  :: ngu_alpha

     !! 0.3 Modified variables


     !! 0.4 Local variables
     INTEGER(i_std)                                         :: i, j, k, ji, jk
     REAL(r_std), DIMENSION(jnlvls)                         :: series_sub1, series_sub2
     REAL(r_std), DIMENSION(jnlvls)                         :: series_sub1_sw, series_sub2_sw  
     REAL(r_std), DIMENSION(jnlvls)                         :: rho_albedo                           ! albedo of each vegetation layer
     REAL(r_std)                                            :: rho_albedo_surf

     REAL(r_std)                                            :: jcounter, jcounter2
     REAL(r_std)                                            :: series_sub1_surf
     REAL(r_std)                                            :: series_sub2_surf, series_sub1_sw_surf
     REAL(r_std)                                            :: temp_surf
     REAL(r_std)                                            :: jfactor
     REAL(r_std), DIMENSION (0:jnlvls+1)                    :: ngu_alpha_insert
     REAL(r_std), DIMENSION (jnlvls)                        :: temp_leaf_pres    ! leaf temperature down the column (K) PRESENT STEP


     ! %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

     DO jk = 1, kjpindex

         temp_leaf_pres(:) = temp_leaf_pres_grid(jk,:)  
         jemissivity(:) = 0.96d0 != 0.96d0
         jomega_sw_surf = 1.0d0
         jomega_surf = 1.0d0         
         jemissivity_surf = 0.96d0 != 0.96d0
         temp_surf =  0.0d0  !(not used)             
         series_sub1(:) = 1.0d0
         series_sub2(:) = 1.0d0 
         series_sub1_sw(:) = 1.0d0
         series_sub2_sw(:) = 1.0d0  
         rho_albedo(:) = 0.4d0 !0.4d0
         rho_albedo_surf = 0.4d0 !0.4d0             
         rho_albedo(:) = 0.4d0
         rho_albedo_surf = 0.4d0    
         jomega_surf = 1.0d0
         jomega_sw_surf = 1.0d0
         jemissivity_surf = 0.96d0
         jemissivity(:) = 0.96d0
         jcounter = 1.0d0
         series_sub1(:) = 1.0d0 
    
         ! calculate omega coefficient for each level
         DO i=1, jnlvls
             jomega(i) = 1 - (exp(-1.0d0 * big_k_lw * pad_deltah(i) ) )
         END DO ! i = 1, jnlvls
    
         DO i=1, jnlvls
             jomega_sw(i) = 1 - (exp(-1.0d0 * big_k_sw * pad_deltah(i) ) ) 
         END DO ! i = 1, jnlvls 

         jcounter = 1.0d0
    
         series_sub1(:) = 1.0d0
    
         DO i = jnlvls, 1, -1
             DO j = jnlvls-1, i, -1
                 series_sub1(i) = series_sub1(i) * ( 1.0d0 - jomega(j+1) ) 
             END DO ! j = jnlvls, i, -1
         END DO ! i = jnlvls, 1, -1
    
         series_sub1_surf = series_sub1(1) * (1.0d0 - jomega(1))
    
         eta_1(1) = (series_sub1(1) * jomega(1) * jemissivity(1))                  
    
         DO i = 2, jnlvls-1
             eta_1(i) = (series_sub1(i) * jomega(i) * jemissivity(i)) 
         END DO ! i = 2, jnlvls-2
    
         eta_1(jnlvls) = series_sub1(jnlvls) * jomega(jnlvls) * jemissivity(jnlvls)
        
         eta_1_surf = jomega_surf * series_sub1_surf * jemissivity_surf
        
         eta_1_above = 0.0d0     
    
         eta_1_above = (series_sub1(1) * jomega(i) * (1.0d0 - jemissivity(1))) 
    
         DO i = 2, jnlvls-1
             eta_1_above = eta_1_above + &
                            & (series_sub1(i) * jomega(i) * (1.0d0 - jemissivity(i))) 
         END DO ! i = 2, jnlvls-2
    
         eta_1_above = eta_1_above + &
                    & series_sub1(jnlvls) * jomega(jnlvls) * (1.0d0 - jemissivity(jnlvls))
    
         eta_1_above = eta_1_above + &
                    & series_sub1_surf * jomega_surf * (1.0d0 - jemissivity(jnlvls))
    
         series_sub1_sw(:) = 1.0d0
    
         DO i = jnlvls, 1, -1
             DO j = jnlvls-1, i, -1
                 series_sub1_sw(i) = series_sub1_sw(i) * ( 1.0d0 - jomega_sw(j+1) ) 
             END DO ! j = jnlvls, i, -1
         END DO ! i = jnlvls, 1, -1
    
         series_sub1_sw_surf = series_sub1_sw(1) * (1.0d0 - jomega_sw(1))
    
         eta_4(1) = (series_sub1_sw(1) * jomega_sw(1) * (1.0d0 - rho_albedo(1))) !+ &
    
         DO i = 2, jnlvls-1
             eta_4(i) = (series_sub1_sw(i) * jomega_sw(i) * (1.0d0 - rho_albedo(i))) !+ &
         END DO ! i = 2, jnlvls-2
    
         eta_4(jnlvls) = series_sub1_sw(jnlvls) &
                           & * jomega_sw(jnlvls) * (1.0d0 - rho_albedo(jnlvls))
    
         eta_4_surf = series_sub1_sw_surf * jomega_sw_surf * (1.0d0 - rho_albedo_surf)
    
         
         ! 200613 old version
    
         eta_4_above = 0.0d0     
    
         eta_4_above = (series_sub1_sw(1) * jomega_sw(1) * rho_albedo_surf) ! - &
    
    
         DO i = 2, jnlvls-1
             eta_4_above = eta_4_above + (series_sub1_sw(i) * jomega_sw(i) * rho_albedo(i))
         END DO ! i = 2, jnlvls-2
    
         eta_4_above = eta_4_above + &
                    & series_sub1_sw(jnlvls) * jomega_sw(jnlvls) * rho_albedo(jnlvls)
         
         eta_4_above = eta_4_above + &
                    & series_sub1_sw_surf* jomega_sw_surf * rho_albedo_surf
    
         jcounter = 1.0d0
         jcounter2 = 1.0d0
         DO j = jnlvls, 1, -1
             jcounter = jcounter * ( 1.0d0 - jomega(j) ) 
             jcounter2 = jcounter2 * ( 1.0d0 - jomega(j) ) 
         END DO ! j = 2, jnlvls  
         series_sub1_surf = jcounter
         series_sub2_surf = jcounter2
         
         jcounter = 1.0d0
         jcounter2 = 1.0d0
         series_sub1_sw(jnlvls) = (1.0d0-rho_albedo(jnlvls)) * jomega_sw(jnlvls)    
         series_sub2_sw(jnlvls) = (1.0d0-rho_albedo(jnlvls)) * jomega_sw(jnlvls)    
         DO i = jnlvls-1, 1, -1
             jcounter = 1.0d0 * jomega_sw(i)
             DO j = jnlvls, i+1, -1
                 jcounter = jcounter * ( 1.0d0 - jomega_sw(j) ) 
             END DO ! j = 2, jnlvls
             series_sub1_sw(i) = (1.0d0-rho_albedo(i)) * jcounter
         END DO ! i = 1, jnlvls 
    
         jcounter = 1.0d0
         DO j = jnlvls, 1, -1
             jcounter = jcounter * ( 1.0d0 - jomega_sw(j) ) 
         END DO ! j = 2, jnlvls  
         series_sub1_sw_surf = (1.0d0-rho_albedo(1)) * jcounter
    
    
         DO i=1, jnlvls
             eta_3(i) = 0.0d0    ! initialise variable
             DO j=1, jnlvls
                 jfactor = 1.0d0
    
                 IF (j .gt. i) THEN
                     IF (j-1 .eq. i) THEN
                         ! no transmission coefficient for layer directly above
                     ELSE
                         DO k = j-1, i+1
                             jfactor = jfactor * (1.0d0 - jomega(k))                
                         END DO ! k = j-1, i+1
                     END IF ! (j-1 .eq. i)                 
                     eta_3(i) = eta_3(i) + &
                          &   ( jomega(i) * jomega(j) * jemissivity (j) * &
                          &     (jfactor) * c_stefan * temp_leaf_pres(j)**4.0d0 )
                  END IF ! (j .gt. i)
    
    
                  IF (j .eq. i) THEN
                      eta_3(i) = eta_3(i)
                  END IF ! (j .eq. i)
    
    
                  IF (j .lt. i) THEN
                      IF (j+1 .eq. i) THEN
                          ! no transmission coefficient for layer directly below
                      ELSE
                          DO k = j+1, i-1               
                              jfactor = jfactor * (1.0d0 - jomega(k))                      
                          END DO ! k = j+1, i-1
                      END IF ! (j+1 .eq. i)                 
                      eta_3(i) = eta_3(i) + &
                           &     ( jomega(i) * jomega(j) * jemissivity (j) * &
                           &     (jfactor) * c_stefan * temp_leaf_pres(j)**4.0d0 )
                  END IF ! (j .lt. i)
             END DO ! j = 1, jnlvls
         END DO ! i = 1, jnlvls    
    
    
         ! for radiation from surface layer
    
         DO i=1, jnlvls
    
              jfactor = 1.0d0
    
              IF (i .eq. 1) THEN
                  ! no transmission coefficient for layer directly above
              ELSE
                  DO k = i, 1
                      jfactor = jfactor * (1.0d0 - jomega(k))                
                  END DO ! k = j-1, i+1
              END IF ! (i .eq. 1)                 
    
              eta_3(i) = eta_3(i) + &
                    &   ( jomega(i) * jomega_surf * jemissivity_surf * &
                    &     (jfactor) * c_stefan * temp_surf_pres**4.0d0 )
    
         END DO ! i = 1, jnlvls    
    
    
    
    
         ! now to calculate eta_3_surf for the surface
    
         eta_3_surf = 0.0d0
    
         DO j = 1, jnlvls
             jfactor = 1.0d0  
             DO k = j, 1
                 jfactor = jfactor * (1.0d0 - jomega(k))
             END DO ! k = jnlvls, 1
             eta_3_surf = eta_3_surf + &
                   & (jomega_surf * jomega(j) * jfactor * c_stefan * temp_leaf_pres(j)**4.0d0)
         END DO ! j = 1, jnlvls
    
    
    
        ! now to calculate eta_3_above to check the above canopy radiation balance
    
        eta_3_above = 0.0d0
    
        DO j = jnlvls, 1
            jfactor = 1.0d0
            DO k = jnlvls, j
                jfactor = jfactor * (1.0d0 - jomega(k))
            END DO ! k = jnlvls, j
            eta_3_above = eta_3_above + & 
                           & (jomega(j) * jfactor * c_stefan * temp_leaf_pres(j)**4.0d0)
        END DO ! j = jnlvls, 1
    
    
    
         eta_3_above = eta_3_above + (jomega_surf * jfactor * c_stefan * temp_surf_pres**4.0d0)
                          
         eta_2(1) = -2.0d0 *  jemissivity(1) * jomega(1) * c_stefan * 4.0d0 *                 &
           & (temp_leaf_pres(1)**3.0d0)                                                      &
           & + jomega(1) *(1-jemissivity(2))* jomega(2) * jemissivity(1) * c_stefan * 4.0d0 * &
           & (temp_leaf_pres(1)**3.0d0)                                                      &                                                  
           & + jomega(1) *(1-jemissivity_surf)* jomega_surf *                                &
           & jemissivity(1) * c_stefan * 4.0d0 * (temp_leaf_pres(1)**3.0d0)                                                                  
    
         eta_3(1) = eta_3(1) + &
           & 2.0d0* jemissivity(1) * jomega(1) * c_stefan *3.0d0* (temp_leaf_pres(1)**4.0d0)  &
           & - jomega(1) * (1-jemissivity(2)) * jomega(2) * jemissivity(1) * c_stefan *       &
           & (3.0d0*temp_leaf_pres(1)**4.0d0)                                                &
           & - jomega(1) * (1-jemissivity_surf) * jomega_surf * jemissivity(1) * c_stefan *   &
           & (3.0d0 * temp_leaf_pres(1)**4.0d0) 
    
         ! eta_4(1) = series_sub1_sw(1)
    
    
    
         DO i = 2, jnlvls-1
        
            eta_2(i) = -2.0d0 *  jemissivity(i) * jomega(i) * c_stefan * 4.0d0 *              &
              & (temp_leaf_pres(i)**3.0d0)                                                   &
              & + jomega(i)*(1-jemissivity(i+1))*jomega(i+1)*jemissivity(i)*c_stefan*4.0d0*   &
              &   (temp_leaf_pres(i)**3.0d0)                                                 &
              & + jomega(i)*(1-jemissivity(i-1))*jomega(i-1)*jemissivity(i)*c_stefan*4.0d0*   &
              &   (temp_leaf_pres(i)**3.0d0)
    
                       
             eta_3(i) = eta_3(i) + 2.0d0 * jemissivity(i) * jomega(i) * c_stefan * 3.0d0 *    &
               & (temp_leaf_pres(i)**4.0d0)                                                  &
               & - jomega(i) *(1-jemissivity(i+1))* jomega(i+1)*jemissivity(i) * c_stefan *   &
               & (3.0d0 * temp_leaf_pres(i)**4.0d0)                                          &
               & - jomega(i) *(1-jemissivity(i-1))* jomega(i-1) * jemissivity(i) * c_stefan * &
               & (3.0d0 * temp_leaf_pres(i)**4.0d0) 
             
         END DO ! i = 1, jnlvls
    
    
    
         eta_2(jnlvls) =-2.0d0* jemissivity(jnlvls) * jomega(jnlvls) * c_stefan * 4.0d0 *  &
               & (temp_leaf_pres(jnlvls)**3.0d0)                                            &
               & + jomega(jnlvls)*(1-jemissivity(jnlvls-1)*jomega(jnlvls-1)               &
               & * jemissivity(jnlvls) * c_stefan * 4.0d0 * (temp_leaf_pres(jnlvls)**3.0d0))
         eta_3(jnlvls) = eta_3(jnlvls) + &
                & 2.0d0 * jemissivity(jnlvls) * jomega(jnlvls) * c_stefan * 3.0d0 *         &
                & (temp_leaf_pres(jnlvls)**4.0d0)                                           &
                & - jomega(jnlvls) * (1-jemissivity(jnlvls-1)) * jomega(jnlvls-1)         &
                & * jemissivity(jnlvls) * c_stefan * (3.0d0 * temp_leaf_pres(jnlvls)**4.0d0) 
    
         
    
         eta_2_surf = -1.0d0 * jemissivity_surf * jomega_surf * c_stefan *                    &
                & 4.0d0 * (temp_surf_pres)**3 +                                              &
                & jomega_surf *(1-jemissivity(1)) * jomega(1) * jemissivity_surf * c_stefan * &
                & 4.0d0 * (temp_surf_pres)**3
    
         eta_3_surf = eta_3_surf + 1.0d0 * jemissivity_surf * jomega_surf * c_stefan *        &
               & (3.0d0 * (temp_surf_pres)**4) +                                             &
               & jomega_surf * (1-jemissivity(1)) * jomega(1) * jemissivity_surf * c_stefan * &
               & (3.0d0 * (temp_surf_pres)**4)
    
    
         eta_2_above = 0.0d0
        
    
         IF (ld_jameswrite) THEN
             DO i = jnlvls, 1, -1
                 WRITE (numout,*) 'eta_1 level ', i, ' is: ', eta_1(i)
             END DO ! i = jnlvls, 1, -1
             DO i = jnlvls, 1, -1
                 WRITE (numout,*) 'eta_2 level ', i, ' is: ', eta_2(i)
             END DO ! i = jnlvls, 1, -1
             DO i = jnlvls, 1, -1
                 WRITE (numout,*) 'eta_3 level ', i, ' is: ', eta_3(i)
             END DO ! i = jnlvls, 1, -1
             DO i = jnlvls, 1, -1
                 WRITE (numout,*) 'eta_4 level ', i, ' is: ', eta_4(i)
             END DO ! i = jnlvls, 1, -1
    
             WRITE (numout,*) 'eta_1_surf level is: ', eta_1_surf
             WRITE (numout,*) 'eta_2_surf level is: ', eta_2_surf
             WRITE (numout,*) 'eta_3_surf level is: ', eta_3_surf
             WRITE (numout,*) 'eta_4_surf level is: ', eta_4_surf    
             WRITE (numout,*) '080313 leaving calc_jradiation_scheme'   
         END IF ! (ld_jameswrite)
    
         CALL calc_jradiation_matrix(pad_deltah, ngu_alpha, ngu_alpha_insert, delta_h)

     END DO ! jk = 1, kjpindex


 END SUBROUTINE calc_jradiation_scheme

 ! ---------------------------------------------------------------------------------------

 !!  ===========================================================================================================================
 !! SUBROUTINE                  : calc_jradiation_matrix
 !!
 !!
 !>\BRIEF                   : calculates the radiation matrix, as outlined in reference below 
 !!
 !! DESCRIPTION                 :
 !! 
 !!
 !! RECENT CHANGE(S): None
 !!
 !! MAIN OUTPUT VARIABLE(S) : ngu_alpha, ngu_alpha_insert
 !!
 !! REFERENCE(S)        : Gu, L. et al., 1999. Micrometeorology, biophysical exchanges and NEE decomposition in a two-storey
 !!                                   boreal forest - development and test of an integrated model. Agricultural and Forest 
 !!                                   Meteorology, 94, pp.123–148.
 !!
 !! FLOWCHART           :
 !_ ==============================================================================================================================



 SUBROUTINE calc_jradiation_matrix(pad_deltah, ngu_alpha, ngu_alpha_insert, delta_h)

    IMPLICIT NONE

    !! 0.1 Input variables
    REAL(r_std), DIMENSION (0:jnlvls), INTENT(in)                   :: pad_deltah
    REAL(r_std), DIMENSION(jnlvls), INTENT(in)                      :: delta_h                ! height of each level (m)

    !! 0.2 Output variables
    REAL(r_std), DIMENSION (0:jnlvls+1, 0:jnlvls+1), INTENT(out)    :: ngu_alpha
    REAL(r_std), DIMENSION (0:jnlvls+1)                             :: ngu_alpha_insert    

    !! 0.3 Modified variables

    !! 0.4 Local variables
    INTEGER(i_std)                                                  :: i, j
    INTEGER(i_std)                                                  :: m
     
    REAL(r_std), DIMENSION (0:jnlvls)                               :: jpad    
    REAL(r_std), DIMENSION (0:jnlvls)                               :: acc_pad        
    REAL(r_std)                                                     :: total_ngu_bign        

    ! read in pad (JR 220713)

    m = jnlvls

    DO i = 0, jnlvls
       jpad(i) = pad_deltah(i)
    END DO ! i = 0, jnlvls


    ! define acc_pad
    
    acc_pad(jnlvls) = 0.0d0
    
    DO i = m-1, 0, -1
       acc_pad(i) = jpad(i+1) + acc_pad(i+1)
    END DO ! i = m-1, 0, -1
                       
    IF (ld_jameswrite) THEN
        WRITE (numout,*) 'in jradiation matrix  acc_pad: ', acc_pad(:)
    END IF ! (ld_jameswrite) THEN
 
 
    DO i = 0, m+1   
       DO j = 0, m+1
 
          IF ((i .eq. 0) .and. (j .eq. 0)) THEN
             ngu_alpha(i,j) = -1.0d0
             ngu_alpha_insert(i) = -1.0d0 

          ELSE IF ((i .eq. 0) .and. ((j .ge. 1) .and. (j .le. m))) THEN
             ngu_alpha(i,j) = gu(acc_pad(0) - acc_pad(j-1)) - gu(acc_pad(0)-acc_pad(j))

          ELSE IF ((i .eq. 0) .and. (j .eq. (m+1)))  THEN
             ngu_alpha(i,j) = gu(acc_pad(0))
    
          ELSE IF (((i .ge. 1) .and. (i .le. m)) &
                    &  .and. ((j .ge. 1) .and. (j .le. (i-1)))) THEN
             ngu_alpha(i,j) = gu(acc_pad(j) - acc_pad(i-1)) - gu(acc_pad(j-1)-acc_pad(i-1)) &
                    & - gu(acc_pad(j) - acc_pad(i)) + gu(acc_pad(j-1)-acc_pad(i))
    
          ELSE IF (((i .ge. 1) .and. (i .le. m)) .and. (j .eq. i))  THEN

             ! 220713 this is a departure from the matrix scheme, because I am treating 
             !   the temperature of the infinitesimal soil surface layer, and each 
             !   vegetation layer, as implicit when (i = j). Radiation from each of the 
             !   other levels is treated explicitly - i.e. by using the Gu scheme

             ! following line is from the full scheme, for completeness
             ! ngu_alpha(i,j) = 2.0d0 * gu(acc_pad(i-1) - acc_pad(i)) - 2.0d0 
             ngu_alpha_insert(i) = gu(acc_pad(i-1) - acc_pad(i)) 
             ! test to see influence of absorbed/emitted from the same layer in the cycle
             ngu_alpha(i,j) = 0.0d0 

          ELSE IF (((i .ge. 1).and.(i .le. m)).and.(((j .ge. (i+1)) .and. (j .le. m))))  THEN
             ngu_alpha(i,j) = gu(acc_pad(i) - acc_pad(j-1)) - gu(acc_pad(i)-acc_pad(j))  &
                         & - gu(acc_pad(i-1) - acc_pad(j-1)) + gu(acc_pad(i-1)-acc_pad(j))

          ELSE IF ((i .eq. (m+1)) .and. (j .eq. 0))  THEN
             ngu_alpha(i,j) = gu(acc_pad(0))

          ELSE IF ((i .eq. (m+1)) .and. ((j .ge. 1) .and. (j .le. m)))  THEN
             ngu_alpha(i,j) = gu(acc_pad(j)) - gu(acc_pad(j-1))   

          ELSE IF ((i .eq. (m+1)) .and. (j .eq. (m+1)))  THEN
             ngu_alpha(i,j) = -1.0d0
             ngu_alpha_insert(i) = -1.0d0   

          ! 220713 extra components, not in the Gu manuscript, but seem logical, 
          !   (probably they were omitted by accident?)

          ELSE IF (((i .ge. 1) .and. (i .le. m)) .and. (j .eq. (0)))  THEN
             ngu_alpha(i,j) = gu(acc_pad(0) - acc_pad(i-1)) - gu(acc_pad(0) - acc_pad(i))

          ELSE IF (((i .ge. 1) .and. (i .le. m)) .and. (j .eq. (m+1)))  THEN
             ngu_alpha(i,j) = gu(acc_pad(i)) - gu(acc_pad(i-1))  

          ELSE 

          END IF

       END DO ! j = 0, m+1
    
    END DO ! i = 0, m+1


 END SUBROUTINE calc_jradiation_matrix




 ! ---------------------------------------------------------------------------------------------------------------------------------

 !!  ===========================================================================================================================
 !! SUBROUTINE                  : jcalc
 !!
 !!
 !>\BRIEF                   : calculates the energy, for either the multilayer or the single layer case 
 !!
 !! DESCRIPTION                 : This module calculates the coefficients that are required to determine the energy 
 !!                                               budget at each time step.
 !! 
 !!
 !! RECENT CHANGE(S): None
 !!
 !! MAIN OUTPUT VARIABLE(S) : big_e, big_f, big_g, flux_h_grid, flux_le_grid, a_t, b_t, c_t, d_t, a_q, b_q, c_q, d_q, 
 !!                               t_a_next_grid, q_a_next_grid, horiz_flux_t, horiz_flux_q, temp_surf_next, flux_ground_h, 
 !!                               flux_ground_le, delta_temp, temp_leaf_next_grid, temp_sol_new, delta_temp_save, epot_air_new, 
 !!                               big_r, big_r_prime, speed, vbeta3, eta_1, eta_2, eta_3, eta_4, eta_1_surf, eta_2_surf, 
 !!                               eta_3_surf, eta_4_surf, qair_new, qair, fluxsens, fluxlat, vbeta2sum, vbeta3sum
 !!
 !! REFERENCE(S)        : Best, M.J. et al., 2004. A proposed structure for coupling tiled surfaces with the planetary boundary 
 !!                               layer. Journal of Hydrometeorology, 5, pp.1271–1278.
 !!
 !!                               Polcher, J. et al., 1998. A proposal for a general interface between land surface schemes and general 
 !!                               circulation models. Global and Planetary Change, 19, pp.261–276
 !!    
 !!                               Ryder, J. Towards a multi-layer representation of the energy balance in a land surface model...
 !!                               working document (see DOFOCO cloud)
 !!
 !! FLOWCHART           : 
 !_ ==============================================================================================================================


 SUBROUTINE jcalc (pad_deltah, layer_height, big_r, big_r_prime, delta_z,                 &
                      & k_eddy, swnet, swdown, lwdown,                                    &
                      & temp_atmos_pres_grid, q_atmos_pres_grid,                          &
                      & a_t, b_t, c_t, d_t,                                               &
                      & a_q, b_q, c_q, d_q,                                               &
                      & big_e, big_f, big_g,                                              &
                      & jalpha, jbeta, temp_leaf_pres_grid, delta_h,                      &
                      & eta_1, eta_2, eta_3, eta_4,                                       &
                      & k_eddy_surf, delta_z_surf, jtheta,                                &
                      & jalpha_surf, jbeta_surf, rau,                                     &
                      & temp_surf_pres, soilcap, soilflx, speed,                          &
                      & q_cdrag,                                                          &
                      & kjpindex,                                                         &
                      & flux_ground_h, flux_ground_le, flux_h_grid, flux_le_grid,         &
                      & u_speed, t_a_next_grid, q_a_next_grid,                            &
                      & temp_surf_next,                                                   &
                      & eta_1_surf, eta_2_surf, eta_3_surf, eta_4_surf,                   &
                      & temp_leaf_next_grid, epot_air_new, delta_temp,                    &
                      & horiz_flux_t, horiz_flux_q,                                       &
                      & ngu_bign, ngu_alpha, ngu_alpha_insert, qair_new, qair,            &
                      & emis, temp_sol_new, vbeta1, vbeta4, vbeta5, vbeta2,               &
                      & vbeta3, vbeta3pot, vbeta, fluxsens, fluxlat, netrad,              &
                      & petAcoef, petBcoef, peqAcoef, peqBcoef,                           &
                      & u, v, dtradia, E_supply, E_supply_sum, E_supply_tot,              &
                      & hydrol_flag, hydrol_flag2,                                        &
                      & hydrol_flag3, vbeta2sum, vbeta3sum, veget_max, veget,             &
                      & delta_temp_save, james_step_count,                                &
                      & Collim_Abs_Tot, Collim_Alb_Tot, matt_surf_absorbed,               &
                      & matt_layer_absorbed, transpir_supply_column, pb, profile_vbeta3,  &
                      & profile_rveget, u_speed_grid,                                     &
                      & flux_netrad, flux_netsw, flux_netlw, flux_rn_grid,                &
                      & ks_tune, ks_slope, ks_veget, t2m_month_out) 



     ! 

     IMPLICIT NONE


     ! 0.1 input variables
   

     INTEGER(i_std), INTENT(in)                           :: kjpindex               !! Domain size (-)

     REAL(r_std), INTENT (in)                             :: dtradia                !! Time (s)
     REAL(r_std), INTENT (in)                             :: layer_height           ! layer height (m)

     REAL(r_std), DIMENSION (kjpindex, jnlvls), INTENT(in)          :: temp_leaf_pres_grid         ! leaf temperature down the column (K) PRESENT STEP
     REAL(r_std), DIMENSION (kjpindex, jnlvls), INTENT(in)          :: temp_atmos_pres_grid        ! atmospheric temperature at the present timestep (K)
     REAL(r_std), DIMENSION (kjpindex, jnlvls), INTENT(in)          :: q_atmos_pres_grid           ! specific humidity at the present timestep (kg/kg)


     REAL(r_std), DIMENSION (kjpindex), INTENT (in)       :: q_cdrag                !! Surface drag (m s^{-1})
     REAL(r_std), DIMENSION (jnlvls), INTENT(in)          :: jtheta                 ! specific heat capacity of air (J / (kg K) )

     REAL(r_std), DIMENSION (jnlvls), INTENT (inout)      :: k_eddy                 ! eddy diffusivity between each level (m^2 / s)
     REAL(r_std), INTENT (in)                             :: k_eddy_surf            ! eddy diffusivity at the lowest level (if used)

     REAL(r_std), DIMENSION (kjpindex), INTENT (in)       :: lwdown                 !! Down-welling long-wave flux (W m^{-2})
     REAL(r_std), DIMENSION (kjpindex), INTENT (in)       :: swnet                  !! Net surface short-wave flux (W m^{-2})
     REAL(r_std), DIMENSION (kjpindex), INTENT (in)       :: swdown                 !! Down-welling short-wave flux (W m^2)
     REAL(r_std), DIMENSION (0:jnlvls), INTENT(in)        :: pad_deltah             ! leaf area density at each level (m^2 / m^3)
     REAL(r_std), DIMENSION (jnlvls), INTENT(in)          :: jalpha, jbeta
     REAL(r_std), DIMENSION(kjpindex), INTENT(in)         :: jalpha_surf, jbeta_surf
     REAL(r_std), INTENT(in)                              :: temp_surf_pres
     REAL(r_std), DIMENSION (jnlvls), INTENT(in)          :: delta_z                ! seperation of levels (m)
     REAL(r_std), INTENT(in)                              :: delta_z_surf           ! surface layer thickness (m)
     REAL(r_std), DIMENSION(jnlvls), INTENT(in)           :: delta_h                ! height of each level (m)


     REAL(r_std),DIMENSION (kjpindex), INTENT (in)        :: soilcap          !! Soil calorific capacity (J K^{-1])
     REAL(r_std), DIMENSION (kjpindex), INTENT (in)       :: soilflx          !!     JJJ     Soil flux (W m^{-2})




     REAL(r_std), DIMENSION (jnlvls), INTENT(in)          :: u_speed                !    wind speed at each level (m/s)

     REAL(r_std), DIMENSION (kjpindex), INTENT (in)       :: emis             !! Emissivity (-)

     REAL(r_std), DIMENSION (kjpindex), INTENT(in)        :: rau              !!    JJJ     Air density (kg m^{-3})

     REAL(r_std), DIMENSION (0:jnlvls+1, 0:jnlvls+1), INTENT(in)      :: ngu_alpha


     REAL(r_std), DIMENSION (kjpindex), INTENT (in)          :: u                !! Eastward Lowest level wind speed  (m s^{-1})
     REAL(r_std), DIMENSION (kjpindex), INTENT (in)          :: v                !! Northward Lowest level wind speed (m s^{-1})
     REAL(r_std), DIMENSION (kjpindex), INTENT (in)          :: petAcoef         !! PetAcoef (see note)
     REAL(r_std), DIMENSION (kjpindex), INTENT (in)          :: petBcoef         !! PetBcoef (see note)
     REAL(r_std), DIMENSION (kjpindex), INTENT (in)          :: peqAcoef         !! PeqAcoef (see note)
     REAL(r_std), DIMENSION (kjpindex), INTENT (in)          :: peqBcoef         !! PeqBcoef (see note) 
     REAL(r_std), DIMENSION (kjpindex), INTENT (inout)       :: vbeta            !! Resistance coefficient (-)
     REAL(r_std), DIMENSION (kjpindex), INTENT (in)          :: vbeta1           !! Snow resistance (-) 
     REAL(r_std), DIMENSION (kjpindex), INTENT (in)          :: vbeta4           !! Bare soil resistance (-)
     REAL(r_std), DIMENSION (kjpindex), INTENT (in)          :: vbeta5           !! Floodplains resistance
     REAL(r_std), DIMENSION (kjpindex,nvm), INTENT (in)      :: vbeta2           !! Interception resistance (-)
     REAL(r_std), DIMENSION (kjpindex,nvm), INTENT (in)      :: vbeta3pot        !! Vegetation resistance for potential transpiration
!    REAL(r_std), DIMENSION (kjpindex,nvm), INTENT (in)     :: vbetaco2         !! Vegetation resistance to CO2 (-)
     REAL(r_std), DIMENSION (kjpindex,nvm), INTENT (in)      :: E_supply         !! Supply of water for transpiration @tex $(m.s^{-1})$ @endtex
     REAL(r_std), DIMENSION(kjpindex), INTENT (in)           :: E_supply_tot     !! total supply of water for transpiration for each domain (m s^{-1}))
     REAL(r_std),  DIMENSION (kjpindex, nvm), INTENT(in)     :: veget_max        !! Max. fraction of vegetation type (LAI -> infty, unitless)
     REAL(r_std),  DIMENSION (kjpindex, nvm), INTENT(in)     :: veget            !! Vegetation fraction for each pft 
     INTEGER, INTENT(in)                                     :: james_step_count
     REAL(r_std), DIMENSION(nlevels_tot, kjpindex, nvm), INTENT(in) &
                                                             :: transpir_supply_column     !! Supply of water for transpiration 

     REAL(r_std),DIMENSION (kjpindex,nvm,nlevels_tot), INTENT(in) &
                                                             :: profile_vbeta3
     REAL(r_std),DIMENSION (kjpindex,nvm,nlevels_tot), INTENT(in) &
                                                             :: profile_rveget
     REAL(r_std),DIMENSION (kjpindex), INTENT(in)            :: t2m_month_out !! An output of t2m_longterm, from stomate season 

     REAL(r_std), INTENT(in)                                 :: ks_tune          !! coefficient for tuning the forest floor evapotranspiration
     REAL(r_std), INTENT(in)                                 :: ks_slope         !! coefficient for S-shape function 
     REAL(r_std), INTENT(in)                                 :: ks_veget         !! coefficient for S-shape function

     ! 0.2 output variables
     REAL(r_std), DIMENSION (jnlvls), INTENT(out)            :: big_e, big_f, big_g
     REAL(r_std), DIMENSION (kjpindex, jnlvls), INTENT(out)  :: flux_h_grid, flux_le_grid  ! sensible and latent heat flux respectively at each level (both W/m^2)
     REAL(r_std), DIMENSION (kjpindex, jnlvls), INTENT(out)  :: flux_rn_grid               ! net radiation flux at each level (W/m^2)
     REAL(r_std), DIMENSION (kjpindex, jnlvls), INTENT(out)  :: u_speed_grid               ! wind profile at each level (m/s)
     REAL(r_std), DIMENSION (jnlvls), INTENT(out)            :: a_t, b_t, c_t, d_t
     REAL(r_std), DIMENSION (jnlvls), INTENT(out)            :: a_q, b_q, c_q, d_q 
     REAL(r_std), DIMENSION (kjpindex, jnlvls), INTENT(out)  :: t_a_next_grid
     REAL(r_std), DIMENSION (kjpindex, jnlvls), INTENT(out)  :: q_a_next_grid
     REAL(r_std), DIMENSION (jnlvls), INTENT(out)            :: horiz_flux_t,  horiz_flux_q
     REAL(r_std), INTENT(out)                                :: temp_surf_next         ! temperature of surface at respectively present and next timestep (K)
     REAL(r_std), INTENT(out)                                :: flux_ground_h, flux_ground_le    ! sensible and latent heat flux respectively from the ground (both W/m^2)
     REAL(r_std), INTENT(out)                                :: delta_temp             ! change in temperature over the time step (Kelvins)
     REAL(r_std), DIMENSION (kjpindex, jnlvls), INTENT(out)  :: temp_leaf_next_grid
     REAL(r_std), INTENT(out)                                :: matt_surf_absorbed
     REAL(r_std), DIMENSION(jnlvls), INTENT(out)             :: matt_layer_absorbed
  
     REAL(r_std), DIMENSION (jnlvls), INTENT(out)            :: flux_netrad            ! net radiation flux for each layer  
     REAL(r_std), DIMENSION (jnlvls), INTENT(out)            :: flux_netsw             ! net radiation contributed from short-wave
     REAL(r_std), DIMENSION (jnlvls), INTENT(out)            :: flux_netlw             ! net radiation contributed from long-wave

     ! 0.3 modified variables
     REAL(r_std), DIMENSION (kjpindex), INTENT(inout)        :: temp_sol_new       !! New ground temperature (K)
     REAL(r_std),DIMENSION (kjpindex), INTENT(inout)         :: delta_temp_save     
     REAL(r_std), DIMENSION (kjpindex), INTENT(inout)                    :: epot_air_new
     REAL(r_std), DIMENSION (jnlvls), INTENT(inout)                      :: big_r, big_r_prime     ! stomatal resistance for sensible and latent heat flux at each level (s/m)    
     REAL(r_std), INTENT(inout)                                          :: speed                  ! wind speed above the canopy (m/s), as defined by old enerbil
     REAL(r_std), DIMENSION (kjpindex,nvm), INTENT (inout)               :: vbeta3           !! Vegetation resistance (-)
     REAL(r_std), DIMENSION (jnlvls), INTENT (inout)                     :: eta_1, eta_2
     REAL(r_std), DIMENSION (jnlvls), INTENT (inout)                     :: eta_3, eta_4
     REAL(r_std), INTENT (inout)                                         :: eta_1_surf, eta_2_surf
     REAL(r_std), INTENT (inout)                                         :: eta_3_surf, eta_4_surf
     REAL(r_std), DIMENSION (kjpindex), INTENT (inout)                   :: qair_new, qair
     REAL(r_std), DIMENSION (kjpindex), INTENT(inout)                    :: fluxsens         !! Sensible heat flux (W m^{-2})
     REAL(r_std), DIMENSION (kjpindex), INTENT(inout)                    :: fluxlat          !! Latent heat flux (W m^{-2})     
     REAL(r_std), DIMENSION (kjpindex), INTENT (inout)                   :: vbeta2sum, vbeta3sum
     REAL(r_std), DIMENSION (kjpindex), INTENT(inout)                    :: netrad           !! Net radiation flux (W m^{-2})   added by YC 
     REAL(r_std),DIMENSION(nlevels_tot), INTENT(inout)                   :: Collim_Abs_Tot   !! collimated total absorption for a given level
     REAL(r_std),DIMENSION(nlevels_tot), INTENT(inout)                   :: Collim_Alb_Tot   !! Collimated (direct) total albedo for a given level

     ! 0.4 local variables
     INTEGER(i_std)                                 :: i, j, m, n, ji, jv, jk, jm
     REAL(r_std), DIMENSION (jnlvls)                :: x_1, x_2, x_3, x_4, x_5, x_6
     REAL(r_std), DIMENSION (jnlvls)                :: y_1, y_2, y_3, y_4, y_5, y_6 
     REAL(r_std), DIMENSION (jnlvls)                :: t_a_pres
     REAL(r_std), DIMENSION (jnlvls)                :: q_a_pres
     REAL(r_std), DIMENSION (jnlvls)                :: t_l_pres, t_l_next
     REAL(r_std), DIMENSION (jnlvls)                :: q_l_pres, q_l_next
     REAL(r_std), DIMENSION (jnlvls)                :: big_e_top
     REAL(r_std), DIMENSION (jnlvls)                :: big_f_top
     REAL(r_std), DIMENSION (jnlvls)                :: big_g_top
     REAL(r_std), DIMENSION (jnlvls)                :: big_efg_bottom
     REAL(r_std), DIMENSION (jnlvls)                :: source_sink_t, source_sink_q
     REAL(r_std), DIMENSION (jnlvls)                :: source_sink_sum_t, source_sink_sum_q
     REAL(r_std), DIMENSION (jnlvls)                :: source_sink_t_l
     REAL(r_std), DIMENSION (jnlvls)                :: temp_leaf_pres         ! leaf temperature down the column (K) PRESENT STEP
     REAL(r_std), DIMENSION (jnlvls)                :: temp_atmos_pres        ! atmospheric temperature at the present timestep (K)
     REAL(r_std), DIMENSION (jnlvls)                :: q_atmos_pres           ! specific humidity at the present timestep (kg/kg)
     REAL(r_std), DIMENSION (jnlvls)                :: flux_h                 ! sensible heat flux at each level (both W/m^2)
     REAL(r_std), DIMENSION (jnlvls)                :: flux_le                ! latent heat flux at each level (both W/m^2)
     REAL(r_std), DIMENSION (jnlvls)                :: t_a_next
     REAL(r_std), DIMENSION (jnlvls)                :: q_a_next

     REAL(r_std)                                    :: big_omega_1, big_omega_2
     REAL(r_std)                                    :: big_omega_3, big_omega_4
     REAL(r_std)                                    :: big_omega_5, big_omega_6
     REAL(r_std)                                    :: big_omega_7, big_omega_8
     REAL(r_std)                                    :: big_omega_5_sub, big_omega_6_sub
     REAL(r_std)                                    :: big_omega_7_sub, big_omega_8_sub
     REAL(r_std)                                    :: xi_1, xi_2, xi_3, xi_4

!    REAL(r_std)                                    :: source_sink_sum_t
!    REAL(r_std)                                    :: source_sink_sum_q

     REAL(r_std)                                    :: fevap, turb_factor
     REAL(r_std)                                    :: M_1_h, N_1_h, M_1_q, N_1_q

     REAL(r_std)                                    :: jtchalev0, jtcp_air, jtrau
     REAL(r_std), DIMENSION (kjpindex)              :: beta_diffuco           ! the 'beta coefficient' from the existing scheme, for the surface layer (-) 
     REAL(r_std), DIMENSION (kjpindex)              :: beta_diffuco_evap           ! the 'beta coefficient' from the existing scheme, for the surface layer (-)
     REAL(r_std), DIMENSION (kjpindex)              :: beta_diffuco_sub           ! the 'beta coefficient' from the existing scheme, for the surface layer (-)
     REAL(r_std), DIMENSION (kjpindex)              :: beta_diffuco_evap_notrans
     REAL(r_std), DIMENSION (kjpindex)              :: beta_diffuco_evap_trans
     REAL(r_std), DIMENSION (kjpindex)              :: E_supply_sum

     REAL(r_std), DIMENSION (kjpindex)              :: evap_fraction          !! a evapotranspiration fraction based on vbeta3, vbeta4 and other paramters  

     REAL(r_std), DIMENSION (jnlvls)                :: delta_t_a, sum_flux_a
     REAL(r_std), DIMENSION (jnlvls)                :: delta_q_a, sum_flux_q_a

     REAL(r_std), DIMENSION (jnlvls)                :: delta_temp_leaf
     REAL(r_std), DIMENSION (jnlvls)                :: sum_flux_leaf

     REAL(r_std), DIMENSION (0:jnlvls+1)            :: gu_alpha_sum
     REAL(r_std), DIMENSION (0:jnlvls+1)            :: ngu_temp_solid, ngu_bign
     REAL(r_std), DIMENSION (0:jnlvls+1)            :: ngu_alpha_insert    
     REAL(r_std), DIMENSION (0:jnlvls+1)            :: ngu_alpha_insert2    

     LOGICAL                                        :: hydrol_flag            !! Flag that recalculates the energy budget for the new supply term (true/false)    
     LOGICAL, DIMENSION (kjpindex), INTENT(in)      :: hydrol_flag2
     LOGICAL, DIMENSION (kjpindex, nvm), INTENT(in) :: hydrol_flag3           !! flag that 'trips' the alternative energy budget for each grid square
 
     REAL(r_std), DIMENSION (0:jnlvls)              :: pad_deltah_temp        ! leaf area density at each level (m^2 / m^3)

     INTEGER                                        :: convergence_count

     REAL(r_std), DIMENSION (jnlvls)                :: phi_trans


     ! 090215 new column variables

     REAL(r_std), DIMENSION (jnlvls)                :: column_air_relhum
     REAL(r_std), DIMENSION (jnlvls)                :: column_q_surf_leaf
     REAL(r_std), DIMENSION (jnlvls)                :: column_r_sv_leaf
     REAL(r_std), DIMENSION (jnlvls)                :: column_r_sto_v_leaf
     REAL(r_std), DIMENSION (jnlvls)                :: column_evap
     REAL(r_std), DIMENSION (jnlvls)                :: column_qsatt_leaf

     REAL(r_std), DIMENSION (kjpindex), INTENT (in)            :: pb                !! Lowest level pressure (Pa)

     REAL(r_std), DIMENSION (kjpindex)              :: ks_weight_1   ! for the calculation of evap_fraction 
     REAL(r_std), DIMENSION (kjpindex)              :: ks_weight_2   ! weight_1 for vbeta3; weight_2 for vbeta4 
     REAL(r_std)                                    :: temp_sol_err  ! the abslute error of temp_sol surface temperatrure 
     REAL(r_std)                                    :: epsilon_err   ! mechain error + numerical error set to 1.0E-15  
     REAL(r_std)                                    :: temp_sol_dummy1
     REAL(r_std)                                    :: temp_sol_dummy2
 !! ==============================================================================================================================
     !      
     epsilon_err = 1.0E-15
     
     ! initialise variables

     phi_trans(:) = 0.0d0 !1.0d4

     flux_h(:) = 0.0d0; flux_le(:) = 0.0d0

     source_sink_t(:) = 0.0d0; source_sink_q(:) = 0.0d0
     temp_surf_next = 0.0d0

     big_e = 0.0d0; big_f = 0.0d0; big_g = 0.0d0

     i = 0; ji = 0
    
     x_1 = 0.0d0; x_2 = 0.0d0; x_3 = 0.0d0; 
     x_4 = 0.0d0; x_5 = 0.0d0; x_6 = 0.0d0
     y_1 = 0.0d0; y_2 = 0.0d0; y_3 = 0.0d0; 
     y_4 = 0.0d0; y_5 = 0.0d0; y_6 = 0.0d0

     ! t_a_pres = temp_atmos_pres
     t_a_next = 0.0d0
     ! q_a_pres = q_atmos_pres
     q_a_next = 0.0d0

     t_l_next = 0.0d0; q_l_pres = 0.0d0; q_l_next = 0.0d0
    
     ! n.b. set resistance of uppermost layer to <approaching infinity>
     big_r(jnlvls) = 1.0d20; big_r_prime(jnlvls) = 1.0d20

     matt_surf_absorbed = 0.0d0
     matt_layer_absorbed = 0.0d0


     ! 121013 we need to re-calculate beta_diffuco_evap_trans, specifically vbeta3sum

     IF (ld_jameswrite) THEN
         WRITE(numout, *) 'in jcalc, big_r_prime is: ', big_r_prime
         WRITE(numout, *) 'in jcalc, profile_rveget is: ', profile_rveget(test_grid,test_pft, :)
     END IF ! (ld_jameswrite)


     hydrol_flag = .FALSE.

    gridloop: DO ji=1, kjpindex  ! -----------------------------------------------------------------

        ! $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$  start convergence loop here  $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
         
         temp_sol_err = 0.0_r_std
         convergence_loop: DO jm = 1, 20
         
         !++++++++++copy the solved temperature for next time step i
         ! from previous covergence loop step result   
         temp_sol_dummy1 = temp_surf_next 
         !++++++++++++++++++++++++++++++

           
                    ! calculate the revised vbeta3sum, based on the revised vbeta3

             IF (hydrol_flag) THEN  ! overall hydrol flag

               
                vbeta3sum(ji) = 0.0d0
                DO jv = 1, nvm
!!$                   !++++++++++
!!$                   IF(ji == test_grid .AND. jv == test_pft)THEN
!!$                      WRITE(numout,*) 'vbeta3 vnkldsvd 1',vbeta3(ji,jv)
!!$                   ENDIF
!!$                   !+++++++++++
                   vbeta3sum(ji) = vbeta3sum(ji) + vbeta3(ji,jv)
                ENDDO ! jv = 1, nvm    

                vbeta(ji) = vbeta4(ji) + vbeta2sum(ji) + vbeta3sum(ji)

             END IF ! IF (hydrol_flag .eq. .TRUE.) 

             temp_leaf_pres(:) = temp_leaf_pres_grid(ji,:)
             t_a_pres(:) = temp_atmos_pres_grid(ji,:) 
             q_a_pres(:) = q_atmos_pres_grid(ji,:)
             t_l_pres(:) = temp_leaf_pres_grid(ji,:)
    
    
             !+++++ CHECK ++++++
             ! Commenting out this loop to fix a bug which caused incorrect results on large scale simulations.
             ! Apparently, if we exit here some variables are not getting properly calculated for the pixels
             ! which have no water stress.  This will cause a performance problem, so best to continue
             ! looking into this.  One variable where it was very noticable was temp_sol_new.
             !!$      IF (hydrol_flag) THEN
             !!$         ! are we on the second call of enerbil, with the corrections to transpiration
             !!$         IF (.NOT. hydrol_flag2(ji)) THEN
             !!$            ! no need to run jcalc again if (E_supply .gt. transpir_mod) for all PFTs at this
             !!$            !   particular grid point
             !!$            ! WRITE (*,*) '091013 hydrol_flag2 is false so we are exiting the gridloop'
             !!$                   t_a_next_grid(ji,:) = temp_atmos_pres_grid(ji,:)
             !!$                   q_a_next_grid(ji,:) = q_atmos_pres_grid(ji,:)
             !!$                   temp_leaf_next_grid(ji,:) = temp_leaf_pres_grid(ji,:)
             !!$
             !!$                   temp_sol_new(ji) = temp_sol_new(ji)
             !!$                   IF(test_grid == ji) WRITE(numout,*) "ruieozenerbil jcalc 1",temp_sol_new(ji)
             !!$                 temp_surf_next = temp_surf_next
             !!$                   qsol_sat_new(ji) = qsol_sat_new(ji)
             !!$                   qair(ji) = qair(ji)
             !!$                   qair_new(ji) = qair(ji)
             !!$
             !!$                   fluxsens(ji) = fluxsens(ji)
             !!$                   fluxlat(ji) = fluxlat(ji)
             !!$                   vbeta2sum(ji) = vbeta2sum(ji)
             !!$                   vbeta3sum(ji) = vbeta3sum(ji)
             !!$                   vbeta3(ji,:) = vbeta3(ji,:)
             !!$
             !!$            CYCLE gridloop
             !!$
             !!$         ELSE
             !!$
             !!$            ! hydrol_flag is not set, so carry on with the energy budget calculation
             !!$
             !!$         END IF ! IF (hydrol_flag2(ji) .eq. .FALSE.)
             !!$      ELSE
             !!$         ! we are on the first call of enerbil
             !!$      END IF ! IF (hydrol_flag .eq. .TRUE.) 
             !+++++++++++++++++++++++++++++++++++++++++++
    
             ! we are inside the loop over grid squares - speed is calculated from the horizontal
             !   velocities u and v, which are the input parameters
             speed = MAX(min_wind, SQRT (u(ji)*u(ji) + v(ji)*v(ji)))
    
    
             ! %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
             !! A note on the A and B coefficients:
             !! The values passed down from the coupled model are petAcoef, petBcoef, peqAcoef and
             !!   peqBcoef, allocated per gridsquare.
             !!
             !! n.b. for the uncoupled mode, only b_t and b_q (at the uppermost level) have a value. 
             !!      All of the other coefficients are zero.
             !!
             !! however, for the coupled mode, a_t, b_t, a_q and b_q should all have values.
             !! c_t, d_t, c_q and d_q should not have values as they are derived internal to enerbil
             !! (they represent the interaction between the atmosphere in the canopy and the vegetation)
             !! and so are all initialised here.
             !!
             !! The interaction between the coefficients within the module 'surf_land_orchidee' (within LMDZ) 
             !! is as follows:
             !! 
             !! petAcoef (orchidee) = petBcoef (lmdz) * delta_t
             !!
             !! petBcoef (orchidee) = petAcoef (lmdz)
             !!
             !! peqAcoef (orchidee) = peqBcoef (lmdz) * delta_t
             !!
             !! peqBcoef (orchidee) = peqAcoef (lmdz)
    
         !! enerbil_equiv ========================================
         a_t(jnlvls) = petAcoef(ji)        ! n.b. kjpindex = 1 corresponds to the bare soil PFT
         b_t(jnlvls) = petBcoef(ji)/cp_air ! n.b. kjpindex = 1 corresponds to the bare soil PFT 
                                           ! (petBcoef is in units of surface static energy - 
                                               ! (cp_air * temp) as pressure difference between the 
                                               ! surface and the lowest layer of the atmosphere is 
                                               ! assumed negligable)
         ! ======================================================
         c_t(jnlvls) = 0.0d0
         d_t(jnlvls) = 0.0d0
    
    
         ! ====================================================================================
         a_q(jnlvls) = peqAcoef(ji) ! n.b. kjpindex = 1 corresponds to the bare soil PFT
         b_q(jnlvls) = peqBcoef(ji) ! n.b. kjpindex = 1 corresponds to the bare soil PFT 
                                    !      (peqBcoef is already in units of (kg/kg)
         ! ====================================================================================
         c_q(jnlvls) = 0.0d0
         d_q(jnlvls) = 0.0d0
    
    
         beta_diffuco(ji) =  1.0d0 * vbeta1(ji) * (un - vbeta5(ji)) + &
               & (un - vbeta1(ji)) * (un - vbeta5(ji)) * vbeta(ji) + vbeta5(ji)
    
    
         ! seperating the beta_diffuco term above into expressions for sublimination (snow) and
         !   evaporation
         beta_diffuco_sub(ji) = vbeta1(ji) * (1.0d0 - vbeta5(ji))
         beta_diffuco_evap(ji) = (1.0d0 - vbeta1(ji)) * &
                                  & (1.0d0 -vbeta5(ji)) * vbeta(ji) + vbeta5(ji)
    
    
         ! the version of the evaporation expression above for which the transpiration term is 
         !   removed
         beta_diffuco_evap_notrans(ji) = (1.0d0 - vbeta1(ji)) * &
                                  & (1.0d0 - vbeta5(ji)) * (vbeta4(ji) + vbeta2sum(ji)) + vbeta5(ji)
    
         ! the version of the evaporation expression above for which the transpiration term is 
         !   included? (check this...)
         beta_diffuco_evap_trans(ji) = (1.0d0 - vbeta1(ji)) * &
                                  & (1.0d0 - vbeta5(ji)) * vbeta3sum(ji)


 
    
    
    
         ! Gu radiation total calculation 
    
         IF (jnlvls .eq. 1) THEN
    
            eta_1_surf = 1.0d0 - (1.0d0 - emis(ji))
            eta_4_surf = 1.0d0
    
            eta_2_surf = - c_stefan * 4.0d0 * emis(ji) * (psold(ji)/cp_air)**3
            eta_3_surf = - 3.0d0 * c_stefan * emis(ji) * ((psold(ji)/cp_air)**4)
    
            DO i = 1, jnlvls
    
                eta_1(i) = 0.0d0 
    
                eta_2(i) = c_stefan * 4.0d0 * temp_leaf_pres(i)**3
    
                eta_3(i) = c_stefan * 3.0d0 * temp_leaf_pres(i)**4 

                eta_4(i) = 0.0d0
    
            END DO ! i = 1, jnlvls

            ELSE

               ngu_temp_solid(0) = c_stefan * ((psold(ji)/cp_air)**4)
        
               DO i = 1, jnlvls
                  ngu_temp_solid(i) = c_stefan * (temp_leaf_pres(i)**4) 
               END DO ! i = 1, jnlvls
    
               ngu_temp_solid(jnlvls+1) = lwdown(ji)
    
               ngu_bign(:) = 0.0d0

               ngu_alpha_insert2(0) = - 1.0d0 * ngu_temp_solid(0)
               DO i = 1, jnlvls
                     ngu_alpha_insert2(i) = (2.0d0 * ngu_alpha_insert(i) - 2.0d0) * ngu_temp_solid(i)
               END DO ! i = 1, jnlvls 
               ngu_alpha_insert2(jnlvls+1) = - 1.0d0 * ngu_temp_solid(jnlvls+1)


               DO i = 0, jnlvls + 1 
                  DO j = 0, jnlvls + 1
                     IF (i .ne. j) THEN
                        ngu_bign(i) = ngu_bign(i) + ( ngu_alpha(i,j) * ngu_temp_solid(j) )
                     ELSE
                        ngu_bign(i) = ngu_bign(i) + 0.0d0
                     END IF ! (i .ne. j)            
                  END DO ! j = 0, jnlvls + 1
               END DO ! i = 0, jnlvls + 1
    
               ! %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        
               DO i = 1, jnlvls
    
                   eta_1(i) = 0.0d0 
    
                   eta_2(i) = (2.0d0*ngu_alpha_insert(i)-2.0d0) &
                            &  * c_stefan * 4.0d0 * temp_leaf_pres(i)**3
    
                   eta_3(i) = -(2.0d0*ngu_alpha_insert(i) - 2.0d0) &
                            &  * c_stefan * 3.0d0 * temp_leaf_pres(i)**4 + ngu_bign(i)
    
               END DO ! i = 1, jnlvls
        
               ! The Gu method assumes black body emitters for simplicity
       
               eta_1_surf = 0.0d0
    
               eta_2_surf = - 4.0d0 * c_stefan * (((1.0d0/cp_air) * psold(ji))**3)      
    
               eta_3_surf = - (c_stefan * (3.0d0 * (psold(ji)/cp_air)**4) + ngu_bign(0))

               ! jjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjj
                IF ((control%ok_new_enerbil_nextstep ).AND. (SUM(collim_abs_tot(:)) > min_sechiba)) THEN
 
                   matt_surf_absorbed = 0.0d0

                   DO i = 1 + 1, jnlvls_canopy + 1
                     
                         ! +++ TEMP +++ 
                         ! test for night time air temperature gradient 
                         ! 230515 add (need to discusse with James for 
                         ! checking the calculation of soil surface absorption
                         ! equals to summation of ( leaf surafce absorption +
                         ! leaf surface albedo ) in each level instead of
                         ! surface level only
                         ! ++++++++++++++++++ 
                           matt_surf_absorbed = matt_surf_absorbed + Collim_Abs_Tot(i) + Collim_Alb_Tot(i)  
                         ! matt_surf_absorbed = matt_surf_absorbed + Collim_Abs_Tot(i)       
                   END DO

                  ! comment out for the testing 
                  ! matt_surf_absorbed = 1.0d0 - (matt_surf_absorbed + Collim_Alb_Tot(jnlvls_canopy+1) )
                   matt_surf_absorbed = (1.0d0 - matt_surf_absorbed )


                   IF (ld_jameswrite) THEN
                       WRITE (numout, *) '230315 matt_surf_absorbed is: ', matt_surf_absorbed 
                       WRITE (numout, *) '110315 Collim_Alb_Tot is: ', Collim_Alb_Tot
                       WRITE (numout, *) '110315 Collim_Abs_Tot is: ', Collim_Abs_Tot
                   END IF ! (ld_jameswrite)!

                   matt_layer_absorbed = 0.0d0

                   DO i = 2, jnlvls_canopy+1
                       matt_layer_absorbed(i + jnlvls_under - 1) = Collim_Abs_Tot(i)
                   END DO

                   IF (ld_jameswrite) THEN
                       WRITE (numout, *) '110315 matt_layer_absorbed is: ', matt_layer_absorbed
                   END IF ! (ld_jameswrite

                   eta_4(:) = matt_layer_absorbed 
                   eta_4_surf = matt_surf_absorbed 

                   IF (ld_jameswrite) THEN
                       WRITE (numout, *) '130515 leaf level absorption eta_4:', eta_4(:)
                       WRITE (numout, *) '130515 soil surface absorption eta_4_surf:', eta_4_surf
                   END IF ! (ld_jameswrite) THEN


               END IF ! (control%ok_new_enerbil_nextstep)

           END IF ! (jnlvls doesn't equal 1)    

          ! this is just for the lowermost vegetation layer...)
          ! special case for 'single layer model', which should reduce to the ORCHIDEE trunk
    
          IF (jnlvls .eq. 1) THEN
    
              x_1(1) = 0.0d0; x_2(1) = 0.0d0; x_3(1) = 0.0d0; x_4(1) = 0.0d0; x_5(1) = 0.0d0
    
              y_1(1) = 0.0d0; y_2(1) = 0.0d0; y_3(1) = 0.0d0; y_4(1) = 0.0d0; y_5(1) = 0.0d0
        
              DO i = jnlvls, jnlvls
    
                   big_e_top(i) = (-(dtradia*a_q(i)*chalev0*rau(ji))/                                &
                      &  (rho_veg*leaf_tks*pad_deltah(i)*delta_h(i)*                                 &
                      &  big_r_prime(i)*jtheta(i)))                                                  &
                      &  - ((dtradia* d_t(i) * cp_air * rau(ji)) /                                   & 
                      & (rho_veg*leaf_tks * pad_deltah(i) * delta_h(i) * big_r(i) * jtheta(i)))   
    
                   big_f_top(i) = (-(dtradia*a_t(i)*cp_air*rau(ji))/                                 &
                      & (rho_veg*leaf_tks*pad_deltah(i)*delta_h(i)*                                  &
                      & big_r(i)*jtheta(i)))                                                         &
                      & -((dtradia* d_q(i) * chalev0 * rau(ji)) /                                    &
                      & (rho_veg*leaf_tks * pad_deltah(i)* delta_h(i) * big_r_prime(i) * jtheta(i)))
    
                   big_g_top(i) = (t_l_pres(i) + dtradia* eta_3(i)/                                  &
                      & (rho_veg*leaf_tks * pad_deltah(i) * delta_h(i) * jtheta(i))                  &  
                      &  + (eta_1(i) * lwdown(ji) * dtradia)/                                        &
                      & (rho_veg*leaf_tks * pad_deltah(i) * delta_h(i) * jtheta(i))                  &
                      &  + (eta_4(i) * swdown(ji) * dtradia)/                                         &
                      & (rho_veg*leaf_tks * pad_deltah(i) * delta_h(i) * jtheta(i))                  &
                      &  + (chalev0 * rau(ji) * dtradia* jbeta(i))/                                  &
                      & (rho_veg*leaf_tks * pad_deltah(i) *delta_h(i) *  big_r_prime(i) * jtheta(i)) &
                      &  - (dtradia* cp_air * rau(ji) * b_t(i))/                                     &
                      & (rho_veg*leaf_tks * pad_deltah(i) * delta_h(i) * big_r(i) * jtheta(i))       &
                      &  - (dtradia* chalev0 * rau(ji) * b_q(i))/                                    &
                      & (rho_veg*leaf_tks * pad_deltah(i) * delta_h(i) * big_r_prime(i) * jtheta(i)))
    
                   big_efg_bottom(i) =                                                               &
                      & ( 1.0d0 - (dtradia* jalpha(i) * chalev0 * rau(ji))/                          &
                      &  (rho_veg*leaf_tks * pad_deltah(i) * delta_h(i) * big_r_prime(i)*jtheta(i))  &
                      &  - (dtradia* cp_air * rau(ji)) /                                             &
                      & (rho_veg*leaf_tks * pad_deltah(i) * delta_h(i) * jtheta(i) * big_r(i))       &
                      &  - (eta_2(i) * dtradia) /                                                    & 
                      & (rho_veg *leaf_tks * pad_deltah(i) * delta_h(i) * jtheta(i))                 &
                      &  + (dtradia* chalev0 * rau(ji) * cp_air * c_t(i)) /                          &
                      & (rho_veg *leaf_tks * pad_deltah(i) * delta_h(i) *                            &
                      &  jtheta(i) * big_r(i)) )
    
                   big_e(i) = big_e_top(i) / big_efg_bottom(i)
     
                   big_f(i) = big_f_top(i) / big_efg_bottom(i)
    
                   big_g(i) = big_g_top(i) / big_efg_bottom(i)
    
              END DO ! i = jnlvls, jnlvls
        
          ELSE ! IF (jnlvls doesn't equal 1)....
        
              DO i = jnlvls, jnlvls

                  IF (hydrol_flag) THEN

                      big_e_top(i) =                                                                     &
                          &   ((dtradia* d_t(i) * cp_air * rau(ji)) /                                    & 
                          & (rho_veg*leaf_tks * pad_deltah(i) * delta_h(i) * jtheta(i) * big_r(i)))   
 
                      big_f_top(i) = ( + (dtradia * a_t(i) * cp_air * rau(ji))/                           &
                          & (rho_veg*leaf_tks * pad_deltah(i)* delta_h(i)* jtheta(i) * big_r(i)) )    

                      big_g_top(i) = (t_l_pres(i) + dtradia* eta_3(i)/                                   &
                          & (rho_veg*leaf_tks * pad_deltah(i) * delta_h(i) * jtheta(i))                  & 
                          &  + (eta_1(i) * lwdown(ji) * dtradia)/                                        &
                          & (rho_veg*leaf_tks * pad_deltah(i) * delta_h(i) * jtheta(i))                  &
                          &  + (eta_4(i) * swdown(ji) * dtradia)/                                         & 
                          & (rho_veg*leaf_tks * pad_deltah(i) * delta_h(i) * jtheta(i))                  &
                          &  + (dtradia * cp_air * rau(ji) * b_t(i))/                                    &
                          & (rho_veg*leaf_tks * pad_deltah(i) * delta_h(i) * jtheta(i) * big_r(i) )      &
                          &  - (dtradia * phi_trans(i)) / (rho_veg*leaf_tks * pad_deltah(i) * delta_h(i) * jtheta(i)))

                     big_efg_bottom(i) =                                                                 &
                         & ( 1.0d0                                                                     &
                         &  + (dtradia * cp_air * rau(ji)) /                                             &
                         & (rho_veg*leaf_tks * pad_deltah(i) * delta_h(i) * jtheta(i) * big_r(i))        &
                         &  - (eta_2(i) * dtradia) /                                                     & 
                         & (rho_veg*leaf_tks * pad_deltah(i) * delta_h(i) * jtheta(i))                   &
                         &  - (dtradia * chalev0 * rau(ji) * cp_air * c_t(i)) /                          &
                         & (rho_veg*leaf_tks * pad_deltah(i) * delta_h(i) * jtheta(i) * big_r(i)) )
    
                     big_e(i) = big_e_top(i) / big_efg_bottom(i)
       
                     big_f(i) = big_f_top(i) / big_efg_bottom(i)
    
                     big_g(i) = big_g_top(i) / big_efg_bottom(i)

                  ELSE ! following if not (hydrol_flag)

                     big_e_top(i) = (+(dtradia*a_q(i)*chalev0*rau(ji))/                                 &
                         &  (rho_veg*leaf_tks*pad_deltah(i)*delta_h(i)*                                 &
                         &  big_r_prime(i)*jtheta(i)))                                                  &
                         &  + ((dtradia* d_t(i) * cp_air * rau(ji)) /                                   & 
                         & (rho_veg*leaf_tks * pad_deltah(i) * delta_h(i) * big_r(i) * jtheta(i)))   
    
                     big_f_top(i) = (+(dtradia*a_t(i)*cp_air*rau(ji))/                                  &
                         & (rho_veg*leaf_tks*pad_deltah(i)*delta_h(i)*                                  &
                         & big_r(i)*jtheta(i)))                                                         &
                         & +((dtradia* d_q(i) * chalev0 * rau(ji)) /                                    & 
                         & (rho_veg*leaf_tks * pad_deltah(i)* delta_h(i) * big_r_prime(i) * jtheta(i)))
    
                     big_g_top(i) = (t_l_pres(i) + dtradia* eta_3(i)/                                   &
                         & (rho_veg*leaf_tks * pad_deltah(i) * delta_h(i) * jtheta(i))                  & 
                         &  + (eta_1(i) * lwdown(ji) * dtradia)/                                        &
                         & (rho_veg*leaf_tks * pad_deltah(i) * delta_h(i) * jtheta(i))                  &
                         &  + (eta_4(i) * swdown(ji) * dtradia)/                                         & 
                         & (rho_veg*leaf_tks * pad_deltah(i) * delta_h(i) * jtheta(i))                  &
                         &  - (chalev0 * rau(ji) * dtradia* jbeta(i))/                                  &
                         & (rho_veg*leaf_tks * pad_deltah(i) *delta_h(i) *  big_r_prime(i) * jtheta(i)) &
                         &  + (dtradia* cp_air * rau(ji) * b_t(i))/                                     &
                         & (rho_veg*leaf_tks * pad_deltah(i) * delta_h(i) * big_r(i) * jtheta(i))       &
                         &  + (dtradia* chalev0 * rau(ji) * b_q(i))/                                    & 
                         & (rho_veg*leaf_tks * pad_deltah(i) * delta_h(i) * big_r_prime(i) * jtheta(i)) )
    
                     big_efg_bottom(i) =                                                                &
                         & ( 1.0d0 + (dtradia* jalpha(i) * chalev0 * rau(ji))/                          &
                         &  (rho_veg*leaf_tks * pad_deltah(i) * delta_h(i) * big_r_prime(i)*jtheta(i))  &
                         &  + (dtradia* cp_air * rau(ji)) /                                             &
                         & (rho_veg*leaf_tks * pad_deltah(i) * delta_h(i) * jtheta(i) * big_r(i))       &
                         &  - (eta_2(i) * dtradia) /                                                    & 
                         & (rho_veg *leaf_tks * pad_deltah(i) * delta_h(i) * jtheta(i))                 &
                         &  - (dtradia* chalev0 * rau(ji) * cp_air * c_t(i)) /                          &
                         & (rho_veg *leaf_tks * pad_deltah(i) * delta_h(i) *                            &
                         &  jtheta(i) * big_r(i)) )
    
                     big_e(i) = big_e_top(i) / big_efg_bottom(i)
       
                     big_f(i) = big_f_top(i) / big_efg_bottom(i)
    
                     big_g(i) = big_g_top(i) / big_efg_bottom(i)
    
                END IF ! (hydrol_flag)

             END DO ! i = jnlvls, jnlvls
        
          DO i=jnlvls-1, 2, -1
    
              ! --------------------------------------------------------------------
    
              IF (hydrol_flag) THEN
    
    
              ! -------------------------------------------------------------------

              x_1(i) = 1 - ( dtradia* ( (a_t(i+1) * (k_eddy(i)/(delta_z(i)*delta_h(i))))  &
                               &           - (k_eddy(i)/(delta_z(i)*delta_h(i)))                  &
                               &           - (k_eddy(i-1)/(delta_z(i-1)*delta_h(i)))              &
                               &           - ((1.0d0)/(big_r(i)*delta_h(i)))))                    &
                               &           - ((k_eddy(i) * c_t(i+1)) /                            &
                               &             (delta_z(i)*delta_h(i))) * big_f(i+1) * dtradia
          
              x_2(i) = t_a_pres(i) + ((b_t(i+1)*k_eddy(i)*dtradia)/(delta_z(i) * delta_h(i)))  &
                               & + ((k_eddy(i) * c_t(i+1)) / (delta_z(i) * delta_h(i))) * big_g(i+1) * dtradia
          
              x_3(i) = (k_eddy(i-1)/(delta_z(i-1)*delta_h(i))) * dtradia
          
              x_4(i) = (k_eddy(i) * d_t(i+1) / (delta_z(i) * delta_h(i))) * dtradia            &
                               &  + ((k_eddy(i) * c_t(i+1)) / (delta_z(i) * delta_h(i))) * big_e(i+1) * dtradia
          
              x_5(i) = + (dtradia) / (delta_h(i) * big_r(i))
    
              ! -------------------------------------------------------------------
    
              y_1(i) = 1 - ( dtradia* ( (a_q(i+1) * k_eddy(i)/(delta_z(i)*delta_h(i)))       &
                        &  - (k_eddy(i)/(delta_z(i)*delta_h(i)))                             &
                        &  - (k_eddy(i-1)/(delta_z(i-1)*delta_h(i)))                         &
                        &  ))  
                        
              y_2(i) = q_a_pres(i) +                                                         &
                        & ((b_q(i+1) * k_eddy(i) * dtradia)/(delta_z(i) * delta_h(i)))       &
                        & + ((k_eddy(i) * c_q(i+1)) / (delta_z(i) * delta_h(i))) * big_g(i+1) * dtradia    &
                        & + (dtradia * phi_trans(i)) / (delta_h(i) * chalev0 * rau(ji))
                                 
              y_3(i) = (k_eddy(i-1)/(delta_z(i-1)*delta_h(i))) * dtradia


             
              y_4(i) = (k_eddy(i) * d_q(i+1) / (delta_z(i) * delta_h(i))) * dtradia          
             
              y_5(i) = 0.0d0 ! + (jalpha(i) * dtradia) / (delta_h(i) * big_r_prime(i))
    
              ! -------------------------------------------------------------------
    
              a_t(i) = x_3(i) / (x_1(i) - (x_4(i) * (y_4(i) / y_1(i))))
              b_t(i) = ( x_2(i) + (x_4(i) * (y_2(i)/y_1(i))) )  /        &
                       & (x_1(i) - (x_4(i) *(y_4(i)/y_1(i))))
              c_t(i) = ( (x_4(i) * (y_5(i)/y_1(i))) + x_5(i)) /          &
                       & (x_1(i) - (x_4(i)*(y_4(i)/y_1(i))))
              d_t(i) = ( x_4(i) * (y_3(i)/y_1(i))  )   /                 &
                        & (x_1(i) - (x_4(i) * (y_4(i) / y_1(i))))
    
              ! -------------------------------------------------------------------
    
              a_q(i) = y_3(i) / (y_1(i) - (y_4(i) * (x_4(i) / x_1(i))))
              b_q(i) = ( y_2(i) + (y_4(i) * (x_2(i)/x_1(i))) )  /        &
                     & (y_1(i) - (y_4(i) *(x_4(i)/x_1(i))))
              c_q(i) = ( (y_4(i) * (x_5(i)/x_1(i))) + y_5(i)) /          &
                     & (y_1(i) - (y_4(i)*(x_4(i)/x_1(i))))
              d_q(i) = ( y_4(i) * (x_3(i)/x_1(i))  )   /                 &
                     & (y_1(i) - (y_4(i) * (x_4(i) / x_1(i))))
    
              ! -------------------------------------------------------------------
    
    
              big_e_top(i) =                                                                     &
                     &   ((dtradia* d_t(i) * cp_air * rau(ji)) /                                    & 
                     & (rho_veg*leaf_tks * pad_deltah(i) * delta_h(i)* jtheta(i) * big_r(i)))   

              big_f_top(i) = ( + (dtradia* a_t(i) * cp_air * rau(ji))/                           &
                     & (rho_veg*leaf_tks * pad_deltah(i)* delta_h(i) * jtheta(i)* big_r(i)) )    
    

              big_g_top(i) = (t_l_pres(i) + dtradia* eta_3(i)/                                   &
                     & (rho_veg*leaf_tks*pad_deltah(i)*delta_h(i)*jtheta(i))                        & 
                     &  + (eta_1(i) * lwdown(ji) * dtradia)/                                        &
                     & (rho_veg*leaf_tks*pad_deltah(i)*delta_h(i)*jtheta(i))                        &
                     &  + (eta_4(i) * swdown(ji) * dtradia)/                                         & 
                     & (rho_veg*leaf_tks*pad_deltah(i)*delta_h(i)*jtheta(i))                        &
                     &  + (dtradia * cp_air * rau(ji) * b_t(i))/                                    &
                     & (rho_veg*leaf_tks*pad_deltah(i)*delta_h(i)* jtheta(i) * big_r(i) )           &
                     !  &  - (dtradia * phi_trans) / (delta_h(i) * jtheta(i) * rho_veg))
                     &  - (dtradia * phi_trans(i)) / (rho_veg*leaf_tks * pad_deltah(i) * delta_h(i) * jtheta(i)))


               big_efg_bottom(i) =                                                                  &
                     & ( 1.0d0                                                                      &
                     &  + (dtradia * cp_air * rau(ji)) /                                            &
                     & (rho_veg*leaf_tks * pad_deltah(i) * delta_h(i) * jtheta(i)  * big_r(i))      &
                     &  - (eta_2(i) * dtradia) /                                                    & 
                     & (rho_veg *leaf_tks * pad_deltah(i) * delta_h(i) * jtheta(i))                 &
                     &  - (dtradia * cp_air * rau(ji) * c_t(i)) /                                   &
                     & (rho_veg *leaf_tks * pad_deltah(i) * delta_h(i) *jtheta(i) * big_r(i)) )


              big_e(i) = big_e_top(i) / big_efg_bottom(i)
     
              big_f(i) = big_f_top(i) / big_efg_bottom(i)
    
              big_g(i) = big_g_top(i) / big_efg_bottom(i)


              ELSE ! following if not (hydrol_flag)

              ! --------------------------------------------------------------------
    
              x_1(i) = 1 - ( dtradia* ( (a_t(i+1) * (k_eddy(i)/(delta_z(i)*delta_h(i))))   &
                        &           - (k_eddy(i)/(delta_z(i)*delta_h(i)))                  &
                        &           - (k_eddy(i-1)/(delta_z(i-1)*delta_h(i)))              &
                        &           - ((1.0d0)/(big_r(i)*delta_h(i)))))                    &
                        &           - ((k_eddy(i) * c_t(i+1)) /                            &
                        &             (delta_z(i)*delta_h(i))) * big_f(i+1) * dtradia
    
              x_2(i) = t_a_pres(i) + ((b_t(i+1)*k_eddy(i)*dtradia)/(delta_z(i) * delta_h(i)))  &
                 & + ((k_eddy(i) * c_t(i+1)) / (delta_z(i) * delta_h(i))) * big_g(i+1) * dtradia
    
              x_3(i) = (k_eddy(i-1)/(delta_z(i-1)*delta_h(i))) * dtradia
    
              x_4(i) = (k_eddy(i) * d_t(i+1) / (delta_z(i) * delta_h(i))) * dtradia            &
                 &  + ((k_eddy(i) * c_t(i+1)) / (delta_z(i) * delta_h(i))) * big_e(i+1) * dtradia
    
              x_5(i) = + (dtradia) / (delta_h(i) * big_r(i))
    
              ! -------------------------------------------------------------------
    
              y_1(i) = 1.0d0 - ( dtradia* ( (a_q(i+1) * (k_eddy(i)/(delta_z(i)*delta_h(i))))       &
                 &  - (k_eddy(i)/(delta_z(i)*delta_h(i)))                                      &
                 &  - (k_eddy(i-1)/(delta_z(i-1)*delta_h(i)))                                  &
                 &  - ((1.0d0)/(big_r_prime(i)*delta_h(i)))))                                  &
                 &  - ((k_eddy(i) * c_q(i+1)) / (delta_z(i)*delta_h(i))) * big_e(i+1) * dtradia
    
              y_2(i) = q_a_pres(i) +                                                           &
                 & ((b_q(i+1) * k_eddy(i) * dtradia)/(delta_z(i) * delta_h(i)))                &
                 & + (jbeta(i) * dtradia)  / (big_r_prime(i)*delta_h(i))                       &
                 & + ((k_eddy(i) * c_q(i+1)) / (delta_z(i) * delta_h(i))) * big_g(i+1) * dtradia
    
              y_3(i) = (k_eddy(i-1)/(delta_z(i-1)*delta_h(i))) * dtradia
    
              y_4(i) = (k_eddy(i) * d_q(i+1) / (delta_z(i) * delta_h(i))) * dtradia            &
                 &  + ((k_eddy(i) * c_q(i+1)) / (delta_z(i) * delta_h(i))) * big_f(i+1) * dtradia
    
              y_5(i) = + (jalpha(i) * dtradia) / (delta_h(i) * big_r_prime(i))
    
              ! -------------------------------------------------------------------
    
              a_t(i) = x_3(i) / (x_1(i) - (x_4(i) * (y_4(i) / y_1(i))))
              b_t(i) = ( x_2(i) + (x_4(i) * (y_2(i)/y_1(i))) )  /        &
                     & (x_1(i) - (x_4(i) *(y_4(i)/y_1(i))))
              c_t(i) = ( (x_4(i) * (y_5(i)/y_1(i))) + x_5(i)) /          &
                     & (x_1(i) - (x_4(i)*(y_4(i)/y_1(i))))
              d_t(i) = ( x_4(i) * (y_3(i)/y_1(i))  )   /                 &
                     & (x_1(i) - (x_4(i) * (y_4(i) / y_1(i))))
    
              ! -------------------------------------------------------------------
    
              a_q(i) = y_3(i) / (y_1(i) - (y_4(i) * (x_4(i) / x_1(i))))
              b_q(i) = ( y_2(i) + (y_4(i) * (x_2(i)/x_1(i))) )  /        &
                     & (y_1(i) - (y_4(i) *(x_4(i)/x_1(i))))
              c_q(i) = ( (y_4(i) * (x_5(i)/x_1(i))) + y_5(i)) /          &
                     & (y_1(i) - (y_4(i)*(x_4(i)/x_1(i))))
              d_q(i) = ( y_4(i) * (x_3(i)/x_1(i))  )   /                 &
                     & (y_1(i) - (y_4(i) * (x_4(i) / x_1(i))))
    
              ! -------------------------------------------------------------------
    
              big_e_top(i) = (+(dtradia*a_q(i)*chalev0*rau(ji))/                             &
                  & (rho_veg*leaf_tks * pad_deltah(i)*delta_h(i)*big_r_prime(i)*jtheta(i)))  &
                  & + ((dtradia* d_t(i) * cp_air * rau(ji)) /                                &
                  & (rho_veg*leaf_tks * pad_deltah(i) * delta_h(i) * big_r(i) * jtheta(i)))   
    
              big_f_top(i) = ( + (dtradia* a_t(i) * cp_air * rau(ji))/                       &
                  & (rho_veg*leaf_tks * pad_deltah(i)* delta_h(i)                            &
                  & * big_r(i) * jtheta(i)) )                                                &
                  & + ((dtradia* d_q(i) * chalev0 * rau(ji)) /                               &
                  & (rho_veg*leaf_tks * pad_deltah(i) * delta_h(i)                           &
                  & * big_r_prime(i) * jtheta(i)))
    
              big_g_top(i) = (t_l_pres(i)+dtradia*eta_3(i)/                                  &
                  & (rho_veg*leaf_tks*pad_deltah(i)*delta_h(i)*jtheta(i))                    & 
                  & + (eta_1(i)*lwdown(ji)*dtradia)/                                         &
                  & (rho_veg*leaf_tks*pad_deltah(i)*delta_h(i)*jtheta(i))                    &
                  & + (eta_4(i)*swdown(ji)*dtradia)/                                          &
                  & (rho_veg*leaf_tks*pad_deltah(i)*delta_h(i)*jtheta(i))                    &
                  & - (chalev0*rau(ji)*dtradia*jbeta(i))/                                    &
                  & (rho_veg*leaf_tks*pad_deltah(i)*delta_h(i) *                             &
                  &  big_r_prime(i)*jtheta(i))                                               &
                  & + (dtradia*cp_air*rau(ji)*b_t(i))/                                       &
                  & (rho_veg*leaf_tks*pad_deltah(i)*delta_h(i)*big_r(i)* jtheta(i))          &
                  & + (dtradia*chalev0*rau(ji)*b_q(i))/                                      &
                  & (rho_veg*leaf_tks*pad_deltah(i)*delta_h(i)*                              &
                  & big_r_prime(i)*jtheta(i)) )
    
              big_efg_bottom(i) =                                                            &
                  & ( 1.0d0 + (dtradia* jalpha(i) * chalev0 * rau(ji)) /                     &
                  & (rho_veg *leaf_tks * pad_deltah(i) * delta_h(i) *                        &
                  & big_r_prime(i) * jtheta(i))                                              &
                  & + (dtradia* cp_air * rau(ji)) /                                          &
                  & (rho_veg *leaf_tks * pad_deltah(i) * delta_h(i) * jtheta(i) * big_r(i))  &
                  & - (eta_2(i) * dtradia) /                                                 &
                  & (rho_veg*leaf_tks * pad_deltah(i)  * delta_h(i) * jtheta(i))             &
                  & - (dtradia* chalev0 * rau(ji) * c_q(i)) /                                & 
                  & (rho_veg *leaf_tks* pad_deltah(i) * delta_h(i) * jtheta(i)               &
                  & * big_r_prime(i))                                                        &
                  & - (dtradia* cp_air * rau(ji) * c_t(i))/                                  &
                  & (rho_veg *leaf_tks * pad_deltah(i) * delta_h(i) * jtheta(i) * big_r(i)))
    
              big_e(i) = big_e_top(i) / big_efg_bottom(i)
     
              big_f(i) = big_f_top(i) / big_efg_bottom(i)
    
              big_g(i) = big_g_top(i) / big_efg_bottom(i)

              END IF ! (hydrol_flag)               
              ! -------------------------------------------------------------------
    
          END DO !i = jnlvls-1, 2, -1

        
              IF (hydrol_flag) THEN

              ! -------------------------------------------------------------------

              x_1(1) = 1.0d0 - dtradia* ((a_t(2) * k_eddy(1)) / (delta_z(1) * delta_h(1))       &
                      &  +  (k_eddy(1) * c_t(2) * big_f(2)) / (delta_z(1) * delta_h(1))       &
                      &  -  (k_eddy(1)/ (delta_z(1) * delta_h(1)))                           &
                      &  -  ( 1.0d0 / (delta_h(1) * big_r(1))) )
    
              x_2(1) = t_a_pres(1)+dtradia*(  (k_eddy(1) * b_t(2))/(delta_z(1) * delta_h(1))  &
                      &   +  (k_eddy(1) * c_t(2) * big_g(2))/(delta_z(1) * delta_h(1)) )
    
              x_3(1) = dtradia* (1.0d0 / delta_h(1)) * ( 1.0d0 / (rau(ji) * cp_air) )    
    
              x_4(1) = dtradia* ( (k_eddy(1) * c_t(2) * big_e(2)) + (k_eddy(1) * d_t(2)) ) /  &
                      & (delta_z(1) * delta_h(1)) 
     
              x_5(1) = + dtradia* ( 1.0d0 / (delta_h(1) * big_r(1)) )
    
              ! -------------------------------------------------------------------
    
              y_1(1) = 1.0d0 - dtradia* (    (a_q(2) * k_eddy(1)) / (delta_z(1)*delta_h(1))   &
                      &  +  (k_eddy(1) * c_q(2) * big_e(2)) / (delta_z(1) * delta_h(1))       &
                      &  -  (k_eddy(1) / (delta_z(1) * delta_h(1)))                           &
                      &  -  ( 1.0d0 / (delta_h(1) * big_r_prime(1))) )
    
              y_2(1) = q_a_pres(1)+dtradia*(  (k_eddy(1) * b_q(2))/(delta_z(1) * delta_h(1))  &
                      &   +  (k_eddy(1) * c_q(2) * big_g(2))/(delta_z(1) * delta_h(1))        &
                      &   + jbeta(1) / (delta_h(1) * big_r_prime(1)) )
    
              y_3(1) = dtradia* (1.0d0 / delta_h(1)) * ( 1.0d0 / (rau(ji) * chalev0) )
    
              y_4(1) = dtradia* ( (k_eddy(1)* c_q(2) * big_f(2)) + (k_eddy(1) * d_q(2)) ) /  &
                      &   (delta_z(1) * delta_h(1)) 
     
              y_5(1) = + dtradia* ( jalpha(1) / (delta_h(1) * big_r_prime(1)) )
    
              ! -------------------------------------------------------------------
    
              a_t(1) = x_3(1) / (x_1(1) - (x_4(1) * (y_4(1) / y_1(1))))
              b_t(1) = ( x_2(1) + (x_4(1) * (y_2(1)/y_1(1))) )  /         &
                       & (x_1(1) - (x_4(1) *(y_4(1)/y_1(1))))
              c_t(1) = ( (x_4(1) * (y_5(1)/y_1(1))) + x_5(1)) /           &
                       & (x_1(1) - (x_4(1)*(y_4(1)/y_1(1))))
              d_t(1) = ( x_4(1) * (y_3(1)/y_1(1))  )   /                  &
                       & (x_1(1) - (x_4(1) * (y_4(1) / y_1(1))))
    
              ! -------------------------------------------------------------------
    
              a_q(1) = y_3(1) / (y_1(1) - (y_4(1) * (x_4(1) / x_1(1))))
              b_q(1) = ( y_2(1) + (y_4(1) * (x_2(1)/x_1(1))) )  /         &
                       & (y_1(1) - (y_4(1) *(x_4(1)/x_1(1))))
              c_q(1) = ( (y_4(1) * (x_5(1)/x_1(1))) + y_5(1)) /           & 
                       & (y_1(1) - (y_4(1)*(x_4(1)/x_1(1))))
              d_q(1) = ( y_4(1) * (x_3(1)/x_1(1))  )   /                  &
                       & (y_1(1) - (y_4(1) * (x_4(1) / x_1(1))))

          ELSE ! following if not (hydrol_flag)

              x_1(1) = 1.0d0 - dtradia* ((a_t(2) * k_eddy(1)) / (delta_z(1)*delta_h(1))       &
                      &  +  (k_eddy(1) * c_t(2) * big_f(2)) / (delta_z(1) * delta_h(1))       &
                      &  -  (k_eddy(1) / (delta_z(1) * delta_h(1)))                           &
                      &  -  ( 1.0d0 / (delta_h(1) * big_r(1))) )
    
              x_2(1) = t_a_pres(1)+dtradia*(  (k_eddy(1) * b_t(2))/(delta_z(1) * delta_h(1))  &
                      &   +  (k_eddy(1) * c_t(2) * big_g(2))/(delta_z(1) * delta_h(1)) )
    
              x_3(1) = dtradia* (1.0d0 / delta_h(1)) * ( 1.0d0 / (rau(ji) * cp_air) )    
    
              x_4(1) = dtradia* ( (k_eddy(1) * c_t(2) * big_e(2)) + (k_eddy(1) * d_t(2)) ) /  &
                      & (delta_z(1) * delta_h(1)) 
     
              x_5(1) = + dtradia* ( 1.0d0 / (delta_h(1) * big_r(1)) )
    
              ! -------------------------------------------------------------------
    
              y_1(1) = 1.0d0 - dtradia* (    (a_q(2) * k_eddy(1)) / (delta_z(1)*delta_h(1))   &
                      &  +  (k_eddy(1) * c_q(2) * big_e(2)) / (delta_z(1) * delta_h(1))       &
                      &  -  (k_eddy(1) / (delta_z(1) * delta_h(1)))                           &
                      &  -  ( 1.0d0 / (delta_h(1) * big_r_prime(1))) )
    
              y_2(1) = q_a_pres(1)+dtradia*(  (k_eddy(1) * b_q(2))/(delta_z(1) * delta_h(1))  &
                      &   +  (k_eddy(1) * c_q(2) * big_g(2))/(delta_z(1) * delta_h(1))        &
                      &   + jbeta(1) / (delta_h(1) * big_r_prime(1)) )
    
              y_3(1) = dtradia* (1.0d0 / delta_h(1)) * ( 1.0d0 / (rau(ji) * chalev0) )
    
              y_4(1) = dtradia* ( (k_eddy(1) * c_q(2) * big_f(2)) + (k_eddy(1) * d_q(2)) ) /  &
                      &   (delta_z(1) * delta_h(1)) 
     
              y_5(1) = + dtradia* ( jalpha(1) / (delta_h(1) * big_r_prime(1)) )
    
              ! -------------------------------------------------------------------
    
              a_t(1) = x_3(1) / (x_1(1) - (x_4(1) * (y_4(1) / y_1(1))))
              b_t(1) = ( x_2(1) + (x_4(1) * (y_2(1)/y_1(1))) )  /         &
                       & (x_1(1) - (x_4(1) *(y_4(1)/y_1(1))))
              c_t(1) = ( (x_4(1) * (y_5(1)/y_1(1))) + x_5(1)) /           &
                       & (x_1(1) - (x_4(1)*(y_4(1)/y_1(1))))
              d_t(1) = ( x_4(1) * (y_3(1)/y_1(1))  )   /                  &
                       & (x_1(1) - (x_4(1) * (y_4(1) / y_1(1))))
    
              ! -------------------------------------------------------------------
    
              a_q(1) = y_3(1) / (y_1(1) - (y_4(1) * (x_4(1) / x_1(1))))
              b_q(1) = ( y_2(1) + (y_4(1) * (x_2(1)/x_1(1))) )  /         &
                       & (y_1(1) - (y_4(1) *(x_4(1)/x_1(1))))
              c_q(1) = ( (y_4(1) * (x_5(1)/x_1(1))) + y_5(1)) /           & 
                       & (y_1(1) - (y_4(1)*(x_4(1)/x_1(1))))
              d_q(1) = ( y_4(1) * (x_3(1)/x_1(1))  )   /                  &
                       & (y_1(1) - (y_4(1) * (x_4(1) / x_1(1))))
    
              ! -------------------------------------------------------------------

              END IF ! (hydrol_flag)
    
          END IF 
    

            big_e_top(1) = (+(dtradia* a_q(1) * chalev0 * rau(ji)) /                              &
               & (rho_veg *leaf_tks * pad_deltah(1) * delta_h(1) * jtheta(1) * big_r_prime(1)))   &
               & +((dtradia* d_t(1) * cp_air * rau(ji)) /                                        &
               & (rho_veg*leaf_tks * pad_deltah(1) * delta_h(1) * jtheta(1) * big_r(1)) )   

    
            big_f_top(1) = (+(dtradia* a_t(1) * cp_air * rau(ji)) /                              &
               & (rho_veg*leaf_tks * pad_deltah(1) * delta_h(1)* jtheta(1) * big_r(1)))          &
               & + ((dtradia* d_q(1) * chalev0 * rau(ji)) /                                      &
               & (rho_veg*leaf_tks * pad_deltah(1) * delta_h(1) * jtheta(1) * big_r_prime(1)))

    
            big_g_top(1) = (t_l_pres(1) + dtradia* eta_3(1) /                                     &
               & (rho_veg*leaf_tks * pad_deltah(1) * delta_h(1) * jtheta(1))                     & 
               & + (eta_1(1) * lwdown(ji) * dtradia)/                                            &
               & (rho_veg *leaf_tks* pad_deltah(1) * delta_h(1) * jtheta(1))                     &
               &  + (eta_4(1) * swdown(ji) * dtradia)/                                            &
               & (rho_veg *leaf_tks* pad_deltah(1) * delta_h(1) * jtheta(1))                     &
               &  - (chalev0 * rau(ji) * dtradia* jbeta(1))/                                     & 
               & (rho_veg *leaf_tks* pad_deltah(1) * delta_h(1) * jtheta(1) * big_r_prime(1))    &
               &  + (dtradia* cp_air * rau(ji) * b_t(1)) /                                       &
               & (rho_veg *leaf_tks* pad_deltah(1) * delta_h(1) * jtheta(1) * big_r(1))          &
               &  + (dtradia * chalev0 * rau(ji) * b_q(1)) /                                     &
               & (rho_veg *leaf_tks * pad_deltah(1) * delta_h(1) * jtheta(1) * big_r_prime(1)))

    
            big_efg_bottom(1) = ( 1.0d0 + (dtradia *jalpha(1) * chalev0 * rau(ji))/               &
               & (rho_veg *leaf_tks * pad_deltah(1) * delta_h(1) * jtheta(1) * big_r_prime(1) )   &
               & + (dtradia * cp_air * rau(ji)) /       +                                         &
               & (rho_veg *leaf_tks * pad_deltah(1) * delta_h(1) * jtheta(1) * big_r(1))          &
               & - (eta_2(1) * dtradia) /                                                         &
               ! & - (0.0d0 * dtradia) /                                                            &
               & (rho_veg*leaf_tks * pad_deltah(1)  * delta_h(1) * jtheta(1))                     &
               & - (dtradia * chalev0 * rau(ji) * c_q(1)) /                                       &
               & (rho_veg *leaf_tks * pad_deltah(1) * delta_h(1) * jtheta(1) * big_r_prime(1))    &
               & - (dtradia * cp_air * rau(ji) * c_t(1))/                                         &
               & (rho_veg *leaf_tks * pad_deltah(1) * delta_h(1) * jtheta(1) * big_r(1)))
    

            big_e(1) = big_e_top(1) / big_efg_bottom(1)
     
            big_f(1) = big_f_top(1) / big_efg_bottom(1)
    
            big_g(1) = big_g_top(1) / big_efg_bottom(1)    

    
          ! --------------------------------------------------------------------------------------------------------------
        
          IF (jnlvls .eq. 1) THEN
              turb_factor = speed * q_cdrag(ji)         ! surface turbulence calculated as in existing ORCHIDEE
          ELSE
              turb_factor = u_speed(1) * q_cdrag(ji) * ks_tune    ! surface turbulence calculated as in existing ORCHIDEE
          END IF

    
          ! components for surface sensible heat flux    
    
          big_omega_1 = 1.0d0 - (rau(ji) * 1.0d0) * turb_factor * &
                     & (a_t(1) + c_t(1) * big_f(1)) 
    
          big_omega_2 = + (rau(ji) * cp_air) * turb_factor * &
                     & (b_t(1) + c_t(1) * big_g(1))
        
          big_omega_3 = - rau(ji) * cp_air * turb_factor   
        
          big_omega_4 = + rau(ji) * cp_air * turb_factor * &
                     & (c_t(1) * big_e(1) + d_t(1))
        
        
    
          ! components for surface latent heat flux (evaporation)
    
          big_omega_5 = 1.0d0 - (rau(ji) * 1.0d0) * turb_factor  * & 
                     & (a_q(1) + c_q(1) * big_e(1)) 
        
          big_omega_6 = + (rau(ji) * chalev0) * turb_factor  * &
                     & (b_q(1) + c_q(1) * big_g(1) - jbeta_surf(ji) )
        
          big_omega_7 = - jalpha_surf(ji) * rau(ji) * chalev0 * turb_factor 
                
          big_omega_8 = + rau(ji) * chalev0 * turb_factor  * &
                     & (c_q(1) * big_f(1) + d_q(1))
    
    
    
    
          ! components for surface latent heat flux (sublimination)
    
          big_omega_5_sub = 1.0d0 - (rau(ji) * 1.0d0) * turb_factor* & 
                     & (a_q(1) + c_q(1) * big_e(1)) 
        
          big_omega_6_sub = + (rau(ji) * chalsu0) * turb_factor * &
                     & (b_q(1) + c_q(1) * big_g(1) - jbeta_surf(ji) )
        
          big_omega_7_sub = - jalpha_surf(ji) * rau(ji) * chalsu0 * turb_factor  
        
          big_omega_8_sub = + rau(ji) * chalsu0 * turb_factor *  &
                     & (c_q(1) * big_f(1) + d_q(1))
    
    
             ! --------------------------------------------------------------------------------------------------------------
    
             xi_1 = ( big_omega_2 + (big_omega_4/big_omega_5) * big_omega_6 )  / &
                  &  ( big_omega_1 - (big_omega_4/big_omega_5) * big_omega_8 )
    
             xi_2 = ( big_omega_3 + (big_omega_4/big_omega_5) * big_omega_7 )  / &
                  &  ( big_omega_1 - (big_omega_4/big_omega_5) * big_omega_8 )

             !+++++++++++++++++++++++++++++++++++++++++++++++++++++
             ! Here we try to paramterize the surface evaporation  
             ! with the combination of the vebta3, vbeta4, ks_slope, ks_veget
             ! and other variables such as pgap(=1-veget), day of growth etc  
             ! air_temperature as an indicator to account 
             ! seasonal grass leaf-fall at forest floor     
             !+++++++++++++++++++++++++++++++++++++++++++++++++++++

             ! Set up a S-shape function (from 0 to 1) based on the site dependent vegettation cover fraction  
             ! for the calculation of the weighting for vebta3 or vbeta4  
             ! paramteriziation the surface evapotranspiration coefficient for the boundary condiction for the 
             ! multilayer scheme
             ks_weight_1(ji) = 1.0_r_std / (1.0_r_std + EXP (-ks_slope * (veget(test_grid, test_pft) - ks_veget) )) 
             ks_weight_2(ji) = 1.0_r_std - ks_weight_1(ji) 
              
             ! Add a condictional rule to consider the temperature constrain the growing season
             ! of forest floor when the vegetation cover below certain threshood or critiria             
             ! "ks_veget" and longterm average temperature below certain value (15oC) 
             ! the grass was assume to be reduced having little contribution on total evapotraspiration.  
             IF ( veget(test_grid,test_pft) .LT. ks_veget) THEN
             ks_weight_1(ji) = 1.0_r_std / (((1.0_r_std + abs( t2m_month_out(test_grid) - 288.15)/15.) ) +  & 
                             &               EXP (-ks_slope *(veget(test_grid, test_pft) - ks_veget) ))
             ks_weight_2(ji) = 1.0_r_std - ks_weight_1(ji)  
 
             ENDIF     

             ! Calculate the total weighting coefficient both from vbeta4 and vbeta3
             evap_fraction(ji) =  (  ks_weight_1(ji) * vbeta4(ji) & 
                                &  + ks_weight_2(ji) * vbeta3(test_grid, test_pft) & 
                                &  )           
            
             ! evap_fraction(ji) =  vbeta3(test_grid,test_pft) 
             ! set up a upper limited for the potential evapotranspiration rate 
             ! thus if the veget ~1.0, there is little forest floor evapotranspiration   
             IF  ( evap_fraction(ji) .GT. 1.0_r_std ) THEN
                 evap_fraction(ji) =  1.0_r_std
             ENDIF
             ! set a lowerbound/miinum of evapofraction, because of we tune this parameter 
             ! IF (evap_fraction(ji) .LT.  0.01_r_std) THEN
             !     evap_fraction(ji) = 0.01_r_std
             ! ENDIF
         
             ! We still need to add some parmeters from the  phenology for the
             ! decedious forest type.
             ! IF the is decidious tree.....
             ! evapo_fraction(ji) = f(day start to growth, vb3, vb4)
             !

           IF (ld_jameswrite) THEN

               WRITE (numout, '(A40,F10.5)') 'veget_fraction:', veget(test_grid, test_pft) 
               WRITE (numout, '(A40,F10.5)') 'beta3:', vbeta3(test_grid, test_pft)
               WRITE (numout, '(A40,F10.5)') 'beta4:', vbeta4(ji)
               WRITE (numout, '(A40,F10.5)') 'ks_weight_1:', ks_weight_1(ji)
               WRITE (numout, '(A40,F10.5)') 'ks_weight_2:', ks_weight_2(ji)
               WRITE (numout, '(A40,F10.5)') 'ks_tune:', ks_tune
               WRITE (numout, '(A40,F10.5)') 'forest_floor_evapo_fraction:', evap_fraction(ji)
               WRITE (numout, '(A40,F10.5)') 'longterm_temperature:', t2m_month_out(test_grid)
               !+++++++++++++++++++++++++++++++++++++++++++++++++++++

          END IF ! (ld_jameswrite) THEN


          xi_3 = evap_fraction(ji)  *                                                         &
                & ( big_omega_6 + (big_omega_8/big_omega_1) * big_omega_2 ) /         &
                & ( big_omega_5 - (big_omega_8/big_omega_1) * big_omega_4 )  +        &
                & beta_diffuco_sub(ji) *                                              &
                & ( big_omega_6_sub + (big_omega_8_sub/big_omega_1) * big_omega_2 ) / &
                & ( big_omega_5_sub - (big_omega_8_sub/big_omega_1) * big_omega_4 ) 
    
          xi_4 = evap_fraction(ji) *                                                         &
                & ( big_omega_7 + (big_omega_8/big_omega_1) * big_omega_3 ) /         &
                &  ( big_omega_5 - (big_omega_8/big_omega_1) * big_omega_4 ) +        &
                & beta_diffuco_sub(ji) *                                              &
                & ( big_omega_7_sub + (big_omega_8_sub/big_omega_1) * big_omega_3 ) / &
                &  ( big_omega_5_sub - (big_omega_8_sub/big_omega_1) * big_omega_4 ) 
    
          IF (ld_jameswrite) THEN

              WRITE (numout, *) 'prior to temp_surf_next'
              WRITE (numout, *) 'psold(ji)/cp_air:', psold(ji)/cp_air
              WRITE (numout, *) 'dtradia:', dtradia
              WRITE (numout, *) '(dtradia/soilcap(ji)):', (dtradia/soilcap(ji))
              WRITE (numout, *) 'xi_1:', xi_1
              WRITE (numout, *) 'xi_2:', xi_2
              WRITE (numout, *) 'xi_3:', xi_3
              WRITE (numout, *) 'xi_4:', xi_4

              WRITE (numout, *) 'eta_1_surf*lwdown(ji):', eta_1_surf*lwdown(ji)
              WRITE (numout, *) 'eta_2_surf:', eta_2_surf
              WRITE (numout, *) 'eta_3_surf:', eta_3_surf
              WRITE (numout, *) 'eta_4_surf:', eta_4_surf
              WRITE (numout, *) 'swnet(ji):', swnet(ji)
              WRITE (numout, *) 'swdown(ji):', swdown(ji)
              WRITE (numout, *) 'eta_4_surf*swnet(ji):', eta_4_surf*swdown(ji)
    
              WRITE (numout, *) 'soilflx(ji):', soilflx(ji)

              WRITE (numout, *) 'x_1:', x_1(1)
              WRITE (numout, *) 'x_2:', x_2(1)
              WRITE (numout, *) 'x_3:', x_3(1)
              WRITE (numout, *) 'x_4:', x_4(1)
              WRITE (numout, *) 'x_5:', x_5(1)

              WRITE (numout, *) 'y_1:', y_1(1)
              WRITE (numout, *) 'y_2:', y_2(1)
              WRITE (numout, *) 'y_3:', y_3(1)
              WRITE (numout, *) 'y_4:', y_4(1)
              WRITE (numout, *) 'y_5:', y_5(1)

              WRITE (numout, *) 'a_t(1):', a_t(1)
              WRITE (numout, *) 'b_t(1):', b_t(1)
              WRITE (numout, *) 'c_t(1):', c_t(1)
              WRITE (numout, *) 'd_t(1):', d_t(1)

              WRITE (numout, *) 'a_q(1):', a_q(1)
              WRITE (numout, *) 'b_q(1):', b_q(1)
              WRITE (numout, *) 'c_q(1):', c_q(1)
              WRITE (numout, *) 'd_q(1):', d_q(1)

              WRITE (numout, *) 'a_t(jnlvls):', a_t(jnlvls)
              WRITE (numout, *) 'b_t(jnlvls):', b_t(jnlvls)
              WRITE (numout, *) 'c_t(jnlvls):', c_t(jnlvls)
              WRITE (numout, *) 'd_t(jnlvls):', d_t(jnlvls)

              WRITE (numout, *) 'a_q(jnlvls):', a_q(jnlvls)
              WRITE (numout, *) 'b_q(jnlvls):', b_q(jnlvls)
              WRITE (numout, *) 'c_q(jnlvls):', c_q(jnlvls)
              WRITE (numout, *) 'd_q(jnlvls):', d_q(jnlvls)

              WRITE (numout, *) 'big_r_prime(1): ', big_r_prime(1)

              WRITE (numout, *) 'big_omega_1: ', big_omega_1
              WRITE (numout, *) 'big_omega_2: ', big_omega_2
              WRITE (numout, *) 'big_omega_3: ', big_omega_3
              WRITE (numout, *) 'big_omega_4: ', big_omega_4
              WRITE (numout, *) 'big_omega_5: ', big_omega_5
              WRITE (numout, *) 'big_omega_6: ', big_omega_6
              WRITE (numout, *) 'big_omega_7: ', big_omega_7
              WRITE (numout, *) 'big_omega_8: ', big_omega_8

              WRITE (numout, *) 'big_e(1): ', big_e(1)
              WRITE (numout, *) 'big_f(1): ', big_f(1)
              WRITE (numout, *) 'big_g(1): ', big_g(1)

          END IF ! (ld_jameswrite)

          temp_surf_next = ( psold(ji)/cp_air  + (dtradia/soilcap(ji)) * ( xi_1 + xi_3   &
                & + eta_1_surf*lwdown(ji)                                              &
                & + eta_4_surf*swdown(ji)                                               &
                & - eta_3_surf + soilflx(ji)) ) /                                      &
                & ( 1.0d0 - (xi_2 + xi_4 + eta_2_surf) * (dtradia/soilcap(ji)))
              
          temp_sol_new(ji) = temp_surf_next
    

          psnew(ji) = temp_surf_next * cp_air
    
          qsol_sat_new(ji) = qsol_sat(ji) + &
                      &  (1.0d0/(cp_air)) * pdqsold(ji) * &
                      &  (psnew(ji) - psold(ji))

    
          delta_temp = temp_surf_next - psold(ji)/cp_air
    
          epot_air_new(ji) = (1.0d0/( rau(ji) * speed * q_cdrag(ji))) *                 &
                       & (cp_air * b_t(1) - psold(ji) - cp_air * delta_temp) /          &
                       & ((1.0d0/( rau(ji) * speed * q_cdrag(ji))) - a_t(1) ) +         &
                       & cp_air * temp_surf_next
                
    
          ! %%%%% JR 060913 section modified from enerbil_flux %%%%%%%%%%%%%%%%%
    
          !! \latexonly 
          !!     \input{enerbilsurftemp17.tex}
          !! \endlatexonly
        
          fevap = (chalev0 * beta_diffuco_evap(ji) +                                         &
                  & chalsu0 * beta_diffuco_sub(ji)) * ((b_q(1) - qsol_sat(ji)) /             &
                  & ((1.0d0/(rau(ji)*speed*q_cdrag(ji))) - a_q(1) ) )                        &
                  & - (chalev0 * beta_diffuco_evap(ji) + chalsu0 * beta_diffuco_sub(ji))     &
                  & * ((pdqsold(ji)/cp_air) /                                                &
                  & ((1.0d0/(rau(ji)*speed*q_cdrag(ji))) - a_q(1) )  )  *                    &
                  & (cp_air*temp_surf_next - psold(ji))
    
    
          IF ( ABS(fevap) < EPSILON(un) ) THEN
            
              !! \latexonly 
              !!     \input{enerbilsurftemp18.tex}
              !! \endlatexonly
    
              qair_new(ji) = qair(ji)                                                                   
          ELSE
              !! \latexonly 
              !!     \input{enerbilsurftemp19.tex}
              !! \endlatexonly     
    
              qair_new(ji) = (1.0d0/( rau(ji) * speed * q_cdrag(ji))) * un / ( chalsu0 *  vbeta1(ji) * (un - vbeta5(ji)) + &
                 & chalev0 * ((un - vbeta1(ji))*(un - vbeta5(ji)) * vbeta(ji) * 1.0d0 + vbeta5(ji)) ) &
                 & * fevap + qsol_sat_new(ji)  
 
          ENDIF
                
    
          ! 201013 calculate new series of beta_v3
          
          IF (hydrol_flag) THEN  ! overall hydrol flag
          ! vbeta2sum(:) = zero
             IF (hydrol_flag2(ji)) THEN
                DO jv = 1, nvm
                   IF (.NOT. hydrol_flag3(ji, jv)) THEN 
                      ! this is for cases when the supply term is higher than the 
                      !     transpiration) - i.e. the normal state of affairs
                      ! WRITE(numout,*) '201013 vbeta3(ji,jv) is unchanged as: ', vbeta3(ji,jv)
                   ELSE
                     ! WRITE(numout,*) '261113 vbeta3(ji,jv) is originally: ', vbeta3(ji,jv)
  
                      vbeta3(ji,jv) = ( veget_max(ji,jv)* E_supply(ji,jv) ) / &
                          & ( dtradia * (1.0d0 - vbeta1(ji)) * (qsol_sat_new(ji) - qair_new(ji)) * ( rau(ji) * speed * q_cdrag(ji)) )
                     ! WRITE(numout,*) '261113 new vbeta3(ji,jv) is: ', vbeta3(ji,jv)    
                   END IF ! (hydrol_flag3(ji, jv) .eq. .FALSE.)
               ENDDO ! jv = 1, nvm
            ELSE
               ! do nothing
            END IF ! (hydrol_flag2(ji) .eq. .TRUE.)        
         END IF ! IF (hydrol_flag .eq. .TRUE.)       
          
         ! end 281013 calculate new series of beta_v3 

             IF (hydrol_flag) THEN
                ! We aim to calculate a new delta_temp using the new vbeta3
             ELSE
                ! in non-hydrol interface case (the first call to the energy budget in the sechiba cycle,
                !     we can exit this loop straightaway)
                 EXIT convergence_loop 
             END IF 


             !++++ copy the current step in convergence loop for checking the error ++++++ 
              temp_sol_dummy2 = temp_surf_next
             !+++++++++++++++++++++++

             temp_sol_err = abs(temp_sol_dummy2 - temp_sol_dummy1) 
             ! print out the absolute error
             WRITE(numout, '(A40,ES18.6)') "120615 Absolute Error:", temp_sol_err
             
             !+++++++ set a critiria then we exit the loop +++++
             !IF ( temp_sol_error .LT. epsilon_err ) THEN
             !    EXIT convergence_loop 
             !ENDIF    
             !++++++++++++++++++++++++++++++++++++++++++++++++++
             END DO convergence_loop
             ! $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$  end convergence loop here  $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$

    
          flux_ground_h = - (xi_1 + (xi_2 * temp_surf_next))
    
          flux_ground_le = - (xi_3 + (xi_4 * temp_surf_next))   
    
          ! work up the column for atmospheric temperature and humidity, and 
          !    vegetation temperature 
    
          t_l_next(1) = big_e(1) * (flux_ground_le) + big_f(1) * (flux_ground_h) + big_g(1)
    
          t_a_next(1) = (a_t(1) * (flux_ground_h)) + b_t(1) + &
                       & (c_t(1) * t_l_next(1)) + (d_t(1) * (flux_ground_le)) 
    
          q_a_next(1) = (a_q(1) * (flux_ground_le)) + b_q(1) + &
                       & (c_q(1) * t_l_next(1)) + (d_q(1) * (flux_ground_h)) 

          IF (jnlvls .gt. 1) THEN    
              DO i = 2, jnlvls
                  t_l_next(i) = (big_e(i) * q_a_next(i-1)) + &
                           & (big_f(i) * t_a_next(i-1)) + big_g(i)
    
                  t_a_next(i) = (a_t(i) * t_a_next(i-1)) + b_t(i) + &
                           & (c_t(i) * t_l_next(i)) + (d_t(i) * q_a_next(i-1))
    
                  q_a_next(i) = (a_q(i) * q_a_next(i-1)) + b_q(i) + &
                           & (c_q(i) * t_l_next(i)) + (d_q(i) * t_a_next(i-1))
              END DO ! i = 1, jnlvls
          END IF ! (jnlvls .gt. 1) THEN

          t_a_next_grid(ji, :) = t_a_next(:)

          q_a_next_grid(ji, :) = q_a_next(:)
    
          temp_leaf_next_grid(ji, :) = t_l_next(:)

          column_qsatt_leaf(:) = t_l_next(:) * jalpha(:) + jbeta(:)

          ! the lowest level flux is always the flux from the ground
          ! n.b. sign convention: positive flux is the flow of heat from high temperature to low
          !   temperature or from high humidity to low humidity respectably
    
          flux_h(1) = flux_ground_h
          flux_le(1) = flux_ground_le
    
          IF (jnlvls .gt. 1) THEN
              DO i = 2, jnlvls
                  flux_h(i) = - cp_air * rau(ji) * &
                          & (t_a_next(i) - t_a_next(i-1)) * k_eddy(i-1)/delta_z(i-1)
                  flux_le(i) = - chalev0 * rau(ji) * &
                          & (q_a_next(i) - q_a_next(i-1)) * k_eddy(i-1)/delta_z(i-1)
              END DO ! i = 2, jnlvls
          END IF !  (jnlvls .eq. 1) THEN   


          flux_h_grid(ji,  :) = flux_h(:)
          flux_le_grid(ji, :) = flux_le(:)
          u_speed_grid(ji, :) = u_speed(:) ! Although the u_speed is not directly in jcala (from jprofile)  
                                               ! here, we save the wind profile for each grid.  

          IF (control%ok_new_enerbil_nextstep ) THEN 

             DO i = 11, 20 ! 090215 calculate provisional column values of q_surf_leaf
                !  column_q_surf_leaf(i) = vbeta1(ji) * ( ( 1.0d0 - vbeta5(ji)) + vbeta5(ji) ) * column_qsatt_leaf(i) + &
                !                         &  (1.0d0 - vbeta1(ji)) * (1.0d0 - vbeta5(ji)) * (vbeta4(ji) + vbeta2sum(ji) + vbeta3sum(ji)) * column_qsatt_leaf(i)
                column_q_surf_leaf(i) = profile_vbeta3(test_grid,test_pft,(i-10)) * column_qsatt_leaf(i)
                column_q_surf_leaf(i) = MAX(column_q_surf_leaf(i), q_a_next(i))
             END DO ! i = 1, jnlvls
    
          END IF ! (control%ok_new_enerbil_nextstep ) THEN 




          ! Calculation of the net shortwave & longwave radiation
          ! short-wave radiation term can be calculated from eta4 
          ! loing-wave radiation tern can be calculated from eta1 to eta3   
              
          DO i =1, jnlvls

              IF (i .EQ. 1) THEN
                  ! IF (i .eq.1) which means that the surface layer net radiation term shoudle be equals 
                  ! (LE + H + G) + (h_horizontal + le_horizontal)    
                  ! note the sign of eta_3 is not the same for surface and leave layer balance calculation
                  ! here, the longwave and shortwave were separated according to eta1, eta2, eta3 and eta4 
                  flux_netsw(i) = eta_4_surf*swdown(ji) 
                  flux_netsw(i) = flux_netsw(i) + (eta_4(i) * swdown(ji))
                   
                  flux_netlw(i) = eta_1_surf*lwdown(ji) + (eta_2_surf * temp_surf_next) - eta_3_surf 
                  flux_netlw(i) = flux_netlw(i) + (eta_1(i) * lwdown(ji)) + (eta_2(i)*t_l_next(i)) +  eta_3(i) 
              ELSE IF (i .EQ. jnlvls) THEN
                  flux_netsw(i) =  swnet(ji) 
                  flux_netlw(i) =  lwdown(ji) - lwup(ji) 
              ELSE
              ! For Other layers, 
              ! the net rdiation can be calculated as the summation of leave layer energy balance (hrizontal terms) and 
              ! previous layer netradiation term (vertical term)  
              ! here, the longwave and shortwave were separated according to eta1, eta2, eta3 and eta4 
                  flux_netsw(i) =  (eta_4(i) * swdown(ji)) + flux_netsw(i-1)                         
                  flux_netlw(i) =  (eta_1(i) * lwdown(ji)) + (eta_2(i)*t_l_next(i)) +  eta_3(i)  +  flux_netlw(i-1) 
              ENDIF  
                
              flux_netrad(i) = flux_netsw(i) + flux_netlw(i) - soilflx(ji)                                 
       
           ENDDO
                         
           !+++++++++++++++++++++++++++++++++++++++++

           flux_rn_grid(ji, :) = flux_netrad(:)
           flux_h_grid(ji, :)  = flux_h(:)
           flux_le_grid(ji, :) = flux_le(:)
    
           fluxsens(ji) = flux_h(jnlvls)
           fluxlat(ji) = flux_le(jnlvls)  

           IF (control%ok_new_enerbil_nextstep ) THEN 
               DO i = 1, jnlvls ! 090215 calculate relhum of air
                   column_air_relhum(i) = (  (column_q_surf_leaf(i) * pb(ji)) / (tetens_1 + column_q_surf_leaf(i) * tetens_2)   )  / &
                                        & ( (column_qsatt_leaf(i) * pb(ji)) / (tetens_1 + column_qsatt_leaf(i) * tetens_2) )
               END DO ! i = 1, jnlvls
           END IF ! (control%ok_new_enerbil_nextstep ) THEN 




          IF (ld_jameswrite) THEN
        
               WRITE (numout,*) 'balance test delta temp: ', temp_surf_next - (psold(ji)/cp_air)
              
               WRITE (numout,*) 'balance test fluxes sum: ', (-flux_h(1) - flux_le(1) + &
                  & eta_1_surf*lwdown(ji) + eta_4_surf*swdown(ji) + eta_3_surf + soilflx(ji)   &
                  & + eta_2_surf * temp_surf_next) &
                  & * (dtradia/soilcap(ji))
              
               WRITE (numout,*) 'balance test delta temp / (dtradia/soilcap(ji): ', &
                  & (temp_surf_next - (psold(ji)/cp_air)) / (dtradia/soilcap(ji))
              
               WRITE (numout,*) 'balance test fluxes sum: ', -flux_h(1) - flux_le(1) + &
                  & eta_1_surf*lwdown(ji) + eta_4_surf*swdown(ji) + soilflx(ji)   &
                  & + (eta_2_surf * temp_surf_next) - eta_3_surf
            
               WRITE (numout,*) 'total balance test: ', &
                  & (temp_surf_next - (psold(ji)/cp_air)) / (dtradia/soilcap(ji)) - &
                  & (-flux_h(1) - flux_le(1) + &
                  & eta_1_surf*lwdown(ji) + eta_4_surf*swdown(ji) + soilflx(ji)   &
                  & + (eta_2_surf * temp_surf_next) - eta_3_surf)
            
               WRITE (numout,*) 'balance test flux_h(1): ', flux_h(1)
               WRITE (numout,*) 'balance test flux_le(1): ', flux_le(1)
               WRITE (numout,*) 'balance test eta_1_surf*lwdown(ji): ', eta_1_surf*lwdown(ji)
               WRITE (numout,*) 'balance test eta_1_surf', eta_1_surf
               WRITE (numout,*) 'balance test lwdown', lwdown(ji)
               WRITE (numout,*) 'balance test eta_2_surf * temp_surf_next: ', eta_2_surf * temp_surf_next
               WRITE (numout,*) 'balance test eta_3_surf: ', eta_3_surf
               WRITE (numout,*) 'balance test eta_4_surf*swdown(ji): ', eta_4_surf*swdown(ji)
               WRITE (numout,*) 'balance test soilflx(ji): ', soilflx(ji)
        
               ! %%%% JR 260813 write out delta_temp_leaf and sum_flux_leaf series %%%%%%
               ! %%%% this is to verify that energy is conserved %%%%%%
        

          END IF !(ld_jameswrite) 


           source_sink_t(1) = cp_air * rau(ji) * &
                             & ((t_a_next(1) - t_a_pres(1)) * delta_h(1)) / dtradia

           source_sink_q(1) = chalev0 * rau(ji) * &
                             & ((q_a_next(1) - q_a_pres(1)) * delta_h(1)) / dtradia
        
           source_sink_t_l(1) = ((t_l_next(1) - t_l_pres(1)) * delta_h(1)) / dtradia
        
           horiz_flux_t(1) = (t_l_next(1) - t_a_next(1))/big_r(1) * cp_air * rau(ji) 

           horiz_flux_q(1) = (((jalpha(1) * t_l_next(1) + jbeta(1)) - q_a_next(1))/ &
                             & big_r_prime(1)) * chalev0 * rau(ji)

           IF (jnlvls .gt. 1) THEN
               DO i = 2, jnlvls
 
                   source_sink_t(i) = cp_air * rau(ji) * &
                                & ((t_a_next(i) - t_a_pres(i)) * delta_h(i)) / dtradia
                   source_sink_q(i) = chalev0 * rau(ji) * &
                                & ((q_a_next(i) - q_a_pres(i)) * delta_h(i)) / dtradia
        
                   source_sink_t_l(i) = ((t_l_next(i) - t_l_pres(i)) * delta_h(i)) / dtradia
        
                   horiz_flux_t(i) = (t_l_next(i) - t_a_next(i))/big_r(i) * cp_air * rau(ji) 

           !    IF (jr_nextstep) THEN
           !        horiz_flux_q(i) = phi_trans(i)
           !    ELSE
                   horiz_flux_q(i) = (((jalpha(i) * t_l_next(i) + jbeta(i)) - q_a_next(i))/ &
                                 & big_r_prime(i)) * chalev0 * rau(ji)
           !    END IF 

               END DO ! i = 1, jnlvls
           END IF ! (jnlvls .gt. 1) THEN
        
           IF (jnlvls .gt. 1) THEN  

               delta_t_a(1) = cp_air * rau(ji) * ((t_a_next(1) - t_a_pres(1)) &
                           & * delta_h(1)) / dtradia
               sum_flux_a(1) = - flux_h(1+1) + ( flux_h(1) + horiz_flux_t(1) )
        
               delta_q_a(1) = chalev0 * rau(ji) * &
                           & ((q_a_next(1) - q_a_pres(1)) * delta_h(1)) / &
                           & dtradia

               sum_flux_q_a(1) = - flux_le(1+1) + ( flux_le(1) + horiz_flux_q(1) )
        
               DO i = 2, jnlvls-1
                    delta_t_a(i) = cp_air * rau(ji) * ((t_a_next(i) - t_a_pres(i)) &
                                & * delta_h(i)) / dtradia
                    sum_flux_a(i) = - flux_h(i+1) + ( flux_h(i) + horiz_flux_t(i) )
               END DO ! i = 1, jnlvls-1
        
               DO i = 2, jnlvls-1
                    delta_q_a(i) = chalev0 * rau(ji) * &
                               & ((q_a_next(i) - q_a_pres(i)) * delta_h(i)) / &
                               & dtradia
                    sum_flux_q_a(i) = - flux_le(i+1) + ( flux_le(i) + horiz_flux_q(i) )
               END DO ! i = 1, jnlvls-1
        
        
               DO i = 1, jnlvls-1
                     delta_temp_leaf(i) = rho_veg*leaf_tks * pad_deltah(i) * jtheta(i) * &
                                       & ((t_l_next(i) - t_l_pres(i)) * delta_h(i)) / &
                                       & dtradia
                     sum_flux_leaf(i) = (eta_1(i) * lwdown(ji)) + (eta_2(i)*t_l_next(i)) + &
                                       & eta_3(i) + (eta_4(i) * swdown(ji)) - &
                                       & ( horiz_flux_t(i) + horiz_flux_q(i))
               END DO ! i = 1, jnlvls-1
        
               DO i = 1, jnlvls-1
                      IF (ABS(source_sink_t(i) - sum_flux_a(i)) .gt. 1.0d-5) THEN
                          WRITE (numout,*) 'level: ', i, ' does not balance at atmos temperature'
                      ELSE
                          ! do nothing
                      END IF
        
                      IF (ABS(source_sink_q(i) - sum_flux_q_a(i)) .gt. 1.0d-5) THEN
                          WRITE (numout,*) 'level: ', i, ' does not balance at atmos specific humidity'
                      ELSE
                          ! do nothing
                      END IF
           
                      IF (ABS(delta_temp_leaf(i) - sum_flux_leaf(i)) .gt. 1.0d-5) THEN
                          WRITE (numout,*) 'level: ', i, ' does not balance at leaf temperature'
                      ELSE
                          ! do nothing
                      END IF
                  
                END DO ! i = 1, jnlvls-1
       
                DO i = 1, jnlvls-1
                      source_sink_sum_t = (source_sink_sum_t + source_sink_t(i) + horiz_flux_t(i))
                      source_sink_sum_q = (source_sink_sum_q + source_sink_q(i) + horiz_flux_q(i))
                END DO ! i = 1, jnlvls-1

             END IF ! (jnlvls .gt. 1) THEN


    END DO gridloop ! ji=1,kjpindex  ! ------------------------------------------------------------
 

 END SUBROUTINE jcalc


 ! -------------------------------------------------------------------------------------------------

 !!  ===========================================================================================================================
 !! SUBROUTINE                  : calc_jalpha_and_jbeta 
 !!
 !!
 !>\BRIEF                   : calculates the rate of change of saturated surface humidity of the leaf surface 
 !!
 !! DESCRIPTION                 : 
 !! 
 !!
 !! RECENT CHANGE(S): None
 !!
 !! MAIN OUTPUT VARIABLE(S) : jalpha, jbeta, jalpha_surf, jbeta_surf, q_sat_surf, temp_surf_pres
 !!
 !! REFERENCE(S)        : Monteith & Unsworth (2008 edition)
 !!
 !! FLOWCHART           : 
 !_ ==============================================================================================================================


 SUBROUTINE calc_jalpha_and_jbeta (temp_atmos_pres_grid, temp_leaf_pres_grid, jalpha, jbeta, &
               & jalpha_surf, jbeta_surf, temp_surf_pres, q_sat_surf, kjpindex, &
               & pb, temp_sol, jpdqsold)

    ! we use the approach outlined in Monteith & Unsworth (2008 edition)

    IMPLICIT NONE

    !! 0.1 Input variables

    INTEGER(i_std), INTENT(in)                               :: kjpindex                       !! Domain size (-)
    REAL(r_std), DIMENSION (kjpindex), INTENT (in)           :: pb                             !! Lowest level pressure (hPa)
    REAL(r_std), DIMENSION (kjpindex, jnlvls), INTENT (in)   :: temp_atmos_pres_grid           ! atmospheric temperature down the column (K)
    REAL(r_std), DIMENSION (kjpindex, jnlvls), INTENT (in)   :: temp_leaf_pres_grid            ! leaf temperature down the column (K)


    REAL(r_std), DIMENSION (kjpindex), INTENT(in)            :: temp_sol                       !! Soil temperature (K)

    !! 0.2 Output variables

    REAL(r_std), DIMENSION (jnlvls), INTENT (out)            :: jalpha, jbeta
    REAL(r_std), DIMENSION (kjpindex), INTENT (out)          :: jalpha_surf, jbeta_surf
    REAL(r_std), DIMENSION (kjpindex), INTENT (out)          :: q_sat_surf


    !! 0.3 Modified variables
    REAL(r_std), INTENT (inout)                              :: temp_surf_pres                 ! surface temperature at PRESENT time step (Kelvins)

    !! 0.4 Local variables

    REAL(r_std), DIMENSION (jnlvls)                          :: temp_atmos                     ! atmospheric temperature down the column (K)
    REAL(r_std), DIMENSION (jnlvls)                          :: temp_leaf                      ! leaf temperature down the column (K)

    INTEGER(i_std)                                :: ji                           ! grid point loop
    INTEGER(i_std)                                :: i                           ! levels loop

    REAL(r_std)                                   :: lambda_const = 2480.0d0        ! (J g^-1) at 10degC n.b. this changes by 2.4 per degree
    REAL(r_std)                                   :: bigm_w = 18.0d0                ! (g mol^-1) approximate molar mass of water
    REAL(r_std)                                   :: bigr_const = 8.314d0           ! (J mol^-1 K^-1) the molar gas constant
    REAL(r_std)                                   :: eta_const = 0.622d0            ! ratio of molecular mass of water to molecular mass of air
    REAL(r_std)                                   :: biga_const = 17.27d0
    REAL(r_std)                                   :: tstar_const = 273.0d0          ! (Kelvins)
    REAL(r_std)                                   :: tprime_const = 36.0d0          ! (Kelvins)
    REAL(r_std)                                   :: e_sat_tstar = 0.611d0          ! (kPa)
    REAL(r_std)                                   :: zrapp,zcorr,ztemperature,zqsat !! Temporary vector variables      
    REAL(r_std), DIMENSION (jnlvls)               :: q_sat                          ! (kg/kg)
    REAL(r_std), DIMENSION (jnlvls)               :: big_delta                      ! delta q_sat / delta temp at leaf temperature
    REAL(r_std), DIMENSION (jnlvls)               :: leaf_qsol_sat                    !! Soil temperature (K)
    REAL(r_std), DIMENSION (jnlvls)               :: leaf_dev_qsol                    !! Soil temperature (K)
    REAL(r_std), DIMENSION (jnlvls)               :: leaf_pdqsold                     !! Soil temperature (K)
    REAL(r_std), DIMENSION (kjpindex)             :: big_delta_surf
    REAL(r_std), DIMENSION (kjpindex)             :: j_qsol_sat, j_dev_qsol
    REAL(r_std), DIMENSION (kjpindex)             :: jpdqsold
    REAL(r_std), DIMENSION (kjpindex)             :: j_dev_q, j_q_sat


!   =from qsat_moisture=================================================================================================

    !! 1. Initialisation
    zrapp = msmlr_h2o/msmlr_air
    zcorr = 0.00320991_r_std

    jalpha(:) = 0.0d0
    jbeta(:) = 0.0d0
    jalpha_surf(:) = 0.0d0 
    jbeta_surf(:) = 0.0d0

    q_sat(:) = 0.0d0
    big_delta(:) = 0.0d0
    leaf_qsol_sat(:) = 0.0d0
    leaf_dev_qsol(:) = 0.0d0
    leaf_pdqsold(:) = 0.0d0
    big_delta_surf(:) = 0.0d0 
    j_qsol_sat(:) = 0.0d0
    j_dev_qsol(:) = 0.0d0
    jpdqsold(:) = 0.0d0

    DO ji = 1, kjpindex
      
       !! 2. Computes saturated humidity one time and store in qsfrict local array
         
       ztemperature = temp_sol(ji)
       j_qsol_sat(:) = 0.0d0
       j_dev_qsol(:) = 0.0d0


       CALL qsatcalc (kjpindex, temp_sol, pb, j_qsol_sat)

       CALL dev_qsatcalc (kjpindex, temp_sol, pb, j_dev_qsol)

       temp_surf_pres = temp_sol(ji)

       !+++ According to r2093 in the trunk, the line below is modified to avoid possible instability in the coupling.

       ! jpdqsold(ji) = j_dev_qsol(ji) * ( pb(ji)**kappa ) / cp_air

       jpdqsold(ji) = j_dev_qsol(ji)

       q_sat(:) = 0.0d0
       big_delta(:) = 0.0d0

       ! calculate jalpha_surf and jbeta_surf first of all:

       ! Using Tetens' method for close approximation of saturated specific humidity 
       !   (e.g. 2.27 of Monteith & Unsworth (2008)) and the specific humidity approximation 
       !   in terms of vapour pressure (e.g. 2.32 of Monteith & Unsworth (2008))
        

       q_sat_surf(ji) = (eta_const / 100000.0d0)  * (e_sat_tstar*1000.0d0) *  &
          & EXP((biga_const*(temp_surf_pres - tstar_const)/(temp_surf_pres - tprime_const)))

 
       big_delta_surf(ji) = ( lambda_const * bigm_w * q_sat_surf(ji) ) / &
                      & ( bigr_const * (temp_surf_pres)**2 )

       IF (ld_jameswrite) THEN
           WRITE (numout,*) 'in calc_jalpha_jbeta, big_delta_surf is: ', big_delta_surf(ji)    
       END IF !(ld_jameswrite)

       ! jalpha_surf(ji) = j_dev_qsol(ji)
       jalpha_surf(ji) = jpdqsold(ji)
       jbeta_surf(ji) = j_qsol_sat(ji) - temp_sol(ji) * jpdqsold(ji)
       q_sat_surf(ji) = j_qsol_sat(ji)

       IF (jnlvls .gt. 1) THEN

           DO i=1, jnlvls

               temp_atmos(i) = temp_atmos_pres_grid(ji,i)
               temp_leaf(i) = temp_leaf_pres_grid(ji,i)

               ! Using Tetens' method for close approximation of saturated specific humidity 
               !   (see 2.27 of Monteith & Unsworth (2008)) and the specific humidity 
               !   approximation in terms of vapour pressure (see 2.32 of Monteith & Unsworth (2008))
        
               q_sat(i) = (eta_const / 100000.0d0)  * (e_sat_tstar*1000.0d0) *  &
                   & EXP((biga_const*(temp_leaf(i) - tstar_const)/(temp_leaf(i) - tprime_const)))

               big_delta(i) = ( lambda_const * bigm_w * q_sat(i) ) / &
                   & ( bigr_const * temp_leaf(i)**2 )

               jalpha(i) = big_delta(i) 
               jbeta(i) = q_sat(i) - (temp_leaf(i) * jalpha(i))


               CALL qsatcalc (kjpindex, temp_leaf_pres_grid(:,i), pb, j_q_sat)

               !   CALL dev_qsatcalc (kjpindex, temp_sol, pb, dev_qsol) jtest3
               CALL dev_qsatcalc (kjpindex, temp_leaf_pres_grid(:,i), pb, j_dev_q)

               jalpha(i) = j_dev_q(ji) * ( pb(ji)**kappa ) / cp_air
           
               jbeta(i) = j_q_sat(ji) - temp_leaf(i) * j_dev_q(ji) * ( pb(ji)**kappa ) / cp_air

           END DO !i = 1, jnlvls

       END IF ! IF (jnlvls .gt. 1)



    END DO ! ji = 1, kjpindex

 END SUBROUTINE calc_jalpha_and_jbeta 


 ! -------------------------------------------------------------------------------------------------

 !!  ===========================================================================================================================
 !! SUBROUTINE                  : jprofile
 !!
 !!
 !>\BRIEF                   : calculates the rate of change of saturated surface humidity of the leaf surface 
 !!
 !! DESCRIPTION                 : This routine will allocate a LAD to each level of the canopy using the approach 
 !!                                           of Yang et al. (1999) and the parameters they allocate to an aspen stand (if not 
 !!                                           already known through read-in data). Based on this LAD profile, canopy turbulence 
 !!                                           statistics will be allocated using the method of Massman & Weil (1999).
 !! 
 !!
 !! RECENT CHANGE(S): None
 !!
 !! MAIN OUTPUT VARIABLE(S) : jalpha, jbeta, jalpha_surf, jbeta_surf, q_sat_surf, temp_surf_pres
 !!
 !! REFERENCE(S)        : Massman, W.J. & Weil, J.C., 1999. An analytical one-dimensional second-order 
 !!                               closure model of turbu lence statistics and the lagrangian time scale within and 
 !!                               above plant canopies of arbitrary structure. Boundary-Layer Meteorology, 91, 
 !!                               pp.81–107.
 !!
 !!                               Raupach, M.R., Antonia R.A. and Rajagpalan, S., 1991. 'Rough-wall turbulent 
 !!                               boundary layers', Applied Mechanics Review, 44, pp. 1-25
 !!
 !!                               Yang, X., Witcosky, J.J. & Miller, D.R., 1999. Vertical overstory canopy 
 !!                               architecture of temperate deciduous in the Eastern United States. Forest Science, 
 !!                               45(3), pp.349–358.
 !!
 !!                               Wohlfahrt G. and A. Cernusca 2002, Monmentum tansfer by a mountain
 !!                               meadow canopy: a simulation analysis based on Massman's (1997) model,
 !!                               Boundary Layer Meteorology, 103, 391-407  
 !!
 !! FLOWCHART           : 
 !_ ==============================================================================================================================


 SUBROUTINE jprofile (pad_deltah, layer_height, ustar_ac, delta_z, &
                      & delta_h, box_height, k_eddy, delta_h_surf, delta_z_surf, &
                      & box_height_surf, u_star, speed, u_speed, big_pad, &
                      & k_eddy_ceil,k_eddy_slope,k_eddy_ustar, k_eddy_surf, surf_lev, top_lev, swnet, jrpad, &
                      & a_1, a_2, a_3, a_4, a_5, WC02)

    IMPLICIT NONE





    !! 0 Variable and parameter description



    !! 0.1 Input variables
    REAL(r_std), DIMENSION (0:jnlvls), INTENT(in)    :: pad_deltah             ! Plant Area Density as a function of height above the ground (m^2/m^3)

    REAL(r_std), DIMENSION(jnlvls+1), INTENT(in)     :: box_height             ! height of each box edge, starting at the surface layer (m)
    REAL(r_std), INTENT(in)                          :: box_height_surf        ! height of bottom of surface layer box (m)
    REAL(r_std), INTENT(in)                          :: surf_lev
    REAL(r_std), DIMENSION(jnlvls+1), INTENT(in)     :: top_lev

    REAL(r_std), INTENT (in)                         :: swnet                  !! Net surface short-wave flux (W m^{-2})

    REAL(r_std), INTENT(in)                          :: k_eddy_ceil            !! for k_eddy parameterization 
    REAL(r_std), INTENT(in)                          :: k_eddy_slope           !! for k_eddy parameterization  
    REAL(r_std), INTENT(in)                          :: k_eddy_ustar           !! for k_eddy parameterization 

    !! 0.2 Output variables
    REAL(r_std), INTENT(out)                         :: u_star                 ! friction velocity (m/s)
    REAL(r_std), INTENT(out)                         :: layer_height           ! the height of each layer in the canopy (assumed equal for now) (m)
    REAL(r_std), DIMENSION (jnlvls)                  :: cum_pad_deltah         ! Cumulative Plant Area Density multiplied by layer height (m^2/m^3)
    REAL(r_std), DIMENSION (jnlvls), INTENT(out)     :: delta_z                ! change in height (m)
    REAL(r_std), DIMENSION (jnlvls), INTENT(out)     :: u_speed                ! vertical wind speed (m/s)
    REAL(r_std), DIMENSION (jnlvls), INTENT(out)     :: big_pad 
    REAL(r_std), DIMENSION (jnlvls), INTENT(out)     :: k_eddy                 ! calculated eddy diffusivity coefficient (m^2/s)
    REAL(r_std), DIMENSION (jnlvls), INTENT(out)     :: delta_h                ! height of each level (m)
    REAL(r_std), DIMENSION (0:jnlvls), INTENT(out)   :: jrpad                  ! Plant Area Density, replacement for values above (as definitions between different
                                                                               !    parts of the model was becoming confusing)
                                                                               ! 'jrpad' is the total one-sided leaf area per unit of layer volume

    !! 0.3 Modified variables




    !! 0.4 Local variables
    REAL(r_std), DIMENSION (jnlvls)                   :: cum_jrpad         ! Cumulative Plant Area Density (m^2/m^3)

    !! 0.4 Local variables
    REAL(r_std)                                       :: speed
    REAL(r_std)                                       :: param_b=0.426d0
    REAL(r_std)                                       :: param_c=1.856d0
    REAL(r_std)                                       :: c_d=0.2d0
    REAL(r_std)                                       :: p_m=1.0d0
    REAL(r_std)                                       :: jlambda
    REAL(r_std)                                       :: jalpha=0.05d0
    REAL(r_std)                                       :: small_n
    INTEGER(i_std)                                    :: i
    INTEGER(i_std)                                    :: j, k, m, n
    REAL(r_std)                                       :: c_1, c_2, c_3          ! coefficients as defined in Massman & Weil (1999) p. 83
    REAL(r_std), DIMENSION (jnlvls+1)                 :: jepsilon               ! epsilon coefficient as in Massman & Weil (1999), 
                                                                                !   equation (6) and just after
    REAL(r_std), DIMENSION (jnlvls)                  :: eta_z                  ! eta coefficient as in Massman & Weil (1999), equation (3)
    REAL(r_std), DIMENSION (jnlvls)                  :: sigma_exp_1
    REAL(r_std), DIMENSION (jnlvls)                  :: sigma_exp_2
    REAL(r_std), DIMENSION (jnlvls)                  :: sigma_exp_3
    REAL(r_std), DIMENSION (jnlvls)                  :: sigma_e_over_u_star    ! sigma_e normalised by u_star (-)
    REAL(r_std), DIMENSION (jnlvls)                  :: sigma_u_over_u_star    ! sigma_u normalised by u_star (-)
    REAL(r_std), DIMENSION (jnlvls)                  :: sigma_v_over_u_star    ! sigma_v normalised by u_star (-)
    REAL(r_std), DIMENSION (jnlvls)                  :: sigma_w_over_u_star    ! sigma_w normalised by u_star (-)
    REAL(r_std), DIMENSION (jnlvls)                  :: sigma_u                ! standard deviation in u-stream (horizontal) velocity (m/s); Massman & Weil eqn. (11)
    REAL(r_std), DIMENSION (jnlvls)                  :: sigma_v                ! standard deviation in v-stream (horizontal) velocity (m/s); Massman & Weil eqn. (11)
    REAL(r_std), DIMENSION (jnlvls)                  :: sigma_w                ! standard deviation in w-stream (vertical) velocity (m/s); Massman & Weil eqn. (11)

    REAL(r_std), DIMENSION (timesteps)                :: ustar_ac               ! ?? not used ?? Friction velocity (m/s)
    REAL(r_std)                                       :: delta_h_surf           ! ?? not used ?? height of surface level (m)
    REAL(r_std)                                       :: delta_z_surf           ! ?? not used ?? vertical difference between the middle of the surface layer
                                                                                             !     and the middle of the first layer (m)
    REAL(r_std)                                       :: k_eddy_surf            ! ?? not used ?? 

    REAL(r_std)                                       :: gamma_1=2.4d0
    REAL(r_std)                                       :: gamma_2=1.9d0
    REAL(r_std)                                       :: gamma_3=1.25d0
    REAL(r_std)                                       :: nu_1, nu_2, nu_3
    REAL(r_std)                                       :: bigb1
    REAL(r_std)                                       :: rp_a_1 = 1.25d0        ! Raupach et al. (1991) co-efficients
    REAL(r_std)                                       :: rp_a_0 = 0.25d0        ! Raupach et al. (1991) co-efficients
    REAL(r_std)                                       :: rp_c_0 = 0.3d0         ! Raupach et al. (1991) co-efficients 
    REAL(r_std), DIMENSION (0:jnlvls)                 :: gu_l
    REAL(r_std)                                       :: gu_ext = 0.7d0
    REAL(r_std), DIMENSION (jnlvls)                   :: middle_layer_height    ! the height in the middle of each layer in the canopy (m)
    REAL(r_std), DIMENSION (jnlvls+1)                 :: z_h                    ! height above ground at the base of each layer (m)
    REAL(r_std), DIMENSION (jnlvls+1)                 :: jeta
    REAL(r_std), DIMENSION (jnlvls)                   :: t_lagrangian           ! Lagrangian timescale (s)

    ! Parameter for dynamic drag coefficient cd_z
    ! Ref:  Wohlfahrt G. and A. Cernusca 2002, Monmentum tansfer by a mountain
    ! meadow canopy: a simulation analysis based on Massman's (1997) model,
    ! Boundary Layer Meteorology, 103, 391-407 
    ! a negtive sign was adeed on a_4 (this could be the typo in origial paper) 
    LOGICAL, INTENT(in)                               :: WC02                     ! flage for the drag_coefficient is depend on LAI profile
    REAL(r_std), DIMENSION(jnlvls)                    :: c_dz
    REAL(r_std), INTENT(in)                           :: a_1 
    REAL(r_std), INTENT(in)                           :: a_2 
    REAL(r_std), INTENT(in)                           :: a_3 
    REAL(r_std), INTENT(in)                           :: a_4 
    REAL(r_std), INTENT(in)                           :: a_5 




    layer_height = 0.0_r_std
    middle_layer_height = 0.0_r_std
    z_h = 0.0_r_std
    jepsilon = 0.0_r_std
    cum_pad_deltah = 0.0_r_std
    small_n = 0.0_r_std
    bigb1 = 0.0_r_std
    sigma_exp_1 = 0.0_r_std
    sigma_exp_2 = 0.0_r_std
    sigma_exp_3 = 0.0_r_std

    nu_1 = (gamma_1**2.0_r_std+gamma_2**2.0_r_std+gamma_3**2.0_r_std)**(-0.5_r_std)
    nu_3 = (gamma_1**2.0_r_std+gamma_2**2.0_r_std+gamma_3**2.0_r_std)**(3.0_r_std/2.0_r_std)   
    nu_2 = nu_3/6.0_r_std-(gamma_3**2.0_r_std/(2.0_r_std*nu_1))

    DO i=1, jnlvls
        delta_h(i) = box_height(i+1) - box_height(i)
    END DO !i=1, jnlvls

    DO i=1, jnlvls-1
        delta_z(i) = (box_height(i+1) + (delta_h(i+1)/2.0_r_std)) - &
                         &  (box_height(i) + (delta_h(i)/2.0_r_std))
    END DO !i=1, jnlvls

    DO i=1, jnlvls+1
        z_h(i) = box_height(i)
        jeta(i) = z_h(i) / canopy_height
        ! this is from Yang et al. (1999), equation 1
        !++++TEMP++++
        ! for single layer case without STOMATE
        !jepsilon(i) = 1.0_r_std - exp ((-1.0_r_std)*(( (1-jeta(i)) /param_b ) ** (param_c)))
        jepsilon(i) = 1.0_r_std 
        !+++++++++++++
    END DO !i=1, jnlvls+1

    DO i=1, jnlvls
        middle_layer_height(i) = (z_h(i) + z_h(i+1)) / 2.0_r_std
    END DO !i=1, jnlvls


    delta_h(1) = top_lev(1) - surf_lev
     
    DO i = 2, jnlvls
        delta_h(i) = top_lev(i) - top_lev(i-1)
    END DO ! i = 2, jnlvls 

    DO i = 1, jnlvls-1
        delta_z(i) = (delta_h(i) + delta_h(i+1))/2.0_r_std
    END DO ! i = 1, jnlvls-1


    ! coefficients c_1, c_2 and c_3 as defined in Massman & Weil (1999) p. 83

    c_1 = 0.320_r_std
    c_2 = 0.264_r_std
    c_3 = 15.1_r_std

    cum_pad_deltah(jnlvls) = pad_deltah(jnlvls)
    DO i=jnlvls-1, 1, -1
        cum_pad_deltah(i) = cum_pad_deltah(i+1) + pad_deltah(i)
    END DO !i=1, jnlvls

    DO i=1, jnlvls
        big_pad(i) = pad_deltah(i) / delta_h(i)
    END DO ! i=1, jnlvls

    ! here we define jrpad correctly - the total one-sided leaf area per unit of layer volume
    DO i=1, jnlvls
        jrpad(i) = pad_deltah(i) / delta_h(i)
    END DO !i=1, jnlvls

!    eta_z(1) = (( c_d * pad_deltah(1) ) / p_m )
!    DO i=2, jnlvls
!        eta_z(i) = (( c_d * pad_deltah(i) ) / p_m ) + eta_z(i-1)
!    END DO !i=1, jnlvls

    !+++++ CHECK +++++++
    !Calculate the eta_z with Dynamic Drag Coeffiient (c_dz) or Static Drag Coefficient Constance(c_d)
    IF ( WC02 ) THEN
    !Dyanaic Drage Coefficient vertical
        c_dz(:) = c_d !set default value c_dz to c_d
        DO i = 1, jnlvls
           c_dz(i) =  a_1**(-1.0_r_std*cum_pad_deltah(i)/a_2) + a_3**(-1.0_r_std*cum_pad_deltah(i)/a_4) + a_5   
           !WRITE(numout, *) 'C_Deff(z):', c_dz(i), 'PAIcum:',cum_pad_deltah(i)
        ENDDO 
           eta_z(1) = (( c_dz(1) * pad_deltah(1)  )  )
        DO i=2, jnlvls
           eta_z(i) = (( c_dz(i) * pad_deltah(i)  )  ) + eta_z(i-1)
        !   WRITE(numout, *) 'eta_z:', eta_z(i)
        END DO 
            
    ELSE
    !Constant Drag Coefficient
           eta_z(1) = (( c_d * pad_deltah(1) ) / p_m )
        DO i=2, jnlvls
           eta_z(i) = (( c_d * pad_deltah(i) ) / p_m ) + eta_z(i-1)
        END DO 
             
    ENDIF
   !+++++++++++++++++++++++

    

    u_star = speed * (c_1 - c_2 * exp (- c_3 * eta_z(jnlvls) ) )


    jlambda = SQRT( 7.0_r_std / (3.0_r_std * jalpha**2.0_r_std * nu_1 * nu_3 ) + & 
                      & (1.0_r_std / 3.0_r_std - (gamma_3**2.0_r_std * nu_1**2.0_r_std ))/  &
                      & (3.0_r_std * jalpha**2.0_r_std * nu_1 * nu_2))

    small_n = eta_z(jnlvls)/((2.0_r_std*(u_star**2.0_r_std))/(speed**2.0_r_std))


    bigb1 = -(9.0_r_std* u_star / speed)/(2.0_r_std* jalpha * nu_1 * & 
               & (9.0_r_std/4.0_r_std - ((jlambda**2.0_r_std*u_star**4.0_r_std)/speed**4.0_r_std)))
    DO i=1, jnlvls
        sigma_exp_1(i) = -jlambda * eta_z(jnlvls) * ( 1.0_r_std - eta_z(i) / eta_z(jnlvls))
        sigma_exp_2(i) = -3.0_r_std * small_n * ( 1.0_r_std - ( eta_z(i) / eta_z(jnlvls)))
        sigma_exp_3(i) = -jlambda * eta_z(jnlvls) * (1.0_r_std - eta_z(i) / eta_z(jnlvls))
        sigma_e_over_u_star(i) = ( nu_3 * &
                             & EXP (sigma_exp_1(i)) + bigb1 *( EXP( sigma_exp_2(i)) - &
                             & EXP( sigma_exp_3(i))))**(1.0_r_std/3.0_r_std)
        sigma_u_over_u_star(i) = gamma_1 * nu_1 * sigma_e_over_u_star(i)
        sigma_v_over_u_star(i) = gamma_2 * nu_1 * sigma_e_over_u_star(i)
        sigma_w_over_u_star(i) = gamma_3 * nu_1 * sigma_e_over_u_star(i)
        sigma_u(i) = sigma_u_over_u_star(i) * u_star
        sigma_v(i) = sigma_v_over_u_star(i) * u_star
        sigma_w(i) = sigma_w_over_u_star(i) * u_star
        u_speed(i) = speed * exp (-small_n * (1-(eta_z(i)/eta_z(jnlvls))))

        !++++SET a MINIMUM wind speed  +++++
        u_speed(i) = MAX (0.05_r_std, u_speed(i) ) 
        !++++++++++++++++++++++++++++++++++++   
     END DO !i=1, jnlvls


    IF (jnlvls .eq. 1) THEN
        u_speed(1) = speed
    END IF

    ! approximate the Lagrangian timescale using the parameterisation of Raupach (1989)
    ! where 0.4_r_std is the von Karmen constant

    DO i=1, jnlvls
        t_lagrangian(i) = (canopy_height/u_star) *                                      &
                       &    (max (rp_c_0, 0.4_r_std * ( z_h(i) - (0.63_r_std*canopy_height) )   &
                       &     / (rp_a_1 * canopy_height)))
    END DO !i=1, jnlvls

    ! this is the eddy diffusivity approximation for the far-field (also Raupach, 1989)

    DO i=1, jnlvls

        k_eddy(i) = (sigma_w(i))**2 * t_lagrangian(i)

    END DO !i=1, jnlvls


    !+++ FOR NIGHTTIME K_EDDY was parameterized as a S-shape funtion based on threshood ustar value ++++
    ! Here we use a paramterization method for scaling the K_eddy based on u*
    ! 
    ! u_star < 0.3 cut 50% 
    ! u_star > 0.3 increase from 50 %
    ! K_EDDY_SLOPE determine the increasing/decreasing rate 
    !     
      k_eddy = k_eddy * (1.0 / (1.0 + EXP (-k_eddy_slope * (u_star- k_eddy_ustar))) ) 
    !++++++++++++++++++++++++++++++++++++++

 END SUBROUTINE jprofile

    
 !!  ===========================================================================================================================
 !! SUBROUTINE                  : calc_stomatal_resistance
 !!
 !!
 !>\BRIEF                   : 
 !!
 !! DESCRIPTION                 : 
 !! 
 !!
 !! RECENT CHANGE(S): None
 !!
 !! MAIN OUTPUT VARIABLE(S) :     down_sw_tab, down_lw_tab, big_r_prime
 !!
 !! REFERENCE(S)        :     Gao, W., Wesely, M.L. & Doskey, P.V., 1993. Numerical modeling of the turbulent 
 !!                                   diffusion and chemistry of NOx, O3, Isoprene, and other reactive trace gases in 
 !!                                   and above a forest canopy. Journal of Geophysical Research, 98(D10), pp.18339–18353
 !!
 !!                                   Guimberteau, M., 2010. Modélisation de l’hydrologie continentale et influences de 
 !!                                   l'irrigation sur le cycle de l'eau.
 !!
 !!                                   Monteith J., Unsworth M., 2008. Principles of environmental physics. Published 
 !!                                   Academic Press (Elsevier), ISBN: 978-0-12-505103-3
 !!
 !! FLOWCHART           : 
 !_ ==============================================================================================================================


 ! -------------------------------------------------------------------------------------------------

 SUBROUTINE calc_stomatal_resistance (pad_deltah, swnet, lwdown, pb, &
                      & down_sw_tab, down_lw_tab, big_r_prime, temp_atmos_pres_grid, &
                      & q_atmos_pres_grid, big_r, u_speed, r_sto_v_fac, big_r_h2o_fac, &
                      & rau, kjpindex, sr_fac, hd_big_r_prime, q_cdrag, delta_h, jrpad, &
                      & profile_rveget)




    IMPLICIT NONE
    

    !! 0.1 Input variables
    INTEGER(i_std), INTENT(in)                                   :: kjpindex         !! Domain size (-)
    REAL(r_std), DIMENSION (kjpindex), INTENT(in)                :: rau              !! Air density (kg m^{-3})
    REAL(r_std),DIMENSION (kjpindex), INTENT(in)                 :: swnet            !! Net surface short-wave flux (W m^{-2})
    REAL(r_std),DIMENSION (kjpindex), INTENT(in)                 :: lwdown           !! Down-welling long-wave flux (W m^{-2})
    REAL(r_std),DIMENSION (kjpindex), INTENT(in)                 :: pb               !! Lowest level pressure (hPa)
    REAL(r_std), DIMENSION (0:jnlvls), INTENT(in)                :: pad_deltah       !! Plant Area Density multiplied by layer height
    REAL(r_std), DIMENSION (kjpindex, jnlvls), INTENT(in)        :: temp_atmos_pres_grid
    REAL(r_std), DIMENSION (kjpindex, jnlvls), INTENT(in)        :: q_atmos_pres_grid
    REAL(r_std), DIMENSION(jnlvls), INTENT(in)                   :: big_r    
    REAL(r_std), DIMENSION(jnlvls), INTENT(in)                   :: u_speed
    REAL(r_std), INTENT(in)                                      :: r_sto_v_fac, big_r_h2o_fac    
    REAL(r_std), INTENT(in)                                      :: sr_fac           !! parameterisation factor   
    REAL(r_std), INTENT(in)                                      :: hd_big_r_prime   !! parameterisation factor
    REAL(r_std), DIMENSION (kjpindex), INTENT (in)               :: q_cdrag          !! Surface drag (m s^{-1})
    REAL(r_std), DIMENSION(jnlvls), INTENT(in)                   :: delta_h          !! height of each level (m)
    REAL(r_std),DIMENSION (kjpindex,nvm,nlevels_tot), INTENT(in) :: profile_rveget
    REAL(r_std), DIMENSION (0:jnlvls), INTENT(in)                :: jrpad            !! Plant Area Density, replacement for values above
                                                                                     !!   (as definitions between different
                                                                                     !! parts of the model was becoming confusing)
                                                                                     !!   'jrpad' is the total one-sided leaf area per unit of layer volume

    !! 0.2 Output variables
    REAL(r_std), DIMENSION (jnlvls), INTENT (out)                :: down_sw_tab, down_lw_tab
    REAL(r_std), DIMENSION (jnlvls), INTENT(out)                 :: big_r_prime



    !! 0.3 Modified variables


    !! 0.4 Local variables

    INTEGER(i_std)                                     :: i, j, jk             !! loop counters
    
    REAL(r_std), DIMENSION (jnlvls)                    :: temp_atmos_pres    
    REAL(r_std), DIMENSION (jnlvls)                    :: q_atmos_pres    
    ! constants from Tetens' expression for saturated vapor pressure (see Monteith &
    !   Unsworth 2.1)  bigA (-), tstar (K), tprime(K)
    REAL(r_std)                                        :: biga_const = 17.27_r_std
    REAL(r_std)                                        :: tstar_const = 273.0_r_std
    REAL(r_std)                                        :: tprime_const = 36.0_r_std 
    REAL(r_std)                                        :: e_sat_tstar = 0.611_r_std  !vapor pressure at T* (kPa)
    !! laitab is the accumulated LAI from the top of the canopy (see Gao et al., 1993)
    REAL(r_std), DIMENSION (jnlvls)                              :: laitab
    REAL(r_std), DIMENSION (jnlvls)                              :: e_sat_tair 
    !! extinction coefficient used is that from from src_parameters of 0.5_r_std for all PFTs               
    REAL(r_std)                                        :: ext_coef_sw = 0.5_r_std
    REAL(r_std)                                        :: ext_coef_lw = 0.8_r_std
    ! this is the assumed minimum average stomatal resistance (s/m)
    REAL(r_std)                                        :: k_0 = 25.0d-2
    ! PFT dependent number that is characterised by vegetation (value for PFT8, 
    !   from Guimberteau, Table 2.2) (-)
    REAL(r_std)                                        :: bigr_0 = 125.0_r_std ! solar radiation constant (Wm^-2)
    REAL(r_std)                                        :: lambda = 1.5d-3
    ! coefficient for the increase of stomatal closure under hydrological deficit 
    !   (m^2 s kg^(-1))
    REAL(r_std), DIMENSION (jnlvls)                    :: delta_c
    ! deficit of water in the atmosphere at the level in question (kg m^-3)
    REAL(r_std), DIMENSION (jnlvls)                    :: q_sat_ta, r_sto_v
    REAL(r_std)                                        :: big_l =2.46d6           !(J/kg)
    REAL(r_std)                                        :: r_b = 30.0_r_std            ! leaf boundary layer resistance
    ! (see Guimberteau section 4.2)
    REAL(r_std)                                        :: r_sv = 25.0_r_std           ! structural resistance for PFT8 (s/m) 
    REAL(r_std)                                        :: r_a = 0.0_r_std             ! aerodynamic resistance represented by the eddy
    REAL(r_std)                                        :: bigu_sv = 0.8_r_std         ! water uptake function
    REAL(r_std)                                        :: small_d                 ! characteristic leaf length (m)
    REAL(r_std), DIMENSION(jnlvls)                     :: bigd_h2o                ! molecular diffusivity of water
    REAL(r_std), DIMENSION(jnlvls)                     :: nu_air                  ! kinematic viscosity of air
    REAL(r_std), DIMENSION(jnlvls)                     :: schmidt                 ! schmidt number
    REAL(r_std), DIMENSION(jnlvls)                     :: sherwood                ! sherwood number
    REAL(r_std), DIMENSION(jnlvls)                     :: reynolds                ! reynolds number
    REAL(r_std), DIMENSION(jnlvls)                     :: big_r_h2o               ! h2o leaf boundary layer resistance
    REAL(r_std)                                        :: smalla = 23.0_r_std        ! this is the assumed minimum average stomatal resistance (s/m)

    r_sto_v=zero
    big_r_h2o=zero


   !---initialized the big_r_prime
    big_r_prime(:)= 1.0d3
   !----

    IF (ld_jameswrite) WRITE (*,*) '250613 hd_big_r_prime is: ', hd_big_r_prime
 
    IF (control%ok_new_enerbil_nextstep ) THEN 

    DO jk = 1, kjpindex

        temp_atmos_pres(:) = temp_atmos_pres_grid(jk,:)
        q_atmos_pres(:) = q_atmos_pres_grid(jk,:)


        small_d = 0.05_r_std                            ! set the leaf length to 5cm for now
        ! physical dimensions for aspen leaf length: www.ag.ndsu.edu/trees/handbook/th-3-103.pdf
    
        ! n.b. for Eucalyptus delegatensis:
        ! http://davisla.wordpress.com/2012/04/07/plant-of-the-week-eucalyptus-delegatensis/
        ! 'juvenile leaf being up to 25cm long and 10cm broad the mature 18cm long and 3cm broad'
    
    
        ! calculate steps of LAI (using the same formulation as in diffuco.f90 for co2)
        laitab = 0.0_r_std
     
        ! calculate light fraction that comes through at a given LAI for each vegetation type
        ! laitab(jnlvls) = 0
     
        IF (jnlvls .eq. 1) THEN
            ! omit the layered calculation for single layer case
        ELSE
    
            DO i=jnlvls-1, 1, -1
         
                     ! laitab(i) = (pad_deltah(i+1) * delta_h(i+1)) + laitab(i+1)   
                     ! laitab(i) = (pad_deltah(i+1) ) + laitab(i+1)             
                     laitab(i) = (jrpad(i+1) * delta_h(i+1) ) + laitab(i+1)             

                END DO !i=jnlvls, 1, -1
        
                DO i=1, jnlvls
                     down_sw_tab(i) = swnet(1) * exp (-ext_coef_sw * laitab(i))
                     down_lw_tab(i) = lwdown(1) * exp (-ext_coef_lw * laitab(i))
                END DO !i=1, jnlvls


                DO i=1, jnlvls
                     q_sat_ta(i) = (0.622_r_std/ pb(1))  *e_sat_tstar* 1000.0_r_std * &
                       & EXP((biga_const*(temp_atmos_pres(i) - tstar_const) / &
                       & (temp_atmos_pres(i) - tprime_const)))
            
                     delta_c(i) = rau(1) * MAX (q_sat_ta(i) - q_atmos_pres(i), zero) /100.0_r_std
                END DO !i=1, jnlvls
    
                DO i=1, jnlvls
                     IF (down_sw_tab(i) .ne. 0.0d0) THEN
                          r_sto_v(i) = (smalla/k_0) * (1/pad_deltah(i)) * &
                                & ((down_sw_tab(i)+bigr_0)/down_sw_tab(i))  &
                                & * (un + lambda * (delta_c(i)/smalla) )
                     ELSE
                          r_sto_v(i) = 1.0d20
                     END IF
                END DO !i=1,jnlvls

                IF (control%ok_new_enerbil_nextstep ) THEN 
                     r_sto_v(:) = 1.0d20
                     r_sto_v(10:19) = profile_rveget(test_grid,test_pft,1:10) 
                END IF ! (control%ok_new_enerbil_nextstep ) THEN 

                DO i = 1, jnlvls
                     WRITE (numout,*) 'r_sto_v(i) is: ', r_sto_v(i)
                END DO
             
                ! We're using the expression from ORCHIDEE as in Guimberteau (2010)
    
                DO i = 1, jnlvls
    
                   ! n.b. the transition point from laminar to turbulent flow occurs when the 
                   !   Reynolds number passes 8,000 to 25,000 (Baldochhi, 1988)

                   bigd_h2o(i) = 2.26d-5 + (1.51d-7 * (temp_atmos_pres(i) - 273.15_r_std))
                   nu_air(i) = 1.35d-5 + (1.0d-7 * (temp_atmos_pres(i) - 273.15_r_std))
                   schmidt(i) = nu_air(i) / bigd_h2o(i)
    
                   reynolds(i) = (small_d * u_speed(i))/nu_air(i)
    
                   ! is the flow laminar or turbulent?
                   IF (reynolds(i) .le. 8000.0_r_std) THEN
                       sherwood(i) = 0.66_r_std * (reynolds(i))**(0.5_r_std) + (schmidt(i))**(0.33_r_std)
                   ELSE IF (reynolds(i) .gt. 8000.0_r_std) THEN
                       sherwood(i) = 0.03_r_std * (reynolds(i))**(0.8_r_std) + (schmidt(i))**(0.33_r_std)
                   END IF
    
                   big_r_h2o(i) = small_d / (bigd_h2o(i) * sherwood(i))
                   ! +++ TEMP +++ use the same expression of leaf boundary+++++++++++++++++++++++++++++++++++++++
                   ! resistance in the subroutineL diffuco_trans_co2 
                       big_r_h2o(i) = (1.0_r_std /25.0_r_std)/(22.4_r_std*temp_atmos_pres(i)/273._r_std/1000._r_std) 
                       !+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
                       big_r_prime(i) = ((r_sto_v(i) * r_sto_v_fac) + (big_r_h2o(i) * big_r_h2o_fac)) !/ pad_deltah(i)


                END DO ! i=1, jnlvls
    
            END IF ! (jnlvls .eq. 1)
    
            big_r_prime = ((r_sto_v * r_sto_v_fac) + (big_r_h2o * big_r_h2o_fac))
            
            big_r_prime = big_r_prime * sr_fac
    
            big_r_prime(jnlvls) = 1.0d20   ! the top level resistance should be approaching 
                                         !   infinity (see diagram) need to implement this 
                                         !   more elegantly
                                         
            IF (ld_jameswrite) WRITE(numout, *) 'big_r_prime is: ', big_r_prime


          END DO ! jk = 1, kjpindex



    ELSE ! (jr_nextstep)

        DO jk = 1, kjpindex

            temp_atmos_pres(:) = temp_atmos_pres_grid(jk,:)
            q_atmos_pres(:) = q_atmos_pres_grid(jk,:)


            small_d = 0.05_r_std                            ! set the leaf length to 5cm for now
            ! physical dimensions for aspen leaf length: www.ag.ndsu.edu/trees/handbook/th-3-103.pdf
    
            ! n.b. for Eucalyptus delegatensis:
            ! http://davisla.wordpress.com/2012/04/07/plant-of-the-week-eucalyptus-delegatensis/
            ! 'juvenile leaf being up to 25cm long and 10cm broad the mature 18cm long and 3cm broad'
    
    
            ! calculate steps of LAI (using the same formulation as in diffuco.f90 for co2)
            laitab = 0.0_r_std
     
            ! calculate light fraction that comes through at a given LAI for each vegetation type
            ! laitab(jnlvls) = 0
     
            IF (jnlvls .eq. 1) THEN
                ! omit the layered calculation for single layer case
            ELSE
    
                DO i=jnlvls-1, 1, -1
         
                     ! laitab(i) = (pad_deltah(i+1) * delta_h(i+1)) + laitab(i+1)   
                     ! laitab(i) = (pad_deltah(i+1) ) + laitab(i+1)             
                     laitab(i) = (jrpad(i+1) * delta_h(i+1) ) + laitab(i+1)             

                END DO !i=jnlvls, 1, -1
        
                DO i=1, jnlvls
                     down_sw_tab(i) = swnet(1) * exp (-ext_coef_sw * laitab(i))
                     down_lw_tab(i) = lwdown(1) * exp (-ext_coef_lw * laitab(i))
                END DO !i=1, jnlvls

                    IF (ld_jameswrite) THEN
                       DO i=jnlvls-1, 1, -1
                           WRITE (numout,*) 'laitab, level: ', i, ' is: ', laitab(i)
                       END DO !i=1, jnlvls
                    END IF ! (ld_jameswrite)       

                    IF (ld_jameswrite) THEN
                       DO i=jnlvls-1, 1, -1
                           WRITE (numout,*) 'down_sw_tab, level: ', i, ' is: ', down_sw_tab(i)
                       END DO !i=1, jnlvls
                    END IF ! (ld_jameswrite)       

                    IF (ld_jameswrite) THEN
                       DO i=jnlvls-1, 1, -1
                           WRITE (numout,*) 'down_lw_tab, level: ', i, ' is: ', down_lw_tab(i)
                       END DO !i=1, jnlvls
                    END IF ! (ld_jameswrite)       


                ! DO i=1, jnlvls
                !      q_sat_ta(i) = (0.622_r_std/ pb(1))  *e_sat_tstar* 1000.0_r_std * &
                !        & EXP((biga_const*(temp_atmos_pres(i) - tstar_const) / &
                !        & (temp_atmos_pres(i) - tprime_const)))
                ! 
                !      delta_c(i) = rau(1) * MAX (q_sat_ta(i) - q_atmos_pres(i), zero) /100.0_r_std
                ! END DO !i=1, jnlvls
    
                DO i=1, jnlvls
                     q_sat_ta(i) = (0.622_r_std/ pb(1))  *e_sat_tstar* 1000.0_r_std * &
                       & EXP((biga_const*(temp_atmos_pres(i) - tstar_const) / &
                       & (temp_atmos_pres(i) - tprime_const)))
            
                     delta_c(i) = rau(1) * MAX (q_sat_ta(i) - q_atmos_pres(i), zero) /100.0_r_std
                END DO !i=1, jnlvls
    
                DO i=1, jnlvls
                     IF (down_sw_tab(i) .ne. 0.0d0) THEN
                          r_sto_v(i) = (smalla/k_0) * (1/pad_deltah(i)) * &
                                & ((down_sw_tab(i)+bigr_0)/down_sw_tab(i))  &
                                & * (un + lambda * (delta_c(i)/smalla) )
                     ELSE
                          r_sto_v(i) = 1.0d20
                     END IF
                END DO !i=1,jnlvls
         
             
                ! We're using the expression from ORCHIDEE as in Guimberteau (2010)
    
                DO i = 1, jnlvls
     
                   ! n.b. the transition point from laminar to turbulent flow occurs when the 
                   !   Reynolds number passes 8,000 to 25,000 (Baldochhi, 1988)
    
                   bigd_h2o(i) = 2.26d-5 + (1.51d-7 * (temp_atmos_pres(i) - 273.15_r_std))
                   nu_air(i) = 1.35d-5 + (1.0d-7 * (temp_atmos_pres(i) - 273.15_r_std))
                   schmidt(i) = nu_air(i) / bigd_h2o(i)
    
                   reynolds(i) = (small_d * u_speed(i))/nu_air(i)
    
                   ! is the flow laminar or turbulent?
                   IF (reynolds(i) .le. 8000.0_r_std) THEN
                       sherwood(i) = 0.66_r_std * (reynolds(i))**(0.5_r_std) + (schmidt(i))**(0.33_r_std)
                   ELSE IF (reynolds(i) .gt. 8000.0_r_std) THEN
                       sherwood(i) = 0.03_r_std * (reynolds(i))**(0.8_r_std) + (schmidt(i))**(0.33_r_std)
                   END IF
    
                   big_r_h2o(i) = small_d / (bigd_h2o(i) * sherwood(i))

                   big_r_prime(i) = ((r_sto_v(i) * r_sto_v_fac) + (big_r_h2o(i) * big_r_h2o_fac)) !/ pad_deltah(i)

                END DO ! i=1, jnlvls
    
            END IF ! (jnlvls .eq. 1)
    
            big_r_prime = ((r_sto_v * r_sto_v_fac) + (big_r_h2o * big_r_h2o_fac))
    
            big_r_prime = big_r_prime * sr_fac
    
       !    big_r_prime(:) = 1.0d4
         
            big_r_prime(jnlvls) = 1.0d20   ! the top level resistance should be approaching 
                                         !   infinity (see diagram) need to implement this 
                                         !   more elegantly
                                         
            IF (ld_jameswrite) WRITE (numout, *) 'big_r_prime is: ', big_r_prime

          END DO ! jk = 1, kjpindex

     END IF ! (jr_nextstep)
    
 END SUBROUTINE calc_stomatal_resistance

    
 ! -------------------------------------------------------------------------------------------------

 !!  ===========================================================================================================================
 !! SUBROUTINE                  : calc_boundarylayer_resistance
 !!
 !!
 !>\BRIEF                   : 
 !!
 !! DESCRIPTION                 : 
 !! 
 !!
 !! RECENT CHANGE(S): None
 !!
 !! MAIN OUTPUT VARIABLE(S) :     down_sw_tab, down_lw_tab, big_r_prime
 !!
 !! REFERENCE(S)        :     Baldocchi, D.D., 1988. A multi-layer model for estimating sulfur dioxide deposition 
 !!                                   to a deciduous oak forest canopy. Atmospheric Environment (1967), 22(5), 
 !!                                   pp.869–884
 !!
 !! FLOWCHART           : 
 !_ ==============================================================================================================================



 SUBROUTINE calc_boundarylayer_resistance (big_r, u_speed, temp_atmos_pres_grid, &
                                            & big_r_heat_fac, kjpindex, &
                                            & br_fac, hd_big_r, q_cdrag, &
                                            & pad_deltah, delta_h, jrpad)
  



    IMPLICIT NONE

    !! 0.1 Input variables

    INTEGER(i_std), INTENT(in)                                     :: kjpindex         !! Domain size (-)
    REAL(r_std), DIMENSION (kjpindex, jnlvls), INTENT(in)          :: temp_atmos_pres_grid ! atmospheric temperature down the column (K) PRESENT STEP
    REAL(r_std), DIMENSION (jnlvls), INTENT(in)                    :: u_speed
    REAL(r_std), INTENT(in)                                        :: big_r_heat_fac
    REAL(r_std), INTENT(in)                                        :: br_fac
    REAL(r_std), INTENT(in)                                        :: hd_big_r
    REAL(r_std), DIMENSION (kjpindex), INTENT (in)                 :: q_cdrag          !! Surface drag (m s^{-1})
    REAL(r_std), DIMENSION (0:jnlvls), INTENT(in)                  :: pad_deltah       !! Plant Area Density multiplied by layer height
    REAL(r_std), DIMENSION(jnlvls), INTENT(in)                     :: delta_h                ! height of each level (m)
    REAL(r_std), DIMENSION (0:jnlvls), INTENT(in)                  :: jrpad   ! Plant Area Density, replacement for values above (as definitions between different
                                                                              !    parts of the model was becoming confusing)
                                                                              ! 'jrpad' is the total one-sided leaf area per unit of layer volume


    !! 0.2 Output variables
    REAL(r_std), DIMENSION(jnlvls), INTENT(out)                    :: big_r

    !! 0.3 Modified variables


    !! 0.4 Local variables

    REAL(r_std), DIMENSION (jnlvls)                                :: temp_atmos_pres ! atmospheric temperature down the column (K) PRESENT STEP
    REAL(r_std)                                                    :: r_a = 0.0_r_std     ! aerodynamic resistance is represented by the eddy
    ! diffusivity coefficients so I have set this to zero 
    REAL(r_std)                                                    :: r_b = 30.0_r_std    ! leaf boundary layer resistance
    INTEGER(i_std)                                                 :: i, jk
    REAL(r_std)                                                    :: small_d         ! characteristic leaf length (m)
    REAL(r_std)                                                    :: big_l           ! characteristic leaf dimension (m)
    REAL(r_std)                                                    :: big_d           ! molecular diffusivity of air
    REAL(r_std)                                                    :: sherwood        ! Sherwood number
    REAL(r_std)                                                    :: prandtl         ! Prandtl number 
    REAL(r_std), DIMENSION (jnlvls)                                :: nusselt         ! Nusselt number
    REAL(r_std), DIMENSION (jnlvls)                                :: bigd_h          ! heat diffusivity of air (m^2 s^-1)
    REAL(r_std), DIMENSION (jnlvls)                                :: nu_air          ! the kinematic viscosity of air
    REAL(r_std), DIMENSION (jnlvls)                                :: reynolds        ! reynolds number
    REAL(r_std), DIMENSION(jnlvls)                                 :: big_r_heat


    DO jk = 1, kjpindex

        temp_atmos_pres(:) = temp_atmos_pres_grid(jk,:)
 
        IF (jnlvls .eq. 1) THEN

            big_r(:)=un

            ! omit the layered calculation for single layer case
        ELSE
    
            small_d = 0.01_r_std                                ! set the leaf length to 1cm for now
            big_l = 0.01_r_std                                  ! set the leaf length to 1cm for now
            prandtl = 1.0_r_std                                 ! set the Prandtl number to 1 for now
    
            DO i = 1, jnlvls
                big_r(i) = 200.0_r_std * (0.05_r_std / u_speed (i))
            END DO ! i = 1, jnlvls
    
            DO i = 1, jnlvls
    
                bigd_h(i) = 1.9d-5 + (1.26d-5 * (temp_atmos_pres(i) - 273.15_r_std))
                nu_air(i) = 1.35d-5 + (1.0d-7 * (temp_atmos_pres(i) - 273.15_r_std))
                reynolds(i) = (big_l * u_speed(i)) / nu_air(i)
    
                ! is the flow laminar or turbulent?
                IF (prandtl .lt. 0.6_r_std) THEN
                    nusselt(i) = 0.332_r_std * (reynolds(i))**(1.0_r_std/2.0_r_std) + (prandtl)**(1.0_r_std/3.0_r_std)
                ELSE IF (prandtl .ge. 0.6_r_std) THEN
                    nusselt(i) = 0.0296_r_std * (reynolds(i))**(4.0_r_std/5.0_r_std) + (prandtl)**(1.0_r_std/3.0_r_std)
                END IF
    
                big_r_heat(i) = small_d / (bigd_h(i) * nusselt(i))
    
            END DO ! i=1, jnlvls
    
        END IF ! (jnlvls .eq. 1)
    
        big_r = big_r +  big_r_heat 
        
        big_r = big_r * br_fac
    
        big_r(jnlvls) = 1.0d20         ! n.b. the top level resistance should be approaching 
                                       !   infinity (see diagram) need to implement this more 
                                       !   elegantly 
    END DO ! jk = 1, kjpindex

 END SUBROUTINE calc_boundarylayer_resistance

    
 ! -------------------------------------------------------------------------------------------------


 SUBROUTINE jcalc_fluxes(temp_atmos_next, q_atmos_next, k_eddy, delta_z,  &
                          & h_flux, le_flux)

    ! calculate flux profiles

    IMPLICIT NONE

    !! 0.1 Input variables
    !! 0.2 Output variables
    !! 0.3 Modified variables
    !! 0.4 Local variables


    REAL(r_std), DIMENSION (jnlvls)               :: temp_atmos_next    ! atmospheric temperature down the column (K) NEXT STEP
    REAL(r_std), DIMENSION (jnlvls)               :: q_atmos_next       
    REAL(r_std), DIMENSION (jnlvls)               :: k_eddy             ! eddy transport coefficient (m^2 s-1)
    REAL(r_std), DIMENSION (jnlvls)               :: delta_z            ! vertical difference between the middle of each box (m)
    REAL(r_std), DIMENSION (jnlvls)               :: h_flux, le_flux
    INTEGER(i_std)                                :: i
    REAL(r_std)                                   :: rho_air            ! temporary for number checking   

    IF (jnlvls .gt. 1) THEN

        DO i = 1, jnlvls-1
            h_flux(i) = -1.0_r_std * cp_air * rho_air * &
              & ((temp_atmos_next(i+1) - temp_atmos_next(i)) * k_eddy(i))/delta_z(i)
        END DO ! i = 1, jnlvls-1

        IF (ld_jameswrite) THEN
            WRITE(numout,*) 'cp_air * rho_air is: ', cp_air * rho_air
            WRITE(numout,*) 'h_flux_top is: ', h_flux(jnlvls-1)
        END IF !(ld_jameswrite)

        DO i = 1, jnlvls-1
            le_flux(i) = -1.0_r_std * chalev0 * rho_air * &
              & ((q_atmos_next(i+1) - q_atmos_next(i)) * k_eddy(i))/delta_z(i)
        END DO ! i = 1, jnlvls-1

    END IF ! (jnlvls .gt. 1) THEN

    IF (ld_jameswrite) THEN
       WRITE(numout,*) 'chalev0 * rho_air is: ', chalev0 * rho_air
       WRITE(numout,*) 'le_flux_top is: ', le_flux(jnlvls-1)
    END IF !(ld_jameswrite)

 END SUBROUTINE jcalc_fluxes


 ! ---------------------------------------------------------------------------------------

  !!  ====================================================================================
  !! SUBROUTINE                 : jenerbil_begin
  !!
  !>\BRIEF                  Preliminary variables required for the 
  !! calculation of the energy budget are derived.
  !!
  !! DESCRIPTION                : This routines computes preliminary 
  !! variables required for the calculation of the energy budget: the old surface static 
  !! energy (psold), the surface saturation humidity (qsol_sat), the derivative of satured 
  !! specific humidity at the old temperature (pdqsold) and the net radiation (netrad).
  !!
  !! MAIN OUTPUT VARIABLE(S)                    : psold, qsol_sat, pdqsold, netrad
  !!
  !! REFERENCE(S)               :
  !! - Best, MJ, Beljaars, A, Polcher, J & Viterbo, P, 2004. A proposed structure for 
  !! coupling tiled surfaces with the planetary boundary layer. Journal of 
  !! Hydrometeorology, 5, pp.1271-1278
  !! - de Noblet-Ducoudré, N, Laval, K & Perrier, A, 1993. SECHIBA, a new set of 
  !! parameterisations of the hydrologic exchanges at the land-atmosphere interface within
  !! the LMD Atmospheric General Circulation Model. Journal of Climate, 6, pp.248-273
  !! - Dufresne, J-L & Ghattas, J, 2009. Description du schéma de la couche limite 
  !! turbulente et la interface avec la surface planetaire dans LMDZ, Technical note, 
  !! available (22/12/11):
  !! http://lmdz.lmd.jussieu.fr/developpeurs/notes-techniques/ressources/pbl_surface.pdf
  !! - Polcher, J. McAvaney, B, Viterbo, P, Gaertner, MA, Hahmann, A, Mahfouf, J-F, 
  !! Noilhan, J Phillips, TJ, Pitman, AJ, Schlosser, CA, Schulz, J-P, Timbal, B, Verseghy,
  !! D Xue, Y, 1998. A proposal for a general interface between land surface schemes and
  !! general circulation models. Global and Planetary Change, 19, pp.261-276
  !! - Richtmeyer, RD, Morton, KW, 1967. Difference Methods for Initial-Value Problems.
  !! Interscience Publishers\n
  !! - Schulz, Jan-Peter, Lydia Dümenil, Jan Polcher, 2001: On the Land Surface–Atmosphere 
  !! Coupling and Its Impact in a Single-Column Atmospheric Model. J. Appl. Meteor., 
  !! 40, 642–663.
  !!
  !! FLOWCHART  : None                     
  !! \n
  !_ =====================================================================================
  
  SUBROUTINE jenerbil_begin (kjpindex, temp_sol, lwdown, swnet, pb, psold, qsol_sat, &
                               & pdqsold, netrad, emis, dev_qsol)

    !! 0. Variable and parameter declaration

    !! 0.1 Input variables

    INTEGER(i_std), INTENT(in)                         :: kjpindex         !! Domain size (-)
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)      :: temp_sol         !! Soil temperature (K)
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)      :: lwdown           !! Down-welling long-wave flux (W m^{-2})
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)      :: swnet            !! Net surface short-wave flux (W m^{-2})
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)      :: pb               !! Lowest level pressure (hPa)
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)      :: emis             !! Emissivity (-)

    
    !! 0.2 Output variables

    REAL(r_std),DIMENSION (kjpindex), INTENT (out)     :: psold            !! Old surface dry static energy (J kg^{-1})
    REAL(r_std),DIMENSION (kjpindex), INTENT (out)     :: qsol_sat     !! Saturated specific humudity for old temperature 
                                                                           !! (kg kg^{-1})    
    REAL(r_std),DIMENSION (kjpindex), INTENT (out)     :: pdqsold      !! Derivative of satured specific humidity at the old 
                                                                           !! temperature (kg (kg s)^{-1})
    REAL(r_std),DIMENSION (kjpindex), INTENT (out)     :: netrad           !! Net radiation (W m^{-2})
    REAL(r_std), DIMENSION(kjpindex)                   :: dev_qsol

    !! 0.3 Modified variables

    !! 0.4 Local variables

    INTEGER(i_std)                                     :: ji

    REAL(r_std), PARAMETER                             :: missing = 999998.
!_ ================================================================================================================================
  
  !  write(numout, *) 'Jenerbil_begin lwdown:', lwdown(:)
  !! 1. Computes psold (the surface static energy for the old timestep)
   
    !! We here define the surface static energy for the 'old' timestep, in terms of the surface
    !! temperature and heat capacity.
    !! \latexonly 
    !!     \input{enerbilbegin1.tex}
    !! \endlatexonly
    psold(:) = temp_sol(:)*cp_air
    
  !! 2. Computes qsol_sat (the surface saturated humidity).

    !! We call the routine 'qsatcalc' from within the module 'src_parameters/constantes_veg'

    CALL qsatcalc (kjpindex, temp_sol, pb, qsol_sat)

    IF ( diag_qsat ) THEN
      IF ( ANY(ABS(qsol_sat(:)) .GT. missing) ) THEN
        DO ji = 1, kjpindex
          IF ( ABS(qsol_sat(ji)) .GT. missing) THEN
            WRITE(numout,*) 'ERROR on ji = ', ji
            WRITE(numout,*) 'temp_sol(ji),  pb(ji) :', temp_sol(ji),  pb(ji)
            CALL ipslerr (3,'enerbil_begin', &
 &           'qsol too high ','','')
          ENDIF
        ENDDO
      ENDIF
    ENDIF
    
  !! 3. Computes pdqsold 
    
    !! Computes pdqsold (the derivative of the saturated humidity with respect to temperature
    !! analysed at the surface temperature at the 'old' timestep.
    !! We call the routine 'dev_qsatcalc' from within the module 'src_parameters/constantes_veg'.
    CALL dev_qsatcalc (kjpindex, temp_sol, pb, dev_qsol)
    
     IF (ld_jameswrite) THEN
!!$       WRITE(numout,*) '***** 010213 *****'
!!$       WRITE(numout,*) ' '
!!$       WRITE(numout,*) 'after CALL dev_qsatcalc'
!!$       WRITE(numout,*) 'temp_sol(1) is: ', temp_sol(1)
!!$       WRITE(numout,*) 'pb(1) is: ', pb(1)
!!$       WRITE(numout,*) 'dev_qsol(1) is: ', dev_qsol(1)        
!!$       WRITE(numout,*) 'EFFECTIVE jalpha is: ', dev_qsol(1)
!!$       WRITE(numout,*) 'EFFECTIVE jbeta is: ', qsol_sat(1) - temp_sol(1) * dev_qsol(1)
     ENDIF


!+++ According to r2093 in the trunk, this causes and instabilty in the coupling
!    DO ji = 1, kjpindex
!!$    
!!$    !! \latexonly 
!!$    !!     \input{enerbilbegin2.tex}
!!$    !! \endlatexonly
!      pdqsold(ji) = dev_qsol(ji) * ( pb(ji)**kappa ) / cp_air
!    ENDDO

    pdqsold(:) = dev_qsol(:)
!+++++++++

    IF ( diag_qsat ) THEN
      IF ( ANY(ABS( pdqsold(:)) .GT. missing) ) THEN
        DO ji = 1, kjpindex
          IF ( ABS( pdqsold(ji)) .GT. missing ) THEN
            WRITE(numout,*) 'ERROR on ji = ', ji
            WRITE(numout,*) 'temp_sol(ji),  pb(ji) :', temp_sol(ji),  pb(ji)
            CALL ipslerr (3,'enerbil_begin', &
 &           'pdqsold too high ','','')
          ENDIF
        ENDDO
      ENDIF
    ENDIF


  !! 4. Computes the net radiation and the absorbed LW radiation absorbed at the surface. 

    !! Long wave radiation absorbed by the surface is the product of emissivity and downwelling LW radiation    
    !! \latexonly 
    !!     \input{enerbilbegin3.tex}
    !! \endlatexonly
    lwabs(:) = emis(:) * lwdown(:)

    !! Net radiation is calculated as:
    !! \latexonly 
    !!     \input{enerbilbegin4.tex}
    !! \endlatexonly
    netrad(:) = lwdown(:) + swnet (:) - (emis(:) * c_stefan * temp_sol(:)**4 + &
                 & (un - emis(:)) * lwdown(:)) 
    !
    IF (long_print) WRITE (numout,*) ' jenerbil_begin done '


    !+++ TEMP FOR NIGHTTIME K_EDDY_FAC ++++
    !DO ji =1, kjpindex
    !      IF (swnet(ji) <= 0 ) THEN
    !          k_eddy_fac =0.01
    !      ELSE
    !          CALL getin_p('K_EDDY_FAC', k_eddy_fac) 
    !      ENDIF
    !ENDDO
    !+++++++++++++++++++++++++++++++++++++


  END SUBROUTINE jenerbil_begin

 ! %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%





 !! =============================================================================================================================
 !! SUBROUTINE                  : jenerbil_pottemp
 !!
 !>\BRIEF       This subroutine computes the surface temperature and humidity should the surface been unstressed    
 !!
 !! DESCRIPTION             : 
 !!
 !! MAIN OUTPUT VARIABLE(S) : q_sol_pot, temp_sol_pot
 !!
 !! REFERENCE(S)        :
 !!
 !! FLOWCHART    : None
 !!
 !_ ==============================================================================================================================  

  SUBROUTINE jenerbil_pottemp (kjpindex, dtradia, zlev, emis, epot_air, &
     & petAcoef, petBcoef, qair, peqAcoef, peqBcoef, soilflx, rau, u, v, q_cdrag, vbeta,&
     & valpha, vbeta1, vbeta5, soilcap, lwdown, swnet, q_sol_pot, temp_sol_pot) 

    !! 0.1 Input variables
    !! 0.2 Output variables
    !! 0.3 Modified variables
    !! 0.4 Local variables

    ! interface 
    ! input scalar 
    INTEGER(i_std), INTENT(in)                               :: kjpindex      !! Domain size
    REAL(r_std), INTENT(in)                                  :: dtradia       !! Time step in seconds
    ! input fields
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: zlev          !! Height of first layer
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: emis          !! Emissivity
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: epot_air      !! Air potential energy
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: petAcoef      !! PetAcoef
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: petBcoef      !! PetBcoef
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: qair          !! Lowest level specific humidity
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: peqAcoef      !! PeqAcoef
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: peqBcoef      !! PeqBcoef
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: soilflx       !! Soil flux
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: rau           !! Density
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: u, v          !! Wind
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: q_cdrag       !!
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: vbeta         !! Resistance coefficient
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: valpha        !! Resistance coefficient
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: vbeta1        !! Snow resistance
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: vbeta5        !! Floodplains resistance
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: soilcap       !! Soil calorific capacity
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: lwdown        !! Down-welling long-wave flux
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: swnet         !! Net surface short-wave flux
    ! output fields
    REAL(r_std),DIMENSION (kjpindex), INTENT (out)           :: q_sol_pot     !! Potential surface air moisture
    REAL(r_std),DIMENSION (kjpindex), INTENT (out)           :: temp_sol_pot  !! Potential soil temperature


    ! local declaration
    INTEGER(i_std)                  :: ji
    REAL(r_std),DIMENSION (kjpindex) :: zicp
    REAL(r_std)                      :: fevap
    REAL(r_std)                      :: sensfl_old, larsub_old, lareva_old, dtheta, sum_old, sum_sns
    REAL(r_std)                      :: zikt, zikq, netrad_sns, sensfl_sns, larsub_sns, lareva_sns
    REAL(r_std)                      :: speed

!_ ==============================================================================================================================

    zicp = un / cp_air

    ! initialisation of variables
    temp_sol_pot(:)=zero
    q_sol_pot(:)=zero

    !
    DO ji=1,kjpindex

       dtheta = zero
       fevap = zero

       temp_sol_pot(ji) = temp_sol_pot(ji) + dtheta

       q_sol_pot(ji) = q_sol_pot(ji) + fevap

    ENDDO

  END SUBROUTINE jenerbil_pottemp



  ! ------------------------------------------------------------------------------------------------------------------------------




  SUBROUTINE jenerbil_evapveg (kjpindex, dtradia, vbeta1, vbeta2, vbeta3, vbeta3pot, &
     & vbeta4, vbeta5, rau, u, v, q_cdrag, qair, humrel, &
!!$     & vbeta4, vbeta5, cimean, ccanopy, rau, u, v, q_cdrag, qair, humrel, &
     & vevapsno, vevapnu , vevapflo, vevapwet, transpir, transpot, evapot)

    !! 0. Variable and parameter declaration
    
    !! 0.1 Input variables

    INTEGER(i_std), INTENT(in)                          :: kjpindex          !! Domain size (-)
    REAL(r_std), INTENT(in)                             :: dtradia           !! Time step (s)
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)       :: vbeta1            !! Snow resistance (-)
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)       :: vbeta4            !! Bare soil resistance (-)
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)       :: vbeta5            !! Floodplains resistance
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)       :: rau               !! Density (kg m^{-3})
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)       :: u, v              !! Wind velocity in directions u and v (m s^{-1})
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)       :: q_cdrag           !! Surface drag (m s^{-1})
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)       :: qair              !! Lowest level specific humidity (kg kg^{-1})
!!$    REAL(r_std),DIMENSION (kjpindex), INTENT (in)       :: ccanopy           !! CO2 concentration in the canopy (ppm)
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)       :: evapot            !! Soil Potential Evaporation (mm day^{-1})
    REAL(r_std),DIMENSION (kjpindex, nvm), INTENT (in)  :: humrel            !! Soil moisture stress (within range 0 to 1)
    !!$ DS 15022011 humrel was used in a previous version of Orchidee, developped by Nathalie. Need to be discussed if it should be introduce again             
    REAL(r_std),DIMENSION (kjpindex,nvm), INTENT (in)   :: vbeta2            !! Interception resistance (-)
    REAL(r_std),DIMENSION (kjpindex,nvm), INTENT (in)   :: vbeta3            !! Vegetation resistance (-)
    REAL(r_std),DIMENSION (kjpindex,nvm), INTENT (in)   :: vbeta3pot         !! Vegetation resistance for potential transpiration
!!$    REAL(r_std),DIMENSION (kjpindex,nvm), INTENT (in)   :: cimean            !! STOMATE: mean intercellular ci \mu mole m^{-2} s^{-1}
    
    !! 0.2 Output variables

    REAL(r_std),DIMENSION (kjpindex), INTENT (out)      :: vevapsno          !! Snow evaporation (mm day^{-1})
    REAL(r_std),DIMENSION (kjpindex), INTENT (out)      :: vevapnu           !! Bare soil evaporation (mm day^{-1})
    REAL(r_std),DIMENSION (kjpindex), INTENT (out)      :: vevapflo         !! Floodplains evaporation
    REAL(r_std),DIMENSION (kjpindex,nvm), INTENT (out)  :: transpir          !! Transpiration (mm day^{-1})
    REAL(r_std),DIMENSION (kjpindex,nvm), INTENT (out)  :: transpot          !! Potential Transpiration
    ! JR GPP calculation is now carried out in diffuco 
!   REAL(r_std),DIMENSION (kjpindex,nvm), INTENT (out)  :: gpp               !! Assimilation (gC m^{-2}, total area)
    REAL(r_std),DIMENSION (kjpindex,nvm), INTENT (out)  :: vevapwet          !! Interception (mm day^{-1})

    !! 0.3 Modified variables

    !! 0.4 Local variables

    INTEGER(i_std)                                      :: ji, jv
    REAL(r_std), DIMENSION(kjpindex)                    :: xx
    REAL(r_std), DIMENSION(kjpindex)                    :: vbeta2sum, vbeta3sum
    REAL(r_std)                                     :: speed         !! Speed (m s^{-1})
!_ ==============================================================================================================================
    ! initialisation: utile pour calculer l'evaporation des floodplains dans lesquelles il y a de la vegetation
    vbeta2sum(:) = 0.
    vbeta3sum(:) = 0.
    DO jv = 1, nvm
      vbeta2sum(:) = vbeta2sum(:) + vbeta2(:,jv)
      vbeta3sum(:) = vbeta3sum(:) + vbeta3(:,jv)
    ENDDO 
    
    IF (ld_jameswrite) WRITE (numout,*) '041113 vbeta3sum is: ', vbeta3sum   

    !! 1. Compute vevapsno (evaporation from snow) and vevapnu (bare soil evaporation)

    DO ji=1,kjpindex 

        !! \latexonly 
        !!     \input{enerbilevapveg1.tex}
        !! \endlatexonly
        speed = MAX(min_wind, SQRT (u(ji)*u(ji) + v(ji)*v(ji)))

      
        !! 1.1 Snow sublimation
        !! \latexonly 
        !!     \input{enerbilevapveg2.tex}
        !! \endlatexonly
        vevapsno(ji) = (un - vbeta5(ji)) * vbeta1(ji) * dtradia * rau(ji) * speed * &
                      & q_cdrag(ji) * (qsol_sat_new(ji) - qair(ji))

        !! 1.2 Bare soil evaporation
        !! \latexonly 
        !!     \input{enerbilevapveg3.tex}
        !! \endlatexonly
        vevapnu(ji) = (un - vbeta1(ji)) * (un-vbeta5(ji)) * vbeta4(ji) * dtradia * &
                     & rau(ji) * speed * q_cdrag(ji) * (qsol_sat_new(ji) - qair(ji))

        !
        ! 1.3 floodplains evaporation - transpiration et interception prioritaires dans les floodplains
        !
        vevapflo(ji) = vbeta5(ji) * (1 - vbeta2sum(ji) - vbeta3sum(ji)) &
             & * dtradia * rau(ji) * speed * q_cdrag(ji) * (qsol_sat_new(ji) - qair(ji))

    END DO

    !! 2. Compute transpir (transpiration) and vevapwet (interception)
  
    !! Preliminaries
    DO ji = 1, kjpindex
       
        !! \latexonly 
        !!     \input{enerbilevapveg4.tex}
        !! \endlatexonly
        speed = MAX(min_wind, SQRT (u(ji)*u(ji) + v(ji)*v(ji)))
       
        !! \latexonly 
        !!     \input{enerbilevapveg5.tex}
        !! \endlatexonly
        !+++TEMP+++
!!$        IF (ld_jameswrite) THEN
!!$           WRITE(numout,*) 'Checking transpir'
!!$           WRITE(numout,*) 'vbeta1(ji), ',vbeta1(ji)
!!$           WRITE(numout,*) 'qsol_sat_new(ji)', qsol_sat_new(ji)
!!$           WRITE(numout,*) 'qair(ji), ', qair(ji)
!!$           WRITE(numout,*) 'rau(ji), ', rau(ji)
!!$           WRITE(numout,*) 'speed, ', speed
!!$           WRITE(numout,*) 'q_cdrag(ji)', q_cdrag(ji)
!!$           WRITE(numout,*) 'vbeta3(ji,jv)', vbeta3(ji,:)
!!$        ENDIF
        xx(ji) = dtradia * (un-vbeta1(ji)) * (qsol_sat_new(ji)-qair(ji)) &
                 & * rau(ji) * speed * q_cdrag(ji)
        !++++++++++
     
    ENDDO
    
    DO jv=1,nvm 
      DO ji=1,kjpindex 
        
        !! 2.1 Calculate interception loss
        !! \latexonly 
        !!     \input{enerbilevapveg6.tex}
        !! \endlatexonly
        vevapwet(ji,jv) = xx(ji) * vbeta2(ji,jv)
        ! 
        !! 2.2 Calculate transpiration
        !! \latexonly 
        !!     \input{enerbilevapveg7.tex}
        !! \endlatexonly
        transpir (ji,jv) = xx(ji) * vbeta3(ji,jv)

           ! IF (jv .eq. 4) THEN
           !      WRITE(numout,*) '261113b in first calc PFT4, vbeta3(ji,jv) is: ', vbeta3(ji,jv)
           !      WRITE(numout,*) '261113b in first calc PFT4, transpir (ji,jv) is: ', transpir (ji,jv)
           !      WRITE(numout,*) '261113b in first calc PFT4, (1.0_r_std - vbeta1(ji)) is: ', (1.0_r_std - vbeta1(ji))
           !      WRITE(numout,*) '261113b in first calc PFT4, qsol_sat_new(ji) is: ', qsol_sat_new(ji)
           !      WRITE(numout,*) '261113b in first calc PFT4, qair(ji) is: ', qair(ji)
           !      WRITE(numout,*) '261113b in first calc PFT4, ( rau(ji) * speed * q_cdrag(ji)) is: ', ( rau(ji) * speed * q_cdrag(ji))
           ! END IF

        transpot(ji,jv) = xx(ji) * vbeta3pot(ji,jv)

      END DO
    END DO

    IF (long_print) WRITE (numout,*) ' enerbil_evapveg done '

  END SUBROUTINE jenerbil_evapveg


  !!  =============================================================================================================================
  !! SUBROUTINE                 : enerbil_flux
  !!
  !>\BRIEF                  Computes the new soil temperature, net radiation and
  !! latent and sensible heat flux for the new time step.
  !!
  !! DESCRIPTION                : This routine diagnoses, based on the new soil temperature, 
  !! the net radiation, the total evaporation, the latent heat flux, the sensible heat flux and the 
  !! sublimination flux. It also diagnoses the potential evaporation used for the fluxes (evapot) and the potential
  !! as defined by Penman & Monteith (Monteith, 1965) based on the correction term developed by Chris 
  !! Milly (1992). This Penman-Monteith formulation is required for the estimation of bare soil evaporation 
  !! when the 11 layer CWRR moisture scheme is used for the soil hydrology.
  !!
  !! MAIN OUTPUT VARIABLE(S)    : qsurf, fluxsens, fluxlat, fluxsubli, vevapp, lwup, lwnet,
  !! tsol_rad, netrad, evapot, evapot_corr
  !!
  !! REFERENCE(S)                   :
  !! - Best, MJ, Beljaars, A, Polcher, J & Viterbo, P, 2004. A proposed structure for coupling tiled
  !! surfaces with the planetary boundary layer. Journal of Hydrometeorology, 5, pp.1271-1278
  !! - de Noblet-Ducoudré, N, Laval, K & Perrier, A, 1993. SECHIBA, a new set of parameterisations
  !! of the hydrologic exchanges at the land-atmosphere interface within the LMD Atmospheric General
  !! Circulation Model. Journal of Climate, 6, pp.248-273
  !! - Dufresne, J-L & Ghattas, J, 2009. Description du schéma de la couche limite turbulente et la
  !! interface avec la surface planetaire dans LMDZ, Technical note, available (22/12/11):
  !! http://lmdz.lmd.jussieu.fr/developpeurs/notes-techniques/ressources/pbl_surface.pdf
  !! - Guimberteau, M, 2010. Modélisation de l'hydrologie continentale et influences de l'irrigation
  !! sur le cycle de l'eau, PhD Thesis, available (22/12/11):
  !! http://www.sisyphe.upmc.fr/~guimberteau/docs/manuscrit_these.pdf
  !! - Monteith, JL, 1965. Evaporation and Environment, paper presented at Symposium of the Society
  !! for Experimental Biology
  !! - Monteith & Unsworth, 2008. Principles of Environmental Physics (third edition), published Elsevier
  !! ISBN 978-0-12-505103-3
  !! - Milly, P. C. D., 1992: Potential Evaporation and Soil Moisture in General Circulation Models. 
  !! Journal of Climate, 5, pp. 209–226.
  !! - Polcher, J. McAvaney, B, Viterbo, P, Gaertner, MA, Hahmann, A, Mahfouf, J-F, Noilhan, J
  !! Phillips, TJ, Pitman, AJ, Schlosser, CA, Schulz, J-P, Timbal, B, Verseghy, D &
  !! Xue, Y, 1998. A proposal for a general interface between land surface schemes and
  !! general circulation models. Global and Planetary Change, 19, pp.261-276
  !! - Richtmeyer, RD, Morton, KW, 1967. Difference Methods for Initial-Value Problems.
  !! Interscience Publishers
  !! - Schulz, Jan-Peter, Lydia Dümenil, Jan Polcher, 2001: On the Land Surface–Atmosphere 
  !! Coupling and Its Impact in a Single-Column Atmospheric Model. J. Appl. Meteor., 40, 642–663.
  !!
  !! FLOWCHART                  : None
  !! \n
  !_ ==============================================================================================================================

  SUBROUTINE jenerbil_flux (kjpindex, dtradia, emis, temp_sol, rau, u, v, q_cdrag, vbeta, valpha, vbeta1, vbeta5, &
       & qair, epot_air, psnew, qsurf, fluxsens, fluxlat, fluxsubli, vevapp, temp_sol_new, &
       & lwdown, swnet, lwup, lwnet, pb, tsol_rad, netrad, evapot, evapot_corr)

    !! 0. Variable and parameter declaration
    
    !! 0.1 Input variables

    INTEGER(i_std), INTENT(in)                               :: kjpindex      !! Domain size (-)
    REAL(r_std), INTENT(in)                                  :: dtradia       !! Time step (s)
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: emis          !! Emissivity (-)
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: temp_sol      !! Surface temperature (K)
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: rau           !! Density (kg m^{-3})
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: u,v           !! Wind velocity in components u and v (m s^{-1}) 
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: q_cdrag       !! Surface drag (m s^{-1})
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: vbeta         !! Resistance coefficient (-)
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: valpha        !! Resistance coefficient (-)
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: vbeta1        !! Snow resistance  (-)
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: vbeta5        !! Flood resistance 
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: qair          !! Lowest level specific humidity (kg kg^{-1})
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: epot_air      !! Air potential energy (J)
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: psnew         !! New surface static energy (J kg^{-1})
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: temp_sol_new  !! New soil temperature (K)
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: pb            !! Lowest level pressure (hPa)

    
    !! 0.2 Output variables

    REAL(r_std),DIMENSION (kjpindex), INTENT (out)           :: qsurf         !! Surface specific humidity (kg kg^{-1})
    REAL(r_std),DIMENSION (kjpindex)                         :: fluxsens      !! Sensible heat flux (W m^{-2})   ! JR 060913 not output here, calculated in jcalc instead
    REAL(r_std),DIMENSION (kjpindex)                         :: fluxlat       !! Latent heat flux (W m^{-2})     ! JR 060913 not output here, calculated in jcalc instead
    REAL(r_std),DIMENSION (kjpindex), INTENT (out)           :: fluxsubli     !! Energy of sublimation (W m^{-2})
    REAL(r_std),DIMENSION (kjpindex), INTENT (out)           :: vevapp        !! Total of evaporation (mm day^{-1})
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: lwdown        !! Downward long wave radiation (W m^{-2})
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: swnet         !! Net short wave radiation (W m^{-2})
    REAL(r_std),DIMENSION (kjpindex), INTENT (out)           :: lwup          !! Long-wave up radiation (W m^{-2})
    REAL(r_std),DIMENSION (kjpindex), INTENT (out)           :: lwnet         !! Long-wave net radiation (W m^{-2})
    REAL(r_std),DIMENSION (kjpindex), INTENT (out)           :: tsol_rad      !! Radiative surface temperature (W m^{-2})
    REAL(r_std),DIMENSION (kjpindex), INTENT (inout)         :: netrad        !! Net radiation (W m^{-2})
    REAL(r_std),DIMENSION (kjpindex), INTENT (inout)         :: evapot        !! Soil Potential Evaporation (mm day^{-1})
    REAL(r_std),DIMENSION (kjpindex), INTENT (inout)         :: evapot_corr   !! Soil Potential Evaporation Correction 
                                                                              !! (?? mm day^{-1})

    !! 0.3 Modified variables

    !! 0.4 Local variables

    INTEGER(i_std)                                               :: ji
    REAL(r_std),DIMENSION (kjpindex)                         :: grad_qsat
    REAL(r_std)                                              :: correction
    REAL(r_std)                                          :: speed              !! Speed (m s^{-1}), 
    REAL(r_std)                                          :: qc          !! Surface drag coefficient (??)
    LOGICAL,DIMENSION (kjpindex)                             :: warning_correction
!_ ================================================================================================================================
    
    DO ji=1,kjpindex

       !! 1. Determination of 'housekeeping' variables

       !! The horizontal wind speed is calculated from the velocities along 
       !! each axis using Pythagorus' theorem. 
       !! \latexonly 
       !!     \input{enerbilflux1.tex}
       !! \endlatexonly
       speed = MAX(min_wind, SQRT (u(ji)*u(ji) + v(ji)*v(ji)))

       !! From definition, the surface drag coefficient is defined:
       !! \latexonly 
       !!     \input{enerbilflux2.tex}
       !! \endlatexonly
       qc = speed * q_cdrag(ji)

       !! 2. Calculation of the net upward flux of longwave radiation
       !! We first of all calculate the radiation as a result of the Stefan-Boltzmann equation,
       !! which is the sum of the calculated values at the surface temperature  at the 'old' 
       !! temperature and the value that corresponds to the difference between the old temperature
       !! and the temperature at the 'new' timestep.
       !! \latexonly 
       !!     \input{enerbilflux3.tex}
       !! \endlatexonly
       lwup(ji) = emis(ji) * c_stefan * temp_sol(ji)**4 + &
            &     quatre * emis(ji) * c_stefan * temp_sol(ji)**3 * &
            &     (temp_sol_new(ji) - temp_sol(ji))      

       !! We then add to this value that from the reflected longwave radiation:
       !! \latexonly 
       !!     \input{enerbilflux4.tex}
       !! \endlatexonly
       lwup(ji) = lwup(ji)  +  (un - emis(ji)) * lwdown(ji)

       !! The radiative surface temperature is calculated according to the Stefan-Boltzmann relation:
       !! \latexonly 
       !!     \input{enerbilflux5.tex}
       !! \endlatexonly      
       ! tsol_rad(ji) = emis(ji) * c_stefan * temp_sol(ji)**4 + lwup(ji)

       !! The implicit solution computes an emitted long-wave flux which is a limited Taylor expansion 
       !! of the future surface temperature around the current values. Thus the long-wave flux does not  
       !! correspond to the new surface temperature but some intermediate value. So we need to deduce the 
       !! radiative surface temperature to which this flux corresponds. 
       !! 
       tsol_rad(ji) = (lwup(ji)/ (emis(ji) * c_stefan)) **(1./quatre) 

       !! qsurf (the surface specific humidity) is a simple diagnostic which will be used by the 
       !! GCM to compute the dependence of of the surface layer stability on moisture.
       !! \latexonly 
       !!     \input{enerbilflux6.tex}
       !! \endlatexonly
       qsurf(ji) = (vbeta1(ji) * (un - vbeta5(ji)) + vbeta5(ji)) * qsol_sat_new(ji) + &
            & (un - vbeta1(ji))*(un - vbeta5(ji)) * vbeta(ji) * valpha(ji) * qsol_sat_new(ji)

       IF (ji .eq. 1 .AND. ld_enerbil) THEN
          WRITE(numout,*) "qsurf(1) is: ", qsurf(1) 
          WRITE(numout,*) "qair(1) is: ", qair(1) 
       END IF

       !! \latexonly 
       !!     \input{enerbilflux7.tex}
       !! \endlatexonly
       qsurf(ji) = MAX(qsurf(ji), qair(ji))

       !! Net downward radiation is the sum of the down-welling less the up-welling long wave flux, plus
       !! the short wave radiation.
       !! \latexonly 
       !!     \emissivity absorbed lw radiationinput{enerbilflux8.tex}
       !! \endlatexonly
       netrad(ji) = lwdown(ji) + swnet(ji) - lwup(ji) 

       !! 'vevapp' is the sum of the total evaporative processes (snow plus non-snow processes).
       !! \latexonly 
       !!     \input{enerbilflux9.tex}
       !! \endlatexonly
       vevapp(ji) = dtradia * rau(ji) * qc * (vbeta1(ji) * (un - vbeta5(ji)) + vbeta5(ji)) * &
            & (qsol_sat_new(ji) - qair(ji)) + &
            &  dtradia * rau(ji) * qc * (un - vbeta1(ji))*(un-vbeta5(ji)) * vbeta(ji) * &
            & (valpha(ji) * qsol_sat_new(ji) - qair(ji))

       !! The total latent heat flux is the sum of the snow plus non-snow processes.
       !! \latexonly 
       !!     \input{enerbilflux10.tex}
       !! \endlatexonly
       ! ( JR 050913 commented out here, as calculated in jcalc for now)
       ! fluxlat(ji) = chalsu0 * rau(ji) * qc * vbeta1(ji) * (un - vbeta5(ji)) * &
       !      & (qsol_sat_new(ji) - qair(ji)) + &
       !      &  chalev0 * rau(ji) * qc * vbeta5(ji) *&
       !      & (qsol_sat_new(ji) - qair(ji)) + &
       !      &  chalev0 * rau(ji) * qc * (un - vbeta1(ji)) * (un - vbeta5(ji)) * vbeta(ji) * &
       !     & (valpha(ji) * qsol_sat_new(ji) - qair(ji))

       !! The sublimination flux concerns is calculated using vbeta1, the snow resistance.
       !! \latexonly 
       !!     \input{enerbilflux11.tex}
       !! \endlatexonly
       fluxsubli(ji) = chalsu0 * rau(ji) * qc * vbeta1(ji) * (un - vbeta5(ji)) * &
            & (qsol_sat_new(ji) - qair(ji)) 

       !! The sensible heat flux is a factor of the difference between the new surface static energy
       !! and the potential energy of air.
       !! \latexonly 
       !!     \input{enerbilflux12.tex}
       !! \endlatexonly
       !fluxsens(ji) =  rau(ji) * qc * (psnew(ji) - epot_air(ji))

       !! This is the net longwave downwards radiation.
       !! \latexonly 
       !!     \input{enerbilflux13.tex}
       !! \endlatexonly
       lwnet(ji) = lwdown(ji) - lwup(ji)

       !! Diagnoses the potential evaporation used for the fluxes (evapot)
       !! \latexonly 
       !!     \input{enerbilflux14.tex}
       !! \endlatexonly  
       evapot(ji) =  MAX(zero, dtradia * rau(ji) * qc * (qsol_sat_new(ji) - qair(ji)))

       !! From definition we can say:
       !! \latexonly 
       !!     \input{enerbilflux15.tex}
       !! \endlatexonly 

       tair(ji)  =  epot_air(ji) / cp_air

    ENDDO

    !! 3. Define qsat_air with the subroutine src_parameter:

    ! WRITE(*,*) '181113a test, pre call to qsatcalc enerbil line 6210; tair is: ', tair
    CALL qsatcalc(kjpindex, tair, pb, qsat_air)
    CALL dev_qsatcalc(kjpindex, tair, pb, grad_qsat)

    ! grad_qsat(:)= (qsol_sat_new(:)- qsat_air(:)) / ((psnew(:) - epot_air(:)) / cp_air) ! * dtradia
    !- Penser a sortir evapot en meme temps qu'evapot_corr tdo.

    warning_correction(:)=.FALSE.
    DO ji=1,kjpindex

       !! \latexonly 
       !!     \input{enerbilflux16.tex}
       !! \endlatexonly 
       speed = MAX(min_wind, SQRT (u(ji)*u(ji) + v(ji)*v(ji)))

       !! \latexonly 
       !!     \input{enerbilflux17.tex}
       !! \endlatexonly 
       qc = speed * q_cdrag(ji)

       !! Derive the potential as defined by Penman & Monteith (Monteith, 1965) based on the correction 
       !! term developed by Chris Milly (1992).
       IF ((evapot(ji) .GT. zero) .AND. ((psnew(ji) - epot_air(ji)) .NE. zero )) THEN
          
          !! \latexonly 
          !!     \input{enerbilflux18.tex}
          !! \endlatexonly 
          correction =  (quatre * emis(ji) * c_stefan * tair(ji)**3 + rau(ji) * qc * cp_air + &
               &                  chalev0 * rau(ji) * qc * grad_qsat(ji) * vevapp(ji) / evapot(ji) )

          !! \latexonly 
          !!     \input{enerbilflux19.tex}
          !! \endlatexonly 
          IF (ABS(correction) .GT. min_sechiba) THEN
             correction = chalev0 * rau(ji) * qc * grad_qsat(ji) * (un - vevapp(ji)/evapot(ji)) / correction
          ELSE
             warning_correction(ji)=.TRUE.
          ENDIF

       ELSE

          correction = zero

       ENDIF

       correction = MAX (zero, correction)

       !! \latexonly 
       !!     \input{enerbilflux20.tex}
       !! \endlatexonly 
       evapot_corr(ji) = evapot(ji) / (un + correction)

    ENDDO

    IF ( ANY(warning_correction) ) THEN
       DO ji=1,kjpindex
          IF ( warning_correction(ji) ) THEN
             WRITE(numout,*) ji,"Denumerator of the Milly Correction is zero, so no correction has been applied"
          ENDIF
       ENDDO
    ENDIF

    IF (long_print) WRITE (numout,*) ' enerbil_flux done '

  END SUBROUTINE jenerbil_flux


 !!  ===========================================================================================================================
  !! SUBROUTINE                 : enerbil_write
  !!
  !!
  !>\BRIEF                  Write a bunch of variables to the history file
  !!
  !! DESCRIPTION                : 
  !! Originally the routine enerbil_main was used for this, but that seemed to create
  !! problems with uninitialized variables when it was called after jdriver.
  !! \n
  !!
  !! RECENT CHANGE(S): None
  !!
  !! MAIN OUTPUT VARIABLE(S)    : None
  !!
  !! REFERENCE(S)       :
  !!
  !! FLOWCHART          :
  !! \n
  !_ ==============================================================================================================================
   
  SUBROUTINE enerbil_write (kjit, kjpindex, &
       & index, lwdown, temp_sol, &
!!$       & index, lwdown, ccanopy, temp_sol, &
       & temp_sol_new, evapot, evapot_corr, hist_id, hist2_id) 

    !! 0 Variable and parameter description

    !! 0.1 Input variables

    INTEGER(i_std), INTENT(in)                           :: kjit          !! Time step number (-)
    INTEGER(i_std), INTENT(in)                           :: kjpindex      !! Domain size (-)
    INTEGER(i_std),INTENT (in)                           :: hist_id       !! _Restart_ file and _history_ file identifier (-)
    INTEGER(i_std),INTENT (in)                           :: hist2_id      !! _history_ file 2 identifier (-)
    
    INTEGER(i_std),DIMENSION (kjpindex), INTENT (in)     :: index         !! Indeces of the points on the map (-)
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)        :: lwdown        !! Down-welling long-wave flux (W m^{-2})
!!$    REAL(r_std),DIMENSION (kjpindex), INTENT (in)        :: ccanopy       !! CO_2 concentration in the canopy (ppm)
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)        :: evapot        !! Soil Potential Evaporation (mm day^{-1})
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)        :: evapot_corr   !! Soil Potential Evaporation Correction (mm day^{-1})
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)        :: temp_sol      !! Soil temperature (K)
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)        :: temp_sol_new  !! New soil temperature (K)

    !! 0.2 Output variables
    
    !! 0.3 Modified variables
    
    
    !! 0.4 Local variables


!_ ================================================================================================================================
    
    
    
     !CALL xios_orchidee_send_field("netrad",netrad)
     CALL xios_orchidee_send_field("evapot",evapot*one_day/dt_sechiba)
     CALL xios_orchidee_send_field("evapot_alma",evapot/dt_sechiba)
     CALL xios_orchidee_send_field("evapot_corr",evapot_corr*one_day/dt_sechiba)
     CALL xios_orchidee_send_field("evapot_corr_alma",evapot_corr/dt_sechiba)
     CALL xios_orchidee_send_field("lwdown",lwabs)
     CALL xios_orchidee_send_field("lwnet",lwnet)
     CALL xios_orchidee_send_field("Qv",fluxsubli)
     CALL xios_orchidee_send_field("PotSurfT",temp_sol_pot)
     
     IF ( .NOT. almaoutput ) THEN
        !CALL histwrite_p(hist_id, 'netrad', kjit, netrad, kjpindex, index)
        CALL histwrite_p(hist_id, 'evapot', kjit, evapot, kjpindex, index)
        CALL histwrite_p(hist_id, 'evapot_corr', kjit, evapot_corr, kjpindex, index)
        CALL histwrite_p(hist_id, 'lwdown', kjit, lwabs,  kjpindex, index)
        CALL histwrite_p(hist_id, 'lwnet',  kjit, lwnet,  kjpindex, index)
        IF ( hist2_id > 0 ) THEN
           !CALL histwrite_p(hist2_id, 'netrad', kjit, netrad, kjpindex, index)
           CALL histwrite_p(hist2_id, 'evapot', kjit, evapot, kjpindex, index)
           CALL histwrite_p(hist2_id, 'evapot_corr', kjit, evapot_corr, kjpindex, index)
           CALL histwrite_p(hist2_id, 'lwdown', kjit, lwabs,  kjpindex, index)
           CALL histwrite_p(hist2_id, 'lwnet',  kjit, lwnet,  kjpindex, index)
        ENDIF
     ELSE
        CALL histwrite_p(hist_id, 'LWnet', kjit, lwnet, kjpindex, index)
        CALL histwrite_p(hist_id, 'Qv', kjit, fluxsubli, kjpindex, index)
        CALL histwrite_p(hist_id, 'PotEvap', kjit, evapot_corr, kjpindex, index)
        CALL histwrite_p(hist_id, 'PotEvapOld', kjit, evapot, kjpindex, index)
        CALL histwrite_p(hist_id, 'PotSurfT', kjit, temp_sol_pot, kjpindex, index)
        IF ( hist2_id > 0 ) THEN
           CALL histwrite_p(hist2_id, 'LWnet', kjit, lwnet, kjpindex, index)
           CALL histwrite_p(hist2_id, 'Qv', kjit, fluxsubli, kjpindex, index)
           CALL histwrite_p(hist2_id, 'PotEvap', kjit, evapot_corr, kjpindex, index)
        ENDIF
     ENDIF
     
    IF (long_print) WRITE (numout,*) ' enerbil_main Done '

  END SUBROUTINE enerbil_write

  ! --------------------------------------------------------------------------------------------------------

!! ADD HEADER

  SUBROUTINE reallocate_column(present_height, next_height, temp_atmos_pres_grid, q_atmos_pres_grid, temp_leaf_pres_grid,   &
                               &   jtheta, rho_veg, leaf_tks, rau, pad_deltah, kjpindex)

  
!!$    REAL, DIMENSION (:), ALLOCATABLE        :: present_height             ! present height in column (m)
!!$    REAL, DIMENSION (:), ALLOCATABLE        :: next_height                ! next height in column (m)
    REAL(r_std), DIMENSION (0:jnlvls)                                         :: present_height             ! present height above zero in column (m)
    REAL(r_std), DIMENSION (0:jnlvls)                                         :: next_height                ! next height above zero in column (m)
    REAL(r_std), DIMENSION (jnlvls)                                           :: delta_h_present             ! present box height in column (m)
    REAL(r_std), DIMENSION (jnlvls)                                           :: delta_h_next                ! next box height in column (m)
    REAL(r_std), DIMENSION (kjpindex, jnlvls)                                 :: temp_atmos_pres_grid
    REAL(r_std), DIMENSION (kjpindex, jnlvls)                                 :: q_atmos_pres_grid
    REAL(r_std), DIMENSION (kjpindex, jnlvls)                                 :: temp_leaf_pres_grid
    REAL(r_std), DIMENSION (jnlvls)                                           :: present_energy             ! energy stored in present box space (J)
    REAL(r_std), DIMENSION (jnlvls)                                           :: next_energy                ! energy stored in next box space (J)
    REAL(r_std)                                                               :: sum_next_energy            ! energy stored in the next step over the whole column
    REAL(r_std), DIMENSION (jnlvls_canopy)                                    :: present_energy_tree        ! energy stored in present vegetation box space (J)
    REAL(r_std), DIMENSION (jnlvls_canopy)                                    :: next_energy_tree           ! energy stored in next vegetation box space (J)
    INTEGER(i_std)                                                            :: i, j                       ! integers for counting
    INTEGER(i_std)                                                            :: m, n, l                    ! number of levels in each part of the canopy 
                                                                                                            !  (lower canopy, over canopy, above canopy)
    INTEGER(i_std)                                                            :: total_layers               ! total number of layers in model
    REAL(r_std)                                                               :: rho_veg                    ! temporary for number checking
    REAL(r_std)                                                               :: leaf_tks                   ! leaf thickness
    REAL(r_std), DIMENSION (jnlvls)                                           :: jtheta
    REAL(r_std), DIMENSION (kjpindex), INTENT (in)                            :: rau                        !! Air density (kg m^{-3})
    REAL(r_std), DIMENSION (0:jnlvls)                                         :: pad_deltah                 ! Plant Area Density multiplied by layer height
    INTEGER(i_std), INTENT(in)                                                :: kjpindex                   !! Domain size (-) 

    !_ ======================================================================================================================

    m = jnlvls_under ! 5 ! number of levels in the lower canopy (treated as empty space for now)
    n = jnlvls_canopy ! 5 ! number of levels in the over canopy (where the vegetation is)
    l = jnlvls_over ! 5 ! number of levels above the canopy, but below the interface point with LMDz 

    total_layers = m + n + l

    ! ALLOCATE ( present_height(0:total_layers) )
    ! ALLOCATE ( next_height(0:total_layers) )

    ! ALLOCATE ( present_energy(total_layers) )
    ! ALLOCATE ( next_energy(total_layers) )

    ! ALLOCATE ( present_energy_tree(n) )
    ! ALLOCATE ( next_energy_tree(n) )

    present_energy(:) = 1.0d0


    DO i=1, jnlvls
        delta_h_present(i) = present_height(i) - present_height(i-1)
    END DO !i=1, jnlvls

    DO i=1, jnlvls
        delta_h_next(i) = next_height(i) - next_height(i-1)
    END DO !i=1, jnlvls

    DO i=1, jnlvls
        present_energy(i) = delta_h_present(i) * cp_air * rau(1) * temp_atmos_pres_grid(1, i) ! temporary rau set to first pixel in series
    END DO !i=1, jnlvls

    DO i=1, jnlvls_canopy
        present_energy_tree(i) = delta_h_present(i) * rho_veg * leaf_tks * pad_deltah(i) * jtheta(i) * temp_leaf_pres_grid(1, i) ! temporary rau set to first pixel in series
    END DO !i=1, jnlvls



    ! initialise variables (just use simple ones here, whilst testing the arithmetic)
    ! present_energy(:) = 1.0d0
    next_energy(:) = 0.0d0
    sum_next_energy = 0.0d0
    next_energy_tree(:) = 0.0d0



    DO j = 0, total_layers           ! next step loop
      DO i = 0, total_layers       ! present step loop
        ! WRITE(numout,*) j,' ; ', next_height(j),' ; ', i, ' ; ', present_height(i)   
      END DO
    END DO


    DO i = 0, total_layers - 1           ! next step loop
   !     WRITE(numout, *) 'present level depth ', i, ' : ', present_height(i+1) - present_height(i)
    END DO

    DO i = 0, total_layers - 1          ! next step loop
!        WRITE(numout, *) 'next level depth ', i, ' : ', next_height(i+1) - next_height(i)
    END DO


    ! we re-allocate the energy in each atmospheric level (to reflect the new profile)
    ! I hope that the below catches all the possible variations

    DO j = 0, total_layers-1           ! next step loop
      DO i = 0, total_layers-1         ! present step loop
         IF ((present_height(i) .ge. next_height(j)) .and. (present_height(i) .lt. next_height(j+1))) THEN
            IF (present_height(i+1) .gt. next_height(j+1)) THEN
                next_energy(j+1) = next_energy(j+1) + &
                                   & present_energy(i+1) * (next_height(j+1) - present_height(i))  &
                                   & / (present_height(i+1) - present_height(i))
            ELSE
                next_energy(j+1) = next_energy(j+1) + present_energy(i+1)
            ENDIF
         ENDIF

         
         IF ((present_height(i) .lt. next_height(j)) .and. (present_height(i+1) .gt. next_height(j+1))) THEN
              next_energy(j+1) = next_energy(j+1) + &
                                  & present_energy(i+1) * (next_height(j+1) - next_height(j)) &
                                  & / (present_height(i+1) - present_height(i))             
         ENDIF

         IF ((present_height(i) .lt. next_height(j)) .and. (present_height(i+1) .le. next_height(j+1))) THEN
            IF (present_height(i+1) .gt. next_height(j)) THEN
               next_energy(j+1) = next_energy(j+1) + &
                                  &  present_energy(i+1) * (present_height(i+1) - next_height(j)) &
                                  &  / (present_height(i+1) - present_height(i))
            ENDIF
         ENDIF
      END DO ! i = 0, total_layers   ! next step loop
    END DO ! j = 0, total_layers     ! present step loop


    ! the heat storage in leaves is only for the layers in the upper canopy (at least for the moment)

    DO i = 0, n-1
    ! DO i = 0, total_layers-1
        next_energy_tree(i + 1) = ((next_height(m+i+1) - next_height(m+i)) / &
                                & (present_height(m+i+1) - present_height(m+i))) &
                                & * (present_energy_tree(i + 1)) 
    END DO ! i = 0, total_layers-1

    ! WRITE(numout, *) '140115d next_energy(:) is: ', next_energy
    ! WRITE(numout, *) '140115d next_energy_tree(:) is: ', next_energy_tree


    DO i=1, jnlvls
        temp_atmos_pres_grid(1, i) = present_energy(i) / ( delta_h_present(i) * cp_air * rau(1) ) 
        ! WRITE(numout, *) '140115f temp_atmos_pres_grid(1, i) is: ', temp_atmos_pres_grid(1, i)
    END DO !i=1, jnlvls

    DO i=1, jnlvls_canopy
        temp_leaf_pres_grid(1, i) = present_energy_tree(i) / ( delta_h_present(i) * rho_veg * leaf_tks * pad_deltah(i) * jtheta(i) )
        ! WRITE(numout, *) '140115f temp_leaf_pres_grid(1, i) is: ', temp_leaf_pres_grid(1, i)
    END DO !i=1, jnlvls


  !  DO i = 1, total_layers
  !    WRITE(numout,*) 'level: ', i, ' next_energy(j) is: ', next_energy(i)
  !  END DO ! i = 1, total_layers


  !  DO i = 1, m
  !    WRITE(numout,*) 'level: ', i, ' next_energy_tree(j) is: ', next_energy_tree(i)
  !  END DO ! i = 1, total_layers


  END SUBROUTINE reallocate_column 


END MODULE enerbil
